@@ -18,7 +18,7 @@ new_production(Grammar *g, char *name) {
     FREE(name);
     return p;
   }
-  p = MALLOC(sizeof(Production));
+  p = (Production*)MALLOC(sizeof(Production));
   memset(p, 0, sizeof(Production));
   vec_add(&g->productions, p);
   p->name = name;
@@ -117,6 +117,11 @@ unescape_term_string(Term *t) {
   for (ss = s = t->string; *s; s++) {
     if (*s == '\\') {
       switch (s[1]) {
+        case '\\':
+    if (t->kind == TERM_STRING)
+      { *ss = '\\'; s++; break; }
+    else
+      goto Ldefault;
 	case 'b': *ss = '\b'; s++; break;
 	case 'f': *ss = '\f'; s++; break;
 	case 'n': *ss = '\n'; s++; break;
@@ -226,7 +231,7 @@ new_utf8_char(Grammar *g, char *s, char *e, Rule *r) {
       utf32_code += base * (*e - 'A' + 10);
   }
   if (utf32_code < 0x80) {
-    utf8_code[0] = utf32_code;
+    utf8_code[0] = (int8)utf32_code;
     len = 1;
   } else if (utf32_code < 0x800) {
     utf8_code[0] = 0xc0 | ((utf32_code >> 6) & 0x1f);
@@ -335,7 +340,7 @@ add_declaration(Grammar *g, char *start, char *end, uint kind, uint line) {
 
 D_Pass *
 find_pass(Grammar *g, char *start, char *end) {
-  int i, l;
+  size_t i, l;
   while (*start && isspace_(*start)) start++;
   l = end - start;
   for (i = 0; i < g->passes.n; i++)
@@ -378,7 +383,7 @@ new_internal_production(Grammar *g, Production *p) {
   char *n = p ? p->name : " _synthetic";
   char *name = MALLOC(strlen(n) + 21);
   Production *pp = NULL, *tp = NULL, *ttp;
-  int i, found = 0;
+  uint i, found = 0;
   sprintf(name, "%s__%d", n, g->productions.n);
   pp = new_production(g, name);
   pp->internal = INTERNAL_HIDDEN;
@@ -510,8 +515,8 @@ finish_productions(Grammar *g) {
 }
 
 Production *
-lookup_production(Grammar *g, char *name, int l) {
-  int i;
+lookup_production(Grammar *g, char *name, size_t l) {
+  uint i;
   
   for (i = 0; i < g->productions.n; i++) {
     Production *pp = g->productions.v[i];
@@ -523,8 +528,8 @@ lookup_production(Grammar *g, char *name, int l) {
 }
 
 static Term *
-lookup_token(Grammar *g, char *name, int l) {
-  int i;
+lookup_token(Grammar *g, char *name, size_t l) {
+  uint i;
   
   for (i = 0; i < g->terminals.n; i++) {
     Term *t = g->terminals.v[i];
@@ -538,7 +543,7 @@ lookup_token(Grammar *g, char *name, int l) {
 
 static Term *
 unique_term(Grammar *g, Term *t) {
-  int i;
+  uint i;
   for (i = 0; i < g->terminals.n; i++) 
     if (t->kind == g->terminals.v[i]->kind && 
 	t->string_len == g->terminals.v[i]->string_len &&
@@ -554,7 +559,7 @@ unique_term(Grammar *g, Term *t) {
 
 static void
 compute_nullable(Grammar *g) {
-  int i, j, k, changed = 1;
+  uint i, j, k, changed = 1;
   Elem *e;
     
   /* ensure that the trivial case is the first cause */
@@ -592,7 +597,8 @@ compute_nullable(Grammar *g) {
 */
 static void
 resolve_grammar(Grammar *g) {
-  int i, j, k, l;
+  uint i, j, k;
+  size_t l;
   Production *p, *pp;
   Rule *r;
   Elem *e;
@@ -647,7 +653,7 @@ resolve_grammar(Grammar *g) {
 
 static void
 merge_identical_terminals(Grammar *g) {
-  int i, j, k;
+  uint i, j, k;
   Production *p;
   Rule *r;
   Elem *e;
@@ -713,7 +719,7 @@ struct EnumStr {
 
 static char *
 assoc_str(uint e) {
-  int i;
+  uint i;
 
   for (i = 0; i < sizeof(assoc_strings) / sizeof(assoc_strings[0]); i++)
     if (e == assoc_strings[i].e)
@@ -723,7 +729,7 @@ assoc_str(uint e) {
 
 void
 print_rule(Rule *r) {
-  int k;
+  uint k;
 
   printf("%s: ", r->prod->name);
   for (k = 0; k < r->elems.n; k++)
@@ -782,7 +788,7 @@ print_grammar(Grammar *g) {
 
 static void
 print_item(Item *i) {
-  int j, end = 1;
+  uint j, end = 1;
 
   printf("\t%s: ", i->rule->prod->name);
   for (j = 0; j < i->rule->elems.n; j++) {
@@ -810,7 +816,8 @@ print_conflict(char *kind, int *conflict) {
 
 static void
 print_state(State *s) {
-  int j, conflict = 0;
+  uint j;
+  int conflict = 0;
 
   printf("STATE %d (%d ITEMS)%s\n", s->index, s->items.n,
 	 s->accept ? " ACCEPT" : "");
@@ -848,15 +855,15 @@ print_state(State *s) {
 
 void
 print_states(Grammar *g) {
-  int i;
+  uint i;
 
   for (i = 0; i < g->states.n; i++)
     print_state(g->states.v[i]);
 }
 
 int
-state_for_declaration(Grammar *g, int iproduction) {
-  int i;
+state_for_declaration(Grammar *g, uint iproduction) {
+  uint i;
   for (i = 0; i < g->declarations.n; i++)
     if (g->declarations.v[i]->kind == DECLARE_STATE_FOR &&
 	g->declarations.v[i]->elem->e.nterm->index == iproduction)
@@ -866,7 +873,7 @@ state_for_declaration(Grammar *g, int iproduction) {
 
 static void
 make_elems_for_productions(Grammar *g) {
-  int i, j, k, l;
+  uint i, j, k, l;
   Rule *rr;
   Production *pp, *ppp;
 
@@ -903,14 +910,14 @@ make_elems_for_productions(Grammar *g) {
 
 static void
 convert_regex_production_one(Grammar *g, Production *p) {
-  int j, k, l;
+  uint j, k, l;
   Production *pp;
   Rule *r, *rr;
   Elem *e;
   Term *t;
   int circular = 0;
   char *buf = 0, *b, *s;
-  int buf_len = 0;
+  size_t buf_len = 0;
 
   if (p->regex_term) /* already done */
     return;
@@ -1019,7 +1026,7 @@ convert_regex_production_one(Grammar *g, Production *p) {
 
 static void
 convert_regex_productions(Grammar *g) {
-  int i, j, k;
+  uint i, j, k;
   Production *p;
   Rule *r;
 
@@ -1060,7 +1067,8 @@ typedef struct {
 
 void
 build_eq(Grammar *g) {
-  int i, j, k, changed = 1, x, xx;
+  uint i, j, k;
+  int changed = 1, x, xx;
   State *s, *ss;
   EqState *eq, *e, *ee;
 
@@ -1182,7 +1190,7 @@ new_D_Grammar(char *pathname) {
 
 static void
 free_rule(Rule *r) {
-  int i;
+  uint i;
   FREE(r->end);
   if (r->final_code.code)
     FREE(r->final_code.code);
@@ -1199,7 +1207,7 @@ free_rule(Rule *r) {
 
 void
 free_D_Grammar(Grammar *g) {
-  int i, j, k;
+  uint i, j, k;
 
   for (i = 0; i < g->productions.n; i++) {
     Production *p = g->productions.v[i];
@@ -1337,7 +1345,7 @@ scanner_declaration(Declaration *d) {
 
 static void
 set_declaration_group(Production *p, Production *root, Declaration *d) {
-  int i, j;
+  uint i, j;
   if (p->declaration_group[d->kind] == root)
     return;
   if (d->kind == DECLARE_TOKENIZE && p->declaration_group[d->kind]) {
@@ -1355,7 +1363,7 @@ set_declaration_group(Production *p, Production *root, Declaration *d) {
 
 static void
 propogate_declarations(Grammar *g) {
-  int i, j, k;
+  uint i, j, k;
   Production *p, *start = g->productions.v[0];
   Rule *r;
   Elem *e;
@@ -1423,7 +1431,7 @@ propogate_declarations(Grammar *g) {
 
 static void
 merge_shift_actions(State *to, State *from) {
-  int i, j;
+  uint i, j;
   for (i = 0; i < from->shift_actions.n; i++) {
     for (j = 0; j < to->shift_actions.n; j++)
       if (from->shift_actions.v[i]->term == to->shift_actions.v[j]->term)
@@ -1436,7 +1444,9 @@ merge_shift_actions(State *to, State *from) {
 static void
 compute_declaration_states(Grammar *g, Production *p, Declaration *d) {
   State *s, *base_s = NULL;
-  int j, k, scanner = scanner_declaration(d);
+  uint j, k;
+  int scanner = scanner_declaration(d);
+  (void)p;
 
   for (j = 0; j < g->states.n; j++) {
     s = g->states.v[j];
@@ -1475,7 +1485,7 @@ compute_declaration_states(Grammar *g, Production *p, Declaration *d) {
 
 static void
 map_declarations_to_states(Grammar *g) {
-  int i;
+  uint i;
   State *s;
   
   for (i = 0; i < g->states.n; i++) {
@@ -1572,7 +1582,7 @@ print_element_escaped(Elem *ee, int double_escaped) {
 
 static void
 print_global_code(Grammar *g) {
-  int i;
+  uint i;
   int print_stdio_h = 1;
   printf("%%<");
   for (i = 0; i < g->ncode; i++) {
@@ -1665,7 +1675,7 @@ static void print_declare(char *s, char *n) {
 
 static void
 print_declarations(Grammar *g) {
-  int i;
+  uint i;
 
   if (g->tokenizer)
     printf("${declare tokenize}\n");
