@@ -42,7 +42,7 @@ static void free_PNode(Parser *p, PNode *pn);
 
 void
 print_paren(Parser *pp, PNode *p) {
-  int i;
+  uint i;
   char *c;
   LATEST(pp, p);
   if (!p->error_recovery) {
@@ -64,11 +64,11 @@ print_paren(Parser *pp, PNode *p) {
 
 void
 xprint_paren(Parser *pp, PNode *p) {
-  int i;
+  uint i;
   char *c;
   LATEST(pp, p);
   if (!p->error_recovery) {
-    printf("[%p %s]", p, pp->t->symbols[p->parse_node.symbol].name);
+    printf("[%p %s]", (void*)p, pp->t->symbols[p->parse_node.symbol].name);
     if (p->children.n) {
       printf("(");
       for (i = 0; i < p->children.n; i++)
@@ -99,7 +99,7 @@ void PP(Parser *pp, PNode *p) { print_paren(pp, p); printf("\n"); }
 D_ParseNode *
 d_get_child(D_ParseNode *apn, int child) {
   PNode *pn = D_ParseNode_to_PNode(apn);
-  if (child < 0 || child >= pn->children.n)
+  if (child < 0 || child >= (long int)pn->children.n)
     return NULL;
   return &pn->children.v[child]->parse_node;
 }
@@ -114,7 +114,7 @@ D_ParseNode *
 d_find_in_tree(D_ParseNode *apn, int symbol) {
   PNode *pn = D_ParseNode_to_PNode(apn);
   D_ParseNode *res;
-  int i;
+  uint i;
 
   if (pn->parse_node.symbol == symbol)
     return apn;
@@ -127,25 +127,27 @@ d_find_in_tree(D_ParseNode *apn, int symbol) {
 char *
 d_ws_before(D_Parser *ap, D_ParseNode *apn) {
   PNode *pn = D_ParseNode_to_PNode(apn);
+  (void)ap;
   return pn->ws_before;
 }
 
 char *
 d_ws_after(D_Parser *ap, D_ParseNode *apn) {
   PNode *pn = D_ParseNode_to_PNode(apn);
+  (void)ap;
   return pn->ws_after;
 }
 
 #define SNODE_HASH(_s, _sc, _g) ((((uintptr_t)(_s)) << 12) + (((uintptr_t)(_sc))) + ((uintptr_t)(_g)))
 
 SNode *
-find_SNode(Parser *p, uint state, D_Scope *sc, void *g) {
+find_SNode(Parser *p, size_t state, D_Scope *sc, void *g) {
   SNodeHash *ph = &p->snode_hash;
   SNode *sn;
-  uint h = SNODE_HASH(state, sc, g);
+  uintptr_t h = SNODE_HASH(state, sc, g);
   if (ph->v)
     for (sn = ph->v[h % ph->m]; sn; sn = sn->bucket_next)
-      if (sn->state - p->t->state == state &&
+      if (sn->state == state + p->t->state &&
 	  sn->initial_scope == sc &&
 	  sn->initial_globals == g)
 	return sn;
@@ -155,12 +157,12 @@ find_SNode(Parser *p, uint state, D_Scope *sc, void *g) {
 void
 insert_SNode_internal(Parser *p, SNode *sn) {
   SNodeHash *ph = &p->snode_hash;
-  uint h = SNODE_HASH(sn->state - p->t->state, sn->initial_scope, sn->initial_globals), i;
+  uintptr_t h = SNODE_HASH(sn->state - p->t->state, sn->initial_scope, sn->initial_globals), i;
   SNode *t;
 
   if (ph->n + 1 > ph->m) {
     SNode **v = ph->v;
-    int m = ph->m;
+    uint m = ph->m;
     ph->i++;
     ph->m = d_prime2[ph->i];
     ph->v = (SNode**)MALLOC(ph->m * sizeof(*ph->v));
@@ -190,7 +192,7 @@ static SNode *
 new_SNode(Parser *p, D_State *state, d_loc_t *loc, D_Scope *sc, void *g) {
   SNode *sn = p->free_snodes;
   if (!sn)
-    sn = MALLOC(sizeof *sn);
+    sn = (SNode*)MALLOC(sizeof *sn);
   else
     p->free_snodes = sn->all_next;
   sn->depth = 0;
@@ -223,7 +225,7 @@ static ZNode *
 new_ZNode(Parser *p, PNode *pn) {
   ZNode *z = p->free_znodes;
   if (!z)
-    z = MALLOC(sizeof *z);
+    z = (ZNode*)MALLOC(sizeof *z);
   else
     p->free_znodes = znode_next(z);
   z->pn = pn;
@@ -235,7 +237,7 @@ new_ZNode(Parser *p, PNode *pn) {
 static void
 free_PNode(Parser *p, PNode *pn) {
   PNode *amb;
-  int i;
+  uint i;
   if (p->user.free_node_fn)
     p->user.free_node_fn(&pn->parse_node);
   for (i = 0; i < pn->children.n; i++)
@@ -268,7 +270,7 @@ free_PNode(Parser *p, PNode *pn) {
 #ifndef USE_GC
 static void
 free_ZNode(Parser *p, ZNode *z, SNode *s) {
-  int i;
+  uint i;
   unref_pn(p, z->pn);
   for (i = 0; i < z->sns.n; i++)
     if (s != z->sns.v[i])
@@ -284,7 +286,7 @@ free_ZNode(Parser *p, ZNode *z, SNode *s) {
 
 static void
 free_SNode(Parser *p, struct SNode *s) {
-  int i;
+  uint i;
   for (i = 0; i < s->zns.n; i++)
     if (s->zns.v[i])
       free_ZNode(p, s->zns.v[i], s);
@@ -306,10 +308,10 @@ free_SNode(Parser *p, struct SNode *s) {
 ((((uintptr_t)_si) << 8) + (((uintptr_t)_ei) << 16) + (((uintptr_t)_s)) + (((uintptr_t)_sc)) + (((uintptr_t)_g)))
 
 PNode *
-find_PNode(Parser *p, char *start, char *end_skip, int symbol, D_Scope *sc, void *g, uint *hash) {
+find_PNode(Parser *p, char *start, char *end_skip, int symbol, D_Scope *sc, void *g, uintptr_t *hash) {
   PNodeHash *ph = &p->pnode_hash;
   PNode *pn;
-  uint h = PNODE_HASH(start, end_skip, symbol, sc, g);
+  uintptr_t h = PNODE_HASH(start, end_skip, symbol, sc, g);
   *hash = h;
   if (ph->v)
     for (pn = ph->v[h % ph->m]; pn; pn = pn->bucket_next)
@@ -328,13 +330,13 @@ find_PNode(Parser *p, char *start, char *end_skip, int symbol, D_Scope *sc, void
 void
 insert_PNode_internal(Parser *p, PNode *pn) {
   PNodeHash *ph = &p->pnode_hash;
-  uint h = PNODE_HASH(pn->parse_node.start_loc.s, pn->parse_node.end_skip,
+  uintptr_t h = PNODE_HASH(pn->parse_node.start_loc.s, pn->parse_node.end_skip,
 		      pn->parse_node.symbol, pn->initial_scope, pn->initial_globals), i;
   PNode *t;
 
   if (ph->n + 1 > ph->m) {
     PNode **v = ph->v;
-    int m = ph->m;
+    uint m = ph->m;
     ph->i++;
     ph->m = d_prime2[ph->i];
     ph->v = (PNode**)MALLOC(ph->m * sizeof(*ph->v));
@@ -361,8 +363,8 @@ insert_PNode(Parser *p, PNode *pn) {
 
 static void
 free_old_nodes(Parser *p) {
-  int i;
-  uint h;
+  uint i;
+  uintptr_t h;
   PNode *pn = p->pnode_hash.all, *tpn, **lpn;
   SNode *sn = p->snode_hash.all, *tsn, **lsn;
   while (sn) {
@@ -419,7 +421,7 @@ alloc_parser_working_data(Parser *p) {
 
 static void
 free_parser_working_data(Parser *p) {
-  int i;
+  uint i;
 
   free_old_nodes(p);
   free_old_nodes(p); /* to catch SNodes saved for error repair */
@@ -472,9 +474,9 @@ free_parser_working_data(Parser *p) {
   p->ncode_shifts = 0;
 }
 
-static int
+static uint
 znode_depth(ZNode *z) {
-  int i, d = 0;
+  uint i, d = 0;
   if (!z)
     return INT_MAX;
   for (i = 0; i < z->sns.n; i++)
@@ -503,7 +505,7 @@ add_Reduction(Parser *p, ZNode *z, SNode *sn, D_Reduction *reduction) {
   {
     Reduction *r = p->free_reductions;
     if (!r)
-      r = MALLOC(sizeof *r);
+      r = (Reduction*)MALLOC(sizeof *r);
     else
       p->free_reductions = r->next;
     r->znode = z;
@@ -522,7 +524,7 @@ add_Shift(Parser *p, SNode *snode) {
   Shift *x, **l = &p->shifts_todo;
   Shift *s = p->free_shifts;
   if (!s)
-    s = MALLOC(sizeof *s);
+    s = (Shift*)MALLOC(sizeof *s);
   else
     p->free_shifts = s->next;
   s->snode = snode;
@@ -535,7 +537,7 @@ add_Shift(Parser *p, SNode *snode) {
 
 static SNode *
 add_SNode(Parser *p, D_State *state, d_loc_t *loc, D_Scope *sc, void *g) {
-  int i;
+  uint i;
   SNode *sn = find_SNode(p, state - p->t->state, sc, g);
   if (sn)
     return sn;
@@ -550,7 +552,7 @@ add_SNode(Parser *p, D_State *state, d_loc_t *loc, D_Scope *sc, void *g) {
 
 static int
 reduce_actions(Parser *p, PNode *pn, D_Reduction *r) {
-  int i, height = 0;
+  uint i, height = 0;
   PNode *c;
 
   for (i = 0; i < pn->children.n; i++) {
@@ -619,6 +621,7 @@ check_child(int ppri, AssocKind passoc, int cpri, AssocKind cassoc,
       cpri < ppri ? 1 : ( 2 + (
 	(IS_RIGHT_ASSOC(cassoc) ? 2 : 0) +
 	(IS_RIGHT_ASSOC(passoc) ? 1 : 0))));
+  (void)left; /* WHY?! */
   return child_table[p][c][r];
 }
 
@@ -661,7 +664,8 @@ check_assoc_priority(PNode *pn0, PNode *pn1, PNode *pn2) {
    the associativity and priority of its operators */
 static int
 check_path_priorities_internal(VecZNode *path) {
-  int i = 0, j, k, jj, kk, one = 0;
+  uint i = 0, j, k, jj, kk;
+  int one = 0;
   ZNode *z, *zz, *zzz;
   PNode *pn0, *pn1;
 
@@ -723,8 +727,8 @@ check_path_priorities_internal(VecZNode *path) {
    check_path_priorities_internal(_p))
 
 static int
-compare_priorities(int xpri[], int xn, int ypri[], int yn) {
-  int i = 0;
+compare_priorities(int xpri[], size_t xn, int ypri[], size_t yn) {
+  size_t i = 0;
 
   while (i < xn && i < yn) {
     if (xpri[i] > ypri[i])
@@ -737,7 +741,7 @@ compare_priorities(int xpri[], int xn, int ypri[], int yn) {
 }
 
 static void
-intreverse(int *xp, int n) {
+intreverse(int *xp, size_t n) {
   int *a = xp, *b = xp + n -1;
   while (a < b) {
     int t = *a;
@@ -769,7 +773,7 @@ priority_insert(StackPNode *psx, PNode *x) {
 
 static void
 get_exp_all(Parser *p, PNode *x, StackInt *psx) {
-  int i;
+  uint i;
 
   if (x->assoc)
     stack_push(psx, x->priority);
@@ -782,7 +786,7 @@ get_exp_all(Parser *p, PNode *x, StackInt *psx) {
 
 static void
 get_exp_one(Parser *p, PNode *x, StackPNode *psx, StackInt *isx) {
-  int i;
+  uint i;
 
   LATEST(p, x);
   if (!IS_NARY_ASSOC(x->assoc))
@@ -797,7 +801,7 @@ get_exp_one(Parser *p, PNode *x, StackPNode *psx, StackInt *isx) {
 
 static void
 get_exp_one_down(Parser *p, PNode *x, StackPNode *psx, StackInt *isx) {
-  int i;
+  uint i;
 
   LATEST(p, x);
   stack_push(isx, x->priority);
@@ -875,7 +879,7 @@ cmp_priorities(Parser *p, PNode *x, PNode *y) {
 
 static void
 get_all(Parser *p, PNode *x, VecPNode *vx) {
-  int i;
+  uint i;
   if (set_add(vx, x)) {
     for (i = 0; i < x->children.n; i++) {
       PNode *pn = x->children.v[i];
@@ -887,7 +891,7 @@ get_all(Parser *p, PNode *x, VecPNode *vx) {
 
 static void
 get_unshared_pnodes(Parser *p, PNode *x, PNode *y, VecPNode *pvx, VecPNode *pvy) {
-  int i;
+  uint i;
   VecPNode vx, vy;
   vec_clear(&vx); vec_clear(&vy);
   LATEST(p, x); LATEST(p, y);
@@ -906,17 +910,17 @@ static int
 greedycmp(const void *ax, const void *ay) {
   PNode *x = *(PNode**)ax;
   PNode *y = *(PNode**)ay;
-  // first by start
+  /* first by start */
   if (x->parse_node.start_loc.s < y->parse_node.start_loc.s)
     return -1;
   if (x->parse_node.start_loc.s > y->parse_node.start_loc.s)
     return 1;
-  // second by symbol
+  /* second by symbol */
   if (x->parse_node.symbol < y->parse_node.symbol)
     return -1;
   if (x->parse_node.symbol > y->parse_node.symbol)
     return 1;
-  // third by length
+  /* third by length */
   if (x->parse_node.end < y->parse_node.end)
     return -1;
   if (x->parse_node.end > y->parse_node.end)
@@ -929,7 +933,8 @@ greedycmp(const void *ax, const void *ay) {
 static int
 cmp_greediness(Parser *p, PNode *x, PNode *y) {
   VecPNode pvx, pvy;
-  int ix = 0, iy = 0, ret = 0;
+  uint ix = 0, iy = 0;
+  int ret = 0;
   vec_clear(&pvx); vec_clear(&pvy);
   get_unshared_pnodes(p, x, y, &pvx, &pvy);
   /* set_to_vec(&pvx); set_to_vec(&pvy); */
@@ -1002,10 +1007,10 @@ cmp_pnodes(Parser *p, PNode *x, PNode *y) {
 }
 
 static PNode *
-make_PNode(Parser *p, uint hash, int symbol, d_loc_t *start_loc, char *e, PNode *pn,
+make_PNode(Parser *p, uintptr_t hash, int symbol, d_loc_t *start_loc, char *e, PNode *pn,
 	   D_Reduction *r, VecZNode *path, D_Shift *sh, D_Scope *scope)
 {
-  int i, l = sizeof(PNode) - sizeof(d_voidp) + p->user.sizeof_user_parse_node;
+  uint i, l = (uint)(sizeof(PNode) - sizeof(d_voidp) + p->user.sizeof_user_parse_node);
   PNode *new_pn = p->free_pnodes;
   if (!new_pn)
     new_pn = MALLOC(l);
@@ -1056,8 +1061,8 @@ make_PNode(Parser *p, uint hash, int symbol, d_loc_t *start_loc, char *e, PNode
     }
   } else if (r) {
     if (path)
-      for (i = path->n - 1; i >= 0; i--) {
-	PNode *latest = path->v[i]->pn;
+      for (i = path->n; i > 0; ) {
+	PNode *latest = path->v[--i]->pn;
 	LATEST(p, latest);
    ref_pn(latest);
 	vec_add(&new_pn->children, latest);
@@ -1067,12 +1072,12 @@ make_PNode(Parser *p, uint hash, int symbol, d_loc_t *start_loc, char *e, PNode
       return NULL;
     }
     if (path && path->n > 1) {
-      int n = path->n;
+      uint n = path->n;
       for (i = 0; i < n; i += n-1) {
 	PNode *child = new_pn->children.v[i];
 	if (child->assoc &&
 	    !check_child(new_pn->priority, new_pn->assoc,
-			 child->priority, child->assoc, i == 0, i == n - 1))
+			 child->priority, child->assoc, i == 0, i == (uint)(n - 1)))
 	{
 	  free_PNode(p, new_pn);
 	  return NULL;
@@ -1085,7 +1090,7 @@ make_PNode(Parser *p, uint hash, int symbol, d_loc_t *start_loc, char *e, PNode
 
 static int
 PNode_equal(Parser *p, PNode *pn, D_Reduction *r, VecZNode *path, D_Shift *sh) {
-  int i, n = pn->children.n;
+  uint i, n = pn->children.n;
   if (sh)
     return sh == pn->shift;
   if (r != pn->reduction)
@@ -1111,7 +1116,7 @@ add_PNode(Parser *p, int symbol, d_loc_t *start_loc, char *e, PNode *pn,
 	  D_Reduction *r, VecZNode *path, D_Shift *sh)
 {
   D_Scope *scope = equiv_D_Scope(pn->parse_node.scope);
-  uint hash;
+  uintptr_t hash;
   PNode *old_pn = find_PNode(p, start_loc->s, e, symbol, scope, pn->parse_node.globals, &hash),
     *new_pn;
   if (old_pn) {
@@ -1165,7 +1170,7 @@ static void set_add_znode(VecZNode *v, ZNode *z);
 
 static void
 set_union_znode(VecZNode *v, VecZNode *vv) {
-  int i;
+  uint i;
   for (i = 0; i < vv->n; i++)
     if (vv->v[i])
       set_add_znode(v, vv->v[i]);
@@ -1195,7 +1200,7 @@ set_find_znode(VecZNode *v, PNode *pn) {
 
 static void
 set_add_znode_hash(VecZNode *v, ZNode *z) {
-  int i, j, n = v->n;
+  uint i, j, n = v->n;
   VecZNode vv;
   vec_clear(&vv);
   if (n) {
@@ -1254,7 +1259,7 @@ goto_PNode(Parser *p, d_loc_t *loc, PNode *pn, SNode *ps) {
   SNode *new_ps, *pre_ps;
   ZNode *z = NULL;
   D_State *state;
-  int i, j, k, state_index;
+  uint i, j, k, state_index;
 
   if (!IS_BIT_SET(ps->state->goto_valid, pn->parse_node.symbol))
     return NULL;
@@ -1320,11 +1325,12 @@ parse_whitespace(D_Parser *ap, d_loc_t *loc, void **p_globals) {
       pp->accept = NULL;
     }
   }
+  (void)p_globals;
 }
 
 static void
 shift_all(Parser *p, char *pos) {
-  int i, j, nshifts = 0, ncode = 0;
+  uint i, j, nshifts = 0, ncode = 0;
   d_loc_t loc, skip_loc;
   D_WhiteSpaceFn skip_fn = NULL;
   PNode *new_pn;
@@ -1364,7 +1370,7 @@ shift_all(Parser *p, char *pos) {
       }
     }
     if (state->scanner_table) {
-      int n = scan_buffer(&loc, state, &p->shift_results[nshifts]);
+      uint n = scan_buffer(&loc, state, &p->shift_results[nshifts]);
       for (i = 0; i < n; i++)
 	p->shift_results[nshifts + i].snode = s->snode;
       nshifts += n;
@@ -1438,8 +1444,8 @@ shift_all(Parser *p, char *pos) {
 static VecZNode path1; /* static first path for speed */
 
 static VecZNode *
-new_VecZNode(VecVecZNode *paths, int n, int parent) {
-  int i;
+new_VecZNode(VecVecZNode *paths, uint n, int parent) {
+  uint i;
   VecZNode *pv;
 
   if (!paths->n)
@@ -1457,7 +1463,7 @@ static void
 build_paths_internal(ZNode *z, VecVecZNode *paths, int parent,
 		     int n, int n_to_go)
 {
-  int j, k, l;
+  uint j, k, l;
 
   vec_add(paths->v[parent], z);
   if (n_to_go <= 1)
@@ -1486,7 +1492,7 @@ build_paths(ZNode *z, VecVecZNode *paths, int nchildren_to_go) {
 
 static void
 free_paths(VecVecZNode *paths) {
-  int i;
+  uint i;
   vec_free(&path1);
   for (i = 1; i < paths->n; i++) {
     vec_free(paths->v[i]);
@@ -1500,7 +1506,7 @@ reduce_one(Parser *p, Reduction *r) {
   SNode *sn = r->snode;
   PNode *pn, *last_pn;
   ZNode *first_z;
-  int i, j, n = r->reduction->nelements;
+  uint i, j, n = r->reduction->nelements;
   VecVecZNode paths;
   VecZNode *path;
 
@@ -1542,7 +1548,7 @@ reduce_one(Parser *p, Reduction *r) {
 
 static int
 VecSNode_equal(VecSNode *vsn1, VecSNode *vsn2) {
-  int i, j;
+  uint i, j;
   if (vsn1->n != vsn2->n)
     return 0;
   for (i = 0; i < vsn1->n; i++) {
@@ -1667,7 +1673,7 @@ cmp_stacks(Parser *p) {
 
 static void
 free_ParseTreeBelow(Parser *p, PNode *pn) {
-  int i;
+  uint i;
   PNode *amb;
 
   for (i = 0; i < pn->children.n; i++)
@@ -1707,7 +1713,7 @@ ambiguity_abort_fn(D_Parser *pp, int n, D_ParseNode **v) {
 
 static int
 final_actionless(PNode *pn) {
-  int i;
+  uint i;
   if (pn->reduction && pn->reduction->final_code)
     return 0;
   for (i = 0; i < pn->children.n; i++)
@@ -1727,7 +1733,8 @@ resolve_ambiguities(Parser *p, PNode *pn) {
   efa = is_epsilon_PNode(pn) && final_actionless(pn);
   vec_add(&pns, &pn->parse_node);
   for (amb = pn->ambiguities; amb; amb = amb->ambiguities) {
-    int i, found = 0;
+    uint i; 
+    int found = 0;
     LATEST(p, amb);
     if (!p->user.dont_merge_epsilon_trees)
       if (efa && is_epsilon_PNode(amb) && final_actionless(amb))
@@ -1785,7 +1792,8 @@ fixup_internal_symbol(Parser *p, PNode *pn, int ichild) {
 
 static PNode *
 commit_tree(Parser *p, PNode *pn) {
-  int i, fixup_ebnf = 0, fixup = 0, internal = 0;
+  uint i;
+  int fixup_ebnf = 0, fixup = 0, internal = 0;
   LATEST(p, pn);
   if (pn->evaluated)
     return pn;
@@ -1813,7 +1821,9 @@ commit_tree(Parser *p, PNode *pn) {
     }
   }
   if (pn->reduction)
+  {
     DBG(printf("commit %p (%s)\n", pn, p->t->symbols[pn->parse_node.symbol].name));
+  }
   if (pn->reduction && pn->reduction->final_code)
     pn->reduction->final_code(
       pn, (void**)&pn->children.v[0], pn->children.n,
@@ -1850,7 +1860,7 @@ commit_stack(Parser *p, SNode *sn) {
 
 static const char *
 find_substr(const char *str, const char *s) {
-  int len = strlen(s);
+  size_t len = strlen(s);
   if (len == 1) {
     while (*str && *str != *s) str++;
     if (*str == *s)
@@ -1892,10 +1902,11 @@ static int
 error_recovery(Parser *p) {
   SNode *sn, *best_sn = NULL;
   const char *best_s = NULL, *ss, *s;
-  int i, j, head = 0, tail = 0, res = 1;
+  uint i, j;
+  int head = 0, tail = 0, res = 1;
   D_ErrorRecoveryHint *best_er = NULL;
   SNode **q = 0;
-  PNode *best_pn;
+  PNode *best_pn = NULL;
 
   if (!p->snode_hash.last_all)
     return res;
@@ -2004,7 +2015,8 @@ pass_call(Parser *p, D_Pass *pp, PNode *pn) {
 
 static void
 pass_preorder(Parser *p, D_Pass *pp, PNode *pn) {
-  int found = PASS_CODE_FOUND(pp, pn), i;
+  int found = PASS_CODE_FOUND(pp, pn);
+  uint i;
   pass_call(p, pp, pn);
   if ((pp->kind & D_PASS_FOR_ALL) ||
       ((pp->kind & D_PASS_FOR_UNDEFINED) && !found))
@@ -2014,7 +2026,8 @@ pass_preorder(Parser *p, D_Pass *pp, PNode *pn) {
 
 static void
 pass_postorder(Parser *p, D_Pass *pp, PNode *pn) {
-  int found = PASS_CODE_FOUND(pp, pn), i;
+  int found = PASS_CODE_FOUND(pp, pn);
+  uint i;
   if ((pp->kind & D_PASS_FOR_ALL) ||
       ((pp->kind & D_PASS_FOR_UNDEFINED) && !found))
     for (i = 0; i < pn->children.n; i++)
@@ -2023,7 +2036,7 @@ pass_postorder(Parser *p, D_Pass *pp, PNode *pn) {
 }
 
 void
-d_pass(D_Parser *ap, D_ParseNode *apn, int pass_number) {
+d_pass(D_Parser *ap, D_ParseNode *apn, uint pass_number) {
   PNode *pn = D_ParseNode_to_PNode(apn);
   Parser *p = (Parser*)ap;
   D_Pass *pp;
@@ -2127,6 +2140,8 @@ static void
 white_space(D_Parser *p, d_loc_t *loc, void **p_user_globals) {
   int rec = 0;
   char *s = loc->s, *scol = 0;
+  (void)p_user_globals;
+  (void)p;
 
   if (*s == '#' && loc->col == 0) {
   Ldirective:
@@ -2196,14 +2211,19 @@ white_space(D_Parser *p, d_loc_t *loc, void **p_user_globals) {
   }
  Ldone:
   if (scol)
-    loc->col = s - scol;
+    loc->col = (int)(s - scol);
   else
-    loc->col += s - loc->s;
+    loc->col += (int)(s - loc->s);
   loc->s = s;
   return;
 }
 
-void null_white_space(D_Parser *p, d_loc_t *loc, void **p_globals) { }
+void null_white_space(D_Parser *p, d_loc_t *loc, void **p_globals) 
+{ 
+  (void)p;
+  (void)loc;
+  (void)p_globals;
+}
 
 D_Parser *
 new_D_Parser(D_ParserTables *t, int sizeof_ParseNode_User) {
@@ -2286,7 +2306,7 @@ free_whitespace_parser(Parser *p) {
 
 static PNode *
 handle_top_level_ambiguities(Parser *p, SNode *sn) {
-  int i;
+  uint i;
   ZNode *z = 0;
   PNode *pn = NULL, *last = NULL, *x;
   for (i = 0; i < sn->zns.n; i++) {
@@ -2314,7 +2334,7 @@ handle_top_level_ambiguities(Parser *p, SNode *sn) {
 }
 
 D_ParseNode *
-dparse(D_Parser *ap, char *buf, int buf_len) {
+dparse(D_Parser *ap, char *buf, size_t buf_len) {
   int r;
   Parser *p = (Parser *)ap;
   SNode *sn;
