@@ -23,11 +23,6 @@
 #ifndef __BLOCKLIST_H
 #define __BLOCKLIST_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-
   typedef unsigned short Bool;
   typedef struct HFnode* HFcursor;
 
@@ -42,10 +37,6 @@ extern "C" {
   typedef tBlock* BList;
 
 
-
-
-
-
   void BLinit(BList*);
 
 #define BLfirst(list) (list->first)
@@ -70,8 +61,4 @@ extern "C" {
   unsigned long Bfrequency(tBlock*);
   void Bfree(tBlock*);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
