@@ -12,7 +12,7 @@
 #ifndef MCRL2_ATERMPP_ALGORITHM_H
 #define MCRL2_ATERMPP_ALGORITHM_H
 
-#include "boost/type_traits/add_reference.hpp"
+#include <type_traits>
 
 #include "mcrl2/atermpp/aterm.h"
 #include "mcrl2/atermpp/aterm_appl.h"
@@ -30,7 +30,7 @@ namespace atermpp
 template <typename UnaryFunction, typename Term>
 UnaryFunction for_each(Term t, UnaryFunction op)
 {
-  return detail::for_each_impl< typename boost::add_reference< UnaryFunction >::type >(aterm_traits<Term>::term(t), op);
+  return detail::for_each_impl< typename std::add_lvalue_reference< UnaryFunction >::type >(t, op);
 }
 
 /// \brief Finds a subterm of t that matches a given predicate.
@@ -38,10 +38,10 @@ UnaryFunction for_each(Term t, UnaryFunction op)
 /// \param match The predicate that determines if a subterm is a match
 /// \return A subterm that matches the given predicate, or aterm_appl() if none was found.
 template <typename Term, typename MatchPredicate>
-aterm_appl find_if(Term t, MatchPredicate match)
+aterm_appl find_if(const Term &t, MatchPredicate match)
 {
   aterm_appl output;
-  detail::find_if_impl< typename boost::add_reference< MatchPredicate >::type >(t, match, output);
+  detail::find_if_impl< typename std::add_lvalue_reference< MatchPredicate >::type >(t, match, output);
   return output;
 }
 
@@ -55,15 +55,7 @@ aterm_appl find_if(Term t, MatchPredicate match)
 template <typename Term, typename MatchPredicate, typename StopPredicate>
 aterm_appl partial_find_if(Term t, MatchPredicate match, StopPredicate stop)
 {
-  try
-  {
-    detail::partial_find_if_impl< typename boost::add_reference< MatchPredicate >::type >(aterm_traits<Term>::term(t), match, stop);
-  }
-  catch (detail::found_term_exception& e)
-  {
-    return e.t;
-  }
-  return aterm_appl();
+  return detail::partial_find_if_impl<typename std::add_lvalue_reference<MatchPredicate>::type>(t, match, stop);
 }
 
 /// \brief Finds all subterms of t that match a given predicate, and writes the found terms
@@ -72,10 +64,10 @@ aterm_appl partial_find_if(Term t, MatchPredicate match, StopPredicate stop)
 /// \param match The predicate that determines if a subterm is a match
 /// \param destBegin The iterator range to which output is written.
 template <typename Term, typename MatchPredicate, typename OutputIterator>
-void find_all_if(Term t, MatchPredicate match, OutputIterator destBegin)
+void find_all_if(const Term &t, MatchPredicate match, OutputIterator destBegin)
 {
   OutputIterator i = destBegin; // we make a copy, since a reference to an iterator is needed
-  detail::find_all_if_impl< typename boost::add_reference< MatchPredicate >::type >(aterm_traits<Term>::term(t), match, i);
+  detail::find_all_if_impl< typename std::add_lvalue_reference< MatchPredicate >::type >(t, match, i);
 }
 
 /// \brief Finds all subterms of t that match a given predicate, and writes the found terms
@@ -90,8 +82,8 @@ template <typename Term, typename MatchPredicate, typename StopPredicate, typena
 void partial_find_all_if(Term t, MatchPredicate match, StopPredicate stop, OutputIterator destBegin)
 {
   OutputIterator i = destBegin; // we make a copy, since a reference to an iterator is needed
-  detail::partial_find_all_if_impl< typename boost::add_reference< MatchPredicate >::type,
-         typename boost::add_reference< StopPredicate >::type >(aterm_traits<Term>::term(t), match, stop, i);
+  detail::partial_find_all_if_impl< typename std::add_lvalue_reference< MatchPredicate >::type,
+         typename std::add_lvalue_reference< StopPredicate >::type >(t, match, stop, i);
 }
 
 /// \brief Replaces each subterm x of t by r(x). The ReplaceFunction r has
@@ -103,10 +95,9 @@ void partial_find_all_if(Term t, MatchPredicate match, StopPredicate stop, Outpu
 /// \param r The replace function that is applied to subterms.
 /// \return The result of the replacement.
 template <typename Term, typename ReplaceFunction>
-Term replace(Term t, ReplaceFunction r)
+Term replace(const Term &t, ReplaceFunction r)
 {
-  ATerm x = detail::replace_impl< typename boost::add_reference< ReplaceFunction >::type >(aterm_traits<Term>::term(t), r);
-  return Term(reinterpret_cast<ATermAppl>(x));
+  return vertical_cast<Term>(detail::replace_impl< typename std::add_lvalue_reference< ReplaceFunction >::type >(t, r));
 }
 
 /// \brief Replaces each subterm in t that is equal to old_value with new_value.
@@ -117,7 +108,7 @@ Term replace(Term t, ReplaceFunction r)
 /// \param new_value The value that will be substituted.
 /// \return The result of the replacement.
 template <typename Term>
-Term replace(Term t, aterm_appl old_value, aterm_appl new_value)
+Term replace(const Term &t, const aterm &old_value, const aterm &new_value)
 {
   return replace(t, detail::default_replace(old_value, new_value));
 }
@@ -134,8 +125,8 @@ Term replace(Term t, aterm_appl old_value, aterm_appl new_value)
 template <typename Term, typename ReplaceFunction>
 Term bottom_up_replace(Term t, ReplaceFunction r)
 {
-  ATerm x = detail::bottom_up_replace_impl< typename boost::add_reference< ReplaceFunction >::type >(aterm_traits<Term>::term(t), r);
-  return Term(reinterpret_cast<ATermAppl>(x));
+  aterm x = detail::bottom_up_replace_impl< typename std::add_lvalue_reference< ReplaceFunction >::type >(t, r);
+  return Term(down_cast<aterm_appl>(x));
 }
 
 /// \brief Replaces each subterm in t that is equal to old_value with new_value.
@@ -165,8 +156,8 @@ Term bottom_up_replace(Term t, aterm_appl old_value, aterm_appl new_value)
 template <typename Term, typename ReplaceFunction>
 Term partial_replace(Term t, ReplaceFunction r)
 {
-  ATerm x = detail::partial_replace_impl< typename boost::add_reference< ReplaceFunction >::type >(aterm_traits<Term>::term(t), r);
-  return Term(reinterpret_cast<ATermAppl>(x));
+  aterm x = detail::partial_replace_impl< typename std::add_lvalue_reference< ReplaceFunction >::type >(t, r);
+  return Term(down_cast<aterm_appl>(x));
 }
 
 } // namespace atermpp
