@@ -12,14 +12,13 @@
 #ifndef MCRL2_ATERMPP_DETAIL_ATERM_LIST_UTILITY_H
 #define MCRL2_ATERMPP_DETAIL_ATERM_LIST_UTILITY_H
 
-#include "boost/static_assert.hpp"
-
 #include "mcrl2/atermpp/aterm_list.h"
 
 namespace atermpp
 {
 namespace detail
 {
+
 /// \brief Returns the concatenation of the lists l and m
 /// \param l A sequence of data expressions
 /// \param m A sequence of data variables
@@ -27,11 +26,11 @@ namespace detail
 /// \note Candidate for moving to atermpp library
 template < typename Expression, typename CompatibleExpression >
 inline
-atermpp::term_list< Expression > operator+(atermpp::term_list< Expression > l, atermpp::term_list< CompatibleExpression > m)
+atermpp::term_list< Expression > operator+(const atermpp::term_list< Expression > &l, const atermpp::term_list< CompatibleExpression > &m)
 {
-  BOOST_STATIC_ASSERT((boost::is_convertible< CompatibleExpression, Expression >::value));
+  static_assert(std::is_convertible< CompatibleExpression, Expression >::value,"Concatenated lists must be of the same type (1)");
 
-  return atermpp::term_list< Expression >(ATconcat(l, m));
+  return atermpp::term_list< Expression >(l+container_cast< atermpp::term_list< Expression > >(m));
 }
 
 /// \brief Returns the concatenation of the lists l and m
@@ -41,11 +40,10 @@ atermpp::term_list< Expression > operator+(atermpp::term_list< Expression > l, a
 /// \note Candidate for moving to atermpp library
 template < typename Expression, typename CompatibleExpression >
 inline
-atermpp::term_list< Expression > operator+(atermpp::term_list< Expression > l, CompatibleExpression const& m)
+atermpp::term_list< Expression > operator+(atermpp::term_list< Expression > l, const CompatibleExpression &m)
 {
-  BOOST_STATIC_ASSERT((boost::is_convertible< CompatibleExpression, Expression >::value));
-
-  return atermpp::push_back(l, static_cast< Expression >(m));
+  l.push_back(down_cast< Expression >(m));
+  return l;
 }
 
 /// \brief Returns the concatenation of the lists l and m
@@ -55,11 +53,10 @@ atermpp::term_list< Expression > operator+(atermpp::term_list< Expression > l, C
 /// \note Candidate for moving to atermpp library
 template < typename Expression, typename CompatibleExpression >
 inline
-atermpp::term_list< Expression > operator+(CompatibleExpression const& m, atermpp::term_list< Expression > l)
+atermpp::term_list< Expression > operator+(const CompatibleExpression &m, atermpp::term_list< Expression > l)
 {
-  BOOST_STATIC_ASSERT((boost::is_convertible< CompatibleExpression, Expression >::value));
-
-  return atermpp::push_front(l, static_cast< Expression >(m));
+  l.push_front(m);
+  return l;
 }
 
 
