@@ -12,94 +12,84 @@
 #ifndef MCRL2_ATERMPP_INDEXED_SET_H
 #define MCRL2_ATERMPP_INDEXED_SET_H
 
-#include <boost/shared_ptr.hpp>
-#include "mcrl2/atermpp/aterm.h"
-#include "mcrl2/atermpp/aterm_list.h"
+#include <vector>
+#include <stack>
+#include <cassert>
+#include "mcrl2/atermpp/detail/aterm_implementation.h"
 
 namespace atermpp
 {
-/// \cond INTERNAL_DOCS
-struct indexed_set_deleter
-{
-  /// \brief Function call operator
-  /// \param s An ATerm indexed set
-  void operator()(ATermIndexedSet s)
-  {
-    ATindexedSetDestroy(s);
-  }
-};
-/// \endcond
 
-/// \brief Indexed set. N.B. Copies of an indexed_set refer to the same object.
+/// \brief Indexed set. 
+template <class ELEMENT>
 class indexed_set
 {
   protected:
-    boost::shared_ptr<_ATermTable> m_set; /**< the wrapped ATermTable */
-    std::size_t m_size;                   /**< the number of entries of the table */
+    size_t sizeMinus1;
+    unsigned int max_load;
+    size_t nr_of_insertions_until_next_rehash;
+    std::vector<size_t> hashtable;
+    std::deque <ELEMENT > m_keys;
+    std::stack < size_t > free_positions; 
+
+    /* Find whether the key is already inserted in the hashtable.
+       If no, insert n as the index for key. If yes return its already
+       existing index */
+    size_t put_in_hashtable(const ELEMENT& key, size_t n);
+
+    /* Double the size of the hashtable. */
+    void resize_hashtable();
 
   public:
+    /// \brief A constant that if returned as an index means that the index does not exist.
+    //         In general this means that a requested element is not in the set.
+    static const size_t npos=static_cast<size_t>(-1);
+
     /// Create a new indexed_set.
     /// \param initial_size The initial capacity of the set.
     /// \param max_load_pct The maximum load percentage.
-    indexed_set(size_t initial_size = 100, unsigned int max_load_pct = 75)
-      : m_set(ATindexedSetCreate(initial_size, max_load_pct), indexed_set_deleter()),
-        m_size(0)
-    {}
+    indexed_set(size_t initial_size = 100, unsigned int max_load_pct = 75);
 
     /// \brief Clear the hash table in the set.
     /// This function clears the hash table in the set, but does not release the memory.
     /// Using indexed_set_reset instead of indexed_set_destroy is preferable when indexed sets of
     /// approximately the same size are being used.
-    void reset()
-    {
-      ATindexedSetReset(m_set.get());
-      m_size = 0;
-    }
+    void clear();
 
-    /// \brief Enter elem into the set.
+    /// \brief Enter an element into the set.
     /// This functions enters elem into the set. If elem was already in the set the previously
-    /// assigned index of elem is returned, and new is set to false. If elem did not yet occur in set a
-    /// new number is assigned, and new is set to true.  This number can either be the number of an
+    /// assigned index of elem is returned, and the boolean is set to false. If elem did not yet occur in set a
+    /// new number is assigned, and the boolean is set to true.  This number can either be the number of an
     /// element that has been removed, or, if such a number is not available, the lowest non used number
     /// is assigned to elem and returned. The lowest number that is used is 0.
     /// \param elem A term.
     /// \return A pair denoting the index of the element in the set, and a boolean denoting whether the term
     /// was already contained in the set.
-    std::pair<size_t, bool> put(const aterm& elem)
-    {
-      bool b;
-      size_t l = ATindexedSetPut(m_set.get(), elem, &b);
-      if (b == true)
-      {
-        m_size++;
-      }
-      return std::make_pair(l, b == true);
-    }
+    std::pair<size_t, bool> put(const ELEMENT& elem);
 
     /// \brief Find the index of elem in set.
     /// The index assigned to elem is returned, except when elem is not in the set, in
-    /// which case the return value is (size_t)-1, i.e. the largest number in size_t.
+    /// which case the return value is indexed_set::npos, i.e. the largest number in size_t.
     /// \param elem An element of the set.
     /// \return The index of the element.
-    size_t index(const aterm& elem) const
-    {
-      return ATindexedSetGetIndex(m_set.get(), elem);
-    }
+    ssize_t index(const ELEMENT& elem) const;
 
     /// \brief Find the index of elem in set.
     /// The index assigned to elem is returned. When elem is not in the set, it
     /// will be added first.
     /// \param elem An element of the set.
     /// \return The index of the element.
-    size_t operator[](const aterm& elem)
+    size_t operator[](const ELEMENT& elem)
     {
-      std::size_t result = ATindexedSetGetIndex(m_set.get(), elem);
-      if (result == (std::size_t) -1)
+      std::size_t result = index(elem);
+      if (result == npos)
       {
-        bool b;
-        result = ATindexedSetPut(m_set.get(), elem, &b);
-        m_size++;
-        assert(b);
+        std::pair<size_t, bool> p=put(elem);
+        if(!p.second)
+        {
+          throw std::runtime_error("failed to add element to indexed set");
+        }
+        result = p.first;
       }
       return result;
     }
@@ -109,39 +99,30 @@ class indexed_set
     /// to this index. If it is invoked with an invalid index, effects are not predictable.
     /// \param index A positive number.
     /// \return The element in the set with the given index.
-    aterm get(size_t index) const
-    {
-      return ATindexedSetGetElem(m_set.get(), index);
-    }
+    const ELEMENT& get(size_t index) const;
 
-    /// \brief Remove elem from set.
-    /// The elem is removed from the indexed set, and if a number was assigned to elem,
-    /// it is freed to be reassigned to an element, that may be put into the set at some later instance.
-    /// \param elem An element of the set.
-    void remove(const aterm& elem)
-    {
-      if (ATindexedSetRemove(m_set.get(), elem))
-      {
-        m_size--;
-      }
-    }
+    /// \brief Indicates whether a certain index is defined.
+    /// \param index A positive number.
+    /// \return The element in the set with the given index.
+    bool defined(size_t index) const;
+
+    /// \brief Remove elem from set. 
+    /// The elem is removed from the indexed set, and if a number was assigned to elem, 
+    /// it is freed to be reassigned to an element, that may be put into the set at some later instance. 
+    /// \param elem An element of the set. 
+    /// \return whether the element was successfully removed. 
+    bool erase(const ELEMENT& elem); 
 
-    /// \brief Retrieve all elements in set.
-    /// A list with all valid elements stored in the indexed set is returned.  The list is
-    /// ordered from element with index 0 onwards.
-    /// \return An ordered list containing the elements of the set.
-    aterm_list elements() const
-    {
-      return aterm_list(ATindexedSetElements(m_set.get()));
-    }
-    
     /// \brief Returns the size of the indexed set.
     std::size_t size() const
     {
-      return m_size;
+      return m_keys.size()-free_positions.size();
     }
 };
 
 } // namespace atermpp
 
+#include "mcrl2/atermpp/detail/indexed_set.h"
+
+
 #endif // MCRL2_ATERMPP_INDEXED_SET_H
