@@ -12,7 +12,6 @@
 #ifndef MCRL2_ATERMPP_MAKE_LIST_H
 #define MCRL2_ATERMPP_MAKE_LIST_H
 
-#include "mcrl2/atermpp/aterm.h"
 #include "mcrl2/atermpp/aterm_list.h"
 
 namespace atermpp
@@ -20,20 +19,25 @@ namespace atermpp
 /// \brief Makes a list with a fixed number of arguments.
 /// \param t0 A list element.
 /// \return The generated list.
-template <typename T0>
-aterm_list make_list(T0 t0)
+template <typename T>
+term_list<T> make_list(const T &t0)
 {
-  return aterm_list(ATmakeList1(aterm_traits<T0>::term(t0)));
+  term_list<T> result;
+  result.push_front(t0);
+  return result;
 }
 
 /// \brief Makes a list with a fixed number of arguments.
 /// \param t0 A list element.
 /// \param t1 A list element.
 /// \return The generated list.
-template <typename T0, typename T1>
-aterm_list make_list(T0 t0, T1 t1)
+template <typename T>
+term_list<T> make_list(const T &t0, const T &t1)
 {
-  return aterm_list(ATmakeList2(aterm_traits<T0>::term(t0), aterm_traits<T1>::term(t1)));
+  term_list<T> result;
+  result.push_front(t1);
+  result.push_front(t0);
+  return result;
 }
 
 /// \brief Makes a list with a fixed number of arguments.
@@ -41,10 +45,14 @@ aterm_list make_list(T0 t0, T1 t1)
 /// \param t1 A list element.
 /// \param t2 A list element.
 /// \return The generated list.
-template <typename T0, typename T1, typename T2>
-aterm_list make_list(T0 t0, T1 t1, T2 t2)
+template <typename T>
+term_list<T> make_list(const T &t0, const T &t1, const T &t2)
 {
-  return aterm_list(ATmakeList3(aterm_traits<T0>::term(t0), aterm_traits<T1>::term(t1), aterm_traits<T2>::term(t2)));
+  term_list<T> result;
+  result.push_front(t2);
+  result.push_front(t1);
+  result.push_front(t0);
+  return result;
 }
 
 /// \brief Makes a list with a fixed number of arguments.
@@ -53,10 +61,15 @@ aterm_list make_list(T0 t0, T1 t1, T2 t2)
 /// \param t2 A list element.
 /// \param t3 A list element.
 /// \return The generated list.
-template <typename T0, typename T1, typename T2, typename T3>
-aterm_list make_list(T0 t0, T1 t1, T2 t2, T3 t3)
+template <typename T>
+term_list<T> make_list(const T &t0, const T &t1, const T &t2, const T &t3)
 {
-  return aterm_list(ATmakeList4(aterm_traits<T0>::term(t0), aterm_traits<T1>::term(t1), aterm_traits<T2>::term(t2), aterm_traits<T3>::term(t3)));
+  term_list<T> result;
+  result.push_front(t3);
+  result.push_front(t2);
+  result.push_front(t1);
+  result.push_front(t0);
+  return result;
 }
 
 } // namespace atermpp
