@@ -9,21 +9,18 @@
 /// \file algorithm_test.cpp
 /// \brief Add your file description here.
 
+#include <vector>
 #include <iostream>
 #include <iterator>
 #include <set>
 #include <string>
 #include <boost/test/minimal.hpp>
 
-#include "mcrl2/atermpp/aterm.h"
-#include "mcrl2/atermpp/aterm_init.h"
+#include "mcrl2/atermpp/aterm_io.h"
 #include "mcrl2/atermpp/aterm_int.h"
 #include "mcrl2/atermpp/aterm_list.h"
 #include "mcrl2/atermpp/aterm_string.h"
-#include "mcrl2/atermpp/aterm_int.h"
-#include "mcrl2/atermpp/utility.h"
 #include "mcrl2/atermpp/algorithm.h"
-#include "mcrl2/atermpp/vector.h"
 
 using namespace std;
 using namespace atermpp;
@@ -39,18 +36,18 @@ struct is_f
 
 void test_algorithm()
 {
-  aterm_appl a = make_term("h(g(x),f(y),p(a(x,y),q(f(z))))");
-  aterm_appl b = make_term("h(g(x),p(a(x,y),q(g(z))))");
+  aterm_appl a (read_term_from_string("h(g(x),f(y),p(a(x,y),q(f(z))))"));
+  aterm_appl b (read_term_from_string("h(g(x),p(a(x,y),q(g(z))))"));
 
   aterm_appl t = find_if(a, is_f());
-  BOOST_CHECK(t == make_term("f(y)"));
+  BOOST_CHECK(t == read_term_from_string("f(y)"));
   aterm_appl t1 = find_if(b, is_f());
   BOOST_CHECK(t1 == aterm_appl());
 
-  atermpp::vector<aterm_appl> v;
+  std::vector<aterm_appl> v;
   find_all_if(a, is_f(), back_inserter(v));
-  BOOST_CHECK(v.front() == make_term("f(y)"));
-  BOOST_CHECK(v.back() == make_term("f(z)"));
+  BOOST_CHECK(v.front() == read_term_from_string("f(y)"));
+  BOOST_CHECK(v.back() == read_term_from_string("f(z)"));
 }
 
 struct for_each_proc
@@ -61,6 +58,18 @@ struct for_each_proc
     : m_names(names)
   {}
 
+  bool operator()(aterm t)
+  {
+    if(t.type_is_appl())
+    {
+      return (*this)(down_cast<aterm_appl>(t));
+    }
+    else
+    {
+      return false;
+    }
+  }
+
   bool operator()(aterm_appl t)
   {
     m_names.insert(t.function().name());
@@ -70,9 +79,9 @@ struct for_each_proc
 
 void test_for_each()
 {
-  aterm_appl t = make_term("h(g(x),f(y))");
+  aterm_appl t (read_term_from_string("h(g(x),f(y))"));
   std::set<std::string> names;
-  for_each(t, for_each_proc(names));
+  atermpp::for_each(t, for_each_proc(names));
   for (std::set<std::string>::iterator i = names.begin(); i != names.end(); ++i)
   {
     std::cout << *i << " ";
@@ -87,8 +96,8 @@ void test_for_each()
 void test_operators()
 {
   {
-    aterm_appl a1 = make_term("a1");
-    aterm_appl a2 = make_term("a2");
+    aterm_appl a1 (read_term_from_string("a1"));
+    aterm_appl a2 (read_term_from_string("a2"));
     bool b = (a1 < a2);
     std::clog << "b = " << (b?"true":"false") << std::endl;
   }
@@ -101,15 +110,15 @@ void test_operators()
   }
 
   {
-    aterm_int a1 = make_term("1");
-    aterm_int a2 = make_term("2");
+    aterm_int a1 (read_term_from_string("1"));
+    aterm_int a2 (read_term_from_string("2"));
     bool b = (a1 < a2);
     std::clog << "b = " << (b?"true":"false") << std::endl;
   }
 
   {
-    aterm_list a1 = make_term("[1,2]");
-    aterm_list a2 = make_term("[3,2]");
+    aterm_list a1 (read_term_from_string("[1,2]"));
+    aterm_list a2 (read_term_from_string("[3,2]"));
     bool b = (a1 < a2);
     std::clog << "b = " << (b?"true":"false") << std::endl;
   }
@@ -117,8 +126,6 @@ void test_operators()
 
 int test_main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
   test_algorithm();
   test_operators();
   test_for_each();
