@@ -11,31 +11,51 @@
 
 #include <iostream>
 #include <string>
+#include <sstream>
 #include <boost/test/minimal.hpp>
 
-#include "mcrl2/atermpp/aterm.h"
+#include "mcrl2/atermpp/aterm_io.h"
 #include "mcrl2/atermpp/aterm_int.h"
-#include "mcrl2/atermpp/aterm_init.h"
-#include "mcrl2/atermpp/utility.h"
+#include "mcrl2/atermpp/aterm_string.h"
 
 using namespace std;
 using namespace atermpp;
 
 void test_aterm()
 {
-  atermpp::aterm a = make_term("f(x)");
-  atermpp::aterm label = make_term("label");
+  atermpp::aterm a = read_term_from_string("f(x)");
+  atermpp::aterm label = read_term_from_string("label");
 
   atermpp::aterm d = aterm_int(10);
   BOOST_CHECK(aterm_int(d).value() == 10);
 
   atermpp::aterm e = atermpp::aterm();
-  BOOST_CHECK(!e);
+  BOOST_CHECK(e==atermpp::aterm());
+}
+
+void test_aterm_string()
+{
+  aterm_string empty = empty_string();
+  std::ostringstream out;
+  out << empty;
+  BOOST_CHECK(out.str() == "");
+}
+
+void test_aterm_io()
+{
+  aterm_string empty = empty_string();
+  std::ostringstream out;
+  write_term_to_text_stream(empty, out);
+  std::istringstream in(out.str());
+  aterm_string empty2 = down_cast<aterm_string>(read_term_from_text_stream(in));
+  BOOST_CHECK(empty == empty2);
 }
 
 int test_main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
   test_aterm();
+  test_aterm_string();
+  test_aterm_io();
+
   return 0;
 }
