@@ -14,19 +14,15 @@
 
 #include <cassert>
 #include <string>
+#include <type_traits>
 
-#include <boost/utility/enable_if.hpp>
-#include <boost/type_traits/is_base_of.hpp>
-
-#include "mcrl2/atermpp/aterm_access.h"
 #include "mcrl2/atermpp/aterm_appl.h"
-#include "mcrl2/atermpp/set.h"
-#include "mcrl2/atermpp/vector.h"
 #include "mcrl2/utilities/detail/join.h"
-#include "mcrl2/core/detail/constructors.h"
-#include "mcrl2/core/detail/struct_core.h"
+#include "mcrl2/core/detail/default_values.h"
+#include "mcrl2/core/detail/function_symbols.h"
 #include "mcrl2/core/detail/soundness_checks.h"
 #include "mcrl2/core/detail/precedence.h"
+#include "mcrl2/core/index_traits.h"
 #include "mcrl2/core/identifier_string.h"
 #include "mcrl2/core/term_traits.h"
 #include "mcrl2/core/print.h"
@@ -38,6 +34,8 @@ namespace mcrl2
 namespace bes
 {
 
+typedef core::identifier_string boolean_variable_key_type;
+
 template <typename T> std::string pp(const T& t);
 
 using namespace core::detail::precedences;
@@ -49,15 +47,15 @@ class boolean_expression: public atermpp::aterm_appl
   public:
     /// \brief Default constructor.
     boolean_expression()
-      : atermpp::aterm_appl(core::detail::constructBooleanExpression())
+      : atermpp::aterm_appl(core::detail::default_values::BooleanExpression)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    boolean_expression(const atermpp::aterm_appl& term)
+    explicit boolean_expression(const atermpp::aterm& term)
       : atermpp::aterm_appl(term)
     {
-      assert(core::detail::check_rule_BooleanExpression(m_term));
+      assert(core::detail::check_rule_BooleanExpression(*this));
     }
 };
 
@@ -65,7 +63,49 @@ class boolean_expression: public atermpp::aterm_appl
 typedef atermpp::term_list<boolean_expression> boolean_expression_list;
 
 /// \brief vector of boolean_expressions
-typedef atermpp::vector<boolean_expression>    boolean_expression_vector;
+typedef std::vector<boolean_expression>    boolean_expression_vector;
+
+// prototypes
+inline bool is_true(const atermpp::aterm_appl& x);
+inline bool is_false(const atermpp::aterm_appl& x);
+inline bool is_not(const atermpp::aterm_appl& x);
+inline bool is_and(const atermpp::aterm_appl& x);
+inline bool is_or(const atermpp::aterm_appl& x);
+inline bool is_imp(const atermpp::aterm_appl& x);
+inline bool is_boolean_variable(const atermpp::aterm_appl& x);
+
+/// \brief Test for a boolean_expression expression
+/// \param x A term
+/// \return True if \a x is a boolean_expression expression
+inline
+bool is_boolean_expression(const atermpp::aterm_appl& x)
+{
+  return bes::is_true(x) ||
+         bes::is_false(x) ||
+         bes::is_not(x) ||
+         bes::is_and(x) ||
+         bes::is_or(x) ||
+         bes::is_imp(x) ||
+         bes::is_boolean_variable(x);
+}
+
+// prototype declaration
+std::string pp(const boolean_expression& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const boolean_expression& x)
+{
+  return out << bes::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(boolean_expression& t1, boolean_expression& t2)
+{
+  t1.swap(t2);
+}
 
 
 /// \brief The value true for boolean expressions
@@ -74,25 +114,43 @@ class true_: public boolean_expression
   public:
     /// \brief Default constructor.
     true_()
-      : boolean_expression(core::detail::constructBooleanTrue())
+      : boolean_expression(core::detail::default_values::BooleanTrue)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    true_(const atermpp::aterm_appl& term)
+    explicit true_(const atermpp::aterm& term)
       : boolean_expression(term)
     {
-      assert(core::detail::check_term_BooleanTrue(m_term));
+      assert(core::detail::check_term_BooleanTrue(*this));
     }
 };
 
 /// \brief Test for a true expression
-/// \param t A term
-/// \return True if it is a true expression
+/// \param x A term
+/// \return True if \a x is a true expression
 inline
-bool is_true(const boolean_expression& t)
+bool is_true(const atermpp::aterm_appl& x)
 {
-  return core::detail::gsIsBooleanTrue(t);
+  return x.function() == core::detail::function_symbols::BooleanTrue;
+}
+
+// prototype declaration
+std::string pp(const true_& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const true_& x)
+{
+  return out << bes::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(true_& t1, true_& t2)
+{
+  t1.swap(t2);
 }
 
 
@@ -102,25 +160,43 @@ class false_: public boolean_expression
   public:
     /// \brief Default constructor.
     false_()
-      : boolean_expression(core::detail::constructBooleanFalse())
+      : boolean_expression(core::detail::default_values::BooleanFalse)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    false_(const atermpp::aterm_appl& term)
+    explicit false_(const atermpp::aterm& term)
       : boolean_expression(term)
     {
-      assert(core::detail::check_term_BooleanFalse(m_term));
+      assert(core::detail::check_term_BooleanFalse(*this));
     }
 };
 
 /// \brief Test for a false expression
-/// \param t A term
-/// \return True if it is a false expression
+/// \param x A term
+/// \return True if \a x is a false expression
 inline
-bool is_false(const boolean_expression& t)
+bool is_false(const atermpp::aterm_appl& x)
 {
-  return core::detail::gsIsBooleanFalse(t);
+  return x.function() == core::detail::function_symbols::BooleanFalse;
+}
+
+// prototype declaration
+std::string pp(const false_& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const false_& x)
+{
+  return out << bes::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(false_& t1, false_& t2)
+{
+  t1.swap(t2);
 }
 
 
@@ -130,35 +206,53 @@ class not_: public boolean_expression
   public:
     /// \brief Default constructor.
     not_()
-      : boolean_expression(core::detail::constructBooleanNot())
+      : boolean_expression(core::detail::default_values::BooleanNot)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    not_(const atermpp::aterm_appl& term)
+    explicit not_(const atermpp::aterm& term)
       : boolean_expression(term)
     {
-      assert(core::detail::check_term_BooleanNot(m_term));
+      assert(core::detail::check_term_BooleanNot(*this));
     }
 
     /// \brief Constructor.
     not_(const boolean_expression& operand)
-      : boolean_expression(core::detail::gsMakeBooleanNot(operand))
+      : boolean_expression(atermpp::aterm_appl(core::detail::function_symbol_BooleanNot(), operand))
     {}
 
-    boolean_expression operand() const
+    const boolean_expression& operand() const
     {
-      return atermpp::arg1(*this);
+      return atermpp::down_cast<boolean_expression>((*this)[0]);
     }
 };
 
 /// \brief Test for a not expression
-/// \param t A term
-/// \return True if it is a not expression
+/// \param x A term
+/// \return True if \a x is a not expression
 inline
-bool is_not(const boolean_expression& t)
+bool is_not(const atermpp::aterm_appl& x)
 {
-  return core::detail::gsIsBooleanNot(t);
+  return x.function() == core::detail::function_symbols::BooleanNot;
+}
+
+// prototype declaration
+std::string pp(const not_& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const not_& x)
+{
+  return out << bes::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(not_& t1, not_& t2)
+{
+  t1.swap(t2);
 }
 
 
@@ -168,40 +262,58 @@ class and_: public boolean_expression
   public:
     /// \brief Default constructor.
     and_()
-      : boolean_expression(core::detail::constructBooleanAnd())
+      : boolean_expression(core::detail::default_values::BooleanAnd)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    and_(const atermpp::aterm_appl& term)
+    explicit and_(const atermpp::aterm& term)
       : boolean_expression(term)
     {
-      assert(core::detail::check_term_BooleanAnd(m_term));
+      assert(core::detail::check_term_BooleanAnd(*this));
     }
 
     /// \brief Constructor.
     and_(const boolean_expression& left, const boolean_expression& right)
-      : boolean_expression(core::detail::gsMakeBooleanAnd(left, right))
+      : boolean_expression(atermpp::aterm_appl(core::detail::function_symbol_BooleanAnd(), left, right))
     {}
 
-    boolean_expression left() const
+    const boolean_expression& left() const
     {
-      return atermpp::arg1(*this);
+      return atermpp::down_cast<boolean_expression>((*this)[0]);
     }
 
-    boolean_expression right() const
+    const boolean_expression& right() const
     {
-      return atermpp::arg2(*this);
+      return atermpp::down_cast<boolean_expression>((*this)[1]);
     }
 };
 
 /// \brief Test for a and expression
-/// \param t A term
-/// \return True if it is a and expression
+/// \param x A term
+/// \return True if \a x is a and expression
 inline
-bool is_and(const boolean_expression& t)
+bool is_and(const atermpp::aterm_appl& x)
 {
-  return core::detail::gsIsBooleanAnd(t);
+  return x.function() == core::detail::function_symbols::BooleanAnd;
+}
+
+// prototype declaration
+std::string pp(const and_& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const and_& x)
+{
+  return out << bes::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(and_& t1, and_& t2)
+{
+  t1.swap(t2);
 }
 
 
@@ -211,40 +323,58 @@ class or_: public boolean_expression
   public:
     /// \brief Default constructor.
     or_()
-      : boolean_expression(core::detail::constructBooleanOr())
+      : boolean_expression(core::detail::default_values::BooleanOr)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    or_(const atermpp::aterm_appl& term)
+    explicit or_(const atermpp::aterm& term)
       : boolean_expression(term)
     {
-      assert(core::detail::check_term_BooleanOr(m_term));
+      assert(core::detail::check_term_BooleanOr(*this));
     }
 
     /// \brief Constructor.
     or_(const boolean_expression& left, const boolean_expression& right)
-      : boolean_expression(core::detail::gsMakeBooleanOr(left, right))
+      : boolean_expression(atermpp::aterm_appl(core::detail::function_symbol_BooleanOr(), left, right))
     {}
 
-    boolean_expression left() const
+    const boolean_expression& left() const
     {
-      return atermpp::arg1(*this);
+      return atermpp::down_cast<boolean_expression>((*this)[0]);
     }
 
-    boolean_expression right() const
+    const boolean_expression& right() const
     {
-      return atermpp::arg2(*this);
+      return atermpp::down_cast<boolean_expression>((*this)[1]);
     }
 };
 
 /// \brief Test for a or expression
-/// \param t A term
-/// \return True if it is a or expression
+/// \param x A term
+/// \return True if \a x is a or expression
 inline
-bool is_or(const boolean_expression& t)
+bool is_or(const atermpp::aterm_appl& x)
 {
-  return core::detail::gsIsBooleanOr(t);
+  return x.function() == core::detail::function_symbols::BooleanOr;
+}
+
+// prototype declaration
+std::string pp(const or_& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const or_& x)
+{
+  return out << bes::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(or_& t1, or_& t2)
+{
+  t1.swap(t2);
 }
 
 
@@ -254,40 +384,58 @@ class imp: public boolean_expression
   public:
     /// \brief Default constructor.
     imp()
-      : boolean_expression(core::detail::constructBooleanImp())
+      : boolean_expression(core::detail::default_values::BooleanImp)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    imp(const atermpp::aterm_appl& term)
+    explicit imp(const atermpp::aterm& term)
       : boolean_expression(term)
     {
-      assert(core::detail::check_term_BooleanImp(m_term));
+      assert(core::detail::check_term_BooleanImp(*this));
     }
 
     /// \brief Constructor.
     imp(const boolean_expression& left, const boolean_expression& right)
-      : boolean_expression(core::detail::gsMakeBooleanImp(left, right))
+      : boolean_expression(atermpp::aterm_appl(core::detail::function_symbol_BooleanImp(), left, right))
     {}
 
-    boolean_expression left() const
+    const boolean_expression& left() const
     {
-      return atermpp::arg1(*this);
+      return atermpp::down_cast<boolean_expression>((*this)[0]);
     }
 
-    boolean_expression right() const
+    const boolean_expression& right() const
     {
-      return atermpp::arg2(*this);
+      return atermpp::down_cast<boolean_expression>((*this)[1]);
     }
 };
 
 /// \brief Test for a imp expression
-/// \param t A term
-/// \return True if it is a imp expression
+/// \param x A term
+/// \return True if \a x is a imp expression
 inline
-bool is_imp(const boolean_expression& t)
+bool is_imp(const atermpp::aterm_appl& x)
 {
-  return core::detail::gsIsBooleanImp(t);
+  return x.function() == core::detail::function_symbols::BooleanImp;
+}
+
+// prototype declaration
+std::string pp(const imp& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const imp& x)
+{
+  return out << bes::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(imp& t1, imp& t2)
+{
+  t1.swap(t2);
 }
 
 
@@ -295,44 +443,70 @@ bool is_imp(const boolean_expression& t)
 class boolean_variable: public boolean_expression
 {
   public:
+
+
+    const core::identifier_string& name() const
+    {
+      return atermpp::down_cast<core::identifier_string>((*this)[0]);
+    }
+//--- start user section boolean_variable ---//
     /// \brief Default constructor.
     boolean_variable()
-      : boolean_expression(core::detail::constructBooleanVariable())
+      : boolean_expression(core::detail::default_values::BooleanVariable)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    boolean_variable(const atermpp::aterm_appl& term)
+    explicit boolean_variable(const atermpp::aterm& term)
       : boolean_expression(term)
     {
-      assert(core::detail::check_term_BooleanVariable(m_term));
+      assert(core::detail::check_term_BooleanVariable(*this));
     }
 
     /// \brief Constructor.
     boolean_variable(const core::identifier_string& name)
-      : boolean_expression(core::detail::gsMakeBooleanVariable(name))
+      : boolean_expression(atermpp::aterm_appl(core::detail::function_symbol_BooleanVariable(),
+          name,
+          atermpp::aterm_int(core::index_traits<boolean_variable, boolean_variable_key_type, 1>::insert(name)
+        )))
     {}
 
     /// \brief Constructor.
     boolean_variable(const std::string& name)
-      : boolean_expression(core::detail::gsMakeBooleanVariable(core::identifier_string(name)))
+      : boolean_expression(atermpp::aterm_appl(core::detail::function_symbol_BooleanVariable(),
+          core::identifier_string(name),
+          atermpp::aterm_int(core::index_traits<boolean_variable, boolean_variable_key_type, 1>::insert(name)
+        )))
     {}
-
-    core::identifier_string name() const
-    {
-      return atermpp::arg1(*this);
-    }
+//--- end user section boolean_variable ---//
 };
 
 /// \brief Test for a boolean_variable expression
-/// \param t A term
-/// \return True if it is a boolean_variable expression
+/// \param x A term
+/// \return True if \a x is a boolean_variable expression
 inline
-bool is_boolean_variable(const boolean_expression& t)
+bool is_boolean_variable(const atermpp::aterm_appl& x)
 {
-  return core::detail::gsIsBooleanVariable(t);
+  return x.function() == core::detail::function_symbols::BooleanVariable;
 }
 
+// prototype declaration
+std::string pp(const boolean_variable& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const boolean_variable& x)
+{
+  return out << bes::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(boolean_variable& t1, boolean_variable& t2)
+{
+  t1.swap(t2);
+}
 //--- end generated classes ---//
 
 // From the documentation:
@@ -341,33 +515,31 @@ bool is_boolean_variable(const boolean_expression& t)
 /// \brief Returns the precedence of boolean expressions
 // N.B. The is_base_of construction is needed to make sure that the precedence also works on
 // classes of type 'and_', 'or_' and 'imp'.
-inline
-int precedence(const boolean_expression& x)
+inline int left_precedence(const imp&)    { return 2; }
+inline int left_precedence(const or_&)    { return 3; }
+inline int left_precedence(const and_&)   { return 4; }
+inline int left_precedence(const not_&)   { return 5; }
+inline int left_precedence(const boolean_expression& x)
 {
-  if (is_imp(x))
-  {
-    return 2;
-  }
-  else if (is_or(x))
-  {
-    return 3;
-  }
-  else if (is_and(x))
-  {
-    return 4;
-  }
-  else if (is_not(x))
-  {
-    return 5;
-  }
+       if (is_imp(x)) { return left_precedence(static_cast<const imp&>(x)); }
+  else if (is_or(x))  { return left_precedence(static_cast<const or_&>(x)); }
+  else if (is_and(x)) { return left_precedence(static_cast<const and_&>(x)); }
+  else if (is_not(x)) { return left_precedence(static_cast<const not_&>(x)); }
   return core::detail::precedences::max_precedence;
 }
 
-// TODO: is there a cleaner way to make the precedence function work for derived classes like and_ ?
-inline int precedence(const imp& x) { return precedence(static_cast<const boolean_expression&>(x)); }
-inline int precedence(const and_& x) { return precedence(static_cast<const boolean_expression&>(x)); }
-inline int precedence(const or_& x) { return precedence(static_cast<const boolean_expression&>(x)); }
-inline int precedence(const not_& x) { return precedence(static_cast<const boolean_expression&>(x)); }
+inline int right_precedence(const boolean_expression& x)
+{
+  return left_precedence(x);
+}
+
+inline const boolean_expression& unary_operand(const not_& x) { return x.operand(); }
+inline const boolean_expression& binary_left(const and_& x)   { return x.left(); }
+inline const boolean_expression& binary_right(const and_& x)  { return x.right(); }
+inline const boolean_expression& binary_left(const or_& x)    { return x.left(); }
+inline const boolean_expression& binary_right(const or_& x)   { return x.right(); }
+inline const boolean_expression& binary_left(const imp& x)    { return x.left(); }
+inline const boolean_expression& binary_right(const imp& x)   { return x.right(); }
 
 /// \brief Returns true if the operations have the same precedence, but are different
 inline
@@ -386,39 +558,21 @@ bool is_same_different_precedence(const or_&, const boolean_expression& x)
 namespace accessors
 {
 inline
-boolean_expression left(boolean_expression const& e)
+const boolean_expression &left(boolean_expression const& e)
 {
   assert(is_and(e) || is_or(e) || is_imp(e));
-  return atermpp::arg1(e);
+  return atermpp::down_cast<const boolean_expression>(e[0]);
 }
 
 inline
-boolean_expression right(boolean_expression const& e)
+const boolean_expression &right(boolean_expression const& e)
 {
   assert(is_and(e) || is_or(e) || is_imp(e));
-  return atermpp::arg2(e);
+  return atermpp::down_cast<const boolean_expression>(e[1]);
 }
 
 } // namespace accessors
 
-/// \brief Returns true if the term t is a boolean expression
-/// \param t A boolean expression
-/// \return True if the term t is a boolean expression
-inline
-bool is_boolean_expression(atermpp::aterm_appl t)
-{
-  // TODO: this code should be generated
-  return
-    core::detail::gsIsBooleanTrue(t) ||
-    core::detail::gsIsBooleanFalse(t) ||
-    core::detail::gsIsBooleanVariable(t) ||
-    core::detail::gsIsBooleanNot(t) ||
-    core::detail::gsIsBooleanAnd(t) ||
-    core::detail::gsIsBooleanOr(t) ||
-    core::detail::gsIsBooleanImp(t)
-    ;
-}
-
 } // namespace bes
 
 } // namespace mcrl2
@@ -447,7 +601,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   term_type true_()
   {
-    return atermpp::aterm_appl(core::detail::gsMakeBooleanTrue());
+    return term_type(atermpp::aterm_appl(core::detail::function_symbol_BooleanTrue()));
   }
 
   /// \brief The value false
@@ -455,16 +609,16 @@ struct term_traits<bes::boolean_expression>
   static inline
   term_type false_()
   {
-    return atermpp::aterm_appl(core::detail::gsMakeBooleanFalse());
+    return term_type(atermpp::aterm_appl(core::detail::function_symbol_BooleanFalse()));
   }
 
   /// \brief Operator not
   /// \param p A term
   /// \return Operator not applied to p
   static inline
-  term_type not_(term_type p)
+  term_type not_(const term_type& p)
   {
-    return atermpp::aterm_appl(core::detail::gsMakeBooleanNot(p));
+    return term_type(atermpp::aterm_appl(core::detail::function_symbol_BooleanNot(), p));
   }
 
   /// \brief Operator and
@@ -474,7 +628,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   term_type and_(term_type p, term_type q)
   {
-    return atermpp::aterm_appl(core::detail::gsMakeBooleanAnd(p, q));
+    return term_type(atermpp::aterm_appl(core::detail::function_symbol_BooleanAnd(), p, q));
   }
 
   /// \brief Operator or
@@ -484,7 +638,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   term_type or_(term_type p, term_type q)
   {
-    return atermpp::aterm_appl(core::detail::gsMakeBooleanOr(p, q));
+    return term_type(atermpp::aterm_appl(core::detail::function_symbol_BooleanOr(), p, q));
   }
 
   /// \brief Implication
@@ -494,7 +648,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   term_type imp(term_type p, term_type q)
   {
-    return atermpp::aterm_appl(core::detail::gsMakeBooleanImp(p, q));
+    return term_type(atermpp::aterm_appl(core::detail::function_symbol_BooleanImp(), p, q));
   }
 
   /// \brief Test for value true
@@ -503,7 +657,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   bool is_true(const term_type& t)
   {
-    return core::detail::gsIsBooleanTrue(t);
+    return bes::is_true(t);
   }
 
   /// \brief Test for value false
@@ -512,7 +666,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   bool is_false(const term_type& t)
   {
-    return core::detail::gsIsBooleanFalse(t);
+    return bes::is_false(t);
   }
 
   /// \brief Test for operator not
@@ -521,7 +675,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   bool is_not(const term_type& t)
   {
-    return core::detail::gsIsBooleanNot(t);
+    return bes::is_not(t);
   }
 
   /// \brief Test for operator and
@@ -530,7 +684,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   bool is_and(const term_type& t)
   {
-    return core::detail::gsIsBooleanAnd(t);
+    return bes::is_and(t);
   }
 
   /// \brief Test for operator or
@@ -539,7 +693,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   bool is_or(const term_type& t)
   {
-    return core::detail::gsIsBooleanOr(t);
+    return bes::is_or(t);
   }
 
   /// \brief Test for implication
@@ -548,7 +702,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   bool is_imp(const term_type& t)
   {
-    return core::detail::gsIsBooleanImp(t);
+    return bes::is_imp(t);
   }
 
   /// \brief Test for boolean variable
@@ -557,7 +711,7 @@ struct term_traits<bes::boolean_expression>
   static inline
   bool is_variable(const term_type& t)
   {
-    return core::detail::gsIsBooleanVariable(t);
+    return bes::is_boolean_variable(t);
   }
 
   /// \brief Test for propositional variable
@@ -573,56 +727,56 @@ struct term_traits<bes::boolean_expression>
   /// \param t A term
   /// \return The requested argument
   static inline
-  term_type arg(const term_type& t)
+  const term_type &arg(const term_type& t)
   {
     assert(is_not(t));
-    return atermpp::arg1(t);
+    return atermpp::down_cast<const term_type>(t[0]);
   }
 
   /// \brief Returns the left argument of a term of type and, or or imp
   /// \param t A term
   /// \return The left argument of the term
   static inline
-  term_type left(const term_type& t)
+  const term_type &left(const term_type& t)
   {
     assert(is_and(t) || is_or(t) || is_imp(t));
-    return atermpp::arg1(t);
+    return atermpp::down_cast<const term_type>(t[0]);
   }
 
   /// \brief Returns the right argument of a term of type and, or or imp
   /// \param t A term
   /// \return The right argument of the term
   static inline
-  term_type right(const term_type& t)
+  const term_type &right(const term_type& t)
   {
     assert(is_and(t) || is_or(t) || is_imp(t));
-    return atermpp::arg2(t);
+    return atermpp::down_cast<const term_type>(t[1]);
   }
 
   /// \brief Returns the argument of a term of type not
   /// \param t A term
   static inline
-  term_type not_arg(const term_type& t)
+  const term_type &not_arg(const term_type& t)
   {
     assert(is_not(t));
-    return atermpp::arg1(t);
+    return atermpp::down_cast<const term_type>(t[0]);
   }
 
   /// \brief Returns the name of a boolean variable
   /// \param t A term
   /// \return The name of the boolean variable
   static inline
-  string_type name(const term_type& t)
+  const string_type &name(const term_type& t)
   {
     assert(is_variable(t));
-    return atermpp::arg1(t);
+    return atermpp::down_cast<const string_type>(t[0]);
   }
 
   /// \brief Conversion from variable to term
   /// \param v A variable
   /// \returns The converted variable
   static inline
-  term_type variable2term(variable_type v)
+  const term_type &variable2term(const variable_type &v)
   {
     return v;
   }
@@ -631,9 +785,9 @@ struct term_traits<bes::boolean_expression>
   /// \param t a term
   /// \returns The converted term
   static inline
-  variable_type term2variable(const term_type& t)
+  const variable_type &term2variable(const term_type& t)
   {
-    return t;
+    return atermpp::down_cast<const variable_type>(t);
   }
 
   /// \brief Pretty print function
@@ -680,11 +834,11 @@ boolean_expression join_and(FwdIt first, FwdIt last)
   return utilities::detail::join(first, last, tr::and_, tr::true_());
 }
 
-inline bool
+/* inline bool
 operator<(const boolean_expression& x, const boolean_expression& y)
 {
-  return ATermAppl(x) < ATermAppl(y);
-}
+  return aterm_appl(x) < aterm_appl(y);
+} */
 
 /// \brief Splits a disjunction into a sequence of operands
 /// Given a boolean expression of the form p1 || p2 || .... || pn, this will yield a
@@ -693,11 +847,11 @@ operator<(const boolean_expression& x, const boolean_expression& y)
 /// \param expr A boolean expression
 /// \return A sequence of operands
 inline
-atermpp::set<boolean_expression> split_or(const boolean_expression& expr)
+std::set<boolean_expression> split_or(const boolean_expression& expr)
 {
   using namespace accessors;
-  atermpp::set<boolean_expression> result;
-  utilities::detail::split(expr, std::insert_iterator<atermpp::set<boolean_expression> >(result, result.begin()), is_or, left, right);
+  std::set<boolean_expression> result;
+  utilities::detail::split(expr, std::insert_iterator<std::set<boolean_expression> >(result, result.begin()), is_or, left, right);
   return result;
 }
 
@@ -708,11 +862,11 @@ atermpp::set<boolean_expression> split_or(const boolean_expression& expr)
 /// \param expr A boolean expression
 /// \return A sequence of operands
 inline
-atermpp::set<boolean_expression> split_and(const boolean_expression& expr)
+std::set<boolean_expression> split_and(const boolean_expression& expr)
 {
   using namespace accessors;
-  atermpp::set<boolean_expression> result;
-  utilities::detail::split(expr, std::insert_iterator<atermpp::set<boolean_expression> >(result, result.begin()), is_and, left, right);
+  std::set<boolean_expression> result;
+  utilities::detail::split(expr, std::insert_iterator<std::set<boolean_expression> >(result, result.begin()), is_and, left, right);
   return result;
 }
 
