@@ -12,7 +12,6 @@
 #ifndef MCRL2_BES_PRINT_H
 #define MCRL2_BES_PRINT_H
 
-#include <boost/foreach.hpp>
 #include "mcrl2/bes/boolean_equation_system.h"
 #include "mcrl2/bes/traverser.h"
 #include "mcrl2/core/print.h"
@@ -51,12 +50,7 @@ struct printer: public bes::add_traverser_boolean_expressions<core::detail::prin
     derived().leave(x);
   }
 
-#ifdef BOOST_MSVC
-  void operator()(const bes::boolean_equation_system<>& x)
-#else
-  template <typename Container>
-  void operator()(const bes::boolean_equation_system<Container>& x)
-#endif
+  void operator()(const bes::boolean_equation_system& x)
   {
     print_list(x.equations(), "pbes\n    ", ";\n\n", ";\n    ");
     derived().print("init ");
