@@ -12,19 +12,17 @@
 #include <sstream>
 #include <string>
 #include <boost/test/minimal.hpp>
-#include "mcrl2/core/garbage_collection.h"
 #include "mcrl2/bes/boolean_equation_system.h"
 #include "mcrl2/bes/normal_forms.h"
 #include "mcrl2/bes/parse.h"
 #include "mcrl2/bes/print.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 using namespace mcrl2::bes;
 
 void test_standard_recursive_form(const std::string& bes_spec, bool recursive_form = false)
 {
-  boolean_equation_system<> b;
+  boolean_equation_system b;
   std::stringstream from(bes_spec);
   from >> b;
   std::cout << "before\n" << bes::pp(b) << std::endl;
@@ -58,13 +56,10 @@ void test_standard_recursive_form()
   test_standard_recursive_form(bes2, false);
   test_standard_recursive_form(bes2, true);
 
-  core::garbage_collect();
 }
 
 int test_main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT_DEBUG(argc, argv)
-
   test_standard_recursive_form();
 
   return 0;
