@@ -12,7 +12,6 @@
 #include <sstream>
 #include <string>
 #include <boost/test/included/unit_test_framework.hpp>
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/utilities/test_utilities.h"
 #include "mcrl2/bes/small_progress_measures.h"
 #include "mcrl2/bes/gauss_elimination.h"
@@ -26,16 +25,14 @@ BOOST_GLOBAL_FIXTURE(collect_after_test_case)
 
 void run_all_algorithms(std::string const& b, bool expected_outcome)
 {
-  boolean_equation_system<> b1;
+  boolean_equation_system b1;
   std::stringstream from;
   from << "pbes\n" << b << std::endl;
   from >> b1;
 
   std::clog << "solving the following input bes: \n" << bes::pp(b1) << std::endl;
 
-  core::garbage_collect();
   BOOST_CHECK_EQUAL(small_progress_measures(b1), expected_outcome);
-  core::garbage_collect();
   BOOST_CHECK_EQUAL(gauss_elimination(b1), expected_outcome);
 }
 
@@ -73,7 +70,5 @@ BOOST_AUTO_TEST_CASE(test_simple)
 
 boost::unit_test::test_suite* init_unit_test_suite(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
   return 0;
 }
