@@ -43,7 +43,7 @@ class add_binding : public Traverser<Derived>
 
     /// \brief Add a sequence of variables to the multiset of bound variables.
     template <typename Container>
-    void increase_bind_count(const Container& variables, typename atermpp::detail::enable_if_container<Container, variable_type>::type* = 0)
+    void increase_bind_count(const Container& variables, typename atermpp::enable_if_container<Container, variable_type>::type* = 0)
     {
       for (typename Container::const_iterator i = variables.begin(); i != variables.end(); ++i)
       {
@@ -59,7 +59,7 @@ class add_binding : public Traverser<Derived>
 
     /// \brief Remove a sequence of variables from the multiset of bound variables.
     template <typename Container>
-    void decrease_bind_count(const Container& variables, typename atermpp::detail::enable_if_container<Container, variable_type>::type* = 0)
+    void decrease_bind_count(const Container& variables, typename atermpp::enable_if_container<Container, variable_type>::type* = 0)
     {
       for (typename Container::const_iterator i = variables.begin(); i != variables.end(); ++i)
       {
@@ -74,6 +74,12 @@ class add_binding : public Traverser<Derived>
       return m_bound_variables.find(v) != m_bound_variables.end();
     }
 
+    /// \brief Returns the bound variables.
+    const std::multiset<variable_type>& bound_variables() const
+    {
+      return m_bound_variables;
+    }
+
     /// \brief Returns the bind count of the variable v.
     typename std::multiset<variable_type>::size_type bind_count(const variable_type& v)
     {
