@@ -23,9 +23,8 @@
 #include <vector>
 #include "mcrl2/utilities/exception.h"
 #include "mcrl2/utilities/logger.h"
-#include "mcrl2/atermpp/aterm.h"
+#include "mcrl2/atermpp/detail/utility.h"
 #include "mcrl2/atermpp/aterm_appl.h"
-#include "mcrl2/atermpp/set.h"
 #include "mcrl2/core/traverser.h"
 #include "mcrl2/core/detail/precedence.h"
 #include "mcrl2/core/print_format.h"
@@ -68,13 +67,9 @@ struct printer: public core::traverser<Derived>
   }
 
   template <typename T>
-  void print_expression(const T& x, int prec = 5)
+  void print_expression(const T& x, int context_precedence, int x_precedence)
   {
-#ifdef MCRL2_DEBUG_PRECEDENCE
-    std::cout << "<print_expression> precedence = " << prec << std::endl;
-    std::cout << "<x>" << x.to_string() << " precedence = " << precedence(x) << std::endl;
-#endif
-    bool print_parens = (precedence(x) < prec);
+    bool print_parens = (x_precedence < context_precedence);
     if (print_parens)
     {
       derived().print("(");
@@ -86,25 +81,25 @@ struct printer: public core::traverser<Derived>
     }
   }
 
+  template <typename T>
+  void print_expression(const T& x, int context_precedence = 5)
+  {
+    print_expression(x, context_precedence, left_precedence(x));
+  }
+
   template <typename T>
   void print_unary_operation(const T& x, const std::string& op)
   {
     derived().print(op);
-    print_expression(x.operand(), precedence(x));
+    print_expression(unary_operand(x), left_precedence(x));
   }
 
   template <typename T>
   void print_binary_operation(const T& x, const std::string& op)
   {
-#ifdef MCRL2_DEBUG_PRECEDENCE
-    std::cout << "<binary>" << std::endl;
-    std::cout << "<x>" << x.to_string() << " precedence = " << precedence(x) << std::endl;
-    std::cout << "<left>" << x.left().to_string() << " precedence = " << precedence(x.left()) << std::endl;
-    std::cout << "<right>" << x.right().to_string() << " precedence = " << precedence(x.right()) << std::endl;
-#endif
-    print_expression(x.left(), is_same_different_precedence(x, x.left()) ? precedence(x) + 1 : precedence(x));
+    print_expression(binary_left(x), is_same_different_precedence(x, binary_left(x)) ? left_precedence(x) + 1 : left_precedence(x));
     derived().print(op);
-    print_expression(x.right(), is_same_different_precedence(x, x.right()) ? precedence(x) + 1 : precedence(x));
+    print_expression(binary_right(x), is_same_different_precedence(x, binary_right(x)) ? left_precedence(x) + 1 : left_precedence(x), right_precedence(binary_right(x)));
   }
 
   template <typename Container>
@@ -134,76 +129,75 @@ struct printer: public core::traverser<Derived>
   template <typename T>
   void operator()(const atermpp::term_appl<T>& x)
   {
-    static_cast<Derived&>(*this).enter(x);
-    static_cast<Derived&>(*this).print(x.to_string());
-    static_cast<Derived&>(*this).leave(x);
+    derived().enter(x);
+    derived().print(utilities::to_string(x));
+    derived().leave(x);
   }
 
   template <typename T>
   void operator()(const std::list<T>& x)
   {
-    static_cast<Derived&>(*this).enter(x);
+    derived().enter(x);
     print_list(x, "", "", ", ");
-    static_cast<Derived&>(*this).leave(x);
+    derived().leave(x);
   }
 
   template <typename T>
   void operator()(const atermpp::term_list<T>& x)
   {
-    static_cast<Derived&>(*this).enter(x);
+    derived().enter(x);
     print_list(x, "", "", ", ");
-    static_cast<Derived&>(*this).leave(x);
+    derived().leave(x);
   }
 
   template <typename T>
   void operator()(const std::set<T>& x)
   {
-    static_cast<Derived&>(*this).enter(x);
+    derived().enter(x);
     print_list(x, "", "", ", ");
-    static_cast<Derived&>(*this).leave(x);
-  }
-
-  template <typename T>
-  void operator()(const atermpp::set<T>& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    print_list(x, "", "", ", ");
-    static_cast<Derived&>(*this).leave(x);
+    derived().leave(x);
   }
 
   void operator()(const core::identifier_string& x)
   {
-    static_cast<Derived&>(*this).enter(x);
+    derived().enter(x);
     if (x == core::identifier_string())
     {
-      static_cast<Derived&>(*this).print("@NoValue");
+      derived().print("@NoValue");
     }
     else
     {
-      static_cast<Derived&>(*this).print(std::string(x));
+      derived().print(std::string(x));
     }
-    static_cast<Derived&>(*this).leave(x);
+    derived().leave(x);
   }
 
-  void operator()(aterm::ATerm x)
+  void operator()(const core::nil& x)
   {
-    static_cast<Derived&>(*this).enter(x);
-    static_cast<Derived&>(*this).print(atermpp::aterm(x).to_string());
-    static_cast<Derived&>(*this).leave(x);
+    derived().enter(x);
+    derived().print("nil");
+    derived().leave(x);
   }
 
-  void operator()(aterm::ATermList x)
+  void operator()(atermpp::aterm x)
   {
-    static_cast<Derived&>(*this).enter(x);
-    static_cast<Derived&>(*this).print(atermpp::aterm_list(x).to_string());
-    static_cast<Derived&>(*this).leave(x);
+    derived().enter(x);
+    derived().print(utilities::to_string(x));
+    derived().leave(x);
   }
 
-  void operator()(aterm::ATermAppl x)
+  void operator()(atermpp::aterm_list x)
   {
-    static_cast<Derived&>(*this).enter(x);
-    static_cast<Derived&>(*this).print(atermpp::aterm_appl(x).to_string());
-    static_cast<Derived&>(*this).leave(x);
+    derived().enter(x);
+    derived().print(utilities::to_string(x));
+    derived().leave(x);
+  }
+
+  void operator()(atermpp::aterm_appl x)
+  {
+    derived().enter(x);
+    derived().print(utilities::to_string(x));
+    derived().leave(x);
   }
 };
 
@@ -251,8 +245,7 @@ std::string pp(const T& x)
 }
 
 /// \brief Prototypes for aterm overloads
-std::string pp(const atermpp::aterm& x);
-std::string pp(const atermpp::aterm_appl& x);
+std::string pp(const nil& x);
 
 } // namespace core
 
