@@ -13,9 +13,7 @@
 
 #include <sstream>
 #include <boost/test/minimal.hpp>
-#include "mcrl2/core/garbage_collection.h"
 #include "mcrl2/core/print.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 
@@ -36,14 +34,10 @@ void test_print()
   // print using the two argument function print
   BOOST_CHECK(core::pp(s) == "abc");
 
-  core::garbage_collect();
 }
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc,argv)
-
   test_print();
-
   return 0;
 }
