@@ -2,15 +2,12 @@
 #include "mcrl2/data/data_specification.h"
 #include "mcrl2/data/parse.h"
 #include "mcrl2/data/rewriter.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2::data;
 
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
-  data_specification data_spec;
+ data_specification data_spec;
   data_spec.add_context_sort(sort_nat::nat());
   rewriter r(data_spec);
 
