@@ -4,15 +4,12 @@
 #include <cassert>
 #include "mcrl2/utilities/text_utility.h"
 #include "mcrl2/data/parse.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
-  if (argc < 2)
+  if (argc < 2 || std::string(argv[1]) == "-h")
   {
     std::cout << "Usage: parse_data FILENAME" << std::endl;
     return 0;
@@ -22,7 +19,6 @@ int main(int argc, char* argv[])
   try
   {
     data::data_specification d = data::parse_data_specification(text);
-//    std::cout << data::pp(d) << std::endl;
   }
   catch (std::runtime_error e)
   {
