@@ -16,7 +16,7 @@
 #include "mcrl2/data/data_expression.h"
 #include "mcrl2/data/variable.h"
 #include "mcrl2/data/binder_type.h"
-#include "mcrl2/core/detail/constructors.h"
+#include "mcrl2/core/detail/default_values.h"
 #include "mcrl2/core/detail/soundness_checks.h"
 
 namespace mcrl2
@@ -25,50 +25,68 @@ namespace mcrl2
 namespace data
 {
 
-//--- start generated class abstraction ---//
 /// \brief An abstraction expression.
 class abstraction: public data_expression
 {
   public:
     /// \brief Default constructor.
     abstraction()
-      : data_expression(core::detail::constructBinder())
+      : data_expression(core::detail::default_values::Binder)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    abstraction(const atermpp::aterm_appl& term)
+    abstraction(const atermpp::aterm& term)
       : data_expression(term)
     {
-      assert(core::detail::check_term_Binder(m_term));
+      assert(core::detail::check_term_Binder(*this));
     }
 
     /// \brief Constructor.
     abstraction(const binder_type& binding_operator, const variable_list& variables, const data_expression& body)
-      : data_expression(core::detail::gsMakeBinder(binding_operator, variables, body))
+      : data_expression(atermpp::aterm_appl(core::detail::function_symbol_Binder(), binding_operator, variables, body))
     {}
 
     /// \brief Constructor.
     template <typename Container>
-    abstraction(const binder_type& binding_operator, const Container& variables, const data_expression& body, typename atermpp::detail::enable_if_container<Container, variable>::type* = 0)
-      : data_expression(core::detail::gsMakeBinder(binding_operator, atermpp::convert<variable_list>(variables), body))
+    abstraction(const binder_type& binding_operator, const Container& variables, const data_expression& body, typename atermpp::enable_if_container<Container, variable>::type* = 0)
+      : data_expression(atermpp::aterm_appl(core::detail::function_symbol_Binder(), binding_operator, variable_list(variables.begin(), variables.end()), body))
     {}
 
-    binder_type binding_operator() const
+    const binder_type& binding_operator() const
     {
-      return atermpp::arg1(*this);
+      return atermpp::down_cast<const binder_type>((*this)[0]);
     }
 
-    variable_list variables() const
+    const variable_list& variables() const
     {
-      return atermpp::list_arg2(*this);
+      return atermpp::down_cast<const variable_list>((*this)[1]);
     }
 
-    data_expression body() const
+    const data_expression& body() const
     {
-      return atermpp::arg3(*this);
+      return atermpp::down_cast<const data_expression>((*this)[2]);
     }
 };
+
+//--- start generated class abstraction ---//
+// prototype declaration
+std::string pp(const abstraction& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const abstraction& x)
+{
+  return out << data::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(abstraction& t1, abstraction& t2)
+{
+  t1.swap(t2);
+}
 //--- end generated class abstraction ---//
 
 } // namespace data
