@@ -35,28 +35,34 @@ struct add_data_variable_binding: public core::add_binding<Builder, Derived, var
   using super::enter;
   using super::leave;
   using super::operator();
-  using super::bind_count;
+  using super::bound_variables;
   using super::increase_bind_count;
   using super::decrease_bind_count;
 
+  void increase_bind_count(const assignment_list& assignments)
+  {
+    for (auto i = assignments.begin(); i != assignments.end(); ++i)
+    {
+      increase_bind_count(i->lhs());
+    }
+  }
+
+  void decrease_bind_count(const assignment_list& assignments)
+  {
+    for (auto i = assignments.begin(); i != assignments.end(); ++i)
+    {
+      decrease_bind_count(i->lhs());
+    }
+  }
+
   void enter(const data::where_clause& x)
   {
-    increase_bind_count(make_assignment_left_hand_side_range(x.declarations()));
+    increase_bind_count(x.assignments());
   }
 
   void leave(const data::where_clause& x)
   {
-    decrease_bind_count(make_assignment_left_hand_side_range(x.declarations()));
-  }
-
-  void enter(const data::assignment& x)
-  {
-    increase_bind_count(x.lhs());
-  }
-
-  void leave(const data::assignment& x)
-  {
-    decrease_bind_count(x.lhs());
+    decrease_bind_count(x.assignments());
   }
 
   void enter(const data::forall& x)
