@@ -13,13 +13,11 @@
 #define MCRL2_DATA_BASIC_SORT_H
 
 #include <string>
-#include "mcrl2/atermpp/aterm_access.h"
 #include "mcrl2/atermpp/aterm_appl.h"
-#include "mcrl2/atermpp/vector.h"
-#include "mcrl2/core/detail/constructors.h"
+#include "mcrl2/core/detail/default_values.h"
 #include "mcrl2/core/detail/soundness_checks.h"
 #include "mcrl2/core/identifier_string.h"
-#include "mcrl2/core/detail/struct_core.h"
+#include "mcrl2/core/detail/function_symbols.h"
 #include "mcrl2/data/sort_expression.h"
 
 namespace mcrl2
@@ -35,38 +33,56 @@ class basic_sort: public sort_expression
   public:
     /// \brief Default constructor.
     basic_sort()
-      : sort_expression(core::detail::constructSortId())
+      : sort_expression(core::detail::default_values::SortId)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    basic_sort(const atermpp::aterm_appl& term)
+    explicit basic_sort(const atermpp::aterm& term)
       : sort_expression(term)
     {
-      assert(core::detail::check_term_SortId(m_term));
+      assert(core::detail::check_term_SortId(*this));
     }
 
     /// \brief Constructor.
     basic_sort(const core::identifier_string& name)
-      : sort_expression(core::detail::gsMakeSortId(name))
+      : sort_expression(atermpp::aterm_appl(core::detail::function_symbol_SortId(), name))
     {}
 
     /// \brief Constructor.
     basic_sort(const std::string& name)
-      : sort_expression(core::detail::gsMakeSortId(core::identifier_string(name)))
+      : sort_expression(atermpp::aterm_appl(core::detail::function_symbol_SortId(), core::identifier_string(name)))
     {}
 
-    core::identifier_string name() const
+    const core::identifier_string& name() const
     {
-      return atermpp::arg1(*this);
+      return atermpp::down_cast<core::identifier_string>((*this)[0]);
     }
 };
+
+// prototype declaration
+std::string pp(const basic_sort& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const basic_sort& x)
+{
+  return out << data::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(basic_sort& t1, basic_sort& t2)
+{
+  t1.swap(t2);
+}
 //--- end generated class basic_sort ---//
 
 /// \brief list of basic sorts
 typedef atermpp::term_list<basic_sort> basic_sort_list;
 /// \brief vector of basic sorts
-typedef atermpp::vector<basic_sort> basic_sort_vector;
+typedef std::vector<basic_sort> basic_sort_vector;
 
 } // namespace data
 
