@@ -14,9 +14,8 @@
 
 #include <iostream>
 
-#include "mcrl2/atermpp/aterm_access.h"
 #include "mcrl2/core/identifier_string.h"
-#include "mcrl2/core/detail/struct_core.h"
+#include "mcrl2/core/detail/function_symbols.h"
 #include "mcrl2/data/container_type.h"
 #include "mcrl2/data/sort_expression.h"
 
@@ -33,32 +32,50 @@ class container_sort: public sort_expression
   public:
     /// \brief Default constructor.
     container_sort()
-      : sort_expression(core::detail::constructSortCons())
+      : sort_expression(core::detail::default_values::SortCons)
     {}
 
     /// \brief Constructor.
     /// \param term A term
-    container_sort(const atermpp::aterm_appl& term)
+    explicit container_sort(const atermpp::aterm& term)
       : sort_expression(term)
     {
-      assert(core::detail::check_term_SortCons(m_term));
+      assert(core::detail::check_term_SortCons(*this));
     }
 
     /// \brief Constructor.
     container_sort(const container_type& container_name, const sort_expression& element_sort)
-      : sort_expression(core::detail::gsMakeSortCons(container_name, element_sort))
+      : sort_expression(atermpp::aterm_appl(core::detail::function_symbol_SortCons(), container_name, element_sort))
     {}
 
-    container_type container_name() const
+    const container_type& container_name() const
     {
-      return atermpp::arg1(*this);
+      return atermpp::down_cast<container_type>((*this)[0]);
     }
 
-    sort_expression element_sort() const
+    const sort_expression& element_sort() const
     {
-      return atermpp::arg2(*this);
+      return atermpp::down_cast<sort_expression>((*this)[1]);
     }
 };
+
+// prototype declaration
+std::string pp(const container_sort& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const container_sort& x)
+{
+  return out << data::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(container_sort& t1, container_sort& t2)
+{
+  t1.swap(t2);
+}
 //--- end generated class container_sort ---//
 
 /// \brief list of function sorts
@@ -67,7 +84,7 @@ typedef atermpp::term_list<container_sort> container_sort_list;
 
 /// \brief list of function sorts
 ///
-typedef atermpp::vector<container_sort> container_sort_vector;
+typedef std::vector<container_sort> container_sort_vector;
 
 } // namespace data
 
