@@ -52,7 +52,7 @@ class data_expression_with_variables: public data_expression
 
 //      /// \brief Constructor. Creates a data expression with an empty sequence of variables.
 //      /// \param term A term
-//      data_expression_with_variables(ATermAppl term)
+//      data_expression_with_variables(aterm_appl term)
 //        : data_expression(term)
 //      {}
 
@@ -61,7 +61,14 @@ class data_expression_with_variables: public data_expression
     /// \param variables A sequence of data variables
     template < typename Container >
     data_expression_with_variables(data_expression expression, const Container& variables)
-      : data_expression(expression), m_variables(atermpp::convert< variable_list >(variables))
+      : data_expression(expression), m_variables(variable_list(variables.begin(),variables.end()))
+    {}
+
+    /// \brief Constructor.
+    /// \param expression A data expression
+    /// \param variables A sequence of data variables
+    data_expression_with_variables(data_expression expression, const variable_list& variables)
+      : data_expression(expression), m_variables(variables)
     {}
 
     /// \brief Return the variables.
@@ -90,37 +97,4 @@ class data_expression_with_variables: public data_expression
 
 } // namespace mcrl2
 
-/// \cond INTERNAL_DOCS
-namespace atermpp
-{
-template<>
-struct aterm_traits<mcrl2::data::data_expression_with_variables>
-{
-  static void protect(const mcrl2::data::data_expression_with_variables& t)
-  {
-    t.protect();
-    t.variables().protect();
-  }
-  static void unprotect(const mcrl2::data::data_expression_with_variables& t)
-  {
-    t.unprotect();
-    t.variables().unprotect();
-  }
-  static void mark(const mcrl2::data::data_expression_with_variables& t)
-  {
-    t.mark();
-    t.variables().mark();
-  }
-  static ATerm term(mcrl2::data::data_expression_with_variables& t)
-  {
-    return t.term();
-  }
-  static ATerm* ptr(mcrl2::data::data_expression_with_variables& t)
-  {
-    return &t.term();
-  }
-};
-} // namespace atermpp
-/// \endcond
-
 #endif // MCRL2_DATA_DATA_EXPRESSION_WITH_VARIABLES_H
