@@ -15,14 +15,10 @@
 #include <algorithm>
 #include <functional>
 #include <iterator>
-#include <set>
 #include <string>
 #include <utility>
-#include <vector>
 
 #include "mcrl2/atermpp/aterm.h"
-#include "mcrl2/atermpp/set.h"
-#include "mcrl2/atermpp/utility.h"
 #include "mcrl2/data/basic_sort.h"
 #include "mcrl2/data/variable.h"
 #include "mcrl2/data/function_symbol.h"
