@@ -13,9 +13,8 @@
 #define BDD2DOT_H
 
 #include <fstream>
-#include "mcrl2/aterm/aterm2.h"
+#include "mcrl2/atermpp/aterm.h"
 #include "mcrl2/atermpp/aterm_int.h"
-#include "mcrl2/atermpp/map.h"
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/data/detail/prover/bdd_info.h"
 
@@ -40,7 +39,7 @@ class BDD2Dot
     std::ofstream f_dot_file;
 
     /// \brief A table containing all the visited nodes. It maps these nodes to the corresponding node numbers.
-    atermpp::map < atermpp::aterm_appl, atermpp::aterm_int> f_visited;
+    std::map < atermpp::aterm_appl, atermpp::aterm_int> f_visited;
 
     /// \brief A class that gives information about the structure of BDDs.
     BDD_Info f_bdd_info;
@@ -51,7 +50,7 @@ class BDD2Dot
     /// following restrictions: It either represents the constant true or the constant false, or it is an if-then-else
     /// expression with an expression of Bool as guard, and a then-branch and an else-branch that again follow these
     /// restrictions
-    void aux_output_bdd(const data_expression a_bdd)
+    void aux_output_bdd(const data_expression &a_bdd)
     {
       if (f_visited.count(a_bdd)>0)  // a_bdd has already been visited.
       {
@@ -72,8 +71,8 @@ class BDD2Dot
         const data_expression v_false_branch = f_bdd_info.get_false_branch(a_bdd);
         aux_output_bdd(v_true_branch);
         aux_output_bdd(v_false_branch);
-        int v_true_number = f_visited[v_true_branch].value();
-        int v_false_number = f_visited[v_false_branch].value();
+        size_t v_true_number = f_visited[v_true_branch].value();
+        size_t v_false_number = f_visited[v_false_branch].value();
         const data_expression v_guard = f_bdd_info.get_guard(a_bdd);
         f_dot_file << "  " << f_node_number << " [label=\"" << mcrl2::data::pp(v_guard) << "\"];" << std::endl;
         f_dot_file << "  " << f_node_number << " -> " << v_true_number << ";" << std::endl;
@@ -95,7 +94,7 @@ class BDD2Dot
     /// restrictions
     /// \param a_bdd A binary decision diagram.
     /// \param a_file_name A file name.
-    void output_bdd(const data_expression a_bdd, char const* a_file_name)
+    void output_bdd(const data_expression &a_bdd, char const* a_file_name)
     {
       f_node_number = 0;
       f_dot_file.open(a_file_name);
