@@ -12,7 +12,6 @@
 #ifndef BDD_MANIPULATOR_H
 #define BDD_MANIPULATOR_H
 
-#include "mcrl2/aterm/aterm2.h"
 #include "mcrl2/data/standard.h"
 
 namespace mcrl2
@@ -36,9 +35,9 @@ class BDD_Manipulator
     /// \param[in] a_false_branch A BDD.
     /// \return A BDD.
     inline data_expression make_if_then_else(
-                       const data_expression a_guard, 
-                       const data_expression a_true_branch, 
-                       const data_expression a_false_branch)
+                       const data_expression &a_guard, 
+                       const data_expression &a_true_branch, 
+                       const data_expression &a_false_branch)
     {
       return mcrl2::data::if_(a_guard, a_true_branch, a_false_branch);
     } 
@@ -54,9 +53,9 @@ class BDD_Manipulator
     /// \param[in] a_false_branch A BDD.
     /// \return A reduced BDD.
     inline data_expression make_reduced_if_then_else(
-                       const data_expression a_guard, 
-                       const data_expression a_true_branch, 
-                       const data_expression a_false_branch)
+                       const data_expression &a_guard, 
+                       const data_expression &a_true_branch, 
+                       const data_expression &a_false_branch)
     {
       return (a_true_branch == a_false_branch) ? 
                     a_true_branch : 
