@@ -7,12 +7,11 @@
 // http://www.boost.org/LICENSE_1_0.txt)
 //
 /// \file mcrl2/data/detail/prover/info.h
-/// \brief Interface to classes InternalFormatInfo
+/// \brief Interface to classes Info
 
 #ifndef INFO_H
 #define INFO_H
 
-#include "mcrl2/aterm/aterm_ext.h"
 #include "mcrl2/atermpp/algorithm.h"
 #include "mcrl2/data/rewriter.h"
 #include "mcrl2/data/detail/prover/utilities.h"
@@ -33,14 +32,9 @@ enum Compare_Result
 
 /// \brief Base class for classes that provide information about the structure of
 /// \brief data expressions in one of the internal formats of the rewriter.
-class InternalFormatInfo
+class Info
 {
   protected:
-    /// \brief The rewriter used to translate formulas to the internal format of rewriters.
-    boost::shared_ptr<detail::Rewriter> f_rewriter;
-
-    /// \brief aterm_appl representing the internal \c if \c then \c else function with type Bool -> Bool -> Bool -> Bool.
-    atermpp::aterm_int f_if_then_else_bool;
 
     /// \brief Flag indicating whether or not the arguments of equality functions are taken into account
     /// \brief when determining the order of expressions.
@@ -51,15 +45,21 @@ class InternalFormatInfo
     /// \brief equation, when determining the order of expressions.
     bool f_reverse;
 
-    Compare_Result lexico(Compare_Result a_result1, Compare_Result a_result2)
+    Compare_Result lexico(
+              const Compare_Result& a_result1,
+              const Compare_Result& a_result2)
     {
       return (a_result1 != compare_result_equal) ? a_result1 : a_result2;
     }
 
-    Compare_Result compare_address(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2)
+    Compare_Result compare_address(
+              const atermpp::aterm& a_term1,
+              const atermpp::aterm& a_term2)
     {
-      long v_address1 = reinterpret_cast < long >((ATermAppl)a_term1);
-      long v_address2 = reinterpret_cast < long >((ATermAppl)a_term2);
+      // The code below does not seem to need to use explicit adresses. Code can
+      // directly use <, >, == on aterms, which also compares adresses.
+      long v_address1 = reinterpret_cast < long >(atermpp::detail::address(a_term1));
+      long v_address2 = reinterpret_cast < long >(atermpp::detail::address(a_term2));
 
       if (v_address1 < v_address2)
       {
@@ -72,7 +72,7 @@ class InternalFormatInfo
       return compare_result_equal;
     }
 
-    bool alpha1(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2, size_t a_number)
+    bool alpha1(const data_expression& a_term1, const data_expression& a_term2, size_t a_number)
     {
       if (get_number_of_arguments(a_term1) == a_number)
       {
@@ -80,12 +80,12 @@ class InternalFormatInfo
       }
       else
       {
-        atermpp::aterm_appl v_term = get_argument(a_term1, a_number);
+        data_expression v_term = get_argument(a_term1, a_number);
         return (v_term == a_term2) || lpo1(v_term, a_term2) || alpha1(a_term1, a_term2, ++a_number);
       }
     }
 
-    bool beta1(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2)
+    bool beta1(const data_expression& a_term1, const data_expression& a_term2)
     {
       const atermpp::aterm v_operator_1 = get_operator(a_term1);
       const atermpp::aterm v_operator_2 = get_operator(a_term2);
@@ -95,38 +95,33 @@ class InternalFormatInfo
     class equals // A simple class containing an equality predicate.
     {
       private:
-        const atermpp::aterm_appl m_t; 
+        const atermpp::aterm_appl m_t;
 
       public:
         // constructor
-        equals(const atermpp::aterm_appl t1):
+        equals(const data_expression& t1):
           m_t(t1)
         {}
 
-        bool operator()(const atermpp::aterm_appl t) const
+        bool operator()(const atermpp::aterm_appl& t) const
         {
           return (t==m_t);
         }
     };
 
-    static bool occurs(const atermpp::aterm_appl t1, const atermpp::aterm_appl t2)
+    static bool occurs(const data_expression& t1, const data_expression& t2)
     {
       return atermpp::find_if(t1,equals(t2))!=atermpp::aterm_appl();
     }
 
-    bool gamma1(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2)
+    bool gamma1(const data_expression& a_term1, const data_expression& a_term2)
     {
       const atermpp::aterm v_operator_1 = get_operator(a_term1);
       const atermpp::aterm v_operator_2 = get_operator(a_term2);
       return (v_operator_1 == v_operator_2) && lex1(a_term1, a_term2, 0) && majo1(a_term1, a_term2, 0);
     }
 
-    /* bool delta1(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2)
-    {
-      return occurs(a_term2, a_term1);
-    } */
-
-    bool majo1(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2, size_t a_number)
+    bool majo1(const data_expression& a_term1, const data_expression& a_term2, size_t a_number)
     {
       if (get_number_of_arguments(a_term2) == a_number)
       {
@@ -134,12 +129,12 @@ class InternalFormatInfo
       }
       else
       {
-        atermpp::aterm_appl v_term = get_argument(a_term2, a_number);
+        data_expression v_term = get_argument(a_term2, a_number);
         return lpo1(a_term1, v_term) && majo1(a_term1, a_term2, ++a_number);
       }
     }
 
-    bool lex1(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2, size_t a_number)
+    bool lex1(const data_expression& a_term1, const data_expression& a_term2, size_t a_number)
     {
       if (get_number_of_arguments(a_term1) == a_number)
       {
@@ -147,8 +142,8 @@ class InternalFormatInfo
       }
       else
       {
-        atermpp::aterm_appl v_term1 = get_argument(a_term1, a_number);
-        atermpp::aterm_appl v_term2 = get_argument(a_term2, a_number);
+        data_expression v_term1 = get_argument(a_term1, a_number);
+        data_expression v_term2 = get_argument(a_term2, a_number);
         if (v_term1 == v_term2)
         {
           return lex1(a_term1, a_term2, ++a_number);
@@ -161,7 +156,7 @@ class InternalFormatInfo
     }
 
     /// \brief Returns an integer corresponding to the structure of the guard passed as argument \c a_guard.
-    int get_guard_structure(atermpp::aterm_appl a_guard)
+    int get_guard_structure(const data_expression& a_guard)
     {
       if (is_variable(a_guard))
       {
@@ -169,7 +164,7 @@ class InternalFormatInfo
       }
       if (is_equality(a_guard))
       {
-        atermpp::aterm_appl v_term1, v_term2;
+        data_expression v_term1, v_term2;
 
         v_term1 = get_argument(a_guard, 0);
         v_term2 = get_argument(a_guard, 1);
@@ -183,7 +178,7 @@ class InternalFormatInfo
     }
 
     /// \brief Compares the structure of two guards.
-    Compare_Result compare_guard_structure(const atermpp::aterm_appl a_guard1, const atermpp::aterm_appl a_guard2)
+    Compare_Result compare_guard_structure(const data_expression& a_guard1, const data_expression& a_guard2)
     {
       if (get_guard_structure(a_guard1) < get_guard_structure(a_guard2))
       {
@@ -197,11 +192,11 @@ class InternalFormatInfo
     }
 
     /// \brief Compares two guards by their arguments.
-    Compare_Result compare_guard_equality(const atermpp::aterm_appl a_guard1, const atermpp::aterm_appl a_guard2)
+    Compare_Result compare_guard_equality(const data_expression& a_guard1, const data_expression& a_guard2)
     {
       if (f_full && is_equality(a_guard1) && is_equality(a_guard2))
       {
-        atermpp::aterm_appl v_g1a0, v_g1a1, v_g2a0, v_g2a1;
+        data_expression v_g1a0, v_g1a1, v_g2a0, v_g2a1;
 
         v_g1a0 = get_argument(a_guard1, 0);
         v_g1a1 = get_argument(a_guard1, 1);
@@ -220,7 +215,7 @@ class InternalFormatInfo
     }
 
     /// \brief Compares terms by their type.
-    Compare_Result compare_term_type(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2)
+    Compare_Result compare_term_type(const data_expression& a_term1, const data_expression& a_term2)
     {
       if (is_variable(a_term1) && !is_variable(a_term2))
       {
@@ -234,7 +229,7 @@ class InternalFormatInfo
     }
 
     /// \brief Compares terms by checking whether one is a part of the other.
-    Compare_Result compare_term_occurs(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2)
+    Compare_Result compare_term_occurs(const data_expression& a_term1, const data_expression& a_term2)
     {
       if (occurs(a_term1, a_term2))
       {
@@ -249,30 +244,28 @@ class InternalFormatInfo
 
   public:
     /// \brief Constructor that initializes the rewriter.
-    InternalFormatInfo(boost::shared_ptr<detail::Rewriter> a_rewriter)
+    Info()
     {
-      f_rewriter = a_rewriter;
-      f_if_then_else_bool = (f_rewriter->toRewriteFormat(if_(sort_bool::bool_())))(0);
     }
 
     /// \brief Destructor with no particular functionality.
-    virtual ~InternalFormatInfo()
+    ~Info()
     {}
 
-    /// \brief Sets the flag InternalFormatInfo::f_reverse.
+    /// \brief Sets the flag Info::f_reverse.
     void set_reverse(bool a_reverse)
     {
       f_reverse = a_reverse;
     }
 
-    /// \brief Sets the flag InternalFormatInfo::f_full.
+    /// \brief Sets the flag Info::f_full.
     void set_full(bool a_bool)
     {
       f_full = a_bool;
     }
 
     /// \brief Compares two guards.
-    Compare_Result compare_guard(const atermpp::aterm_appl a_guard1, const atermpp::aterm_appl a_guard2)
+    Compare_Result compare_guard(const data_expression& a_guard1, const data_expression& a_guard2)
     {
       return lexico(
                lexico(
@@ -284,7 +277,7 @@ class InternalFormatInfo
     }
 
     /// \brief Compares two terms.
-    Compare_Result compare_term(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2)
+    Compare_Result compare_term(const data_expression& a_term1, const data_expression& a_term2)
     {
       return lexico(
                lexico(
@@ -296,7 +289,7 @@ class InternalFormatInfo
     }
 
     /// \brief Compares two terms using lpo.
-    bool lpo1(atermpp::aterm_appl a_term1, atermpp::aterm_appl a_term2)
+    bool lpo1(const data_expression& a_term1, const data_expression& a_term2)
     {
       if (is_variable(a_term1) && is_variable(a_term2))
       {
@@ -318,23 +311,18 @@ class InternalFormatInfo
 
     /// \brief Indicates whether or not a term has type bool.
     /// \brief Indicates whether or not a term has type bool.
-    bool has_type_bool(const data_expression a_term)
+    /* bool has_type_bool(const data_expression& a_term)
     {
       return a_term.sort()==sort_bool::bool_();
-    }
-
-    bool has_type_bool(const atermpp::aterm_appl a_term)
-    {
-      return f_rewriter->fromRewriteFormat(a_term).sort()==sort_bool::bool_();
-    }
+    } */
 
     /// \brief Returns the number of arguments of the main operator of a term.
     /// \param a_term An expression in the internal format of the rewriter with the jitty strategy.
     /// \return 0, if \c aterm is a constant or a variable.
     ///         The number of arguments of the main operator, otherwise.
-    size_t get_number_of_arguments(const atermpp::aterm_appl a_term)
+    size_t get_number_of_arguments(const data_expression& a_term)
     {
-      if (!is_true(a_term) && !is_false(a_term) && !is_variable(a_term))
+      if (!is_variable(a_term) && !is_function_symbol(a_term))
       {
         return a_term.size() - 1;
       }
@@ -345,55 +333,47 @@ class InternalFormatInfo
     }
 
     /// \brief Returns the main operator of the term \c a_term;
-    atermpp::aterm get_operator(const atermpp::aterm_appl a_term) 
+    data_expression get_operator(const data_expression& a_term)
     {
-      return a_term(0);
+      if (is_function_symbol(a_term))
+      {
+         return a_term;
+      }
+      const application& a = atermpp::down_cast<application>(a_term);
+      return get_operator(a.head());
     }
 
     /// \brief Returns the argument with number \c a_number of the main operator of term \c a_term.
-    atermpp::aterm_appl get_argument(const atermpp::aterm_appl a_term, const size_t a_number)
+    data_expression get_argument(const data_expression& a_term, const size_t a_number)
     {
-      return a_term(a_number + 1);
+      return data_expression(a_term[a_number + 1]);
     }
 
-    /// \brief Indicates whether or not a term is equal to \c true.
-    bool is_true(const atermpp::aterm_appl a_term)
-    {
-      return a_term==f_rewriter->internal_true;
-    }
-
-    /// \brief Indicates whether or not a term is equal to \c false.
-    bool is_false(const atermpp::aterm_appl a_term)
-    {
-      return a_term == f_rewriter->internal_false;
-    }
-
-    /// \brief Indicates whether or not a term is equal to the \c if \c then \c else function
-    /// \brief with type Bool -> Bool -> Bool -> Bool.
-    bool is_if_then_else_bool(const atermpp::aterm_appl a_term)
-    {
-      atermpp::aterm v_function = a_term(0);
-      return (v_function == f_if_then_else_bool && get_number_of_arguments(a_term) == 3);
-    }
-
-    /// \brief Indicates whether or not a term is a single variable.
-    virtual bool is_variable(const atermpp::aterm_appl a_term)
-    {
-      return mcrl2::data::is_variable(a_term);
-    } 
-
     /// \brief Indicates whether or not a term is an equality.
-    virtual bool is_equality(atermpp::aterm_appl a_term)
+    bool is_equality(const data_expression& a_term)
     {
-      if (get_number_of_arguments(a_term) == 2)
+      if (!is_application(a_term))
       {
-        const data_expression d_term = f_rewriter->fromRewriteFormat(a_term);
-        return detail::equal_symbol().is_application(d_term);
+         return false;
       }
-      else
+
+      const application& a = atermpp::down_cast<application>(a_term);
+      if (a.size() != 2)
       {
         return false;
       }
+
+      if (!is_function_symbol(a.head()))
+      {
+        return false;
+      }
+
+      const data::function_symbol& f = atermpp::down_cast<function_symbol>(a.head());
+      if (static_cast<const std::string&>(f.name())=="==")
+      {
+        return true;
+      }
+      return false;
     }
 };
 
