@@ -15,8 +15,6 @@
 #include <string>
 #include <sstream>
 
-#include "mcrl2/aterm/aterm2.h"
-#include "mcrl2/aterm/aterm_ext.h"
 #include "mcrl2/core/print.h"
 #include "mcrl2/data/standard_utility.h"
 #include "mcrl2/utilities/logger.h"
@@ -54,13 +52,31 @@ class SMT_LIB_Solver: public SMT_Solver
     std::string f_variables_extrafuns;
     std::string f_extrapreds;
     std::string f_formula;
-    atermpp::map < sort_expression, size_t > f_sorts;
-    atermpp::map < function_symbol, size_t > f_operators;
-    atermpp::set < variable > f_variables;
-    atermpp::set < variable > f_nat_variables;
-    atermpp::set < variable > f_pos_variables;
+    std::map < sort_expression, size_t > f_sorts;
+    std::map < function_symbol, size_t > f_operators;
+    std::set < variable > f_variables;
+    std::set < variable > f_nat_variables;
+    std::set < variable > f_pos_variables;
     bool f_bool2pred;
 
+    const data_expression& left(const data_expression& x) const
+    {
+      const application& a = atermpp::down_cast<mcrl2::data::application>(x);
+      return binary_left(a);
+    }
+
+    const data_expression& right(const data_expression& x) const
+    {
+      const application& a = atermpp::down_cast<mcrl2::data::application>(x);
+      return binary_right(a);
+    }
+
+    const data_expression& arg(const data_expression& x) const
+    {
+      const application& a = atermpp::down_cast<mcrl2::data::application>(x);
+      return unary_operand(a);
+    }
+
     void declare_sorts()
     {
       f_extrasorts = "";
@@ -68,7 +84,7 @@ class SMT_LIB_Solver: public SMT_Solver
       {
         f_extrasorts = "  :extrasorts (";
         sort_expression v_sort;
-        for(atermpp::map < sort_expression, size_t >::const_iterator i=f_sorts.begin(); i!=f_sorts.end(); ++i)
+        for(std::map < sort_expression, size_t >::const_iterator i=f_sorts.begin(); i!=f_sorts.end(); ++i)
         {
           if (v_sort != sort_expression())
           {
@@ -89,7 +105,7 @@ class SMT_LIB_Solver: public SMT_Solver
       if (!f_operators.empty())
       {
         f_operators_extrafuns = "  :extrafuns (";
-        for(atermpp::map < function_symbol, size_t >::const_iterator i=f_operators.begin(); i!=f_operators.end(); ++i)
+        for(std::map < function_symbol, size_t >::const_iterator i=f_operators.begin(); i!=f_operators.end(); ++i)
         {
           std::stringstream v_operator_string;
           v_operator_string << "op" << i->second;
@@ -135,7 +151,7 @@ class SMT_LIB_Solver: public SMT_Solver
               }
               else
               {
-                atermpp::map < sort_expression, size_t >::const_iterator j=f_sorts.find(v_sort_domain_elt);
+                std::map < sort_expression, size_t >::const_iterator j=f_sorts.find(v_sort_domain_elt);
                 size_t v_sort_number=f_sorts.size();
                 if (j==f_sorts.end())  // not found
                 {
@@ -166,7 +182,7 @@ class SMT_LIB_Solver: public SMT_Solver
         f_variables_extrafuns = "  :extrafuns (";
       }
 
-      for(atermpp::set < variable > :: const_iterator i=f_variables.begin(); i!=f_variables.end(); ++i)
+      for(std::set < variable > :: const_iterator i=f_variables.begin(); i!=f_variables.end(); ++i)
       {
         const variable v_variable = *i;
         std::string v_variable_string = v_variable.name();
@@ -189,7 +205,7 @@ class SMT_LIB_Solver: public SMT_Solver
         }
         else
         {
-          atermpp::map < sort_expression, size_t >::const_iterator j=f_sorts.find(v_sort);
+          std::map < sort_expression, size_t >::const_iterator j=f_sorts.find(v_sort);
           size_t v_sort_number=f_sorts.size();
           if (j==f_sorts.end())  // not found
           {
@@ -232,7 +248,7 @@ class SMT_LIB_Solver: public SMT_Solver
       if (!f_sorts.empty())
       {
         f_sorts_notes = "  :notes \"";
-        for(atermpp::map < sort_expression, size_t >::const_iterator i=f_sorts.begin(); i!=f_sorts.end(); ++i)
+        for(std::map < sort_expression, size_t >::const_iterator i=f_sorts.begin(); i!=f_sorts.end(); ++i)
         {
           std::stringstream v_sort_string;
           v_sort_string << "sort" << i->second;
@@ -249,7 +265,7 @@ class SMT_LIB_Solver: public SMT_Solver
       if (!f_operators.empty())
       {
         f_operators_notes = "  :notes \"";
-        for(atermpp::map < function_symbol, size_t >::const_iterator i=f_operators.begin(); i!=f_operators.end(); ++i)
+        for(std::map < function_symbol, size_t >::const_iterator i=f_operators.begin(); i!=f_operators.end(); ++i)
         {
           std::stringstream v_operator_string;
           v_operator_string << "op" << i->second;
@@ -271,123 +287,121 @@ class SMT_LIB_Solver: public SMT_Solver
       }
     }
 
-    void translate_clause(const data_expression a_clause, const bool a_expecting_predicate)
+    void translate_clause(const data_expression &a_clause, const bool a_expecting_predicate)
     {
-      if (sort_bool::is_not_application(data_expression(a_clause)))
+      if (sort_bool::is_not_application(a_clause))
       {
         translate_not(a_clause);
       }
-      else if (is_equal_to_application(data_expression(a_clause)))
+      else if (is_equal_to_application(a_clause))
       {
         translate_equality(a_clause);
       }
-      else if (is_not_equal_to_application(data_expression(a_clause)))
+      else if (is_not_equal_to_application(a_clause))
       {
         translate_inequality(a_clause);
       }
-      else if (is_greater_application(data_expression(a_clause)))
+      else if (is_greater_application(a_clause))
       {
         translate_greater_than(a_clause);
       }
-      else if (is_greater_equal_application(data_expression(a_clause)))
+      else if (is_greater_equal_application(a_clause))
       {
         translate_greater_than_or_equal(a_clause);
       }
-      else if (is_less_application(data_expression(a_clause)))
+      else if (is_less_application(a_clause))
       {
         translate_less_than(a_clause);
       }
-      else if (is_less_equal_application(data_expression(a_clause)))
+      else if (is_less_equal_application(a_clause))
       {
         translate_less_than_or_equal(a_clause);
       }
-      else if (sort_real::is_plus_application(data_expression(a_clause)))
+      else if (sort_real::is_plus_application(a_clause))
       {
         translate_plus(a_clause);
       }
-      else if (sort_real::is_negate_application(data_expression(a_clause)))
+      else if (sort_real::is_negate_application(a_clause))
       {
         translate_unary_minus(a_clause);
       }
-      else if (sort_real::is_minus_application(data_expression(a_clause)))
+      else if (sort_real::is_minus_application(a_clause))
       {
         translate_binary_minus(a_clause);
       }
-      else if (sort_real::is_times_application(data_expression(a_clause)))
+      else if (sort_real::is_times_application(a_clause))
       {
         translate_multiplication(a_clause);
       }
-      else if (sort_real::is_maximum_application(data_expression(a_clause)))
+      else if (sort_real::is_maximum_application(a_clause))
       {
         translate_max(a_clause);
       }
-      else if (sort_real::is_minimum_application(data_expression(a_clause)))
+      else if (sort_real::is_minimum_application(a_clause))
       {
         translate_min(a_clause);
       }
-      else if (sort_real::is_abs_application(data_expression(a_clause)))
+      else if (sort_real::is_abs_application(a_clause))
       {
         translate_abs(a_clause);
       }
-      else if (sort_real::is_succ_application(data_expression(a_clause)))
+      else if (sort_real::is_succ_application(a_clause))
       {
         translate_succ(a_clause);
       }
-      else if (sort_real::is_pred_application(data_expression(a_clause)))
+      else if (sort_real::is_pred_application(a_clause))
       {
         translate_pred(a_clause);
       }
-      else if (sort_pos::is_add_with_carry_application(data_expression(a_clause)))
+      else if (sort_pos::is_add_with_carry_application(a_clause))
       {
         translate_add_c(a_clause);
       }
-      else if (sort_nat::is_cnat_application(data_expression(a_clause)))
+      else if (sort_nat::is_cnat_application(a_clause))
       {
         translate_c_nat(a_clause);
       }
-      else if (sort_int::is_cint_application(data_expression(a_clause)))
+      else if (sort_int::is_cint_application(a_clause))
       {
         translate_c_int(a_clause);
-        //} else if (gsIsDataExprCReal(data_expression(a_clause))) {
-        //  translate_c_real(a_clause);
       }
-      else if (sort_int::is_integer_constant(data_expression(a_clause)))
+      else if (sort_int::is_integer_constant(a_clause))
       {
         translate_int_constant(a_clause);
       }
-      else if (sort_nat::is_natural_constant(data_expression(a_clause)))
+      else if (sort_nat::is_natural_constant(a_clause))
       {
         translate_nat_constant(a_clause);
       }
-      else if (sort_pos::is_positive_constant(data_expression(a_clause)))
+      else if (sort_pos::is_positive_constant(a_clause))
       {
         translate_pos_constant(a_clause);
       }
-      else if (sort_bool::is_true_function_symbol(data_expression(a_clause)) && a_expecting_predicate)
+      else if (sort_bool::is_true_function_symbol(a_clause) && a_expecting_predicate)
       {
         translate_true();
       }
-      else if (sort_bool::is_false_function_symbol(data_expression(a_clause)) && a_expecting_predicate)
+      else if (sort_bool::is_false_function_symbol(a_clause) && a_expecting_predicate)
       {
         translate_false();
       }
-      else if (core::detail::gsIsDataVarId(a_clause))
+      else if (is_variable(a_clause))
       {
         if (a_expecting_predicate)
         {
           add_bool2pred_and_translate_clause(a_clause);
         }
-        else if (sort_nat::is_nat(data_expression(a_clause).sort()))
+        else if (sort_nat::is_nat(a_clause.sort()))
         {
-          translate_nat_variable(a_clause);
+          translate_nat_variable(atermpp::down_cast<variable>(a_clause));
         }
-        else if (sort_pos::is_pos(data_expression(a_clause).sort()))
+        else if (sort_pos::is_pos(a_clause.sort()))
         {
-          translate_pos_variable(a_clause);
+          translate_pos_variable(atermpp::down_cast<variable>(a_clause));
         }
         else
         {
-          translate_variable(a_clause);
+          translate_variable(atermpp::down_cast<variable>(a_clause));
         }
       }
       else if (is_application(a_clause))
@@ -401,7 +415,7 @@ class SMT_LIB_Solver: public SMT_Solver
           translate_unknown_operator(a_clause);
         }
       }
-      else if (core::detail::gsIsOpId(a_clause))
+      else if (data::is_function_symbol(a_clause))
       {
         translate_constant(a_clause);
       }
@@ -412,7 +426,7 @@ class SMT_LIB_Solver: public SMT_Solver
       }
     }
 
-    void add_bool2pred_and_translate_clause(const data_expression a_clause)
+    void add_bool2pred_and_translate_clause(const data_expression &a_clause)
     {
       f_bool2pred = true;
       f_formula = f_formula + "(bool2pred ";
@@ -420,18 +434,18 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_not(const data_expression a_clause)
+    void translate_not(const data_expression &a_clause)
     {
-      const data_expression v_clause=application(a_clause).argument(0);
+      const data_expression v_clause=arg(a_clause);
       f_formula = f_formula + "(not ";
       translate_clause(v_clause, true);
       f_formula = f_formula + ")";
     }
 
-    void translate_equality(const data_expression a_clause)
+    void translate_equality(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(= ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -439,10 +453,10 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_inequality(const data_expression a_clause)
+    void translate_inequality(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(distinct ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -450,10 +464,10 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_greater_than(const data_expression a_clause)
+    void translate_greater_than(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(> ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -461,10 +475,10 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_greater_than_or_equal(const data_expression a_clause)
+    void translate_greater_than_or_equal(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(>= ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -472,10 +486,10 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_less_than(const data_expression a_clause)
+    void translate_less_than(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(< ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -483,10 +497,10 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_less_than_or_equal(const data_expression a_clause)
+    void translate_less_than_or_equal(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(<= ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -494,10 +508,10 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_plus(const data_expression a_clause)
+    void translate_plus(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(+ ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -505,18 +519,18 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_unary_minus(const data_expression a_clause)
+    void translate_unary_minus(const data_expression &a_clause)
     {
-      const data_expression v_clause = application(a_clause).argument(0);
+      const data_expression v_clause = arg(a_clause);
       f_formula = f_formula + "(~";
       translate_clause(v_clause, false);
       f_formula = f_formula + ")";
     }
 
-    void translate_binary_minus(const data_expression a_clause)
+    void translate_binary_minus(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(- ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -524,10 +538,10 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_multiplication(const data_expression a_clause)
+    void translate_multiplication(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(* ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -535,10 +549,10 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_max(const data_expression a_clause)
+    void translate_max(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(ite (>= ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -550,10 +564,10 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_min(const data_expression a_clause)
+    void translate_min(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
+      const data_expression v_clause_1 = left(a_clause);
+      const data_expression v_clause_2 = right(a_clause);
       f_formula = f_formula + "(ite (<= ";
       translate_clause(v_clause_1, false);
       f_formula = f_formula + " ";
@@ -565,9 +579,9 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_abs(const data_expression a_clause)
+    void translate_abs(const data_expression &a_clause)
     {
-      const data_expression v_clause = application(a_clause).argument(0);
+      const data_expression v_clause = arg(a_clause);
       f_formula = f_formula + "(ite (< 0 ";
       translate_clause(v_clause, false);
       f_formula = f_formula + ") ~";
@@ -577,27 +591,29 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_succ(const data_expression a_clause)
+    void translate_succ(const data_expression &a_clause)
     {
-      const data_expression v_clause = application(a_clause).argument(0);
+      const data_expression v_clause = arg(a_clause);
       f_formula = f_formula + "(+ ";
       translate_clause(v_clause, false);
       f_formula = f_formula + " 1)";
     }
 
-    void translate_pred(const data_expression a_clause)
+    void translate_pred(const data_expression &a_clause)
     {
-      const data_expression v_clause = application(a_clause).argument(0);
+      const data_expression v_clause = arg(a_clause);
       f_formula = f_formula + "(- ";
       translate_clause(v_clause, false);
       f_formula = f_formula + " 1)";
     }
 
-    void translate_add_c(const data_expression a_clause)
+    void translate_add_c(const data_expression &a_clause)
     {
-      const data_expression v_clause_1 = application(a_clause).argument(0);
-      const data_expression v_clause_2 = application(a_clause).argument(1);
-      const data_expression v_clause_3 = application(a_clause).argument(2);
+      const application& v_clause = atermpp::down_cast<application>(a_clause);
+      application::const_iterator arg = v_clause.begin();
+      const data_expression v_clause_1 = *arg++;
+      const data_expression v_clause_2 = *arg++;
+      const data_expression v_clause_3 = *arg++;
       f_formula = f_formula + "(ite ";
       translate_clause(v_clause_1, true);
       f_formula = f_formula + " (+ ";
@@ -611,22 +627,23 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + "))";
     }
 
-    void translate_c_nat(const data_expression a_clause)
+    void translate_c_nat(const data_expression &a_clause)
     {
-      const data_expression v_clause = application(a_clause).argument(0);
+      const data_expression v_clause = arg(a_clause);
       translate_clause(v_clause, false);
     }
 
-    void translate_c_int(const data_expression a_clause)
+    void translate_c_int(const data_expression &a_clause)
     {
-      const data_expression v_clause = application(a_clause).argument(0);
+      const data_expression v_clause = arg(a_clause);
       translate_clause(v_clause, false);
     }
 
-    void translate_unknown_operator(const data_expression a_clause)
+    void translate_unknown_operator(const data_expression &a_clause)
     {
-      const data_expression v_operator = application(a_clause).head();
-      atermpp::map < function_symbol, size_t >::const_iterator i=f_operators.find(v_operator);
+      data_expression h = application(a_clause).head();
+      const function_symbol& v_operator = atermpp::down_cast<function_symbol>(h);
+      std::map < function_symbol, size_t >::const_iterator i=f_operators.find(v_operator);
 
       size_t v_operator_number=f_operators.size(); // This is the value if v_operator does not occur in f_operators.
       if (i==f_operators.end()) // not found.
@@ -643,8 +660,8 @@ class SMT_LIB_Solver: public SMT_Solver
 
       if (data::is_application(a_clause))
       {
-        data::application a = data::application(data::data_expression(a_clause));
-        for (data_expression_list::iterator i = a.arguments().begin(); i != a.arguments().end(); ++i)
+        const data::application &a = atermpp::down_cast<data::application>(a_clause);
+        for (application::const_iterator i = a.begin(); i != a.end(); ++i)
         {
           f_formula = f_formula + " ";
           translate_clause(*i, false);
@@ -653,7 +670,7 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + ")";
     }
 
-    void translate_variable(const variable a_clause)
+    void translate_variable(const variable &a_clause)
     {
       std::string v_string = a_clause.name();
       f_formula = f_formula + v_string;
@@ -661,7 +678,7 @@ class SMT_LIB_Solver: public SMT_Solver
       f_variables.insert(a_clause);
     }
 
-    void translate_nat_variable(const variable a_clause)
+    void translate_nat_variable(const variable &a_clause)
     {
       std::string v_string = a_clause.name();
       f_formula = f_formula + v_string;
@@ -670,7 +687,7 @@ class SMT_LIB_Solver: public SMT_Solver
       f_nat_variables.insert(a_clause);
     }
 
-    void translate_pos_variable(const variable a_clause)
+    void translate_pos_variable(const variable &a_clause)
     {
       std::string v_string = a_clause.name();
       f_formula = f_formula + v_string;
@@ -679,7 +696,7 @@ class SMT_LIB_Solver: public SMT_Solver
       f_pos_variables.insert(a_clause);
     }
 
-    void translate_int_constant(const data_expression a_clause)
+    void translate_int_constant(const data_expression &a_clause)
     {
       std::string v_value(data::sort_int::integer_constant_as_string(data::data_expression(a_clause)));
       if (v_value[0] == '-')
@@ -693,13 +710,13 @@ class SMT_LIB_Solver: public SMT_Solver
       }
     }
 
-    void translate_nat_constant(const data_expression a_clause)
+    void translate_nat_constant(const data_expression &a_clause)
     {
       std::string v_value(data::sort_nat::natural_constant_as_string(data::data_expression(a_clause)));
       f_formula = f_formula + v_value;
     }
 
-    void translate_pos_constant(const data_expression a_clause)
+    void translate_pos_constant(const data_expression &a_clause)
     {
       std::string v_value(data::sort_pos::positive_constant_as_string(data::data_expression(a_clause)));
       f_formula = f_formula + v_value;
@@ -715,10 +732,11 @@ class SMT_LIB_Solver: public SMT_Solver
       f_formula = f_formula + "false";
     }
 
-    void translate_constant(const data_expression a_clause)
+    void translate_constant(const data_expression &a_clause)
     {
-      const data_expression v_operator = application(a_clause).head();
-      atermpp::map < function_symbol, size_t >::const_iterator i=f_operators.find(v_operator);
+      data_expression h = application(a_clause).head();
+      const function_symbol& v_operator = atermpp::down_cast<function_symbol>(h);
+      std::map < function_symbol, size_t >::const_iterator i=f_operators.find(v_operator);
 
       size_t v_operator_number=f_operators.size(); // This is the value if v_operator does not occur in f_operators.
       if (i==f_operators.end()) // not found.
@@ -737,7 +755,7 @@ class SMT_LIB_Solver: public SMT_Solver
 
     void add_nat_clauses()
     {
-      for(atermpp::set < variable >::const_iterator i=f_nat_variables.begin(); i!=f_nat_variables.end(); ++i)
+      for(std::set < variable >::const_iterator i=f_nat_variables.begin(); i!=f_nat_variables.end(); ++i)
       {
         std::string v_variable_string = i->name();
         f_formula = f_formula + " (>= " + v_variable_string + " 0)";
@@ -746,7 +764,7 @@ class SMT_LIB_Solver: public SMT_Solver
 
     void add_pos_clauses()
     {
-      for(atermpp::set < variable >::const_iterator i=f_pos_variables.begin(); i!=f_pos_variables.end(); ++i)
+      for(std::set < variable >::const_iterator i=f_pos_variables.begin(); i!=f_pos_variables.end(); ++i)
       {
         std::string v_variable_string = i->name();
         f_formula = f_formula + " (>= " + v_variable_string + " 1)";
@@ -769,11 +787,11 @@ class SMT_LIB_Solver: public SMT_Solver
       f_bool2pred = false;
 
       f_formula = "  :formula (and";
-      mCRL2log(log::verbose) << "Formula to be solved: " << data::pp(a_formula) << std::endl;
+      mCRL2log(log::verbose) << "Formula to be solved: " << a_formula << std::endl;
       while (!a_formula.empty())
       {
         v_clause = a_formula.front();
-        a_formula = pop_front(a_formula);
+        a_formula.pop_front();
         f_formula = f_formula + " ";
         translate_clause(v_clause, true);
       }
@@ -875,7 +893,7 @@ class cvc_smt_solver : public SMT_LIB_Solver, public binary_smt_solver< cvc_smt_
     /// precondition: The argument passed as parameter a_formula is a list of expressions of sort Bool in internal mCRL2
     /// format. The argument represents a formula in conjunctive normal form, where the elements of the list represent the
     /// clauses
-    bool is_satisfiable(const data_expression_list a_formula)
+    bool is_satisfiable(const data_expression_list &a_formula)
     {
       translate(a_formula);
 
@@ -912,7 +930,7 @@ class ario_smt_solver : public SMT_LIB_Solver, public binary_smt_solver< ario_sm
     /// precondition: The argument passed as parameter a_formula is a list of expressions of sort Bool in internal mCRL2
     /// format. The argument represents a formula in conjunctive normal form, where the elements of the list represent the
     /// clauses
-    bool is_satisfiable(const data_expression_list a_formula)
+    bool is_satisfiable(const data_expression_list &a_formula)
     {
       translate(a_formula);
 
