@@ -12,7 +12,6 @@
 #ifndef __REWR_PROVER_H
 #define __REWR_PROVER_H
 
-#include "mcrl2/aterm/aterm2.h"
 #include <mcrl2/data/detail/bdd_prover.h>
 #include "mcrl2/data/rewriter.h"
 
@@ -27,10 +26,9 @@ class RewriterProver: public Rewriter
 {
   public:
     BDD_Prover* prover_obj;
-    boost::shared_ptr<detail::Rewriter> rewr_obj;
+    std::shared_ptr<detail::Rewriter> rewr_obj;
 
     typedef Rewriter::substitution_type substitution_type;
-    typedef Rewriter::internal_substitution_type internal_substitution_type;
 
   public:
     RewriterProver(const data_specification& DataSpec, mcrl2::data::rewriter::strategy strat, const used_data_equation_selector& equations_selector);
@@ -39,19 +37,9 @@ class RewriterProver: public Rewriter
     rewrite_strategy getStrategy();
 
     data_expression rewrite(
-         const data_expression term,
+         const data_expression &term,
          substitution_type &sigma);
 
-    atermpp::aterm_appl rewrite_internal(
-         const atermpp::aterm_appl term,
-         internal_substitution_type &sigma);
-
-    atermpp::aterm_appl toRewriteFormat(const data_expression term);
-    // data_expression fromRewriteFormat(const atermpp::aterm_appl term);
-
-    bool addRewriteRule(const data_equation rule);
-    bool removeRewriteRule(const data_equation rule);
-
 };
 
 }
