@@ -47,7 +47,7 @@ class exists: public abstraction
     template < typename Container >
     exists(const Container& variables,
            const data_expression& body,
-           typename atermpp::detail::enable_if_container< Container, variable >::type* = 0)
+           typename atermpp::enable_if_container< Container, variable >::type* = 0)
       : abstraction(exists_binder(), variables, body)
     {
       assert(!variables.empty());
@@ -55,6 +55,26 @@ class exists: public abstraction
 
 }; // class exists
 
+//--- start generated class exists ---//
+// prototype declaration
+std::string pp(const exists& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const exists& x)
+{
+  return out << data::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(exists& t1, exists& t2)
+{
+  t1.swap(t2);
+}
+//--- end generated class exists ---//
+
 } // namespace data
 
 } // namespace mcrl2
