@@ -14,6 +14,8 @@
 
 #include <functional>
 #include "mcrl2/data/builder.h"
+#include "mcrl2/data/standard_utility.h"
+#include "mcrl2/data/standard_container_utility.h"
 
 namespace mcrl2
 {
@@ -36,16 +38,14 @@ class translate_user_notation_builder: public data_expression_builder<Derived>
 
     data_expression operator()(const abstraction& x)
     {
-      //variable_list bound_variables = atermpp::convert<variable_list>((*this)(x.variables()));
       variable_list bound_variables = x.variables();
 
-      if (atermpp::function_symbol(atermpp::arg1(x).function()).name() == "SetComp")
+      if (is_set_comprehension(x))
       {
-        //sort_expression element_sort((*this)(x.variables().begin()->sort()));
         sort_expression element_sort(x.variables().begin()->sort());
         return sort_set::constructor(element_sort, lambda(bound_variables, static_cast<Derived&>(*this)(x.body())),sort_fset::empty(element_sort));
       }
-      else if (atermpp::function_symbol(atermpp::arg1(x).function()).name() == "BagComp")
+      else if (is_bag_comprehension(x))
       {
         sort_expression element_sort(x.variables().begin()->sort());
 
@@ -75,29 +75,35 @@ class translate_user_notation_builder: public data_expression_builder<Derived>
       {
         function_symbol head(x.head());
 
-        if (head.name() == "@ListEnum")
+        if (head.name() == sort_list::list_enumeration_name())
         {
           // convert to snoc list
           sort_expression element_sort(*function_sort(head.sort()).domain().begin());
 
-          return sort_list::list(element_sort, static_cast<Derived&>(*this)(x.arguments()));
+          return sort_list::list(element_sort, static_cast<Derived&>(*this)(data_expression_list(x.begin(), x.end())));
         }
-        else if (head.name() == "@SetEnum")
+        else if (head.name() == sort_set::set_enumeration_name())
         {
           // convert to finite set
           sort_expression element_sort(*function_sort(head.sort()).domain().begin());
-
-          return sort_set::set_fset(element_sort, sort_fset::fset(element_sort, static_cast<Derived&>(*this)(x.arguments())));
+          return sort_fset::fset(element_sort, static_cast<Derived&>(*this)(data_expression_list(x.begin(), x.end())));
         }
-        else if (head.name() == "@BagEnum")
+        else if (head.name() == sort_bag::bag_enumeration_name())
         {
           // convert to finite bag
-          using namespace sort_bag;
           sort_expression element_sort(*function_sort(head.sort()).domain().begin());
-          return sort_bag::bag_fbag(element_sort, sort_fbag::fbag(element_sort, static_cast<Derived&>(*this)(x.arguments())));
+          return sort_fbag::fbag(element_sort, static_cast<Derived&>(*this)(data_expression_list(x.begin(), x.end())));
         }
       }
-      data_expression result = application(static_cast<Derived&>(*this)(x.head()), static_cast<Derived&>(*this)(x.arguments()));
+
+      typedef data::data_expression (Derived::*function_pointer)(const data::data_expression&);
+      function_pointer fp = &Derived::operator();
+      data_expression result = application(
+         static_cast<Derived&>(*this)(x.head()),
+         x.begin(),
+         x.end(),
+         boost::bind(fp, static_cast<Derived*>(this), _1)
+      );
       static_cast<Derived&>(*this).leave(x);
       return result;
     }
@@ -115,7 +121,7 @@ struct translate_user_notation_function: public std::unary_function<data_express
 
 template <typename T>
 void translate_user_notation(T& x,
-                             typename boost::disable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                             typename std::enable_if< !std::is_base_of< atermpp::aterm, T >::value >::type* = 0
                             )
 {
   core::make_update_apply_builder<data::data_expression_builder>(detail::translate_user_notation_function())(x);
@@ -123,7 +129,7 @@ void translate_user_notation(T& x,
 
 template <typename T>
 T translate_user_notation(const T& x,
-                          typename boost::enable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                          typename std::enable_if< std::is_base_of< atermpp::aterm, T >::value>::type* = 0
                          )
 {
   T result = core::make_update_apply_builder<data::data_expression_builder>(detail::translate_user_notation_function())(x);
