@@ -17,17 +17,19 @@
 #ifndef MCRL2_DATA_TRAVERSER_H
 #define MCRL2_DATA_TRAVERSER_H
 
-#include "boost/utility/enable_if.hpp"
-#include "boost/type_traits/is_base_of.hpp"
+#include <type_traits>
 
 #include "mcrl2/core/traverser.h"
 #include "mcrl2/data/assignment.h"
 #include "mcrl2/data/data_specification.h"
-#include "mcrl2/data/multiple_possible_sorts.h"
 #include "mcrl2/data/exists.h"
 #include "mcrl2/data/forall.h"
 #include "mcrl2/data/lambda.h"
-#include "mcrl2/data/unknown_sort.h"
+#include "mcrl2/data/set_comprehension.h"
+#include "mcrl2/data/bag_comprehension.h"
+#include "mcrl2/data/untyped_possible_sorts.h"
+#include "mcrl2/data/untyped_set_or_bag_comprehension.h"
+#include "mcrl2/data/untyped_sort.h"
 
 namespace mcrl2
 {
@@ -45,13 +47,6 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
   using super::leave;
   using super::operator();
 
-  void operator()(const data::identifier& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
   void operator()(const data::variable& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -70,7 +65,7 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.head());
-    static_cast<Derived&>(*this)(x.arguments());
+    for (auto i = x.begin(); i != x.end(); ++i) { static_cast<Derived&>(*this)(*i); }
     static_cast<Derived&>(*this).leave(x);
   }
 
@@ -82,6 +77,13 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::untyped_identifier& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    // skip
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::assignment& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -90,7 +92,7 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const data::identifier_assignment& x)
+  void operator()(const data::untyped_identifier_assignment& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.rhs());
@@ -126,14 +128,14 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const data::unknown_sort& x)
+  void operator()(const data::untyped_sort& x)
   {
     static_cast<Derived&>(*this).enter(x);
     // skip
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const data::multiple_possible_sorts& x)
+  void operator()(const data::untyped_possible_sorts& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.sorts());
@@ -164,6 +166,30 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::set_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.variables());
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
+  void operator()(const data::bag_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.variables());
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
+  void operator()(const data::untyped_set_or_bag_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.variables());
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::structured_sort_constructor_argument& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -178,6 +204,13 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::alias& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.reference());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::data_equation& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -191,30 +224,29 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
   void operator()(const data::data_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::data_expression result;
     if (data::is_abstraction(x))
     {
-      static_cast<Derived&>(*this)(data::abstraction(atermpp::aterm_appl(x)));
-    }
-    else if (data::is_identifier(x))
-    {
-      static_cast<Derived&>(*this)(data::identifier(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::abstraction>(x));
     }
     else if (data::is_variable(x))
     {
-      static_cast<Derived&>(*this)(data::variable(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::variable>(x));
     }
     else if (data::is_function_symbol(x))
     {
-      static_cast<Derived&>(*this)(data::function_symbol(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::function_symbol>(x));
     }
     else if (data::is_application(x))
     {
-      static_cast<Derived&>(*this)(data::application(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::application>(x));
     }
     else if (data::is_where_clause(x))
     {
-      static_cast<Derived&>(*this)(data::where_clause(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::where_clause>(x));
+    }
+    else if (data::is_untyped_identifier(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_identifier>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -222,14 +254,13 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
   void operator()(const data::assignment_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::assignment_expression result;
     if (data::is_assignment(x))
     {
-      static_cast<Derived&>(*this)(data::assignment(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::assignment>(x));
     }
-    else if (data::is_identifier_assignment(x))
+    else if (data::is_untyped_identifier_assignment(x))
     {
-      static_cast<Derived&>(*this)(data::identifier_assignment(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_identifier_assignment>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -237,30 +268,29 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
   void operator()(const data::sort_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::sort_expression result;
     if (data::is_basic_sort(x))
     {
-      static_cast<Derived&>(*this)(data::basic_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::basic_sort>(x));
     }
     else if (data::is_container_sort(x))
     {
-      static_cast<Derived&>(*this)(data::container_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::container_sort>(x));
     }
     else if (data::is_structured_sort(x))
     {
-      static_cast<Derived&>(*this)(data::structured_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::structured_sort>(x));
     }
     else if (data::is_function_sort(x))
     {
-      static_cast<Derived&>(*this)(data::function_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::function_sort>(x));
     }
-    else if (data::is_unknown_sort(x))
+    else if (data::is_untyped_sort(x))
     {
-      static_cast<Derived&>(*this)(data::unknown_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_sort>(x));
     }
-    else if (data::is_multiple_possible_sorts(x))
+    else if (data::is_untyped_possible_sorts(x))
     {
-      static_cast<Derived&>(*this)(data::multiple_possible_sorts(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_possible_sorts>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -268,18 +298,29 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
   void operator()(const data::abstraction& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::abstraction result;
     if (data::is_forall(x))
     {
-      static_cast<Derived&>(*this)(data::forall(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::forall>(x));
     }
     else if (data::is_exists(x))
     {
-      static_cast<Derived&>(*this)(data::exists(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::exists>(x));
     }
     else if (data::is_lambda(x))
     {
-      static_cast<Derived&>(*this)(data::lambda(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::lambda>(x));
+    }
+    else if (data::is_set_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::set_comprehension>(x));
+    }
+    else if (data::is_bag_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::bag_comprehension>(x));
+    }
+    else if (data::is_untyped_set_or_bag_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_set_or_bag_comprehension>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -306,13 +347,6 @@ struct add_traverser_data_expressions: public Traverser<Derived>
   using super::leave;
   using super::operator();
 
-  void operator()(const data::identifier& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
   void operator()(const data::variable& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -331,7 +365,7 @@ struct add_traverser_data_expressions: public Traverser<Derived>
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.head());
-    static_cast<Derived&>(*this)(x.arguments());
+    for (auto i = x.begin(); i != x.end(); ++i) { static_cast<Derived&>(*this)(*i); }
     static_cast<Derived&>(*this).leave(x);
   }
 
@@ -343,6 +377,13 @@ struct add_traverser_data_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::untyped_identifier& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    // skip
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::assignment& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -350,7 +391,7 @@ struct add_traverser_data_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const data::identifier_assignment& x)
+  void operator()(const data::untyped_identifier_assignment& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.rhs());
@@ -378,6 +419,27 @@ struct add_traverser_data_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::set_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
+  void operator()(const data::bag_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
+  void operator()(const data::untyped_set_or_bag_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::data_equation& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -390,30 +452,29 @@ struct add_traverser_data_expressions: public Traverser<Derived>
   void operator()(const data::data_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::data_expression result;
     if (data::is_abstraction(x))
     {
-      static_cast<Derived&>(*this)(data::abstraction(atermpp::aterm_appl(x)));
-    }
-    else if (data::is_identifier(x))
-    {
-      static_cast<Derived&>(*this)(data::identifier(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::abstraction>(x));
     }
     else if (data::is_variable(x))
     {
-      static_cast<Derived&>(*this)(data::variable(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::variable>(x));
     }
     else if (data::is_function_symbol(x))
     {
-      static_cast<Derived&>(*this)(data::function_symbol(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::function_symbol>(x));
     }
     else if (data::is_application(x))
     {
-      static_cast<Derived&>(*this)(data::application(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::application>(x));
     }
     else if (data::is_where_clause(x))
     {
-      static_cast<Derived&>(*this)(data::where_clause(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::where_clause>(x));
+    }
+    else if (data::is_untyped_identifier(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_identifier>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -421,14 +482,13 @@ struct add_traverser_data_expressions: public Traverser<Derived>
   void operator()(const data::assignment_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::assignment_expression result;
     if (data::is_assignment(x))
     {
-      static_cast<Derived&>(*this)(data::assignment(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::assignment>(x));
     }
-    else if (data::is_identifier_assignment(x))
+    else if (data::is_untyped_identifier_assignment(x))
     {
-      static_cast<Derived&>(*this)(data::identifier_assignment(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_identifier_assignment>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -436,18 +496,29 @@ struct add_traverser_data_expressions: public Traverser<Derived>
   void operator()(const data::abstraction& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::abstraction result;
     if (data::is_forall(x))
     {
-      static_cast<Derived&>(*this)(data::forall(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::forall>(x));
     }
     else if (data::is_exists(x))
     {
-      static_cast<Derived&>(*this)(data::exists(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::exists>(x));
     }
     else if (data::is_lambda(x))
     {
-      static_cast<Derived&>(*this)(data::lambda(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::lambda>(x));
+    }
+    else if (data::is_set_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::set_comprehension>(x));
+    }
+    else if (data::is_bag_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::bag_comprehension>(x));
+    }
+    else if (data::is_untyped_set_or_bag_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_set_or_bag_comprehension>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -474,13 +545,6 @@ struct add_traverser_variables: public Traverser<Derived>
   using super::leave;
   using super::operator();
 
-  void operator()(const data::identifier& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
   void operator()(const data::variable& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -499,7 +563,7 @@ struct add_traverser_variables: public Traverser<Derived>
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.head());
-    static_cast<Derived&>(*this)(x.arguments());
+    for (auto i = x.begin(); i != x.end(); ++i) { static_cast<Derived&>(*this)(*i); }
     static_cast<Derived&>(*this).leave(x);
   }
 
@@ -511,6 +575,13 @@ struct add_traverser_variables: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::untyped_identifier& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    // skip
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::assignment& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -519,7 +590,7 @@ struct add_traverser_variables: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const data::identifier_assignment& x)
+  void operator()(const data::untyped_identifier_assignment& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.rhs());
@@ -550,6 +621,30 @@ struct add_traverser_variables: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::set_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.variables());
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
+  void operator()(const data::bag_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.variables());
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
+  void operator()(const data::untyped_set_or_bag_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.variables());
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::data_equation& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -563,30 +658,29 @@ struct add_traverser_variables: public Traverser<Derived>
   void operator()(const data::data_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::data_expression result;
     if (data::is_abstraction(x))
     {
-      static_cast<Derived&>(*this)(data::abstraction(atermpp::aterm_appl(x)));
-    }
-    else if (data::is_identifier(x))
-    {
-      static_cast<Derived&>(*this)(data::identifier(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::abstraction>(x));
     }
     else if (data::is_variable(x))
     {
-      static_cast<Derived&>(*this)(data::variable(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::variable>(x));
     }
     else if (data::is_function_symbol(x))
     {
-      static_cast<Derived&>(*this)(data::function_symbol(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::function_symbol>(x));
     }
     else if (data::is_application(x))
     {
-      static_cast<Derived&>(*this)(data::application(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::application>(x));
     }
     else if (data::is_where_clause(x))
     {
-      static_cast<Derived&>(*this)(data::where_clause(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::where_clause>(x));
+    }
+    else if (data::is_untyped_identifier(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_identifier>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -594,14 +688,13 @@ struct add_traverser_variables: public Traverser<Derived>
   void operator()(const data::assignment_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::assignment_expression result;
     if (data::is_assignment(x))
     {
-      static_cast<Derived&>(*this)(data::assignment(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::assignment>(x));
     }
-    else if (data::is_identifier_assignment(x))
+    else if (data::is_untyped_identifier_assignment(x))
     {
-      static_cast<Derived&>(*this)(data::identifier_assignment(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_identifier_assignment>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -609,18 +702,29 @@ struct add_traverser_variables: public Traverser<Derived>
   void operator()(const data::abstraction& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::abstraction result;
     if (data::is_forall(x))
     {
-      static_cast<Derived&>(*this)(data::forall(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::forall>(x));
     }
     else if (data::is_exists(x))
     {
-      static_cast<Derived&>(*this)(data::exists(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::exists>(x));
     }
     else if (data::is_lambda(x))
     {
-      static_cast<Derived&>(*this)(data::lambda(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::lambda>(x));
+    }
+    else if (data::is_set_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::set_comprehension>(x));
+    }
+    else if (data::is_bag_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::bag_comprehension>(x));
+    }
+    else if (data::is_untyped_set_or_bag_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_set_or_bag_comprehension>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -647,13 +751,6 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
   using super::leave;
   using super::operator();
 
-  void operator()(const data::identifier& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    static_cast<Derived&>(*this)(x.name());
-    static_cast<Derived&>(*this).leave(x);
-  }
-
   void operator()(const data::variable& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -674,7 +771,7 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.head());
-    static_cast<Derived&>(*this)(x.arguments());
+    for (auto i = x.begin(); i != x.end(); ++i) { static_cast<Derived&>(*this)(*i); }
     static_cast<Derived&>(*this).leave(x);
   }
 
@@ -686,6 +783,13 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::untyped_identifier& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.name());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::assignment& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -694,7 +798,7 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const data::identifier_assignment& x)
+  void operator()(const data::untyped_identifier_assignment& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.lhs());
@@ -731,14 +835,14 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const data::unknown_sort& x)
+  void operator()(const data::untyped_sort& x)
   {
     static_cast<Derived&>(*this).enter(x);
     // skip
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const data::multiple_possible_sorts& x)
+  void operator()(const data::untyped_possible_sorts& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.sorts());
@@ -769,6 +873,30 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::set_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.variables());
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
+  void operator()(const data::bag_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.variables());
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
+  void operator()(const data::untyped_set_or_bag_comprehension& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.variables());
+    static_cast<Derived&>(*this)(x.body());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::structured_sort_constructor_argument& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -786,6 +914,14 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
+  void operator()(const data::alias& x)
+  {
+    static_cast<Derived&>(*this).enter(x);
+    static_cast<Derived&>(*this)(x.name());
+    static_cast<Derived&>(*this)(x.reference());
+    static_cast<Derived&>(*this).leave(x);
+  }
+
   void operator()(const data::data_equation& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -799,30 +935,29 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
   void operator()(const data::data_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::data_expression result;
     if (data::is_abstraction(x))
     {
-      static_cast<Derived&>(*this)(data::abstraction(atermpp::aterm_appl(x)));
-    }
-    else if (data::is_identifier(x))
-    {
-      static_cast<Derived&>(*this)(data::identifier(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::abstraction>(x));
     }
     else if (data::is_variable(x))
     {
-      static_cast<Derived&>(*this)(data::variable(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::variable>(x));
     }
     else if (data::is_function_symbol(x))
     {
-      static_cast<Derived&>(*this)(data::function_symbol(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::function_symbol>(x));
     }
     else if (data::is_application(x))
     {
-      static_cast<Derived&>(*this)(data::application(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::application>(x));
     }
     else if (data::is_where_clause(x))
     {
-      static_cast<Derived&>(*this)(data::where_clause(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::where_clause>(x));
+    }
+    else if (data::is_untyped_identifier(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_identifier>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -830,14 +965,13 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
   void operator()(const data::assignment_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::assignment_expression result;
     if (data::is_assignment(x))
     {
-      static_cast<Derived&>(*this)(data::assignment(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::assignment>(x));
     }
-    else if (data::is_identifier_assignment(x))
+    else if (data::is_untyped_identifier_assignment(x))
     {
-      static_cast<Derived&>(*this)(data::identifier_assignment(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_identifier_assignment>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -845,30 +979,29 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
   void operator()(const data::sort_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::sort_expression result;
     if (data::is_basic_sort(x))
     {
-      static_cast<Derived&>(*this)(data::basic_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::basic_sort>(x));
     }
     else if (data::is_container_sort(x))
     {
-      static_cast<Derived&>(*this)(data::container_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::container_sort>(x));
     }
     else if (data::is_structured_sort(x))
     {
-      static_cast<Derived&>(*this)(data::structured_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::structured_sort>(x));
     }
     else if (data::is_function_sort(x))
     {
-      static_cast<Derived&>(*this)(data::function_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::function_sort>(x));
     }
-    else if (data::is_unknown_sort(x))
+    else if (data::is_untyped_sort(x))
     {
-      static_cast<Derived&>(*this)(data::unknown_sort(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_sort>(x));
     }
-    else if (data::is_multiple_possible_sorts(x))
+    else if (data::is_untyped_possible_sorts(x))
     {
-      static_cast<Derived&>(*this)(data::multiple_possible_sorts(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_possible_sorts>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -876,18 +1009,29 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
   void operator()(const data::abstraction& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    data::abstraction result;
     if (data::is_forall(x))
     {
-      static_cast<Derived&>(*this)(data::forall(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::forall>(x));
     }
     else if (data::is_exists(x))
     {
-      static_cast<Derived&>(*this)(data::exists(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::exists>(x));
     }
     else if (data::is_lambda(x))
     {
-      static_cast<Derived&>(*this)(data::lambda(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::lambda>(x));
+    }
+    else if (data::is_set_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::set_comprehension>(x));
+    }
+    else if (data::is_bag_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::bag_comprehension>(x));
+    }
+    else if (data::is_untyped_set_or_bag_comprehension(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::untyped_set_or_bag_comprehension>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
