@@ -9,15 +9,12 @@
 /// \file list_test.cpp
 /// \brief Basic regression test for list expressions.
 
-#include <boost/range/iterator_range.hpp>
 #include <boost/test/minimal.hpp>
 
 #include "mcrl2/data/list.h"
 #include "mcrl2/data/parse.h"
 #include "mcrl2/data/rewriter.h"
 
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 using namespace mcrl2::data;
@@ -45,6 +42,7 @@ void list_expression_test()
 
   specification.add_context_sort(sort_list::list(sort_pos::pos()));
 
+
   data::rewriter normaliser(specification);
 
   variable_vector v;
@@ -53,7 +51,7 @@ void list_expression_test()
   test_data_expression("1 in l", v, is_in_application);
   test_data_expression("10 |> l", v, is_cons_application);
   test_data_expression("l <| 10", v, is_snoc_application);
-  test_data_expression("#l", v, is_count_application);
+  test_data_expression("#l", v, is_count_application); 
   test_data_expression("l ++ [10]", v, is_concat_application);
   test_data_expression("l.1", v, is_element_at_application);
   test_data_expression("head(l)", v, is_head_application);
@@ -62,16 +60,14 @@ void list_expression_test()
   test_data_expression("rtail(l)", v, is_rtail_application);
 
   data_expression e = parse_data_expression("[10]", v.begin(), v.end());
-  BOOST_CHECK(is_cons_application(normaliser(e)));
+  BOOST_CHECK(is_cons_application(normaliser(e))); 
+
 
 }
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv);
-
   list_expression_test();
-  core::garbage_collect();
 
   return EXIT_SUCCESS;
 }
