@@ -9,12 +9,12 @@
 /// \file data_utility_test.cpp
 /// \brief Add your file description here.
 
+#include <iterator>
 #include <iostream>
 #include <string>
 #include <set>
 #include <algorithm>
 
-#include "boost/bind.hpp"
 #include "boost/test/minimal.hpp"
 
 #include "mcrl2/data/representative_generator.h"
@@ -22,9 +22,7 @@
 #include "mcrl2/data/bool.h"
 #include "mcrl2/data/list.h"
 #include "mcrl2/data/structured_sort.h"
-#include "mcrl2/core/garbage_collection.h"
 #include "mcrl2/data/utility.h"
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/data/parse.h"
 
 using namespace mcrl2;
@@ -38,16 +36,16 @@ void test_representative_generator()
                                     "    h__:Set(Real);\n"
                                    );
 
-  atermpp::vector< data::structured_sort_constructor_argument > arguments;
+  std::vector< data::structured_sort_constructor_argument > arguments;
   arguments.push_back(structured_sort_constructor_argument("s", basic_sort("E")));
   arguments.push_back(structured_sort_constructor_argument("n", sort_nat::nat()));
 
-  atermpp::vector< structured_sort_constructor > constructors;
-  constructors.push_back(structured_sort_constructor("d", boost::make_iterator_range(arguments.begin(), arguments.begin() + 1)));
-  constructors.push_back(structured_sort_constructor("e", boost::make_iterator_range(arguments.begin() + 1, arguments.begin() + 2)));
+  std::vector< structured_sort_constructor > constructors;
+  constructors.push_back(structured_sort_constructor("d", structured_sort_constructor_argument_list(arguments.begin(), arguments.begin() + 1)));
+  constructors.push_back(structured_sort_constructor("e", structured_sort_constructor_argument_list(arguments.begin() + 1, arguments.begin() + 2)));
 
-  specification.add_alias(alias(basic_sort("D"), structured_sort(boost::make_iterator_range(constructors.begin(), constructors.begin() + 1))));
-  specification.add_alias(alias(basic_sort("E"), structured_sort(boost::make_iterator_range(constructors.begin() + 1, constructors.begin() + 2))));
+  specification.add_alias(alias(basic_sort("D"), structured_sort(structured_sort_constructor_list(constructors.begin(), constructors.begin() + 1))));
+  specification.add_alias(alias(basic_sort("E"), structured_sort(structured_sort_constructor_list(constructors.begin() + 1, constructors.begin() + 2))));
 
   representative_generator default_expression_generator(specification);
 
@@ -63,25 +61,24 @@ void test_representative_generator()
 
   // Should be e(0), since constants are preferred to other constructors or mappings
   BOOST_CHECK(default_expression_generator(basic_sort("E")) ==
-              boost::next(constructors.begin(), 1)->constructor_function(basic_sort("E"))(default_expression_generator(sort_nat::nat())));
+              constructors[1].constructor_function(basic_sort("E"))(default_expression_generator(sort_nat::nat())));
 
   // Should be d(e(0)), since constants are preferred to other constructors or mappings
   BOOST_CHECK(default_expression_generator(basic_sort("D")) ==
-              boost::next(constructors.begin(), 0)->constructor_function(basic_sort("D"))(default_expression_generator(basic_sort("E"))));
+              constructors[0].constructor_function(basic_sort("D"))(default_expression_generator(basic_sort("E"))));
 
   // Check whether the representative of the set of reals is the empty set of reals.
   BOOST_CHECK(default_expression_generator(container_sort(set_container(),data::sort_real::real_())) ==
-              data::sort_set::empty(data::sort_real::real_()));
+              data::sort_set::constructor(data::sort_real::real_(),
+                                          data::sort_set::false_function(data::sort_real::real_()),
+                                          data::sort_fset::empty(data::sort_real::real_())));
 
 
 }
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
   test_representative_generator();
-  core::garbage_collect();
 
   return 0;
 }
