@@ -15,11 +15,9 @@
 #include <iostream>
 #include <sstream>
 #include <boost/test/included/unit_test_framework.hpp>
-#include "mcrl2/atermpp/aterm_init.h"
-#include "mcrl2/core/detail/pp_deprecated.h"
 #include "mcrl2/data/parse.h"
 #include "mcrl2/data/typecheck.h"
-#include "mcrl2/data/unknown_sort.h"
+#include "mcrl2/data/untyped_sort.h"
 #include "mcrl2/data/data_specification.h"
 #include "mcrl2/data/print.h"
 #include "mcrl2/utilities/test_utilities.h"
@@ -46,7 +44,7 @@ data::sort_expression parse_sort_expression(const std::string& de_in)
   try {
     result = data::parse_sort_expression_new(de_in);
 #ifdef MCRL2_ENABLE_TYPECHECK_PP_TESTS
-    std::string de_out = core::pp_deprecated(result);
+    std::string de_out = data::pp(result);
     if (de_in != de_out)
     {
       std::clog << "aterm : " << result << std::endl;
@@ -70,7 +68,7 @@ data::data_expression parse_data_expression(const std::string& de_in)
   try {
     result = data::parse_data_expression_new(de_in);
 #ifdef MCRL2_ENABLE_TYPECHECK_PP_TESTS
-    std::string de_out = core::pp_deprecated(result);
+    std::string de_out = data::pp(result);
     if (de_in != de_out)
     {
       std::clog << "aterm : " << result << std::endl;
@@ -94,7 +92,7 @@ data::data_specification parse_data_specification(const std::string& de_in, bool
   try {
     result = data::parse_data_specification_new(de_in);
 #ifdef MCRL2_ENABLE_TYPECHECK_PP_TESTS
-    std::string de_out = core::pp_deprecated(result);
+    std::string de_out = data::pp(result);
     if (de_in != de_out)
     {
       std::clog << "aterm : " << data::detail::data_specification_to_aterm_data_spec(result) << std::endl;
@@ -120,14 +118,16 @@ void test_data_expression(const std::string& de_in,
                           const std::string& expected_sort = "",
                           bool test_type_checker = true)
 {
+  data::data_expression x(parse_data_expression(de_in));
   std::clog << std::endl
             << "==========================================" << std::endl
             << "Testing type checking of data expression: " << std::endl
-            << "  de_in: " << de_in << std::endl
+            << "  de_in:  " << de_in << std::endl
+            << "  de_out: " << pp(x) << std::endl
             << "  expect success: " << (expect_success?("yes"):("no")) << std::endl
-            << "  expected type: " << expected_sort << std::endl;
+            << "  expected type: " << expected_sort << std::endl
+            << "  detected type: " << pp(x.sort()) << " (before typecheckeing) " << std::endl;
 
-  data::data_expression x(parse_data_expression(de_in));
 
   if (test_type_checker)
   {
@@ -136,20 +136,17 @@ void test_data_expression(const std::string& de_in,
       BOOST_CHECK_NO_THROW(data::type_check(x, begin, end));
       BOOST_CHECK_NE(x, data::data_expression());
 
-      atermpp::aterm de_aterm = x;
-      // If exception was thrown, x is data_expression()
-      if (x != data::data_expression())
+      std::string de_out = data::pp(x);
+      //std::clog << "The following data expressions should be the same:" << std::endl << "  " << de_in  << std::endl << "  " << de_out << std::endl;
+#ifdef MCRL2_ENABLE_TYPECHECK_PP_TESTS
+      BOOST_CHECK_EQUAL(de_in, de_out);
+#endif
+      // TODO: this check should be uncommented
+      //BOOST_CHECK(!search_sort_expression(x.sort(), data::untyped_sort()));
+      if (expected_sort != "")
       {
-        std::string de_out = core::pp_deprecated((ATerm) de_aterm);
-        //std::clog << "The following data expressions should be the same:" << std::endl << "  " << de_in  << std::endl << "  " << de_out << std::endl;
-        BOOST_CHECK_EQUAL(de_in, de_out);
-        // TODO: this check should be uncommented
-        //BOOST_CHECK(!search_sort_expression(x.sort(), data::unknown_sort()));
-        if (expected_sort != "")
-        {
-          BOOST_CHECK_EQUAL(x.sort(), parse_sort_expression(expected_sort));
-          std::clog << "    expression x: " << x << std::endl;
-        }
+        BOOST_CHECK_EQUAL(x.sort(), parse_sort_expression(expected_sort));
+        std::clog << "    expression x in internal format: " << x << std::endl;
       }
       else
       {
@@ -324,32 +321,32 @@ BOOST_AUTO_TEST_CASE(test_list_is_list_nat)
 
 BOOST_AUTO_TEST_CASE(test_emptyset)
 {
-  test_data_expression("{}", false);
+  test_data_expression("{}", true);
 }
 
 BOOST_AUTO_TEST_CASE(test_emptyset_complement)
 {
-  test_data_expression("!{}", false);
+  test_data_expression("!{}", true);
 }
 
 BOOST_AUTO_TEST_CASE(test_emptyset_complement_subset)
 {
-  test_data_expression("!{} <= {}", false);
+  test_data_expression("!{} <= {}", true);
 }
 
 BOOST_AUTO_TEST_CASE(test_emptyset_complement_subset_reverse)
 {
-  test_data_expression("{} <= !{}", false);
+  test_data_expression("{} <= !{}", true);
 }
 
 BOOST_AUTO_TEST_CASE(test_set_true_false)
 {
-  test_data_expression("{true, false}", true, "Set(Bool)");
+  test_data_expression("{true, false}", true, "FSet(Bool)");
 }
 
 BOOST_AUTO_TEST_CASE(test_set_numbers)
 {
-  test_data_expression("{1, 2, -7}", true, "Set(Int)");
+  test_data_expression("{1, 2, -7}", true, "FSet(Int)");
 }
 
 BOOST_AUTO_TEST_CASE(test_set_comprehension)
@@ -357,14 +354,23 @@ BOOST_AUTO_TEST_CASE(test_set_comprehension)
   test_data_expression("{ x: Nat | x mod 2 == 0 }", true, "Set(Nat)");
 }
 
+BOOST_AUTO_TEST_CASE(test_emptybag)
+{
+  test_data_expression("{:}", true);
+}
+
+BOOST_AUTO_TEST_CASE(test_emptybag_complement)
+{
+  test_data_expression("!{:}", false);
+}
 BOOST_AUTO_TEST_CASE(test_bag_true_false)
 {
-  test_data_expression("{true: 1, false: 2}", true, "Bag(Bool)");
+  test_data_expression("{true: 1, false: 2}", true, "FBag(Bool)");
 }
 
 BOOST_AUTO_TEST_CASE(test_bag_numbers)
 {
-  test_data_expression("{1: 1, 2: 2, -8: 8}", true, "Bag(Int)");
+  test_data_expression("{1: 1, 2: 2, -8: 8}", true, "FBag(Int)");
 }
 
 BOOST_AUTO_TEST_CASE(test_bag_comprehension)
@@ -659,16 +665,17 @@ BOOST_AUTO_TEST_CASE(test_sort_as_variable)
   );
 }
 
-BOOST_AUTO_TEST_CASE(test_predefined_aliases)
+/* BOOST_AUTO_TEST_CASE(test_predefined_aliases)   // This test case leads to a parse error, not a typecheck error.
 {
   test_data_specification(
     "sort Nat = Int;\n",
     false, // parse error
     false  // so do not test type checker
   );
-}
+} */
 
-BOOST_AUTO_TEST_CASE(test_conflicting_aliases)
+/* BOOST_AUTO_TEST_CASE(test_conflicting_aliases) // This test case leads to a parse error, due to the use of Nat. 
+                                               // This is not a typecheck error. Therefore this case is outcommented.
 {
   test_data_specification(
     "sort S = Nat;\n"
@@ -676,9 +683,9 @@ BOOST_AUTO_TEST_CASE(test_conflicting_aliases)
     "     T = Int;\n",
     false
   );
-}
+} */
 
-BOOST_AUTO_TEST_CASE(test_conflicting_aliases_predefined_left)
+/* BOOST_AUTO_TEST_CASE(test_conflicting_aliases_predefined_left)  // This test case leads to a parse error, due to the use of Nat.
 {
   test_data_specification(
     "sort Nat = S;\n"
@@ -687,7 +694,7 @@ BOOST_AUTO_TEST_CASE(test_conflicting_aliases_predefined_left)
     false, // parse error
     false  // so do not test type checker
   );
-}
+} */
 
 BOOST_AUTO_TEST_CASE(test_cyclic_aliases)
 {
@@ -874,9 +881,8 @@ void test_data_expression_in_specification_context(const std::string& de_in,
     if (expect_success)
     {
       data::type_check(de, begin, end, ds);
-      atermpp::aterm de_aterm = de;
 
-      std::string de_out = core::pp_deprecated((ATerm) de_aterm);
+      std::string de_out = data::pp(de);
 
       BOOST_CHECK_EQUAL(de_in, de_out);
       if (expected_sort != "")
@@ -1449,7 +1455,7 @@ BOOST_AUTO_TEST_CASE(test_ambiguous_function_application_recursive3)
     "     f: Pos -> Nat;\n"
     "     f,g: Int -> Int;\n",
     v.begin(), v.end(),
-    true,
+    false,
     "Bool"
   );
 }
@@ -1573,10 +1579,38 @@ BOOST_AUTO_TEST_CASE(test_avoidance_of_possible_types)
   );
 }
 
+/* The next example checks whether Int2Pos is properly typed. */
+BOOST_AUTO_TEST_CASE(test_proper_use_of_int2pos)
+{
+  data::variable_vector v;
+  test_data_expression_in_specification_context(
+    "f(Int2Pos(-1))",
+    "map f:Pos->Bool;\n",
+    v.begin(), v.end(),
+    true,
+    "Bool"
+  );
+}
+
+/* This example checks whether explicit transformations among 
+ * numbers are properly typable.*/
+BOOST_AUTO_TEST_CASE(test_proper_use_of_int2pos1)
+{
+  data::variable_vector v;
+  test_data_expression_in_specification_context(
+    "fpos(Nat2Pos(0)) && fpos(Int2Pos(-1)) && fpos(Real2Pos(1 / 2)) && "
+    "fnat(Int2Nat(-1)) && fnat(Real2Nat(1 / 2)) && "
+    "fint(Real2Int(1 / 2))",
+    "map fpos:Pos->Bool;\n"
+    "    fnat:Nat->Bool;\n"
+    "    fint:Int->Bool;\n",
+    v.begin(), v.end(),
+    true,
+    "Bool"
+  );
+}
 
 boost::unit_test::test_suite* init_unit_test_suite(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
   return 0;
 }
