@@ -17,10 +17,10 @@
 #include <map>
 #include <set>
 #include <vector>
-#include "mcrl2/atermpp/convert.h"
 #include "mcrl2/data/data_expression.h"
 #include "mcrl2/data/variable.h"
 #include "mcrl2/data/rewriter.h"
+#include "mcrl2/data/substitutions/mutable_map_substitution.h"
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/lps/remove.h"
 #include "mcrl2/lps/rewrite.h"
@@ -51,7 +51,7 @@ struct default_global_variable_solver
     data::data_expression r = R(g, sigma);
     if (is_variable(r))
     {
-      data::variable v = r;
+      const data::variable& v = atermpp::down_cast<data::variable>(r);
       if (std::find(V.begin(), V.end(), v) != V.end())
       {
         result[v] = e;
@@ -104,9 +104,9 @@ class constelm_algorithm: public lps::detail::lps_algorithm
       {
         mCRL2log(log::debug) << msg
                         << data::pp(d_j) << "\n"
-                        << "      value before: " << data::pp(Rd_j) << "\n"
-                        << "      value after:  " << data::pp(Rg_ij) << "\n"
-                        << "      replacements: " << data::print_substitution(sigma) << std::endl;
+                        << "      value before: " << Rd_j << "\n"
+                        << "      value after:  " << Rg_ij << "\n"
+                        << "      replacements: " << sigma << std::endl;
       }
     }
 
@@ -120,10 +120,10 @@ class constelm_algorithm: public lps::detail::lps_algorithm
       if (mCRL2logEnabled(log::debug))
       {
         mCRL2log(log::debug) << msg
-                        << data::pp(cond)
-                        << data::print_substitution(sigma)
+                        << cond
+                        << sigma
                         << " -> "
-                        << data::pp(c_i) << std::endl;
+                        << c_i << std::endl;
       }
     }
 
@@ -138,16 +138,15 @@ class constelm_algorithm: public lps::detail::lps_algorithm
     {}
 
     /// \brief Runs the constelm algorithm
-    /// \param p A linear process
-    /// \param e An initial value for the linear process p
-    /// \param R A data rewriter
     /// \param instantiate_global_variables If true, the algorithm is allowed to instantiate free variables
     /// as a side effect
+    /// \param ignore_conditions If true, the algorithm is allowed to ignore the conditions in the LPS.
     void run(bool instantiate_global_variables = false, bool ignore_conditions = false)
     {
       m_instantiate_global_variables = instantiate_global_variables;
       m_ignore_conditions = ignore_conditions;
-      data::data_expression_vector e = atermpp::convert<data::data_expression_vector>(m_spec.initial_process().state(m_spec.process().process_parameters()));
+      const data::data_expression_list &vl=m_spec.initial_process().state(m_spec.process().process_parameters());
+      data::data_expression_vector e(vl.begin(),vl.end());
 
       // essential: rewrite the initial state vector e to normal form. Essential
       // because this value is used in W below, and assigned to the right hand side of a substitution, which
@@ -155,7 +154,8 @@ class constelm_algorithm: public lps::detail::lps_algorithm
       lps::rewrite(e, R);
 
       linear_process& p = m_spec.process();
-      data::variable_list V = atermpp::convert<data::variable_list>(m_spec.global_variables());
+      const std::set<data::variable>&global_vars=m_spec.global_variables();
+      data::variable_list V(global_vars.begin(),global_vars.end());
       const data::variable_list& d = p.process_parameters();
 
       // initialize m_index_of
@@ -208,7 +208,7 @@ class constelm_algorithm: public lps::detail::lps_algorithm
                 {
                   for (data::mutable_map_substitution<>::const_iterator w = W.begin(); w != W.end(); ++w)
                   {
-                    sigma[w->first] = w->second; 
+                    sigma[w->first] = w->second;
                     undo[d_j].insert(w->first);
                   }
                 }
