@@ -13,6 +13,7 @@
 #define MCRL2_LPS_DETAIL_INSTANTIATE_GLOBAL_VARIABLES_H
 
 #include "mcrl2/data/representative_generator.h"
+#include "mcrl2/data/substitutions/mutable_map_substitution.h"
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/lps/replace.h"
 #include "mcrl2/lps/remove.h"
@@ -29,16 +30,17 @@ namespace detail
 inline
 void instantiate_global_variables(specification& spec)
 {
+  mCRL2log(log::verbose) << "Replacing free variables with dummy values." << std::endl;
   data::mutable_map_substitution<> sigma;
   data::representative_generator default_expression_generator(spec.data());
   std::set<data::variable> to_be_removed;
-  const atermpp::set<data::variable>& v = spec.global_variables();
-  for (atermpp::set<data::variable>::const_iterator i = v.begin(); i != v.end(); ++i)
+  const std::set<data::variable>& v = spec.global_variables();
+  for (std::set<data::variable>::const_iterator i = v.begin(); i != v.end(); ++i)
   {
     data::data_expression d = default_expression_generator(i->sort());
-    if (d == data::data_expression())
+    if (!d.defined())
     {
-      throw mcrl2::runtime_error("Error in lps::instantiate_global_variables: could not instantiate " + data::pp(*i));
+      throw mcrl2::runtime_error("Error in lps::instantiate_global_variables: could not instantiate " + data::pp(*i) + ". ");
     }
     sigma[*i] = d;
     to_be_removed.insert(*i);
