@@ -123,14 +123,14 @@ struct lps_well_typed_checker
   }
 
   /// \brief Traverses an action label
-  bool is_well_typed(const action_label&) const
+  bool is_well_typed(const process::action_label&) const
   {
     return true;
   }
 
   /// \brief Traverses an action
   /// \param a An action
-  bool is_well_typed(const action&) const
+  bool is_well_typed(const process::action&) const
   {
     return true;
   }
@@ -292,11 +292,11 @@ struct lps_well_typed_checker
   bool is_well_typed(const specification& spec) const
   {
     std::set<data::sort_expression> declared_sorts = data::detail::make_set(spec.data().sorts());
-    std::set<action_label> declared_labels = data::detail::make_set(spec.action_labels());
+    std::set<process::action_label> declared_labels = data::detail::make_set(spec.action_labels());
     const action_summand_vector& action_summands = spec.process().action_summands();
 
     // check 1)
-    for (action_summand_vector::const_iterator i = action_summands.begin(); i != action_summands.end(); ++i)
+    for (auto i = action_summands.begin(); i != action_summands.end(); ++i)
     {
       if (!(data::detail::check_variable_sorts(i->summation_variables(), declared_sorts)))
       {
@@ -348,28 +348,19 @@ struct lps_well_typed_checker
       return false;
     }
 
-    std::set<data::variable> declared_free_variables  = spec.global_variables();
-    std::set<data::variable> occ1 = lps::find_free_variables(spec.process());
-    std::set<data::variable> occ2 = lps::find_free_variables(spec.initial_process());
-    std::set<data::variable> occurring_free_variables = occ1;
-    occurring_free_variables.insert(occ2.begin(), occ2.end());
-    if (!(std::includes(declared_free_variables.begin(),
-                        declared_free_variables.end(),
-                        occurring_free_variables.begin(),
-                        occurring_free_variables.end()
-                       )
-         ))
+    std::set<data::variable> free_variables = lps::find_free_variables(spec);
+    if (!free_variables.empty())
     {
       mCRL2log(log::error) << "is_well_typed(specification) failed: some of the free variables were not declared\n";
-      mCRL2log(log::error) << "declared global variables: " << data::pp(declared_free_variables) << std::endl;
-      mCRL2log(log::error) << "occurring free variables: " << data::pp(occurring_free_variables) << std::endl;
+      mCRL2log(log::error) << "declared global variables: " << data::pp(spec.global_variables()) << std::endl;
+      mCRL2log(log::error) << "occurring free variables: " << data::pp(free_variables) << std::endl;
       return false;
     }
 
     // check 3)
-    if (!data::detail::unique_names(atermpp::convert<data::variable_list>(spec.global_variables())))
+    if (!data::detail::unique_names(spec.global_variables()))
     {
-      mCRL2log(log::error) << "is_well_typed(specification) failed: global variables " << data::pp(atermpp::convert<data::variable_list>(spec.global_variables())) << " don't have unique names." << std::endl;
+      mCRL2log(log::error) << "is_well_typed(specification) failed: global variables " << data::pp(spec.global_variables()) << " don't have unique names." << std::endl;
       return false;
     }
 
@@ -383,8 +374,6 @@ struct lps_well_typed_checker
   }
 };
 
-} // namespace detail
-
 /// \brief Checks well typedness of an LPS object.
 template <typename Object>
 bool is_well_typed(const Object& o)
@@ -393,6 +382,8 @@ bool is_well_typed(const Object& o)
   return checker(o);
 }
 
+} // namespace detail
+
 } // namespace lps
 
 } // namespace mcrl2
