@@ -19,7 +19,8 @@
 #include "mcrl2/atermpp/aterm.h"
 #include "mcrl2/atermpp/aterm_list.h"
 #include "mcrl2/data/data_specification.h"
-#include "mcrl2/lps/summand.h"
+#include "mcrl2/lps/action_summand.h"
+#include "mcrl2/lps/deadlock_summand.h"
 #include "mcrl2/lps/process_initializer.h"
 
 namespace mcrl2
@@ -29,6 +30,7 @@ namespace lps
 {
 
 class linear_process; // prototype declaration
+bool is_well_typed(const linear_process& proc);
 
 ///////////////////////////////////////////////////////////////////////////////
 // linear_process
@@ -47,49 +49,10 @@ class linear_process
     /// \brief The action summands of the process
     action_summand_vector m_action_summands;
 
-    /// \brief Set the summands of the linear process
-    /// \deprecated
-    void set_summands(const deprecated::summand_list& summands)
-    {
-      m_deadlock_summands.clear();
-      m_action_summands  .clear();
-      for (deprecated::summand_list::iterator j = summands.begin(); j != summands.end(); ++j)
-      {
-        if (j->is_delta())
-        {
-          m_deadlock_summands.push_back(deadlock_summand(j->summation_variables(), j->condition(), j->deadlock()));
-        }
-        else
-        {
-          m_action_summands.push_back(action_summand(j->summation_variables(), j->condition(), j->multi_action(), j->assignments()));
-        }
-      }
-    }
-
-    /// \brief Returns the sequence of LPS summands.
-    /// \return The sequence of LPS summands.
-    /// \deprecated
-    deprecated::summand_list summands() const
-    {
-      deprecated::summand_list result;
-      for (deadlock_summand_vector::const_reverse_iterator i = m_deadlock_summands.rbegin(); i != m_deadlock_summands.rend(); ++i)
-      {
-        deprecated::summand s = atermpp::aterm_appl(deadlock_summand_to_aterm(*i));
-        result = atermpp::push_front(result, s);
-      }
-      for (action_summand_vector::const_reverse_iterator i = m_action_summands.rbegin(); i != m_action_summands.rend(); ++i)
-      {
-        deprecated::summand s = atermpp::aterm_appl(action_summand_to_aterm(*i));
-        result = atermpp::push_front(result, s);
-      }
-      return result;
-    }
-
   public:
     /// \brief Constructor.
     linear_process()
     {
-      m_process_parameters.protect();
     }
 
     /// \brief Copy constructor.
@@ -98,7 +61,6 @@ class linear_process
       m_deadlock_summands(other.m_deadlock_summands),
       m_action_summands(other.m_action_summands)
     {
-      m_process_parameters.protect();
     }
 
     /// \brief Constructor.
@@ -111,26 +73,41 @@ class linear_process
       m_deadlock_summands(deadlock_summands),
       m_action_summands(action_summands)
     {
-      m_process_parameters.protect();
     }
 
     /// \brief Constructor.
     /// \param lps A term
-    linear_process(atermpp::aterm_appl lps)
+    linear_process(const atermpp::aterm_appl& lps)
     {
       assert(core::detail::check_term_LinearProcess(lps));
-
-      // unpack LPS(.,.,.) term
       atermpp::aterm_appl::iterator i = lps.begin();
-      m_process_parameters = *i++;
-      m_process_parameters.protect();
-      set_summands(*i);
+      m_process_parameters = data::variable_list(*i++);
+      atermpp::aterm_list summands = atermpp::down_cast<atermpp::aterm_list>(*i);
+      for (atermpp::aterm_list::iterator j = summands.begin(); j != summands.end(); ++j)
+      {
+        assert(core::detail::check_rule_LinearProcessSummand(*j));
+        atermpp::aterm_appl t = atermpp::down_cast<atermpp::aterm_appl>(*j);
+
+        data::variable_list summation_variables(atermpp::down_cast<atermpp::aterm_list>(t[0]));
+        data::data_expression condition         = data::data_expression(t[1]);
+        data::data_expression time              = data::data_expression(t[3]);
+        data::assignment_list assignments(atermpp::down_cast<atermpp::aterm_list>(t[4]));
+        if (atermpp::down_cast<atermpp::aterm_appl>(t[2]).function() == core::detail::function_symbols::Delta)
+        {
+          m_deadlock_summands.push_back(deadlock_summand(summation_variables, condition, deadlock(time)));
+        }
+        else
+        {
+          assert(lps::is_multi_action(atermpp::down_cast<const atermpp::aterm_appl>(t[2])));
+          process::action_list actions(atermpp::down_cast<atermpp::aterm_list>(atermpp::down_cast<atermpp::aterm_appl>(t[2])[0]));
+          m_action_summands.push_back(action_summand(summation_variables, condition, multi_action(actions, time), assignments));
+        }
+      }
     }
 
     /// \brief Destructor
     ~linear_process()
     {
-      m_process_parameters.unprotect();
     }
 
     /// \brief Returns the number of LPS summands.
@@ -204,64 +181,54 @@ class linear_process
     }
 };
 
-/// \brief Conversion to ATermAppl.
-/// \return The action summand converted to ATerm format.
+/// \brief Conversion to aterm_appl.
+/// \return The action summand converted to aterm format.
 inline
 atermpp::aterm_appl linear_process_to_aterm(const linear_process& p)
 {
-  return core::detail::gsMakeLinearProcess(
-           p.process_parameters(),
-           p.summands()
-         );
-}
-
-namespace deprecated
-{
-/// \brief Set the summands of the linear process
-/// \deprecated
-inline
-void set_linear_process_summands(linear_process& p, const summand_list& summands)
-{
-  p.deadlock_summands().clear();
-  p.action_summands().clear();
-  for (summand_list::iterator j = summands.begin(); j != summands.end(); ++j)
-  {
-    if (j->is_delta())
-    {
-      p.deadlock_summands().push_back(deadlock_summand(j->summation_variables(), j->condition(), j->deadlock()));
-    }
-    else
-    {
-      p.action_summands().push_back(action_summand(j->summation_variables(), j->condition(), j->multi_action(), j->assignments()));
-    }
-  }
-}
-
-/// \brief Returns the sequence of LPS summands.
-/// \return The sequence of LPS summands.
-/// \deprecated
-inline
-summand_list linear_process_summands(const linear_process& p)
-{
-  summand_list result;
+  atermpp::term_list<atermpp::aterm_appl> summands;
   for (deadlock_summand_vector::const_reverse_iterator i = p.deadlock_summands().rbegin(); i != p.deadlock_summands().rend(); ++i)
   {
-    summand s = atermpp::aterm_appl(deadlock_summand_to_aterm(*i));
-    result = atermpp::push_front(result, s);
+    atermpp::aterm_appl s = deadlock_summand_to_aterm(*i);
+    summands.push_front(s);
   }
   for (action_summand_vector::const_reverse_iterator i = p.action_summands().rbegin(); i != p.action_summands().rend(); ++i)
   {
-    summand s = atermpp::aterm_appl(action_summand_to_aterm(*i));
-    result = atermpp::push_front(result, s);
+    atermpp::aterm_appl s = action_summand_to_aterm(*i);
+    summands.push_front(s);
   }
-  return result;
+
+  return atermpp::aterm_appl(core::detail::function_symbol_LinearProcess(),
+           p.process_parameters(),
+           summands
+         );
 }
 
-} // namespace deprecated
+//--- start generated class linear_process ---//
+// prototype declaration
+std::string pp(const linear_process& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const linear_process& x)
+{
+  return out << lps::pp(x);
+}
+//--- end generated class linear_process ---//
+
+/// \brief Test for a linear_process expression
+/// \param x A term
+/// \return True if \a x is a linear process expression
+inline
+bool is_linear_process(const atermpp::aterm_appl& x)
+{
+  return x.function() == core::detail::function_symbols::LinearProcess;
+}
 
 // template function overloads
-std::string pp(const linear_process& x);
-std::set<data::variable> find_variables(const lps::linear_process& x);
+std::set<data::variable> find_all_variables(const lps::linear_process& x);
 std::set<data::variable> find_free_variables(const lps::linear_process& x);
 
 } // namespace lps
