@@ -20,20 +20,22 @@
 #include <string>
 #include <vector>
 #include <boost/iterator/iterator_facade.hpp>
+#include "mcrl2/atermpp/detail/utility.h"
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/atermpp/indexed_set.h"
-#include "mcrl2/core/detail/struct_core.h"
+#include "mcrl2/core/detail/function_symbols.h"
 #include "mcrl2/core/detail/print_utility.h"
-#include "mcrl2/data/classic_enumerator.h"
+#include "mcrl2/data/enumerator.h"
 #include "mcrl2/data/find.h"
 #include "mcrl2/data/parse.h"
 #include "mcrl2/data/print.h"
 #include "mcrl2/data/rewrite_strategy.h"
 #include "mcrl2/data/selection.h"
+#include "mcrl2/data/substitutions/mutable_indexed_substitution.h"
+#include "mcrl2/data/detail/io.h"
 #include "mcrl2/lps/find.h"
-#include "mcrl2/lps/parse.h"
+#include "mcrl2/lps/io.h"
 #include "mcrl2/lps/next_state_generator.h"
-#include "mcrl2/utilities/detail/memory_utility.h"
 
 // For backwards compatibility
 //using namespace mcrl2::log;
@@ -52,13 +54,21 @@ std::vector<std::string> generate_values(const data::data_specification& dataspe
   std::size_t max_internal_variables = 10000;
 
   data::rewriter rewr(dataspec);
-  data::classic_enumerator<data::rewriter> enumerator(dataspec, rewr);
+
+  typedef data::enumerator_list_element_with_substitution<> enumerator_element;
+  typedef data::enumerator_algorithm_with_iterator<> enumerator_type;
+
+  enumerator_type enumerator(rewr, dataspec, rewr, max_internal_variables);
   data::variable x("x", s);
   data::variable_vector v;
   v.push_back(x);
-  for (data::classic_enumerator<data::rewriter>::iterator i = enumerator.begin(v, data::sort_bool::true_(), max_internal_variables); i != enumerator.end() ; ++i)
+  data::mutable_indexed_substitution<> sigma;
+  data::variable_list vl(v.begin(),v.end());
+  std::deque<enumerator_element> enumerator_deque(1, enumerator_element(vl, data::sort_bool::true_()));
+  for (auto i = enumerator.begin(sigma, enumerator_deque); i != enumerator.end() ; ++i)
   {
-    result.push_back((*i)(x).to_string());
+    i->add_assignments(vl, sigma, rewr);
+    result.push_back(to_string(sigma(vl.front())));
     if (result.size() >= max_size)
     {
       break;
@@ -72,7 +82,7 @@ std::vector<std::string> generate_values(const data::data_specification& dataspe
 class pins_data_type
 {
   protected:
-    atermpp::indexed_set m_indexed_set;
+    atermpp::indexed_set<atermpp::aterm> m_indexed_set;
     lps::next_state_generator& m_generator;
     bool m_is_bounded;
 
@@ -195,13 +205,13 @@ class pins_data_type
     }
 
     /// \brief Returns the indexed_set holding the values
-    atermpp::indexed_set& indexed_set()
+    atermpp::indexed_set<atermpp::aterm>& indexed_set()
     {
       return m_indexed_set;
     }
 
     /// \brief Returns the indexed_set holding the values
-    const atermpp::indexed_set& indexed_set() const
+    const atermpp::indexed_set<atermpp::aterm>& indexed_set() const
     {
       return m_indexed_set;
     }
@@ -216,12 +226,12 @@ class state_data_type: public pins_data_type
 
     std::size_t expression2index(const data::data_expression& x)
     {
-      return m_indexed_set[m_generator.get_internal_state_argument(x)];
+      return m_indexed_set[x];
     }
 
     data::data_expression index2expression(std::size_t i) const
     {
-      return m_generator.get_state_argument(m_indexed_set.get(i));
+      return static_cast<data::data_expression>(m_indexed_set.get(i));
     }
 
   public:
@@ -232,14 +242,18 @@ class state_data_type: public pins_data_type
       m_name = data::pp(m_sort);
     }
 
+    // prints the expression as an ATerm string
     std::string serialize(int i) const
     {
-      return index2expression(i).to_string();
+      data::data_expression e = index2expression(i);
+      atermpp::aterm t = data::detail::remove_index(static_cast<atermpp::aterm>(e));
+      return to_string(t);
     }
 
     std::size_t deserialize(const std::string& s)
     {
-      return expression2index(data::data_expression(atermpp::read_from_string(s)));
+      atermpp::aterm t = data::detail::add_index(atermpp::read_term_from_string(s));
+      return expression2index(atermpp::down_cast<data::data_expression>(t));
     }
 
     std::string print(int i) const
@@ -278,12 +292,12 @@ class action_label_data_type: public pins_data_type
 
     std::string serialize(int i) const
     {
-      return m_indexed_set.get(i).to_string();
+      return to_string(m_indexed_set.get(i));
     }
 
     std::size_t deserialize(const std::string& s)
     {
-      return m_indexed_set[atermpp::read_from_string(s)];
+      return m_indexed_set[atermpp::read_term_from_string(s)];
     }
 
     std::string print(int i) const
@@ -321,7 +335,6 @@ class pins
     std::vector<std::vector<size_t> > m_write_group;
     lps::specification m_specification;
     lps::next_state_generator m_generator;
-    lps::next_state_generator::substitution_t m_substitution;
     std::vector<std::string> m_process_parameter_names;
 
     // The type mappings
@@ -423,8 +436,8 @@ class pins
         {
           if(j->lhs() != j->rhs())
           {
-            data::find_variables(j->lhs(), std::inserter(used_write_variables, used_write_variables.end()));
-            data::find_variables(j->rhs(), std::inserter(used_read_variables, used_read_variables.end()));
+            data::find_all_variables(j->lhs(), std::inserter(used_write_variables, used_write_variables.end()));
+            data::find_all_variables(j->rhs(), std::inserter(used_read_variables, used_read_variables.end()));
           }
         }
 
@@ -445,7 +458,7 @@ class pins
                               std::inserter(used_write_parameters,
                                             used_write_parameters.begin()));
 
-        std::vector<data::variable> parameters_list = atermpp::convert<std::vector<data::variable> >(proc.process_parameters());
+        std::vector<data::variable> parameters_list(proc.process_parameters().begin(),proc.process_parameters().end());
 
         for (std::vector<data::variable>::const_iterator j = parameters_list.begin(); j != parameters_list.end(); ++j)
         {
@@ -453,7 +466,7 @@ class pins
           {
             m_read_group[i - summands.begin()].push_back(j - parameters_list.begin());
           }
-          if (!used_read_parameters.empty() && used_write_parameters.find(*j) != used_write_parameters.end())
+          if (!used_write_parameters.empty() && used_write_parameters.find(*j) != used_write_parameters.end())
           {
             m_write_group[i - summands.begin()].push_back(j - parameters_list.begin());
           }
@@ -479,7 +492,7 @@ class pins
     static lps::specification load_specification(const std::string& filename)
     {
       lps::specification specification;
-      specification.load(filename);
+      load_lps(specification, filename);
       return specification;
     }
 
@@ -502,7 +515,7 @@ class pins
       // Each state parameter type gets it's own pins_data_type. State parameters of the same
       // type share the pins_data_type.
       std::map<data::sort_expression, pins_data_type*> existing_type_maps;
-      std::vector<data::variable> parameters = atermpp::convert<std::vector<data::variable> >(process().process_parameters());
+      std::vector<data::variable> parameters(process().process_parameters().begin(),process().process_parameters().end());
       for (std::size_t i = 0; i < params.size(); i++)
       {
         data::sort_expression s = parameters[i].sort();
@@ -525,7 +538,7 @@ class pins
 
       for (std::size_t i = 0; i < m_data_types.size(); i++)
       {
-      	std::vector<pins_data_type*>::const_iterator j = std::find(m_unique_data_types.begin(), m_unique_data_types.end(), m_data_types[i]);
+        std::vector<pins_data_type*>::const_iterator j = std::find(m_unique_data_types.begin(), m_unique_data_types.end(), m_data_types[i]);
         assert(j != m_unique_data_types.end());
         m_unique_data_type_index.push_back(j - m_unique_data_types.begin());
       }
@@ -614,10 +627,10 @@ class pins
     /// \brief Assigns the initial state to s.
     void get_initial_state(ltsmin_state_type& s)
     {
-      lps::next_state_generator::internal_state_t initial_state = m_generator.internal_initial_state();
+      state initial_state = m_generator.initial_state();
       for (size_t i = 0; i < m_state_length; i++)
       {
-        s[i] = state_type_map(i)[initial_state(i)];
+        s[i] = state_type_map(i)[initial_state[i]];
       }
     }
 
@@ -626,8 +639,8 @@ class pins
     std::set<std::string> summand_action_names(std::size_t i) const
     {
       std::set<std::string> result;
-      const action_list& l = process().action_summands()[i].multi_action().actions();
-      for (action_list::const_iterator i = l.begin(); i != l.end(); ++i)
+      auto const& l = process().action_summands()[i].multi_action().actions();
+      for (auto i = l.begin(); i != l.end(); ++i)
       {
         result.insert(std::string(i->label().name()));
       }
@@ -635,37 +648,37 @@ class pins
     }
 
     /// \brief Iterates over the 'next states' of state src, and invokes a callback function for each discovered state.
+    ///
+    /// StateFunction is a callback function that must provide the function operator() with the following interface:
+    /// <code>void operator()(ltsmin_state_type const& next_state, int* const& edge_labels, int group=-1);</code>
+    /// where
+    /// - \a next_state is the target state of the transition
+    /// - \a edge_labels is an array of edge labels
+    /// - \a group is the number of the summand from which the next state was generated, or -1 if it is unknown which summand
+    ///
     /// \param src An LTSMin state
     /// \param f A 'callback' function object
     /// \param dest A destination state, which is modified and passed to the callback.  Must provide space for at least process_parameter_count() items.
     /// \param labels An array of labels, which is modified and passed to the callback.  Must provide space for at least edge_label_count() items.
-    ///
-    /// StateFunction is a callback function that must provide the function operator() with the following interface:
-    ///
-    /// \code
-    /// void operator()(ltsmin_state_type const& next_state, int* const& edge_labels, int group=-1);
-    /// \endcode
-    /// where
-    /// - next_state is the target state of the transition
-    /// - edge_labels is an array of edge labels
-    /// - group is the number of the summand from which the next state was generated, or -1 if it is unknown which summand
     template <typename StateFunction>
     void next_state_all(ltsmin_state_type const& src, StateFunction& f, ltsmin_state_type const& dest, int* const& labels)
     {
       std::size_t nparams = process_parameter_count();
-      MCRL2_SYSTEM_SPECIFIC_ALLOCA(state_arguments, lps::next_state_generator::internal_state_argument_t, nparams);
+      data::data_expression_vector state_arguments(nparams);
       for (size_t i = 0; i < nparams; i++)
       {
-        state_arguments[i] = state_type_map(i).get(src[i]);
+        state_arguments[i] = static_cast<data::data_expression>(state_type_map(i).get(src[i]));
       }
-      lps::next_state_generator::internal_state_t source = m_generator.get_internal_state(state_arguments);
+      // data::data_expression_vector source = state_arguments;
+      state source(state_arguments.begin(),nparams);
 
-      for (next_state_generator::iterator i = m_generator.begin(source, &m_substitution); i; i++)
+      next_state_generator::enumerator_queue_t enumeration_queue;
+      for (next_state_generator::iterator i = m_generator.begin(source, &enumeration_queue); i; i++)
       {
-        lps::next_state_generator::internal_state_t destination = i->internal_state();
+        state destination = i->internal_state();
         for (size_t j = 0; j < nparams; j++)
         {
-          dest[j] = state_type_map(j)[destination(j)];
+          dest[j] = state_type_map(j)[destination[j]];
         }
         labels[0] = action_label_type_map()[detail::multi_action_to_aterm(i->action())];
         f(dest, labels);
@@ -675,37 +688,38 @@ class pins
     /// \brief Iterates over the 'next states' of a particular summand
     /// of state src that are generated by a group of summands, and
     /// invokes a callback function for each discovered state.
-    /// \param src An LTSMin state
-    /// \param group the number of the summand, which
-    /// \param dest A destination state, which is modified and passed to the callback.  Must provide space for at least process_parameter_count() items.
-    /// \param labels An array of labels, which is modified and passed to the callback.  Must provide space for at least edge_label_count() items.
     ///
     /// StateFunction is a callback function that must provide the function operator() with the following interface:
-    ///
-    /// \code
-    /// void operator()(ltsmin_state_type const& next_state, int* const& edge_labels, int group);
-    /// \endcode
+    /// <code>void operator()(ltsmin_state_type const& next_state, int* const& edge_labels, int group);</code>
     /// where
     /// - next_state is the target state of the transition
     /// - edge_labels is an array of edge labels
     /// - group is the number of the summand from which the next state was generated, or -1 if it is unknown which summand
+    ///
+    /// \param src An LTSMin state
+    /// \param group the number of the summand, which
+    /// \param dest A destination state, which is modified and passed to the callback.
+    ///        Must provide space for at least process_parameter_count() items.
+    /// \param labels An array of labels, which is modified and passed to the callback.
+    ///        Must provide space for at least edge_label_count() items.
     template <typename StateFunction>
     void next_state_long(ltsmin_state_type const& src, std::size_t group, StateFunction& f, ltsmin_state_type const& dest, int* const& labels)
     {
       std::size_t nparams = process_parameter_count();
-      MCRL2_SYSTEM_SPECIFIC_ALLOCA(state_arguments, lps::next_state_generator::internal_state_argument_t, nparams);
+      data::data_expression_vector state_arguments(nparams);
       for (size_t i = 0; i < nparams; i++)
       {
-        state_arguments[i] = state_type_map(i).get(src[i]);
+        state_arguments[i] = static_cast<data::data_expression>(state_type_map(i).get(src[i]));
       }
-      lps::next_state_generator::internal_state_t source = m_generator.get_internal_state(state_arguments);
+      state source(state_arguments.begin(),nparams);
 
-      for (next_state_generator::iterator i = m_generator.begin(source, &m_substitution, group); i; i++)
+      next_state_generator::enumerator_queue_t enumeration_queue;
+      for (next_state_generator::iterator i = m_generator.begin(source, group, &enumeration_queue); i; i++)
       {
-        lps::next_state_generator::internal_state_t destination = i->internal_state();
+        state destination = i->internal_state();
         for (size_t j = 0; j < nparams; j++)
         {
-          dest[j] = state_type_map(j)[destination(j)];
+          dest[j] = state_type_map(j)[destination[j]];
         }
         labels[0] = action_label_type_map()[detail::multi_action_to_aterm(i->action())];
         f(dest, labels);
@@ -762,7 +776,7 @@ class pins
             << " bounded = " << std::boolalpha << type.is_bounded() << "\n";
         if (type.is_bounded())
         {
-          out << " possible values: " << core::detail::print_set(type.generate_values(10), core::detail::default_printer()) << std::endl;
+          out << " possible values: " << core::detail::print_set(type.generate_values(10)) << std::endl;
         }
         out << std::endl;
       }
