@@ -15,12 +15,11 @@
 #include <iterator>
 #include <stdexcept>
 #include <sstream>
-#include "mcrl2/atermpp/aterm_traits.h"
 #include "mcrl2/atermpp/make_list.h"
-#include "mcrl2/core/detail/struct_core.h" // gsMakeNil
 #include "mcrl2/data/standard_utility.h"
+#include "mcrl2/data/undefined.h"
 #include "mcrl2/data/utility.h" // substitute
-#include "mcrl2/lps/action.h"
+#include "mcrl2/process/process_expression.h"
 #include "mcrl2/core/print.h"
 
 namespace mcrl2
@@ -29,81 +28,63 @@ namespace mcrl2
 namespace lps
 {
 
+// prototype declaration
+bool is_multi_action(const atermpp::aterm_appl& x);
+
 /// \brief Represents a multi action
-/// \detail Multi actions consist of a list of actions together with an optional time tag.
+/// \details Multi actions consist of a list of actions together with an optional time tag.
 class multi_action
 {
     friend class action_summand;
-    friend struct atermpp::aterm_traits<multi_action>;
 
   protected:
     /// \brief The actions of the summand
-    action_list m_actions;
+    process::action_list m_actions;
 
-    /// \brief The time of the summand. If <tt>m_time == data::data_expression()</tt>
+    /// \brief The time of the summand. If <tt>m_time == data::undefined_real()</tt>
     /// the multi action has no time.
     data::data_expression m_time;
 
-    /// \brief Protects the term from being freed during garbage collection.
-    void protect() const
-    {
-      m_time.protect();
-      m_actions.protect();
-    }
-
-    /// \brief Unprotect the term.
-    /// Releases protection of the term which has previously been protected through a
-    /// call to protect.
-    void unprotect() const
-    {
-      m_time.unprotect();
-      m_actions.unprotect();
-    }
-
-    /// \brief Mark the term for not being garbage collected.
-    void mark() const
-    {
-      m_time.mark();
-      m_actions.mark();
-    }
-
   public:
     /// \brief Constructor
-    multi_action(action_list actions = action_list(), data::data_expression time = atermpp::aterm_appl(core::detail::gsMakeNil()))
+    multi_action(const process::action_list& actions = process::action_list(), data::data_expression time = data::undefined_real())
       : m_actions(actions), m_time(time)
-    {}
-
-    /// \brief Constructor
-    multi_action(const atermpp::aterm_appl& t) : m_time(core::detail::gsMakeNil())
     {
-      assert(core::detail::gsIsAction(t) || core::detail::gsIsMultAct(t));
-      m_actions = (core::detail::gsIsAction(t)) ? atermpp::term_list< action >(atermpp::make_list(t)) : atermpp::term_list< action >(t(0));
+      assert(data::sort_real::is_real(m_time.sort()));
     }
 
     /// \brief Constructor
-    multi_action(const action& l)
-      : m_actions(atermpp::push_front(action_list(), l)),
-        m_time(core::detail::gsMakeNil())
+    explicit multi_action(const atermpp::aterm& t1)
+      : m_time(data::undefined_real())
+    {
+      const atermpp::aterm_appl& t = atermpp::down_cast<atermpp::aterm_appl>(t1);
+      assert(process::is_action(t) || lps::is_multi_action(t));
+      m_actions = process::is_action(t) ? process::action_list(t) : process::action_list(t[0]);
+    }
+
+    /// \brief Constructor
+    multi_action(const process::action& l)
+      : m_actions(atermpp::make_list<process::action>(l)),
+        m_time(data::undefined_real())
     {}
 
     /// \brief Returns true if time is available.
     /// \return True if time is available.
     bool has_time() const
     {
-      // TODO: remove the Nil
-      return m_time != core::detail::gsMakeNil();
+      return m_time != data::undefined_real();
     }
 
     /// \brief Returns the sequence of actions.
     /// \return The sequence of actions.
-    const action_list& actions() const
+    const process::action_list& actions() const
     {
       return m_actions;
     }
 
     /// \brief Returns the sequence of actions.
     /// \return The sequence of actions.
-    action_list& actions()
+    process::action_list& actions()
     {
       return m_actions;
     }
@@ -126,14 +107,14 @@ class multi_action
     /// \return The name of the first action.
     core::identifier_string name() const
     {
-      return front(m_actions).label().name();
+      return m_actions.front().label().name();
     }
 
     /// \brief Returns the arguments of the multi action.
     /// \return The arguments of the multi action.
     data::data_expression_list arguments() const
     {
-      return front(m_actions).arguments();
+      return m_actions.front().arguments();
     }
 
     /// \brief Joins the actions of both multi actions.
@@ -161,32 +142,65 @@ class multi_action
     {
       return m_actions < other.m_actions || (m_actions == other.m_actions && m_time < other.m_time);
     }
+
+    /// \brief Swaps the contents
+    void swap(multi_action& other)
+    {
+      using std::swap;
+      swap(m_actions, other.m_actions);
+      swap(m_time, other.m_time);
+    }
 };
 
+//--- start generated class multi_action ---//
+/// \brief list of multi_actions
+typedef atermpp::term_list<multi_action> multi_action_list;
+
+/// \brief vector of multi_actions
+typedef std::vector<multi_action>    multi_action_vector;
+
+// prototype declaration
+std::string pp(const multi_action& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const multi_action& x)
+{
+  return out << lps::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(multi_action& t1, multi_action& t2)
+{
+  t1.swap(t2);
+}
+//--- end generated class multi_action ---//
+
 /// \brief Returns true if the term t is a multi action
 inline
-bool is_multi_action(const atermpp::aterm_appl& t)
+bool is_multi_action(const atermpp::aterm_appl& x)
 {
-  return core::detail::gsIsMultAct(t);
+  return x.function() == core::detail::function_symbols::MultAct;
 }
 
 // template function overloads
-std::string pp(const multi_action& x);
 void normalize_sorts(multi_action& x, const data::data_specification& dataspec);
 void translate_user_notation(lps::multi_action& x);
-std::set<data::variable> find_variables(const lps::multi_action& x);
+std::set<data::variable> find_all_variables(const lps::multi_action& x);
 std::set<data::variable> find_free_variables(const lps::multi_action& x);
 
 /// \cond INTERNAL_DOCS
 namespace detail
 {
 
-/// \brief Conversion to ATermAppl.
-/// \return The multi action converted to ATerm format.
+/// \brief Conversion to aterm_appl.
+/// \return The multi action converted to aterm format.
 inline
 atermpp::aterm_appl multi_action_to_aterm(const multi_action& m)
 {
-  return core::detail::gsMakeMultAct(m.actions());
+  return atermpp::aterm_appl(core::detail::function_symbol_MultAct(), m.actions());
 }
 
 /// \brief Visits all permutations of the arrays, and calls f for each instance.
@@ -216,13 +230,13 @@ void forall_permutations(Iter first, Iter last, Function f)
 /// \param a A sequence of actions
 /// \param b A sequence of actions
 /// \return True if the actions in a and b have the same names, and the same sorts.
-inline bool equal_action_signatures(const std::vector<action>& a, const std::vector<action>& b)
+inline bool equal_action_signatures(const std::vector<process::action>& a, const std::vector<process::action>& b)
 {
   if (a.size() != b.size())
   {
     return false;
   }
-  std::vector<action>::const_iterator i, j;
+  std::vector<process::action>::const_iterator i, j;
   for (i = a.begin(), j = b.begin(); i != a.end(); ++i, ++j)
   {
     if (i->label() != j->label())
@@ -240,7 +254,7 @@ struct compare_action_labels
   /// \param a An action
   /// \param b An action
   /// \return The function result
-  bool operator()(const action& a, const action& b) const
+  bool operator()(const process::action& a, const process::action& b) const
   {
     return a.label() < b.label();
   }
@@ -253,7 +267,7 @@ struct compare_action_label_arguments
   /// \param a An action
   /// \param b An action
   /// \return The function result
-  bool operator()(const action& a, const action& b) const
+  bool operator()(const process::action& a, const process::action& b) const
   {
     if (a.label() != b.label())
     {
@@ -266,13 +280,13 @@ struct compare_action_label_arguments
 /// \brief Used for building an expression for the comparison of data parameters.
 struct equal_data_parameters_builder
 {
-  const std::vector<action>& a;
-  const std::vector<action>& b;
-  atermpp::set<data::data_expression>& result;
+  const std::vector<process::action>& a;
+  const std::vector<process::action>& b;
+  std::set<data::data_expression>& result;
 
-  equal_data_parameters_builder(const std::vector<action>& a_,
-                                const std::vector<action>& b_,
-                                atermpp::set<data::data_expression>& result_
+  equal_data_parameters_builder(const std::vector<process::action>& a_,
+                                const std::vector<process::action>& b_,
+                                std::set<data::data_expression>& result_
                                )
     : a(a_),
       b(b_),
@@ -282,23 +296,20 @@ struct equal_data_parameters_builder
   /// \brief Adds the expression 'a == b' to result.
   void operator()()
   {
-    using namespace data::lazy;
-    namespace d = data;
-
-    atermpp::vector<data::data_expression> v;
-    std::vector<action>::const_iterator i, j;
+    std::vector<data::data_expression> v;
+    std::vector<process::action>::const_iterator i, j;
     for (i = a.begin(), j = b.begin(); i != a.end(); ++i, ++j)
     {
       data::data_expression_list d1 = i->arguments();
       data::data_expression_list d2 = j->arguments();
       assert(d1.size() == d2.size());
-      data::data_expression_list::iterator i1, i2;
-      for (i1 = d1.begin(), i2 = d2.begin(); i1 != d1.end(); ++i1, ++i2)
+      data::data_expression_list::iterator i1 = d1.begin(), i2 = d2.begin();
+      for (     ; i1 != d1.end(); ++i1, ++i2)
       {
-        v.push_back(d::lazy::equal_to(*i1, *i2));
+        v.push_back(data::lazy::equal_to(*i1, *i2));
       }
     }
-    data::data_expression expr = d::lazy::join_and(v.begin(), v.end());
+    data::data_expression expr = data::lazy::join_and(v.begin(), v.end());
 #ifdef MCRL2_EQUAL_MULTI_ACTIONS_DEBUG
     mCRL2log(debug) << "  <and-term> " << data::pp(expr) << std::endl;
 #endif
@@ -309,13 +320,13 @@ struct equal_data_parameters_builder
 /// \brief Used for building an expression for the comparison of data parameters.
 struct not_equal_multi_actions_builder
 {
-  const std::vector<action>& a;
-  const std::vector<action>& b;
-  atermpp::vector<data::data_expression>& result;
+  const std::vector<process::action>& a;
+  const std::vector<process::action>& b;
+  std::vector<data::data_expression>& result;
 
-  not_equal_multi_actions_builder(const std::vector<action>& a_,
-                                  const std::vector<action>& b_,
-                                  atermpp::vector<data::data_expression>& result_
+  not_equal_multi_actions_builder(const std::vector<process::action>& a_,
+                                  const std::vector<process::action>& b_,
+                                  std::vector<data::data_expression>& result_
                                  )
     : a(a_),
       b(b_),
@@ -327,15 +338,15 @@ struct not_equal_multi_actions_builder
   {
     using namespace data::lazy;
 
-    atermpp::vector<data::data_expression> v;
-    std::vector<action>::const_iterator i, j;
+    std::vector<data::data_expression> v;
+    std::vector<process::action>::const_iterator i, j;
     for (i = a.begin(), j = b.begin(); i != a.end(); ++i, ++j)
     {
       data::data_expression_list d1 = i->arguments();
       data::data_expression_list d2 = j->arguments();
       assert(d1.size() == d2.size());
-      data::data_expression_list::iterator i1, i2;
-      for (i1 = d1.begin(), i2 = d2.begin(); i1 != d1.end(); ++i1, ++i2)
+      data::data_expression_list::iterator i1=d1.begin(), i2=d2.begin();
+      for (   ; i1 != d1.end(); ++i1, ++i2)
       {
         v.push_back(data::not_equal_to(*i1, *i2));
       }
@@ -356,14 +367,14 @@ inline data::data_expression equal_multi_actions(const multi_action& a, const mu
 {
 #ifdef MCRL2_EQUAL_MULTI_ACTIONS_DEBUG
   mCRL2log(debug) << "\n<equal multi actions>" << std::endl;
-  mCRL2log(debug) << "a = " << lps::pp(a.actions()) << std::endl;
-  mCRL2log(debug) << "b = " << lps::pp(b.actions()) << std::endl;
+  mCRL2log(debug) << "a = " << process::pp(a.actions()) << std::endl;
+  mCRL2log(debug) << "b = " << process::pp(b.actions()) << std::endl;
 #endif
   using namespace data::lazy;
 
   // make copies of a and b and sort them
-  std::vector<action> va(a.actions().begin(), a.actions().end()); // protection not needed
-  std::vector<action> vb(b.actions().begin(), b.actions().end()); // protection not needed
+  std::vector<process::action> va(a.actions().begin(), a.actions().end()); // protection not needed
+  std::vector<process::action> vb(b.actions().begin(), b.actions().end()); // protection not needed
   std::sort(va.begin(), va.end(), detail::compare_action_label_arguments());
   std::sort(vb.begin(), vb.end(), detail::compare_action_label_arguments());
 
@@ -371,14 +382,14 @@ inline data::data_expression equal_multi_actions(const multi_action& a, const mu
   {
 #ifdef MCRL2_EQUAL_MULTI_ACTIONS_DEBUG
     mCRL2log(debug) << "different action signatures detected!" << std::endl;
-    mCRL2log(debug) << "a = " << action_list(va.begin(), va.end()) << std::endl;
-    mCRL2log(debug) << "b = " << action_list(vb.begin(), vb.end()) << std::endl;
+    mCRL2log(debug) << "a = " << process::action_list(va.begin(), va.end()) << std::endl;
+    mCRL2log(debug) << "b = " << process::action_list(vb.begin(), vb.end()) << std::endl;
 #endif
     return data::sort_bool::false_();
   }
 
   // compute the intervals of a with equal names
-  typedef std::vector<action>::iterator action_iterator;
+  typedef std::vector<process::action>::iterator action_iterator;
   std::vector<std::pair<action_iterator, action_iterator> > intervals;
   action_iterator first = va.begin();
   while (first != va.end())
@@ -388,7 +399,7 @@ inline data::data_expression equal_multi_actions(const multi_action& a, const mu
     first = next;
   }
 
-  atermpp::set<data::data_expression> z;
+  std::set<data::data_expression> z;
   detail::equal_data_parameters_builder f(va, vb, z);
   detail::forall_permutations(intervals.begin(), intervals.end(), f);
   data::data_expression result = data::lazy::join_or(z.begin(), z.end());
@@ -405,8 +416,8 @@ inline data::data_expression not_equal_multi_actions(const multi_action& a, cons
   using namespace data::lazy;
 
   // make copies of a and b and sort them
-  std::vector<action> va(a.actions().begin(), a.actions().end());
-  std::vector<action> vb(b.actions().begin(), b.actions().end());
+  std::vector<process::action> va(a.actions().begin(), a.actions().end());
+  std::vector<process::action> vb(b.actions().begin(), b.actions().end());
   std::sort(va.begin(), va.end(), detail::compare_action_label_arguments());
   std::sort(vb.begin(), vb.end(), detail::compare_action_label_arguments());
 
@@ -416,7 +427,7 @@ inline data::data_expression not_equal_multi_actions(const multi_action& a, cons
   }
 
   // compute the intervals of a with equal names
-  typedef std::vector<action>::iterator action_iterator;
+  typedef std::vector<process::action>::iterator action_iterator;
   std::vector<std::pair<action_iterator, action_iterator> > intervals;
   action_iterator first = va.begin();
   while (first != va.end())
@@ -425,77 +436,15 @@ inline data::data_expression not_equal_multi_actions(const multi_action& a, cons
     intervals.push_back(std::make_pair(first, next));
     first = next;
   }
-  atermpp::vector<data::data_expression> z;
+  std::vector<data::data_expression> z;
   detail::not_equal_multi_actions_builder f(va, vb, z);
   detail::forall_permutations(intervals.begin(), intervals.end(), f);
   data::data_expression result = data::lazy::join_and(z.begin(), z.end());
   return result;
 }
 
-/// \brief Represents the name of a multi action
-typedef atermpp::multiset<core::identifier_string> multi_action_name;
-
-/// \brief Represents a set of multi action names
-typedef std::set<multi_action_name> multi_action_name_set;
-
-/// \brief Pretty print function for a multi action name
-inline
-std::string pp(const multi_action_name& x)
-{
-  std::ostringstream out;
-  for (multi_action_name::const_iterator i = x.begin(); i != x.end(); ++i)
-  {
-    if (i != x.begin())
-    {
-      out << " | ";
-    }
-    out << core::pp(*i);
-  }
-  return out.str();
-}
-
-/// \brief Pretty print function for a set of multi action names
-inline
-std::string pp(const multi_action_name_set& A)
-{
-  std::ostringstream out;
-  out << "{";
-  for (multi_action_name_set::const_iterator i = A.begin(); i != A.end(); ++i)
-  {
-    if (i != A.begin())
-    {
-      out << ", ";
-    }
-    out << pp(*i);
-  }
-  out << "}";
-  return out.str();
-}
-
 } // namespace lps
 
 } // namespace mcrl2
 
-/// \cond INTERNAL_DOCS
-namespace atermpp
-{
-template<>
-struct aterm_traits<mcrl2::lps::multi_action>
-{
-  static void protect(const mcrl2::lps::multi_action& t)
-  {
-    t.protect();
-  }
-  static void unprotect(const mcrl2::lps::multi_action& t)
-  {
-    t.unprotect();
-  }
-  static void mark(const mcrl2::lps::multi_action& t)
-  {
-    t.mark();
-  }
-};
-} // namespace atermpp
-/// \endcond
-
 #endif // MCRL2_LPS_MULTI_ACTION_H
