@@ -24,7 +24,7 @@ namespace lps
 
 template <typename T>
 void translate_user_notation(T& x,
-                             typename boost::disable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                             typename std::enable_if< !std::is_base_of< atermpp::aterm, T >::value>::type* = 0
                             )
 {
   core::make_update_apply_builder<lps::data_expression_builder>(data::detail::translate_user_notation_function())(x);
@@ -32,7 +32,7 @@ void translate_user_notation(T& x,
 
 template <typename T>
 T translate_user_notation(const T& x,
-                          typename boost::enable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                          typename std::enable_if< std::is_base_of< atermpp::aterm, T >::value>::type* = 0
                          )
 {
   return core::make_update_apply_builder<lps::data_expression_builder>(data::detail::translate_user_notation_function())(x);
