@@ -14,9 +14,9 @@
 #include <algorithm>
 
 #include "mcrl2/utilities/logger.h"
-#include "mcrl2/aterm/aterm_ext.h"
-#include "mcrl2/core/detail/struct_core.h"
+#include "mcrl2/core/detail/function_symbols.h"
 #include "mcrl2/data/detail/bdd_prover.h"
+#include "mcrl2/data/substitutions/mutable_map_substitution.h"
 #include "mcrl2/lps/confluence_checker.h"
 #include "mcrl2/utilities/exception.h"
 
@@ -39,9 +39,9 @@ using namespace mcrl2::core::detail;
 // Auxiliary functions ----------------------------------------------------------------------------
 
 static
-atermpp::map < variable,data_expression> get_substitutions_from_assignments(const assignment_list a_assignments)
+data::mutable_map_substitution<> get_substitutions_from_assignments(const assignment_list a_assignments)
 {
-  atermpp::map < variable,data_expression> v_substitutions;
+  data::mutable_map_substitution<> v_substitutions;
 
   for (assignment_list::const_iterator i=a_assignments.begin(); i!=a_assignments.end(); ++i)
   {
@@ -56,8 +56,8 @@ data_expression get_subst_equation_from_assignments(
   const variable_list a_variables,
   assignment_list a_assignments_1,
   assignment_list a_assignments_2,
-  const atermpp::map<variable,data_expression> &a_substitutions_1,
-  const atermpp::map<variable,data_expression> &a_substitutions_2)
+  data::mutable_map_substitution<>& a_substitutions_1,
+  data::mutable_map_substitution<>& a_substitutions_2)
 {
   data_expression v_result = sort_bool::true_();
 
@@ -74,20 +74,18 @@ data_expression get_subst_equation_from_assignments(
     if (!a_assignments_1.empty() && v_next_1)
     {
       const assignment v_assignment_1 = a_assignments_1.front();
-      a_assignments_1 = pop_front(a_assignments_1);
+      a_assignments_1.pop_front();
       v_variable_1 = v_assignment_1.lhs();
       v_expression_1 = v_assignment_1.rhs();
-      v_expression_1 = data::replace_free_variables(v_expression_1,
-                       data::make_map_substitution(a_substitutions_2));
+      v_expression_1 = data::replace_variables_capture_avoiding(v_expression_1, a_substitutions_2, data::substitution_variables(a_substitutions_2));
     }
     if (!a_assignments_2.empty() && v_next_2)
     {
       const assignment v_assignment_2 = a_assignments_2.front();
-      a_assignments_2 = pop_front(a_assignments_2);
+      a_assignments_2.pop_front();
       v_variable_2 = v_assignment_2.lhs();
       v_expression_2 = v_assignment_2.rhs();
-      v_expression_2 = data::replace_free_variables(v_expression_2,
-                       data::make_map_substitution(a_substitutions_1));
+      v_expression_2 = data::replace_variables_capture_avoiding(v_expression_2, a_substitutions_1, data::substitution_variables(a_substitutions_1));
     }
     while (v_variable != v_variable_1 && v_variable != v_variable_2 && i!=a_variables.end())
     {
@@ -102,16 +100,14 @@ data_expression get_subst_equation_from_assignments(
     }
     else if (v_variable == v_variable_1)
     {
-      data_expression expr = data::replace_free_variables(data_expression(v_variable_1),
-                     data::make_map_substitution(a_substitutions_1));
+      data_expression expr = data::replace_variables_capture_avoiding(data_expression(v_variable_1), a_substitutions_1, data::substitution_variables(a_substitutions_1));
       v_result = sort_bool::and_(data_expression(v_result), equal_to(v_expression_1, expr));
       v_next_1 = true;
       v_next_2 = false;
     }
     else if (v_variable == v_variable_2)
     {
-      data_expression expr = data::replace_free_variables(data_expression(v_variable_2),
-                     data::make_map_substitution(a_substitutions_2));
+      data_expression expr = data::replace_variables_capture_avoiding(data_expression(v_variable_2), a_substitutions_2, data::substitution_variables(a_substitutions_2));
       v_result = sort_bool::and_(data_expression(v_result), equal_to(data_expression(v_expression_2), expr));
       v_next_1 = false;
       v_next_2 = true;
@@ -139,14 +135,14 @@ data_expression get_equation_from_assignments(
       {
         // Create a condition from the assigments from both lists.
         v_result = sort_bool::and_(v_result, equal_to(a_assignments_1.front().rhs(), a_assignments_2.front().rhs()));
-        a_assignments_2=pop_front(a_assignments_2);
+        a_assignments_2.pop_front();
       }
       else
       {
         // Create a condition from first assigment only.
         v_result = sort_bool::and_(v_result, equal_to(a_assignments_1.front().rhs(), v_variable));
       }
-      a_assignments_1=pop_front(a_assignments_1);
+      a_assignments_1.pop_front();
     }
     else
     {
@@ -154,7 +150,7 @@ data_expression get_equation_from_assignments(
       {
         // Create a condition from the second assigments only.
         v_result = sort_bool::and_(v_result, equal_to(v_variable, a_assignments_2.front().rhs()));
-        a_assignments_2=pop_front(a_assignments_2);
+        a_assignments_2.pop_front();
       }
     }
   }
@@ -168,18 +164,17 @@ data_expression get_equation_from_assignments(
 // ----------------------------------------------------------------------------------------------
 static
 data_expression get_subst_equation_from_actions(
-  const action_list a_actions,
-  const atermpp::map<variable,data_expression> &a_substitutions)
+  const process::action_list a_actions,
+  data::mutable_map_substitution<>& a_substitutions)
 {
   data_expression v_result = sort_bool::true_();
 
-  for (action_list::const_iterator i=a_actions.begin(); i!=a_actions.end(); ++i)
+  for (auto i=a_actions.begin(); i!=a_actions.end(); ++i)
   {
     const data_expression_list v_expressions = i->arguments();
     for (data_expression_list::const_iterator j=v_expressions.begin(); j!=v_expressions.end(); ++j)
     {
-      const data_expression v_subst_expression = data::replace_free_variables(*j,
-          data::make_map_substitution(a_substitutions));
+      const data_expression v_subst_expression = data::replace_variables_capture_avoiding(*j, a_substitutions, data::substitution_variables(a_substitutions));
       v_result = sort_bool::and_(data_expression(v_result), equal_to(*j, v_subst_expression));
     }
   }
@@ -200,20 +195,18 @@ data_expression get_confluence_condition(
   const data_expression v_condition_1 = a_summand_1.condition();
   const assignment_list v_assignments_1 = a_summand_1.assignments();
 
-  atermpp::map < variable,data_expression> v_substitutions_1 = get_substitutions_from_assignments(v_assignments_1);
+  data::mutable_map_substitution<> v_substitutions_1 = get_substitutions_from_assignments(v_assignments_1);
   const data_expression v_condition_2 = a_summand_2.condition();
   const data_expression v_lhs = sort_bool::and_(sort_bool::and_(v_condition_1, v_condition_2), a_invariant);
   const assignment_list v_assignments_2 = a_summand_2.assignments();
 
-  atermpp::map < variable,data_expression> v_substitutions_2 = get_substitutions_from_assignments(v_assignments_2);
-  const data_expression v_subst_condition_1 = data::replace_free_variables(v_condition_1,
-      data::make_map_substitution(v_substitutions_2));
-  const data_expression v_subst_condition_2 = data::replace_free_variables(v_condition_2,
-      data::make_map_substitution(v_substitutions_1));
+  data::mutable_map_substitution<> v_substitutions_2 = get_substitutions_from_assignments(v_assignments_2);
+  const data_expression v_subst_condition_1 = data::replace_variables_capture_avoiding(v_condition_1, v_substitutions_2, data::substitution_variables(v_substitutions_2));
+  const data_expression v_subst_condition_2 = data::replace_variables_capture_avoiding(v_condition_2, v_substitutions_1, data::substitution_variables(v_substitutions_1));
 
   const data_expression v_subst_equation = get_subst_equation_from_assignments(a_variables, v_assignments_1, v_assignments_2, v_substitutions_1, v_substitutions_2);
 
-  const action_list v_actions =a_summand_2.multi_action().actions();
+  const process::action_list v_actions =a_summand_2.multi_action().actions();
   data_expression v_rhs;
 
   if (v_actions.empty())
@@ -240,7 +233,7 @@ data_expression get_confluence_condition(
 static
 bool has_ctau_action(const specification& a_lps)
 {
-  const action_label_list v_action_specification = a_lps.action_labels();
+  const process::action_label_list v_action_specification = a_lps.action_labels();
   return std::find(v_action_specification.begin(),v_action_specification.end(),make_ctau_act_id())!=v_action_specification.end();
 }
 
@@ -267,7 +260,7 @@ void Confluence_Checker::print_counter_example()
   if (f_counter_example)
   {
     const data_expression v_counter_example(f_bdd_prover.get_counter_example());
-    std::cerr << "  Counter example: " << data::pp(v_counter_example) << "\n";
+    mCRL2log(info) << "  Counter example: " << data::pp(v_counter_example) << "\n";
   }
 }
 
@@ -453,7 +446,6 @@ Confluence_Checker::Confluence_Checker(
   bool a_path_eliminator,
   smt_solver_type a_solver_type,
   bool a_apply_induction,
-  bool a_no_marking,
   bool a_check_all,
   bool a_counter_example,
   bool a_generate_invariants,
@@ -463,7 +455,6 @@ Confluence_Checker::Confluence_Checker(
   f_bdd_prover(a_lps.data(), used_data_equation_selector(a_lps.data()), a_rewrite_strategy,
                      a_time_limit, a_path_eliminator, a_solver_type, a_apply_induction),
   f_lps(a_lps),
-  f_no_marking(a_no_marking),
   f_check_all(a_check_all),
   f_counter_example(a_counter_example),
   f_dot_file_name(a_dot_file_name),
@@ -511,10 +502,10 @@ specification Confluence_Checker::check_confluence_and_mark(const data_expressio
     v_process_equation.deadlock_summands(),
     v_summands);
 
-  action_label_list v_act_decls=f_lps.action_labels();
+  process::action_label_list v_act_decls=f_lps.action_labels();
   if (v_is_marked && !has_ctau_action(f_lps))
   {
-    v_act_decls = push_front(v_act_decls,make_ctau_act_id());
+    v_act_decls.push_front(make_ctau_act_id());
   }
 
   specification v_lps(f_lps.data(),v_act_decls,f_lps.global_variables(),new_process_equation,f_lps.initial_process());
