@@ -14,10 +14,10 @@
 
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/core/print.h"
-#include "mcrl2/aterm/aterm_ext.h"
 #include "mcrl2/lps/invariant_checker.h"
 #include "mcrl2/data/detail/prover/solver_type.h"
 #include "mcrl2/data/detail/bdd_prover.h"
+#include "mcrl2/data/substitutions/mutable_map_substitution.h"
 #include "mcrl2/utilities/exception.h"
 
 namespace mcrl2
@@ -40,8 +40,8 @@ void Invariant_Checker::print_counter_example()
   if (f_counter_example)
   {
     data_expression v_counter_example(f_bdd_prover.get_counter_example());
-    assert(v_counter_example!=0);
-    std::cerr << "  Counter example: " << data::pp(v_counter_example) << "\n";
+    assert(v_counter_example.defined());
+    mCRL2log(info) << "  Counter example: " << data::pp(v_counter_example) << "\n";
   }
 }
 
@@ -71,15 +71,14 @@ void Invariant_Checker::save_dot_file(size_t a_summand_number)
 
 bool Invariant_Checker::check_init(const data_expression a_invariant)
 {
-  atermpp::map < variable, data_expression> v_substitutions;
+  data::mutable_map_substitution<> v_substitutions;
   const assignment_list l=f_init.assignments();
   for (assignment_list::const_iterator i=l.begin(); i!=l.end(); ++i)
   {
     v_substitutions[i->lhs()]=i->rhs();
   }
 
-  data_expression b_invariant = data::replace_free_variables(a_invariant,
-                                data::make_map_substitution(v_substitutions));
+  data_expression b_invariant = data::replace_variables_capture_avoiding(a_invariant, v_substitutions, data::substitution_variables(v_substitutions));
   f_bdd_prover.set_formula(b_invariant);
   if (f_bdd_prover.is_tautology() == answer_yes)
   {
@@ -107,15 +106,14 @@ bool Invariant_Checker::check_summand(
   const data_expression v_condition = a_summand.condition();
   const assignment_list v_assignments = a_summand.assignments();
 
-  atermpp::map < variable, data_expression> v_substitutions;
+  data::mutable_map_substitution<> v_substitutions;
 
   for (assignment_list::const_iterator i=v_assignments.begin(); i!=v_assignments.end(); ++i)
   {
     v_substitutions[i->lhs()]=i->rhs();
   }
 
-  const data_expression v_subst_invariant = data::replace_free_variables(a_invariant,
-      data::make_map_substitution(v_substitutions));
+  const data_expression v_subst_invariant = data::replace_variables_capture_avoiding(a_invariant, v_substitutions, data::substitution_variables(v_substitutions));
 
   const data_expression v_formula = implies(and_(a_invariant, v_condition), v_subst_invariant);
   f_bdd_prover.set_formula(v_formula);
