@@ -14,8 +14,6 @@
 #include <boost/test/minimal.hpp>
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/lps/linearise.h"
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 using namespace mcrl2::data;
@@ -31,14 +29,10 @@ const std::string SPECIFICATION =
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
   specification spec = linearise(SPECIFICATION);
   data_specification data = spec.data();
   BOOST_CHECK(data.is_certainly_finite(sort_bool::bool_()));
-  core::garbage_collect();
   BOOST_CHECK(!data.is_certainly_finite(sort_nat::nat()));
-  core::garbage_collect();
 
   return 0;
 }
