@@ -14,14 +14,13 @@
 #include <algorithm>
 #include <boost/test/minimal.hpp>
 #include "mcrl2/data/rewriter.h"
-#include "mcrl2/data/detail/parse_substitutions.h"
+#include "mcrl2/data/detail/parse_substitution.h"
 #include "mcrl2/data/detail/data_functional.h"
+#include "mcrl2/data/substitutions/mutable_map_substitution.h"
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/lps/rewrite.h"
 #include "mcrl2/lps/parse.h"
-#include "mcrl2/core/garbage_collection.h"
 #include "mcrl2/lps/detail/specification_property_map.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace std;
 using namespace mcrl2;
@@ -166,8 +165,8 @@ void test_lps_rewriter(std::string src_text, std::string dest_text, std::string
 
   // rewrite the specification src
   data::rewriter R(src.data());
-  data::mutable_map_substitution< > sigma;
-  data::detail::parse_substitutions(sigma_text, src.data(), sigma);
+  data::mutable_map_substitution<> sigma;
+  data::detail::parse_substitution(sigma_text, sigma, src.data());
   lps::rewrite(src, R, sigma);
 
   if (src != dest)
@@ -178,7 +177,6 @@ void test_lps_rewriter(std::string src_text, std::string dest_text, std::string
     std::cerr << lps::pp(dest) << std::endl;
   }
   BOOST_CHECK(src == dest);
-  core::garbage_collect();
 }
 
 void test_lps_rewriter()
@@ -198,8 +196,6 @@ void test_lps_rewriter()
 
 int test_main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
   test1();
   test2();
   test3();
