@@ -14,14 +14,12 @@
 #include <set>
 #include <iterator>
 #include <boost/test/minimal.hpp>
+#include "mcrl2/data/detail/print_utility.h"
 #include "mcrl2/lps/print.h"
 #include "mcrl2/lps/find.h"
 #include "mcrl2/lps/linearise.h"
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/lps/detail/test_input.h"
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/core/detail/print_utility.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 using namespace mcrl2::data;
@@ -32,8 +30,7 @@ void test_find_sort_expressions()
   specification spec = linearise(lps::detail::ABP_SPECIFICATION());
   std::set<sort_expression> s;
   lps::find_sort_expressions(spec, std::inserter(s, s.end()));
-  std::cout << core::detail::print_set(s, lps::stream_printer()) << std::endl;
-  // BOOST_CHECK(false);
+  std::cout << core::detail::print_set(s) << std::endl;
 }
 
 void test_system_defined_sorts()
@@ -50,13 +47,9 @@ void test_system_defined_sorts()
 
 int test_main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
   test_find_sort_expressions();
-  core::garbage_collect();
 
   test_system_defined_sorts();
-  core::garbage_collect();
 
   return 0;
 }
