@@ -2,20 +2,20 @@
 /* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton interface for Bison's Yacc-like parsers in C
-
+   
       Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
+   
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -28,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -36,27 +36,26 @@
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-/* Put the tokens into the symbol table, so that GDB and other debuggers
-   know about them.  */
-enum yytokentype
-{
-  DIGRAPH = 258,
-  GRAPH = 259,
-  STRICT = 260,
-  SUBGRAPH = 261,
-  NODE = 262,
-  EDGE = 263,
-  IS = 264,
-  COMMA = 265,
-  COLON = 266,
-  SEMICOLON = 267,
-  LBRACE = 268,
-  RBRACE = 269,
-  LBRACK = 270,
-  RBRACK = 271,
-  ARROW = 272,
-  ID = 273
-};
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum yytokentype {
+     DIGRAPH = 258,
+     GRAPH = 259,
+     STRICT = 260,
+     SUBGRAPH = 261,
+     NODE = 262,
+     EDGE = 263,
+     IS = 264,
+     COMMA = 265,
+     COLON = 266,
+     SEMICOLON = 267,
+     LBRACE = 268,
+     RBRACE = 269,
+     LBRACK = 270,
+     RBRACK = 271,
+     ARROW = 272,
+     ID = 273
+   };
 #endif
 
 
