@@ -2,20 +2,20 @@
 /* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton interface for Bison's Yacc-like parsers in C
-
+   
       Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
+   
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -28,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -36,33 +36,32 @@
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-/* Put the tokens into the symbol table, so that GDB and other debuggers
-   know about them.  */
-enum yytokentype
-{
-  EOLN = 258,
-  SECSEP = 259,
-  LPAR = 260,
-  RPAR = 261,
-  ARROW = 262,
-  HASH = 263,
-  QMARK = 264,
-  COLON = 265,
-  COMMA = 266,
-  BAG = 267,
-  BAR = 268,
-  KWSTRUCT = 269,
-  SET = 270,
-  LIST = 271,
-  NUMBER = 272,
-  ID = 273,
-  QUOTED = 274,
-  BOOL = 275,
-  POS = 276,
-  NAT = 277,
-  INT = 278,
-  REAL = 279
-};
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum yytokentype {
+     EOLN = 258,
+     SECSEP = 259,
+     LPAR = 260,
+     RPAR = 261,
+     ARROW = 262,
+     HASH = 263,
+     QMARK = 264,
+     COLON = 265,
+     COMMA = 266,
+     BAG = 267,
+     BAR = 268,
+     KWSTRUCT = 269,
+     SET = 270,
+     LIST = 271,
+     NUMBER = 272,
+     ID = 273,
+     QUOTED = 274,
+     BOOL = 275,
+     POS = 276,
+     NAT = 277,
+     INT = 278,
+     REAL = 279
+   };
 #endif
 
 
