@@ -21,7 +21,7 @@
 
 #include <sstream>
 #include "mcrl2/utilities/logger.h"
-#include "mcrl2/core/detail/struct_core.h"
+#include "mcrl2/core/detail/function_symbols.h"
 #include "mcrl2/data/parse.h"
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/lts/lts_lts.h"
@@ -43,7 +43,7 @@ inline void convert_core_lts(CONVERTOR& c,
                              LTS_OUT_TYPE& lts_out);
 
 /** \brief Convert LTSs to each other. The input lts is not usable afterwards.
-    \detail Converts the input lts into the output lts maintaining the nature
+    \details Converts the input lts into the output lts maintaining the nature
             of the lts as good as possible. If no translation can be provided
             because more information is required (such as data types), then
             a mcrl2::runtime error is thrown. This also happens if the particular
@@ -68,7 +68,7 @@ inline void lts_convert(
   const LTS_IN_TYPE& lts_in,
   LTS_OUT_TYPE& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -104,7 +104,7 @@ inline void lts_convert(
   const lts_lts_t& lts_in,
   lts_lts_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -120,13 +120,13 @@ inline void lts_convert(
 class lts_fsm_convertor
 {
   private:
-    std::vector < atermpp::map <data::data_expression , size_t > > state_element_values_sets;
+    std::vector < std::map <data::data_expression , size_t > > state_element_values_sets;
     lts_fsm_t& lts_out;
 
   public:
     lts_fsm_convertor(size_t n, lts_fsm_t& l):
-      state_element_values_sets(std::vector < atermpp::map <data::data_expression , size_t > >
-                                (n,atermpp::map <data::data_expression , size_t >())),
+      state_element_values_sets(std::vector < std::map <data::data_expression , size_t > >
+                                (n,std::map <data::data_expression , size_t >())),
       lts_out(l)
     {
     }
@@ -138,12 +138,11 @@ class lts_fsm_convertor
 
     state_label_fsm translate_state(const state_label_lts& l)
     {
-
       std::vector < size_t > result;
       for (size_t i=0; i<l.size(); ++i)
       {
         const data::data_expression t=l[i];
-        atermpp::map <data::data_expression , size_t >::const_iterator index=state_element_values_sets[i].find(t);
+        std::map <data::data_expression , size_t >::const_iterator index=state_element_values_sets[i].find(t);
         if (index==state_element_values_sets[i].end())
         {
           const size_t element_index=state_element_values_sets[i].size();
@@ -185,7 +184,7 @@ inline void lts_convert(
   const lts_lts_t& lts_in,
   lts_fsm_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -224,7 +223,7 @@ inline void lts_convert(
   const lts_lts_t& lts_in,
   lts_aut_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -273,7 +272,7 @@ inline void lts_convert(
   const lts_lts_t& lts_in,
   lts_dot_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -314,7 +313,7 @@ inline void lts_convert(
   const lts_lts_t& lts_in,
   lts_bcg_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -333,12 +332,12 @@ class aut_lts_convertor
 {
   protected:
     const data::data_specification& m_data;
-    const lps::action_label_list& m_action_labels;
+    const process::action_label_list& m_action_labels;
 
   public:
     aut_lts_convertor(
       const data::data_specification& data,
-      const lps::action_label_list& action_labels):
+      const process::action_label_list& action_labels):
       m_data(data),
       m_action_labels(action_labels)
     {}
@@ -384,7 +383,7 @@ inline void lts_convert(
   const lts_aut_t& lts_in,
   lts_lts_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -416,7 +415,7 @@ inline void lts_convert(
   const lts_aut_t& lts_in,
   lts_aut_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -460,7 +459,7 @@ inline void lts_convert(
   const lts_aut_t& lts_in,
   lts_fsm_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -506,7 +505,7 @@ inline void lts_convert(
   const lts_bcg_t& lts_in,
   lts_dot_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -553,7 +552,7 @@ inline void lts_convert(
   const lts_aut_t& lts_in,
   lts_dot_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -606,7 +605,7 @@ class fsm_lts_convertor
     state_label_lts translate_state(const state_label_fsm& l) const
     {
       // If process_parameters are not empty, we use them to check that the sorts of its variables  match.
-      atermpp::vector < data::data_expression > state_label;
+      std::vector < data::data_expression > state_label;
       size_t idx=0;
       const data::variable_list& parameters=m_lts_out.process_parameters();
       data::variable_list::const_iterator parameter_iterator=parameters.begin();
@@ -643,7 +642,7 @@ inline void lts_convert(
   const lts_fsm_t& lts_in,
   lts_lts_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -689,7 +688,7 @@ inline void lts_convert(
   const lts_fsm_t& lts_in,
   lts_aut_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -714,7 +713,7 @@ inline void lts_convert(
   const lts_fsm_t& lts_in,
   lts_fsm_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -754,7 +753,7 @@ inline void lts_convert(
   const lts_fsm_t& lts_in,
   lts_bcg_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -816,7 +815,7 @@ inline void lts_convert(
   const lts_fsm_t& lts_in,
   lts_dot_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -834,12 +833,12 @@ class bcg_lts_convertor
 {
   protected:
     const data::data_specification& m_data;
-    const lps::action_label_list& m_action_labels;
+    const process::action_label_list& m_action_labels;
 
   public:
     bcg_lts_convertor(
       const data::data_specification& data,
-      const lps::action_label_list& action_labels):
+      const process::action_label_list& action_labels):
       m_data(data),
       m_action_labels(action_labels)
     {}
@@ -885,7 +884,7 @@ inline void lts_convert(
   const lts_bcg_t& lts_in,
   lts_lts_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -915,7 +914,7 @@ class bcg_aut_convertor
     {
       if (l.find('"')!=std::string::npos)
       {
-        throw mcrl2::runtime_error(std::string("The action label \"") + l + 
+        throw mcrl2::runtime_error(std::string("The action label \"") + l +
                      "\" constains a double quote. The resulting aut label will be invalid.");
       }
 
@@ -940,7 +939,7 @@ inline void lts_convert(
   const lts_bcg_t& lts_in,
   lts_aut_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -986,7 +985,7 @@ inline void lts_convert(
   const lts_bcg_t& lts_in,
   lts_fsm_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -1013,7 +1012,7 @@ inline void lts_convert(
   const lts_bcg_t& lts_in,
   lts_bcg_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -1056,7 +1055,7 @@ inline void lts_convert(
   const lts_aut_t& lts_in,
   lts_bcg_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -1073,13 +1072,13 @@ inline void lts_convert(
 class dot_lts_convertor
 {
   private:
-    std::vector < atermpp::map <std::string , size_t > > state_element_values_sets;
+    std::vector < std::map <std::string , size_t > > state_element_values_sets;
     const lts_lts_t& lts_out;
 
   public:
     dot_lts_convertor(lts_lts_t& l):
-      state_element_values_sets(std::vector < atermpp::map <std::string , size_t > >
-                                (2,atermpp::map <std::string , size_t >())),
+      state_element_values_sets(std::vector < std::map <std::string , size_t > >
+                                (2,std::map <std::string , size_t >())),
       lts_out(l)
     {
     }
@@ -1125,7 +1124,7 @@ inline void lts_convert(
   const lts_dot_t& lts_in,
   lts_lts_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -1173,7 +1172,7 @@ inline void lts_convert(
   const lts_dot_t& lts_in,
   lts_aut_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -1191,13 +1190,13 @@ inline void lts_convert(
 class dot_fsm_convertor
 {
   private:
-    std::vector < atermpp::map <std::string , size_t > > state_element_values_sets;
+    std::vector < std::map <std::string , size_t > > state_element_values_sets;
     lts_fsm_t& lts_out;
 
   public:
     dot_fsm_convertor(lts_fsm_t& l):
-      state_element_values_sets(std::vector < atermpp::map <std::string , size_t > >
-                                (2,atermpp::map <std::string , size_t >())),
+      state_element_values_sets(std::vector < std::map <std::string , size_t > >
+                                (2,std::map <std::string , size_t >())),
       lts_out(l)
     {
     }
@@ -1211,7 +1210,7 @@ class dot_fsm_convertor
     {
       std::vector < size_t > result;
       const std::string state=l.name();
-      atermpp::map <std::string , size_t >::const_iterator index=state_element_values_sets[0].find(state);
+      std::map <std::string , size_t >::const_iterator index=state_element_values_sets[0].find(state);
       if (index==state_element_values_sets[0].end())
       {
         const size_t element_index=state_element_values_sets[0].size();
@@ -1258,7 +1257,7 @@ inline void lts_convert(
   const lts_dot_t& lts_in,
   lts_fsm_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
@@ -1298,7 +1297,7 @@ inline void lts_convert(
   const lts_dot_t& lts_in,
   lts_bcg_t& lts_out,
   const data::data_specification& data,
-  const lps::action_label_list& action_labels,
+  const process::action_label_list& action_labels,
   const data::variable_list& process_parameters,
   const bool extra_data_is_defined=true)
 {
@@ -1325,7 +1324,7 @@ inline void lts_convert(
   const lts_dot_t& lts_in,
   lts_dot_t& lts_out,
   const data::data_specification&,
-  const lps::action_label_list&,
+  const process::action_label_list&,
   const data::variable_list&,
   const bool extra_data_is_defined=true)
 {
