@@ -22,17 +22,18 @@
 #include <string>
 #include <vector>
 #include "mcrl2/atermpp/aterm_appl.h"
-// #include "mcrl2/atermpp/aterm_balanced_tree.h"
-#include "mcrl2/core/detail/struct_core.h"
+#include "mcrl2/atermpp/aterm_balanced_tree.h"
+#include "mcrl2/core/detail/function_symbols.h"
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/core/parse.h"
 #include "mcrl2/data/variable.h"
+#include "mcrl2/lps/parse.h"
+#include "mcrl2/lps/print.h"
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/lps/multi_action.h"
-#include "mcrl2/lps/action_parse.h"
+#include "mcrl2/process/action_parse.h"
 #include "mcrl2/lps/typecheck.h"
 #include "mcrl2/lts/lts.h"
-#include "mcrl2/lps/detail/multi_action_print.h"
 
 namespace mcrl2
 {
@@ -41,41 +42,36 @@ namespace lts
 namespace detail
 {
 
+extern std::vector < atermpp::function_symbol > state_function_symbols;
+
 /** \brief This class contains state labels for an labelled transition system in .lts format.
     \details A state label in .lts format consists of a term of the form
              STATE(t1,...,tn) where ti are the data expressions.
 */
 class state_label_lts : public atermpp::aterm_appl
 {
-    friend struct atermpp::aterm_traits<state_label_lts>;
-
   protected:
 
     /** \brief We store functions symbols with varying arity and string "STATE"
-        in this ATermAppl array, as they are automatically protected.
+        in this atermpp::aterm_appl array, as they are automatically protected.
     */
-    static atermpp::vector < ATermAppl > vector_templates;
 
     /** \brief A function to get a protected STATE function symbol of the
                indicated arity.
     */
-    static AFun get_STATE_function_symbol(const size_t arity)
+
+    static atermpp::function_symbol get_STATE_function_symbol(const size_t arity)
     {
-      if (arity>=vector_templates.size())
+
+      if (arity>=state_function_symbols.size())
       {
-        vector_templates.resize(arity+1,NULL);
+        state_function_symbols.resize(arity+1);
       }
-      if (vector_templates[arity]==NULL)
+      if (state_function_symbols[arity]==atermpp::function_symbol())
       {
-        ATermAppl stub=ATmakeAppl0(ATmakeAFun("STUB",0,false));
-        ATermList l=ATempty;
-        for (size_t i=0; i<arity; ++i)
-        {
-          l=ATinsert(l,(ATerm)stub);
-        }
-        vector_templates[arity]=ATmakeApplList(ATmakeAFun("STATE",arity,false),l);
+        state_function_symbols[arity]=atermpp::function_symbol("STATE",arity);
       }
-      return ATgetAFun(vector_templates[arity]);
+      return state_function_symbols[arity];
     }
 
   public:
@@ -88,23 +84,30 @@ class state_label_lts : public atermpp::aterm_appl
 
     /** \brief Constructor. Set the state label to the aterm a.
         \details The aterm a must have the shape "STATE(t1,...,tn).*/
-    state_label_lts(const ATermAppl& a):atermpp::aterm_appl(a)
+    state_label_lts(const atermpp::aterm_appl& a):atermpp::aterm_appl(a)
     {
+      assert(a.function()== get_STATE_function_symbol(a.function().arity()));
+
       // Take care that the STATE function symbol with the desired arity exists.
-      const size_t arity=ATgetArity(ATgetAFun(a));
-      get_STATE_function_symbol(arity);
-      assert(ATgetAFun(a)==ATgetAFun(vector_templates[arity]));
+      /* const size_t arity=a.function().arity();
+      get_STATE_function_symbol(arity); */
     }
 
     /** \brief Construct a state label out of a data_expression_list.
     */
     state_label_lts(const mcrl2::data::data_expression_list& l):
-      atermpp::aterm_appl(get_STATE_function_symbol(l.size()),atermpp::aterm_list(l))
+      atermpp::aterm_appl(get_STATE_function_symbol(l.size()),l.begin(),l.end())
     {}
 
     /** \brief Construct a state label out of a data_expression_vector.
     */
-    state_label_lts(const atermpp::vector < mcrl2::data::data_expression > &l):
+    state_label_lts(const std::vector < mcrl2::data::data_expression >& l):
+      atermpp::aterm_appl(get_STATE_function_symbol(l.size()),l.begin(),l.end())
+    {}
+
+    /** \brief Construct a state label out of a balanced tree of data expressions, representing a state label.
+    */
+    state_label_lts(const atermpp::term_balanced_tree<mcrl2::data::data_expression>& l):
       atermpp::aterm_appl(get_STATE_function_symbol(l.size()),l.begin(),l.end())
     {}
 
@@ -112,10 +115,10 @@ class state_label_lts : public atermpp::aterm_appl
         \param[in] i The index of the state label. Must be smaller than the length of this state label.
         \return The data expression at position i of this state label.
     */
-    mcrl2::data::data_expression operator [](const size_t i) const
+    const mcrl2::data::data_expression& operator [](const size_t i) const
     {
       assert(i<size());
-      return mcrl2::data::data_expression(this->argument(i));
+      return atermpp::down_cast<mcrl2::data::data_expression>(atermpp::aterm_appl::operator[](i));
     }
 
     /** \brief Set the i-th element of this state label to the indicated value.
@@ -124,7 +127,9 @@ class state_label_lts : public atermpp::aterm_appl
     void set_element(const mcrl2::data::data_expression& e, const size_t i)
     {
       assert(i<this->size());
-      set_argument(e,i);
+      std::vector<data::data_expression> v(this->begin(),this->end());
+      v[i]=e;
+      *this=atermpp::aterm_appl(get_STATE_function_symbol(v.size()),v.begin(),v.end());
     }
 };
 
@@ -153,8 +158,6 @@ inline std::string pp(const state_label_lts l)
     \details An action label is a multi_action. */
 class action_label_lts:public mcrl2::lps::multi_action
 {
-    friend struct atermpp::aterm_traits<action_label_lts>;
-
   public:
 
     /** \brief Default constructor. */
@@ -162,12 +165,12 @@ class action_label_lts:public mcrl2::lps::multi_action
     {}
 
     /** \brief Constructor. Sets action label to the multi_action a. */
-    action_label_lts(const ATerm a):mcrl2::lps::multi_action(a)
+    action_label_lts(const atermpp::aterm a):mcrl2::lps::multi_action(a)
     {
     }
 
     /** \brief Constructor. Transforms action label a to form a multi_action a. */
-    action_label_lts(const lps::action a):mcrl2::lps::multi_action(a)
+    action_label_lts(const process::action a):mcrl2::lps::multi_action(a)
     {
     }
 
@@ -178,34 +181,34 @@ class action_label_lts:public mcrl2::lps::multi_action
 
     /** \brief Returns this multi_action as an aterm without the time tag.
     */
-    ATerm aterm_without_time() const
+    atermpp::aterm aterm_without_time() const
     {
       if (this->has_time())
       {
         throw mcrl2::runtime_error("Cannot transform multi action " +
-                                   lps::detail::multi_action_print(*this) + " to an ATerm as it contains time.");
+                                   lps::pp(*this) + " to an atermpp::aterm as it contains time.");
       }
-      return (ATerm)mcrl2::core::detail::gsMakeMultAct(this->actions());
+      return atermpp::aterm_appl(core::detail::function_symbol_MultAct(), this->actions());
     }
 
     /** \brief Hide the actions with labels in tau_actions.
         \return Returns whether the hidden action becomes empty, i.e. a tau or hidden action.
     */
-    bool hide_actions(const std::vector<std::string> &tau_actions)
+    bool hide_actions(const std::vector<std::string>& tau_actions)
     {
       using namespace std;
       using namespace mcrl2::lps;
 
-      const action_list mas = this->actions();
-      action_list new_action_list;
-      for (action_list:: const_iterator i=mas.begin(); i!=mas.end(); ++i)
+      const process::action_list mas = this->actions();
+      process::action_list new_action_list;
+      for (process::action_list:: const_iterator i=mas.begin(); i!=mas.end(); ++i)
       {
-        const action a=*i;
+        const process::action a=*i;
 
         if (std::find(tau_actions.begin(),tau_actions.end(),
                       string(a.label().name()))==tau_actions.end())  // this action must not be hidden.
         {
-          new_action_list=push_front(new_action_list,a);
+          new_action_list.push_front(a);
         }
       }
       const bool is_tau=new_action_list.empty();
@@ -219,7 +222,7 @@ class action_label_lts:public mcrl2::lps::multi_action
 /** \brief Print the action label to string. */
 inline std::string pp(const action_label_lts l)
 {
-  return lps::detail::multi_action_print(mcrl2::lps::multi_action(l));
+  return lps::pp(mcrl2::lps::multi_action(l));
 }
 
 /** \brief Parse a string into an action label.
@@ -233,15 +236,10 @@ inline std::string pp(const action_label_lts l)
 inline action_label_lts parse_lts_action(
   const std::string& multi_action_string,
   const data::data_specification& data_spec,
-  const lps::action_list& act_decls)
+  const process::action_label_list& act_decls)
 {
-  // TODO: rewrite this cryptic code
-  ATermAppl t = mcrl2::lps::detail::multi_action_to_aterm(mcrl2::lps::parse_multi_action_new(multi_action_string));
-  lps::multi_action ma=lps::action_list((ATermList)ATgetArgument(t,0));
-  lps::type_check(ma,data_spec,act_decls);
-  lps::translate_user_notation(ma);
-  lps::normalize_sorts(ma, data_spec);
-  return action_label_lts((ATerm)mcrl2::core::detail::gsMakeMultAct(ma.actions()));
+  lps::multi_action ma = lps::parse_multi_action(multi_action_string, act_decls, data_spec);
+  return action_label_lts(atermpp::aterm_appl(core::detail::function_symbol_MultAct(), ma.actions()));
 }
 
 
@@ -262,7 +260,7 @@ class lts_lts_t : public lts< detail::state_label_lts, detail::action_label_lts
     bool m_has_valid_parameters;
     data::variable_list m_parameters;
     bool m_has_valid_action_decls;
-    lps::action_label_list m_action_decls;
+    process::action_label_list m_action_decls;
 
   public:
 
@@ -272,14 +270,11 @@ class lts_lts_t : public lts< detail::state_label_lts, detail::action_label_lts
       m_has_valid_data_spec(false),
       m_has_valid_parameters(false),
       m_has_valid_action_decls(false)
-    {
-      m_parameters.protect();
-      m_action_decls.protect();
-    };
+    {}
 
     /** \brief Creates an object containing a muCRL specification.
      * \param[in] t The muCRL specification that will be stored in the object. */
-    lts_lts_t(ATerm t);
+    lts_lts_t(const atermpp::aterm& t);
 
     /** \brief Creates an object containing an mCRL2 specification.
      * \param[in] spec The mCRL2 specification that will be stored in the object. */
@@ -294,18 +289,12 @@ class lts_lts_t : public lts< detail::state_label_lts, detail::action_label_lts
       m_parameters(l.m_parameters),
       m_has_valid_action_decls(l.m_has_valid_action_decls),
       m_action_decls(l.m_action_decls)
-    {
-      m_parameters.protect();
-      m_action_decls.protect();
-    }
+    {}
 
     /** \brief Standard destructor for the class lts_lts
     */
     ~lts_lts_t()
-    {
-      m_parameters.unprotect();
-      m_action_decls.unprotect();
-    }
+    {}
 
     /** \brief Swap function for this lts. */
     void swap(lts_lts_t& l)
@@ -338,7 +327,7 @@ class lts_lts_t : public lts< detail::state_label_lts, detail::action_label_lts
         l.m_has_valid_action_decls=aux;
       }
       {
-        const lps::action_label_list aux=m_action_decls;
+        const process::action_label_list aux=m_action_decls;
         m_action_decls=l.m_action_decls;
         l.m_action_decls=aux;
       }
@@ -367,7 +356,7 @@ class lts_lts_t : public lts< detail::state_label_lts, detail::action_label_lts
 
     /** \brief Return action label declarations stored in this LTS.
     */
-    lps::action_label_list action_labels() const
+    process::action_label_list action_labels() const
     {
       assert(m_has_valid_action_decls);
       return m_action_decls;
@@ -376,7 +365,7 @@ class lts_lts_t : public lts< detail::state_label_lts, detail::action_label_lts
     /** \brief Set the action label information for this LTS.
      * \param[in] decls  The action labels to be set in this lts.
     */
-    void set_action_labels(const lps::action_label_list& decls)
+    void set_action_labels(const process::action_label_list& decls)
     {
       m_has_valid_action_decls=true;
       m_action_decls=decls;
@@ -420,7 +409,10 @@ class lts_lts_t : public lts< detail::state_label_lts, detail::action_label_lts
     {
       assert(i<m_parameters.size());
       assert(m_has_valid_parameters);
-      return data::variable(ATelementAt(m_parameters,i));
+      data::variable_list::const_iterator j=m_parameters.begin();
+      for(size_t c=0; c!=i; ++j, ++c)
+      {}
+      return data::variable(*j);
     }
 
 
@@ -448,44 +440,4 @@ class lts_lts_t : public lts< detail::state_label_lts, detail::action_label_lts
 } // namespace lts
 } // namespace mcrl2
 
-/// \cond INTERNAL_DOCS
-namespace atermpp
-{
-template<>
-struct aterm_traits<mcrl2::lts::detail::state_label_lts>
-{
-  static void protect(const mcrl2::lts::detail::state_label_lts& t)
-  {
-    t.protect();
-  }
-  static void unprotect(const mcrl2::lts::detail::state_label_lts& t)
-  {
-    t.unprotect();
-  }
-  static void mark(const mcrl2::lts::detail::state_label_lts& t)
-  {
-    t.mark();
-  }
-};
-
-template<>
-struct aterm_traits<mcrl2::lts::detail::action_label_lts>
-{
-  static void protect(const mcrl2::lts::detail::action_label_lts& t)
-  {
-    t.protect();
-  }
-  static void unprotect(const mcrl2::lts::detail::action_label_lts& t)
-  {
-    t.unprotect();
-  }
-  static void mark(const mcrl2::lts::detail::action_label_lts& t)
-  {
-    t.mark();
-  }
-};
-} // namespace atermpp
-/// \endcond
-
-
 #endif
