@@ -6,13 +6,14 @@
 // (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
 
-#include "mcrl2/lts/detail/exploration.h"
+#include <time.h>
 
-#include "mcrl2/lts/lts_io.h"
-#include "mcrl2/trace/trace.h"
-#include "mcrl2/utilities/logger.h"
 #include "mcrl2/atermpp/aterm_balanced_tree.h"
+#include "mcrl2/utilities/logger.h"
+#include "mcrl2/lps/resolve_name_clashes.h"
 #include "mcrl2/lps/detail/instantiate_global_variables.h"
+#include "mcrl2/lts/detail/exploration.h"
+#include "mcrl2/lts/lts_io.h"
 #include <iomanip>
 
 using namespace mcrl2;
@@ -20,9 +21,9 @@ using namespace mcrl2::log;
 using namespace mcrl2::lps;
 using namespace mcrl2::lts;
 
-bool lps2lts_algorithm::initialise_lts_generation(lts_generation_options* options)
+bool lps2lts_algorithm::initialise_lts_generation(lts_generation_options *options)
 {
-  m_options = *options;
+  m_options=*options;
 
   assert(!(m_options.bithashing && m_options.outformat != lts_aut && m_options.outformat != lts_none));
 
@@ -32,7 +33,7 @@ bool lps2lts_algorithm::initialise_lts_generation(lts_generation_options* option
   }
   else
   {
-    m_state_numbers = atermpp::indexed_set(m_options.initial_table_size, 50);
+    m_state_numbers = atermpp::indexed_set<storage_state_t>(m_options.initial_table_size, 50);
   }
 
   m_num_states = 0;
@@ -43,6 +44,9 @@ bool lps2lts_algorithm::initialise_lts_generation(lts_generation_options* option
   m_maintain_traces = m_options.trace || m_options.save_error_trace;
   m_value_prioritize = (m_options.expl_strat == es_value_prioritized || m_options.expl_strat == es_value_random_prioritized);
 
+  lps::specification specification(m_options.specification);
+  resolve_summand_variable_name_clashes(specification);
+
   if (m_options.outformat == lts_aut)
   {
     mCRL2log(verbose) << "writing state space in AUT format to '" << m_options.lts << "'." << std::endl;
@@ -61,16 +65,14 @@ bool lps2lts_algorithm::initialise_lts_generation(lts_generation_options* option
   {
     mCRL2log(verbose) << "writing state space in " << mcrl2::lts::detail::string_for_type(m_options.outformat)
                       << " format to '" << m_options.lts << "'." << std::endl;
-    m_output_lts.set_data(m_options.specification.data());
-    m_output_lts.set_process_parameters(m_options.specification.process().process_parameters());
-    m_output_lts.set_action_labels(m_options.specification.action_labels());
+    m_output_lts.set_data(specification.data());
+    m_output_lts.set_process_parameters(specification.process().process_parameters());
+    m_output_lts.set_action_labels(specification.action_labels());
   }
 
-  lps::specification specification(m_options.specification);
 
   if (m_options.usedummies)
   {
-    mCRL2log(verbose) << "replacing free variables with dummy values." << std::endl;
     lps::detail::instantiate_global_variables(specification);
   }
 
@@ -138,7 +140,7 @@ bool lps2lts_algorithm::initialise_lts_generation(lts_generation_options* option
     for (size_t i = 0; i < specification.process().action_summands().size(); i++)
     {
       bool found = false;
-      for (action_list::iterator j = specification.process().action_summands()[i].multi_action().actions().begin(); j != specification.process().action_summands()[i].multi_action().actions().end(); j++)
+      for (auto j = specification.process().action_summands()[i].multi_action().actions().begin(); j != specification.process().action_summands()[i].multi_action().actions().end(); j++)
       {
         if (m_options.trace_actions.count(j->label().name()) > 0)
         {
@@ -155,25 +157,25 @@ bool lps2lts_algorithm::initialise_lts_generation(lts_generation_options* option
   {
     for (size_t i = 0; i < specification.process().action_summands().size(); i++)
     {
-      specification.process().action_summands()[i].multi_action().actions() = action_list();
+      specification.process().action_summands()[i].multi_action().actions() = process::action_list();
     }
 
     if (m_use_confluence_reduction)
     {
       for (size_t i = 0; i < nonprioritised_summands.size(); i++)
       {
-        nonprioritised_summands[i].multi_action().actions() = action_list();
+        nonprioritised_summands[i].multi_action().actions() = process::action_list();
       }
       for (size_t i = 0; i < prioritised_summands.size(); i++)
       {
-        prioritised_summands[i].multi_action().actions() = action_list();
+        prioritised_summands[i].multi_action().actions() = process::action_list();
       }
     }
   }
 
   m_generator = new next_state_generator(specification, rewriter, m_options.use_enumeration_caching, m_options.use_summand_pruning);
 
-  if (!prioritised_summands.empty())
+  if (m_use_confluence_reduction)
   {
     m_nonprioritized_subset = next_state_generator::summand_subset_t(m_generator, nonprioritised_summands, m_options.use_summand_pruning);
     m_prioritized_subset = next_state_generator::summand_subset_t(m_generator, prioritised_summands, m_options.use_summand_pruning);
@@ -199,7 +201,7 @@ bool lps2lts_algorithm::initialise_lts_generation(lts_generation_options* option
 
 bool lps2lts_algorithm::generate_lts()
 {
-  generator_state_t initial_state = m_generator->internal_initial_state();
+  storage_state_t initial_state = m_generator->initial_state();
   m_initial_state_number=0;
   if (m_use_confluence_reduction)
   {
@@ -208,13 +210,13 @@ bool lps2lts_algorithm::generate_lts()
 
   if (m_options.bithashing)
   {
-    const std::pair<size_t, bool> p=m_bit_hash_table.add_state(storage_state(initial_state));
-    m_initial_state_number=p.first; 
+    const std::pair<size_t, bool> p=m_bit_hash_table.add_state(initial_state);
+    m_initial_state_number=p.first;
     assert(p.second); // The initial state is new.
   }
   else
   {
-    m_state_numbers.put(storage_state(initial_state));
+    m_state_numbers.put(initial_state);
   }
 
   if (m_options.outformat == lts_aut)
@@ -244,7 +246,14 @@ bool lps2lts_algorithm::generate_lts()
     }
     else
     {
-      generate_lts_breadth();
+      if (m_options.todo_max==std::string::npos)
+      { 
+        generate_lts_breadth_todo_max_is_npos();
+      }
+      else 
+      { 
+        generate_lts_breadth_todo_max_larger_than_0(initial_state);
+      }
     }
 
     mCRL2log(verbose) << "done with state space generation ("
@@ -262,7 +271,14 @@ bool lps2lts_algorithm::generate_lts()
   }
   else if (m_options.expl_strat == es_random || m_options.expl_strat == es_value_random_prioritized)
   {
+    // In order to guarantee that time is different from a previous run to initialise
+    // the seed of a previous run and not get the same results, we wait until time gets
+    // a fresh value. It is better to replace this by c++11 random generators, as this
+    // is an awkward solution.
+    unsigned t=time(NULL);
+    for( ; t==time(NULL) ; ); // Wait until time changes.
     srand((unsigned)time(NULL));
+
     generate_lts_random(initial_state);
 
     mCRL2log(verbose) << "done with random walk of "
@@ -333,42 +349,20 @@ bool lps2lts_algorithm::finalise_lts_generation()
   return true;
 }
 
-lps2lts_algorithm::generator_state_t lps2lts_algorithm::generator_state(lps2lts_algorithm::storage_state_t storage_state)
-{
-  if (m_options.stateformat == lps::GS_STATE_VECTOR)
-  {
-    return storage_state;
-  }
-  else
-  {
-    atermpp::term_balanced_tree<atermpp::aterm_appl> tree(storage_state);
-    return generator_state_t(m_generator->internal_state_function(), tree.begin(), tree.end());
-  }
-}
-
-lps2lts_algorithm::storage_state_t lps2lts_algorithm::storage_state(lps2lts_algorithm::generator_state_t generator_state)
-{
-  if (m_options.stateformat == lps::GS_STATE_VECTOR)
-  {
-    return generator_state;
-  }
-  else
-  {
-    return atermpp::term_balanced_tree<atermpp::aterm_appl>(generator_state.begin(), generator_state.end());
-  }
-}
-
 // Confluence reduction based on S.C.C. Blom, Partial tau-confluence for
 // Efficient State Space Generation, Technical Report SEN-R0123, CWI, Amsterdam, 2001
 
-lps2lts_algorithm::generator_state_t lps2lts_algorithm::get_prioritised_representative(lps2lts_algorithm::generator_state_t state)
+lps2lts_algorithm::storage_state_t lps2lts_algorithm::get_prioritised_representative(const storage_state_t& state1)
 {
+  storage_state_t state=state1;
   assert(m_use_confluence_reduction);
 
-  atermpp::map<generator_state_t, size_t> number;
-  atermpp::map<generator_state_t, size_t> low;
-  atermpp::map<generator_state_t, atermpp::list<generator_state_t> > next;
-  atermpp::map<generator_state_t, generator_state_t> back;
+  std::map<storage_state_t, size_t> number;
+  std::map<storage_state_t, size_t> low;
+  std::map<storage_state_t, std::list<storage_state_t> > next;
+  std::map<storage_state_t, storage_state_t> back;
+  next_state_generator::enumerator_queue_t enumeration_queue;
+
   size_t count = 0;
   number[state] = 0;
 
@@ -380,14 +374,16 @@ lps2lts_algorithm::generator_state_t lps2lts_algorithm::get_prioritised_represen
       count++;
       number[state] = count;
       low[state] = count;
-      next[state] = atermpp::list<generator_state_t>();
+      next[state] = std::list<storage_state_t>();
 
-      for (next_state_generator::iterator i = m_generator->begin(state, &m_substitution, m_prioritized_subset); i; i++)
+      enumeration_queue.clear();
+      for (next_state_generator::iterator i = m_generator->begin(state, m_prioritized_subset, &enumeration_queue); i; i++)
       {
-        next[state].push_back(i->internal_state());
-        if (number.count(i->internal_state()) == 0)
+        const storage_state_t s=i->internal_state();
+        next[state].push_back(s);
+        if (number.count(s) == 0)
         {
-          number[i->internal_state()] = 0;
+          number[s] = 0;
         }
       }
     }
@@ -400,13 +396,13 @@ lps2lts_algorithm::generator_state_t lps2lts_algorithm::get_prioritised_represen
         return state;
       }
       assert(back.count(state) > 0);
-      generator_state_t back_state = back[state];
+      storage_state_t back_state = back[state];
       low[back_state] = low[back_state] < low[state] ? low[back_state] : low[state];
       state = back_state;
     }
     else
     {
-      generator_state_t next_state = next[state].front();
+      storage_state_t next_state = next[state].front();
       next[state].pop_front();
       if (number[next_state] == 0)
       {
@@ -421,24 +417,24 @@ lps2lts_algorithm::generator_state_t lps2lts_algorithm::get_prioritised_represen
   }
 }
 
-void lps2lts_algorithm::value_prioritize(atermpp::list<next_state_generator::transition_t> &transitions)
+void lps2lts_algorithm::value_prioritize(std::vector<next_state_generator::transition_t>& transitions)
 {
   data::data_expression lowest_value;
 
-  for (atermpp::list<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
+  for (std::vector<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
   {
     if (i->action().actions().size() == 1 && i->action().actions().front().arguments().size() > 0)
     {
-      const data::data_expression &argument = i->action().actions().front().arguments().front();
+      const data::data_expression& argument = i->action().actions().front().arguments().front();
       if (mcrl2::data::sort_nat::is_nat(argument.sort()))
       {
-        if (lowest_value == data::data_expression())
+        if (lowest_value==data::data_expression()) // lowest value is undefined.
         {
           lowest_value = argument;
         }
         else
         {
-          data::data_expression result = m_generator->get_rewriter()(data::greater(lowest_value, argument));
+          const data::data_expression result = m_generator->get_rewriter()(data::greater(lowest_value, argument));
 
           if (data::sort_bool::is_true_function_symbol(result))
           {
@@ -453,19 +449,20 @@ void lps2lts_algorithm::value_prioritize(atermpp::list<next_state_generator::tra
     }
   }
 
-  for (atermpp::list<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end();)
+  size_t new_position=0;
+  for (std::vector<next_state_generator::transition_t>::const_iterator i = transitions.begin(); i != transitions.end(); ++i)
   {
     if (i->action().actions().size() != 1)
     {
-      i++;
+      transitions[new_position++]= *i;
     }
     else if (i->action().actions().front().arguments().size() == 0)
     {
-      i++;
+      transitions[new_position++]= *i;
     }
     else if (!mcrl2::data::sort_nat::is_nat(i->action().actions().front().arguments().front().sort()))
     {
-      i++;
+      transitions[new_position++]= *i;
     }
     else
     {
@@ -473,11 +470,11 @@ void lps2lts_algorithm::value_prioritize(atermpp::list<next_state_generator::tra
 
       if (data::sort_bool::is_true_function_symbol(result))
       {
-        i++;
+        transitions[new_position++]= *i;
       }
       else if (data::sort_bool::is_false_function_symbol(result))
       {
-        i = transitions.erase(i);
+        // Skip the transition at position i.
       }
       else
       {
@@ -485,25 +482,27 @@ void lps2lts_algorithm::value_prioritize(atermpp::list<next_state_generator::tra
       }
     }
   }
+  transitions.resize(new_position);
 }
 
-bool lps2lts_algorithm::save_trace(lps2lts_algorithm::generator_state_t state, std::string filename)
+void lps2lts_algorithm::construct_trace(const storage_state_t& state1, mcrl2::trace::Trace& trace)
 {
-  atermpp::deque<generator_state_t> states;
-  atermpp::map<storage_state_t, storage_state_t>::iterator source;
-  while ((source = m_backpointers.find(storage_state(state))) != m_backpointers.end())
+  storage_state_t state=state1;
+  std::deque<storage_state_t> states;
+  std::map<storage_state_t, storage_state_t>::iterator source;
+  while ((source = m_backpointers.find(state)) != m_backpointers.end())
   {
     states.push_front(state);
-    state = generator_state(source->second);
+    state = source->second;
   }
 
-  mcrl2::trace::Trace trace;
-  trace.setState(m_generator->get_state(state));
-  for (atermpp::deque<generator_state_t>::iterator i = states.begin(); i != states.end(); i++)
+  trace.setState(state);
+  next_state_generator::enumerator_queue_t enumeration_queue;
+  for (std::deque<storage_state_t>::iterator i = states.begin(); i != states.end(); i++)
   {
-    for (next_state_generator::iterator j = m_generator->begin(state, &m_substitution); j != m_generator->end(); j++)
+    for (next_state_generator::iterator j = m_generator->begin(state, &enumeration_queue); j != m_generator->end(); j++)
     {
-      generator_state_t destination = j->internal_state();
+      storage_state_t destination = j->internal_state();
       if (m_use_confluence_reduction)
       {
         destination = get_prioritised_representative(destination);
@@ -514,10 +513,18 @@ bool lps2lts_algorithm::save_trace(lps2lts_algorithm::generator_state_t state, s
         break;
       }
     }
+    enumeration_queue.clear();
     state = *i;
-    trace.setState(m_generator->get_state(state));
+    trace.setState(state);
   }
 
+}
+
+// Contruct a trace to state1 and store in in filename.
+bool lps2lts_algorithm::save_trace(const storage_state_t& state1, const std::string& filename)
+{
+  mcrl2::trace::Trace trace;
+  lps2lts_algorithm::construct_trace(state1, trace);
   m_traces_saved++;
 
   try
@@ -531,12 +538,39 @@ bool lps2lts_algorithm::save_trace(lps2lts_algorithm::generator_state_t state, s
   }
 }
 
-bool lps2lts_algorithm::search_divergence(lps2lts_algorithm::generator_state_t state, std::set<lps2lts_algorithm::generator_state_t> &current_path, atermpp::set<lps2lts_algorithm::generator_state_t> &visited)
+// Contruct a trace to state1, then add transition to it and store in in filename.
+bool lps2lts_algorithm::save_trace(const storage_state_t& state1, const next_state_generator::transition_t& transition, const std::string& filename)
+{
+  mcrl2::trace::Trace trace;
+  lps2lts_algorithm::construct_trace(state1, trace);
+  trace.addAction(transition.action());
+  trace.setState(transition.internal_state());
+  m_traces_saved++;
+
+  try
+  {
+    trace.save(filename);
+    return true;
+  }
+  catch(...)
+  {
+    return false;
+  }
+}
+
+
+
+
+bool lps2lts_algorithm::search_divergence(
+              const storage_state_t& state,
+              std::set<storage_state_t>& current_path,
+              std::set<storage_state_t>& visited)
 {
   current_path.insert(state);
 
-  std::vector<generator_state_t> new_states;
-  for (next_state_generator::iterator j = m_generator->begin(state, &m_substitution, m_tau_summands); j != m_generator->end(); j++)
+  std::vector<storage_state_t> new_states;
+  next_state_generator::enumerator_queue_t enumeration_queue;
+  for (next_state_generator::iterator j = m_generator->begin(state, m_tau_summands, &enumeration_queue); j != m_generator->end(); j++)
   {
     assert(j->action().actions().size() == 0);
 
@@ -544,13 +578,13 @@ bool lps2lts_algorithm::search_divergence(lps2lts_algorithm::generator_state_t s
     {
       new_states.push_back(j->internal_state());
     }
-    else if (visited.count(j->internal_state()) != 0)
+    else if (current_path.count(j->internal_state()) != 0)
     {
       return true;
     }
   }
 
-  for (std::vector<generator_state_t>::iterator i = new_states.begin(); i != new_states.end(); i++)
+  for (std::vector<storage_state_t>::iterator i = new_states.begin(); i != new_states.end(); i++)
   {
     if (search_divergence(*i, current_path, visited))
     {
@@ -558,24 +592,25 @@ bool lps2lts_algorithm::search_divergence(lps2lts_algorithm::generator_state_t s
     }
   }
 
+  assert(current_path.count(state)==1);
   current_path.erase(state);
   return false;
 }
 
-void lps2lts_algorithm::check_divergence(lps2lts_algorithm::generator_state_t state)
+void lps2lts_algorithm::check_divergence(const storage_state_t& state)
 {
-  atermpp::set<generator_state_t> visited;
-  std::set<generator_state_t> current_path;
+  std::set<storage_state_t> visited;
+  std::set<storage_state_t> current_path;
   visited.insert(state);
 
   if (search_divergence(state, current_path, visited))
   {
-    size_t state_number = m_state_numbers.index(storage_state(state));
+    size_t state_number = m_state_numbers.index(state);
     if (m_options.trace && m_traces_saved < m_options.max_traces)
     {
-      std::ostringstream reason;
-      reason << "divergence_" << m_traces_saved;
-      std::string filename = m_options.generate_filename_for_trace(m_options.trace_prefix, reason.str(), "trc");
+      std::ostringstream filename_stream;
+      filename_stream << "divergence_" << m_traces_saved;
+      std::string filename = m_options.generate_filename_for_trace(m_options.trace_prefix, filename_stream.str(), "trc");
       if (save_trace(state, filename))
       {
         mCRL2log(info) << "divergence-detect: divergence found and saved to '" << filename
@@ -594,27 +629,27 @@ void lps2lts_algorithm::check_divergence(lps2lts_algorithm::generator_state_t st
   }
 }
 
-void lps2lts_algorithm::save_actions(lps2lts_algorithm::generator_state_t state, const next_state_generator::transition_t &transition)
+void lps2lts_algorithm::save_actions(const storage_state_t& state, const next_state_generator::transition_t& transition)
 {
-  size_t state_number = m_state_numbers.index(storage_state(state));
+  size_t state_number = m_state_numbers.index(state);
   mCRL2log(info) << "Detected action '" << pp(transition.action()) << "' (state index " << state_number << ")";
   if (m_options.trace && m_traces_saved < m_options.max_traces)
   {
-    std::ostringstream reason;
-    reason << "act_" << m_traces_saved;
+    std::ostringstream filename_stream;
+    filename_stream << "act_" << m_traces_saved;
     if (m_options.trace_multiactions.find(transition.action()) != m_options.trace_multiactions.end())
     {
-      reason << "_" << pp(transition.action());
+      filename_stream << "_" << pp(transition.action());
     }
-    for (action_list::iterator i = transition.action().actions().begin(); i != transition.action().actions().end(); i++)
+    for (process::action_list::const_iterator i = transition.action().actions().begin(); i != transition.action().actions().end(); i++)
     {
       if (m_options.trace_actions.count(i->label().name()) > 0)
       {
-        reason << "_" << i->label().name();
+        filename_stream << "_" << i->label().name();
       }
     }
-    std::string filename = m_options.generate_filename_for_trace(m_options.trace_prefix, reason.str(), "trc");
-    if (save_trace(transition.internal_state(), filename))
+    std::string filename = m_options.generate_filename_for_trace(m_options.trace_prefix, filename_stream.str(), "trc");
+    if (save_trace(state,transition, filename))
       mCRL2log(info) << " and saved to '" << filename << "'";
     else
       mCRL2log(info) << " but could not saved to '" << filename << "'";
@@ -622,14 +657,14 @@ void lps2lts_algorithm::save_actions(lps2lts_algorithm::generator_state_t state,
   mCRL2log(info) << std::endl;
 }
 
-void lps2lts_algorithm::save_deadlock(lps2lts_algorithm::generator_state_t state)
+void lps2lts_algorithm::save_deadlock(const storage_state_t& state)
 {
-  size_t state_number = m_state_numbers.index(storage_state(state));
+  size_t state_number = m_state_numbers.index(state);
   if (m_options.trace && m_traces_saved < m_options.max_traces)
   {
-    std::ostringstream reason;
-    reason << "dlk_" << m_traces_saved;
-    std::string filename = m_options.generate_filename_for_trace(m_options.trace_prefix, reason.str(), "trc");
+    std::ostringstream filename_stream;
+    filename_stream << "dlk_" << m_traces_saved;
+    std::string filename = m_options.generate_filename_for_trace(m_options.trace_prefix, filename_stream.str(), "trc");
     if (save_trace(state, filename))
     {
       mCRL2log(verbose) << "deadlock-detect: deadlock found and saved to '" << filename
@@ -647,7 +682,7 @@ void lps2lts_algorithm::save_deadlock(lps2lts_algorithm::generator_state_t state
   }
 }
 
-void lps2lts_algorithm::save_error(lps2lts_algorithm::generator_state_t state)
+void lps2lts_algorithm::save_error(const storage_state_t& state)
 {
   if (m_options.save_error_trace)
   {
@@ -663,21 +698,20 @@ void lps2lts_algorithm::save_error(lps2lts_algorithm::generator_state_t state)
   }
 }
 
-bool lps2lts_algorithm::add_transition(lps2lts_algorithm::generator_state_t state, const next_state_generator::transition_t &transition)
+bool lps2lts_algorithm::add_transition(const storage_state_t& source_state, next_state_generator::transition_t& transition)
 {
-  storage_state_t source = storage_state(state);
-  storage_state_t destination = storage_state(transition.internal_state());
+  storage_state_t destination = transition.internal_state();
 
   size_t source_state_number;
   std::pair<size_t, bool> destination_state_number;
   if (m_options.bithashing)
   {
-    source_state_number = m_bit_hash_table.add_state(source).first;
+    source_state_number = m_bit_hash_table.add_state(source_state).first;
     destination_state_number = m_bit_hash_table.add_state(destination);
   }
   else
   {
-    source_state_number = m_state_numbers[source];
+    source_state_number = m_state_numbers[source_state];
     destination_state_number = m_state_numbers.put(destination);
   }
   if (destination_state_number.second)
@@ -686,7 +720,7 @@ bool lps2lts_algorithm::add_transition(lps2lts_algorithm::generator_state_t stat
     if (m_maintain_traces)
     {
       assert(m_backpointers.count(destination) == 0);
-      m_backpointers[destination] = source;
+      m_backpointers[destination] = source_state;
     }
 
     if (m_options.outformat != lts_none && m_options.outformat != lts_aut)
@@ -700,7 +734,7 @@ bool lps2lts_algorithm::add_transition(lps2lts_algorithm::generator_state_t stat
 
   if (m_options.detect_action && m_detected_action_summands[transition.summand_index()])
   {
-    save_actions(state, transition);
+    save_actions(source_state, transition);
   }
 
   if (m_options.outformat == lts_aut)
@@ -722,33 +756,32 @@ bool lps2lts_algorithm::add_transition(lps2lts_algorithm::generator_state_t stat
 
   m_num_transitions++;
 
-  for (atermpp::set<lps::multi_action>::const_iterator ma = m_options.trace_multiactions.begin(); ma != m_options.trace_multiactions.end(); ++ma)
+  for (std::set<lps::multi_action>::const_iterator ma = m_options.trace_multiactions.begin(); ma != m_options.trace_multiactions.end(); ++ma)
   {
     if (*ma == transition.action())
     {
-      save_actions(state, transition);
+      save_actions(source_state, transition);
     }
   }
 
-  if (m_options.detect_action && m_detected_action_summands[transition.summand_index()])
-  {
-    save_actions(state, transition);
-  }
-
   return destination_state_number.second;
 }
 
-atermpp::list<lps2lts_algorithm::next_state_generator::transition_t> lps2lts_algorithm::get_transitions(lps2lts_algorithm::generator_state_t state)
+void lps2lts_algorithm::get_transitions(const storage_state_t& state,
+                                        std::vector<lps2lts_algorithm::next_state_generator::transition_t>& transitions,
+                                        next_state_generator::enumerator_queue_t& enumeration_queue
+                                       )
 {
+  assert(transitions.empty());
   if (m_options.detect_divergence)
   {
     check_divergence(state);
   }
 
-  atermpp::list<next_state_generator::transition_t> transitions;
   try
   {
-    next_state_generator::iterator it(m_generator->begin(state, &m_substitution, *m_main_subset));
+    enumeration_queue.clear();
+    next_state_generator::iterator it(m_generator->begin(state, *m_main_subset, &enumeration_queue));
     while (it)
     {
       transitions.push_back(*it++);
@@ -756,7 +789,7 @@ atermpp::list<lps2lts_algorithm::next_state_generator::transition_t> lps2lts_alg
   }
   catch (mcrl2::runtime_error& e)
   {
-    std::cerr << "Error while exploring state space: " << e.what() << "\n";
+    mCRL2log(error) << "Error while exploring state space: " << e.what() << "\n";
     save_error(state);
     exit(EXIT_FAILURE);
   }
@@ -773,31 +806,34 @@ atermpp::list<lps2lts_algorithm::next_state_generator::transition_t> lps2lts_alg
 
   if (m_use_confluence_reduction)
   {
-    for (atermpp::list<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
+    for (std::vector<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
     {
       i->internal_state() = get_prioritised_representative(i->internal_state());
     }
   }
-
-  return transitions;
 }
 
-void lps2lts_algorithm::generate_lts_breadth()
+void lps2lts_algorithm::generate_lts_breadth_todo_max_is_npos()
 {
+  assert(m_options.todo_max==std::string::npos);
   size_t current_state = 0;
   size_t start_level_seen = 1;
   size_t start_level_transitions = 0;
+  std::vector<next_state_generator::transition_t> transitions;
   time_t last_log_time = time(NULL) - 1, new_log_time;
+  next_state_generator::enumerator_queue_t enumeration_queue;
 
-  while (!m_must_abort && (current_state < m_state_numbers.size()) && (current_state < m_options.max_states) && (!m_options.trace || m_traces_saved < m_options.max_traces))
+  while (!m_must_abort && (current_state < m_state_numbers.size()) &&
+         (current_state < m_options.max_states) && (!m_options.trace || m_traces_saved < m_options.max_traces))
   {
-    generator_state_t state = generator_state(m_state_numbers.get(current_state));
-    atermpp::list<next_state_generator::transition_t> transitions = get_transitions(state);
+    storage_state_t state=m_state_numbers.get(current_state);
+    get_transitions(state,transitions, enumeration_queue);
 
-    for (atermpp::list<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
+    for (std::vector<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
     {
       add_transition(state, *i);
     }
+    transitions.clear();
 
     current_state++;
     if (current_state == start_level_seen)
@@ -819,9 +855,91 @@ void lps2lts_algorithm::generate_lts_breadth()
                        << "tr.\n";
     }
   }
+
+  if (current_state == m_options.max_states)
+  {
+    mCRL2log(verbose) << "explored the maximum number (" << m_options.max_states << ") of states, terminating." << std::endl;
+  }
 }
 
-void lps2lts_algorithm::generate_lts_breadth_bithashing(generator_state_t initial_state)
+void lps2lts_algorithm::generate_lts_breadth_todo_max_larger_than_0(const storage_state_t& initial_state)
+{
+  assert(m_options.todo_max!=std::string::npos);
+  size_t current_state = 0;
+  size_t start_level_seen = 1;
+  size_t start_level_explored = 0;
+  size_t start_level_transitions = 0;
+  time_t last_log_time = time(NULL) - 1, new_log_time;
+
+  queue<storage_state_t> state_queue;
+  state_queue.set_max_size(m_options.max_states < m_options.todo_max ? m_options.max_states : m_options.todo_max);
+  state_queue.add_to_queue(initial_state);
+  state_queue.swap_queues();
+  std::vector<next_state_generator::transition_t> transitions;
+  next_state_generator::enumerator_queue_t enumeration_queue;
+
+  while (!m_must_abort && (state_queue.remaining() > 0) &&
+         (current_state < m_options.max_states) && (!m_options.trace || m_traces_saved < m_options.max_traces))
+  {
+    const storage_state_t state=state_queue.get_from_queue();
+    get_transitions(state,transitions, enumeration_queue);
+
+    for (std::vector<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
+    {
+      if (add_transition(state, *i))
+      {
+        storage_state_t removed = state_queue.add_to_queue(i->internal_state());
+        if (removed != storage_state_t())
+        {
+          m_num_states--;
+        }
+      }
+    }
+    transitions.clear();
+
+    if (state_queue.remaining() == 0)
+    {
+      state_queue.swap_queues();
+    }
+
+    current_state++;
+    if (current_state == start_level_seen)
+    {
+      if (!m_options.suppress_progress_messages)
+      {
+        mCRL2log(verbose) << "monitor: level " << m_level << " done."
+                          << " (" << (current_state - start_level_explored) << " state"
+                          << ((current_state - start_level_explored)==1?"":"s") << ", "
+                          << (m_num_transitions - start_level_transitions) << " transition"
+                          << ((m_num_transitions - start_level_transitions)==1?")\n":"s)\n");
+      }
+
+      m_level++;
+      start_level_seen = m_num_states;
+      start_level_explored = current_state;
+      start_level_transitions = m_num_transitions;
+    }
+
+    if (!m_options.suppress_progress_messages && time(&new_log_time) > last_log_time)
+    {
+      last_log_time = new_log_time;
+      size_t lvl_states = m_num_states - start_level_seen;
+      size_t lvl_transitions = m_num_transitions - start_level_transitions;
+      mCRL2log(status) << std::fixed << std::setprecision(2)
+                       << m_num_states << "st, " << m_num_transitions << "tr"
+                       << ", explored " << 100.0 * ((float)current_state / m_num_states)
+                       << "%. Last level: " << m_level << ", " << lvl_states << "st, " << lvl_transitions
+                       << "tr.\n";
+    }
+  }
+
+  if (current_state == m_options.max_states)
+  {
+    mCRL2log(verbose) << "explored the maximum number (" << m_options.max_states << ") of states, terminating." << std::endl;
+  }
+}
+
+void lps2lts_algorithm::generate_lts_breadth_bithashing(const storage_state_t& initial_state)
 {
   size_t current_state = 0;
 
@@ -831,19 +949,21 @@ void lps2lts_algorithm::generate_lts_breadth_bithashing(generator_state_t initia
 
   queue<storage_state_t> state_queue;
   state_queue.set_max_size(m_options.max_states < m_options.todo_max ? m_options.max_states : m_options.todo_max);
-  state_queue.add_to_queue(storage_state(initial_state));
+  state_queue.add_to_queue(initial_state);
   state_queue.swap_queues();
+  std::vector<next_state_generator::transition_t> transitions;
+  next_state_generator::enumerator_queue_t enumeration_queue;
 
   while (!m_must_abort && (state_queue.remaining() > 0) && (current_state < m_options.max_states) && (!m_options.trace || m_traces_saved < m_options.max_traces))
   {
-    generator_state_t state = generator_state(state_queue.get_from_queue());
-    atermpp::list<next_state_generator::transition_t> transitions = get_transitions(state);
+    const storage_state_t state=state_queue.get_from_queue();
+    get_transitions(state,transitions, enumeration_queue);
 
-    for (atermpp::list<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
+    for (std::vector<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
     {
       if (add_transition(state, *i))
       {
-        storage_state_t removed = state_queue.add_to_queue(storage_state(i->internal_state()));
+        storage_state_t removed = state_queue.add_to_queue(i->internal_state());
         if (removed != storage_state_t())
         {
           m_bit_hash_table.remove_state_from_bithash(removed);
@@ -851,6 +971,7 @@ void lps2lts_algorithm::generate_lts_breadth_bithashing(generator_state_t initia
         }
       }
     }
+    transitions.clear();
 
     if (state_queue.remaining() == 0)
     {
@@ -883,28 +1004,36 @@ void lps2lts_algorithm::generate_lts_breadth_bithashing(generator_state_t initia
       start_level_transitions = m_num_transitions;
     }
   }
+
+  if (current_state == m_options.max_states)
+  {
+    mCRL2log(verbose) << "explored the maximum number (" << m_options.max_states << ") of states, terminating." << std::endl;
+  }
 }
 
-void lps2lts_algorithm::generate_lts_depth(generator_state_t initial_state)
+void lps2lts_algorithm::generate_lts_depth(const storage_state_t& initial_state)
 {
-  atermpp::list<storage_state_t> stack;
-  stack.push_back(storage_state(initial_state));
+  std::list<storage_state_t> stack;
+  stack.push_back(initial_state);
+  std::vector<next_state_generator::transition_t> transitions;
 
   size_t current_state = 0;
+  next_state_generator::enumerator_queue_t enumeration_queue;
 
   while (!m_must_abort && (!stack.empty()) && (!m_options.trace || m_traces_saved < m_options.max_traces))
   {
-    generator_state_t state = generator_state(stack.back());
+    const storage_state_t state=stack.back();
     stack.pop_back();
-    atermpp::list<next_state_generator::transition_t> transitions = get_transitions(state);
+    get_transitions(state,transitions, enumeration_queue);
 
-    for (atermpp::list<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
+    for (std::vector<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
     {
       if (add_transition(state, *i) && (current_state + stack.size() < m_options.max_states) && (stack.size() < m_options.todo_max))
       {
-        stack.push_back(storage_state(i->internal_state()));
+        stack.push_back(i->internal_state());
       }
     }
+    transitions.clear();
 
     current_state++;
     if (!m_options.suppress_progress_messages && ((current_state % 1000) == 0))
@@ -915,17 +1044,23 @@ void lps2lts_algorithm::generate_lts_depth(generator_state_t initial_state)
                         << " (stacksize is " << stack.size() << ")" << std::endl;
     }
   }
+
+  if (current_state == m_options.max_states)
+  {
+    mCRL2log(verbose) << "explored the maximum number (" << m_options.max_states << ") of states, terminating." << std::endl;
+  }
 }
 
-void lps2lts_algorithm::generate_lts_random(generator_state_t initial_state)
+void lps2lts_algorithm::generate_lts_random(const storage_state_t& initial_state)
 {
-  generator_state_t state = initial_state;
-
+  storage_state_t state = initial_state;
+  std::vector<next_state_generator::transition_t> transitions;
   size_t current_state = 0;
+  next_state_generator::enumerator_queue_t enumeration_queue;
 
   while (!m_must_abort && current_state < m_options.max_states && (!m_options.trace || m_traces_saved < m_options.max_traces))
   {
-    atermpp::list<next_state_generator::transition_t> transitions = get_transitions(state);
+    get_transitions(state,transitions, enumeration_queue);
 
     if (transitions.empty())
     {
@@ -933,9 +1068,9 @@ void lps2lts_algorithm::generate_lts_random(generator_state_t initial_state)
     }
 
     size_t index = rand() % transitions.size();
-    generator_state_t new_state;
+    storage_state_t new_state;
 
-    for (atermpp::list<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
+    for (std::vector<next_state_generator::transition_t>::iterator i = transitions.begin(); i != transitions.end(); i++)
     {
       add_transition(state, *i);
 
@@ -944,6 +1079,7 @@ void lps2lts_algorithm::generate_lts_random(generator_state_t initial_state)
         new_state = i->internal_state();
       }
     }
+    transitions.clear();
 
     state = new_state;
 
@@ -955,4 +1091,9 @@ void lps2lts_algorithm::generate_lts_random(generator_state_t initial_state)
                         << " and encountered " << m_num_states << " unique state" << ((m_num_states==1)?"":"s") << std::endl;
     }
   }
+
+  if (current_state == m_options.max_states)
+  {
+    mCRL2log(verbose) << "explored the maximum number (" << m_options.max_states << ") of states, terminating." << std::endl;
+  }
 }
