@@ -19,8 +19,6 @@
 #include <sstream>
 #include <cstdio>
 #include <boost/test/included/unit_test_framework.hpp>
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/lps/parse.h"
 #include "mcrl2/lts/detail/exploration.h"
@@ -43,30 +41,24 @@ template <class LTS_TYPE>
 LTS_TYPE translate_lps_to_lts(lps::specification const& specification,
                               lts::exploration_strategy const strategy = lts::es_breadth,
                               mcrl2::data::rewrite_strategy const rewrite_strategy = mcrl2::data::jitty,
-                              NextStateFormat format = GS_STATE_VECTOR,
                               std::string priority_action = "")
 {
-  std::clog << "Translating LPS to LTS with exploration strategy " << strategy << ", rewrite strategy " << rewrite_strategy << ", and state format " << format << "." << std::endl;
+  std::clog << "Translating LPS to LTS with exploration strategy " << strategy << ", rewrite strategy " << rewrite_strategy << "." << std::endl;
   lts::lts_generation_options options;
   options.trace_prefix = "lps2lts_test";
   options.specification = specification;
   options.priority_action = priority_action;
   options.strat = rewrite_strategy;
   options.expl_strat = strategy;
-  options.stateformat = format;
 
   options.lts = utilities::temporary_filename("lps2lts_test_file");
 
   LTS_TYPE result;
   options.outformat = result.type();
   lts::lps2lts_algorithm lps2lts;
-  core::garbage_collect();
   lps2lts.initialise_lts_generation(&options);
-  core::garbage_collect();
   lps2lts.generate_lts();
-  core::garbage_collect();
   lps2lts.finalise_lts_generation();
-  core::garbage_collect();
 
   result.load(options.lts);
   remove(options.lts.c_str()); // Clean up after ourselves
@@ -98,25 +90,6 @@ exploration_strategy_vector exploration_strategies()
   return exploration_strategies;
 }
 
-// State formats to be tested;
-typedef std::vector< NextStateFormat > nextstate_format_vector;
-
-static inline
-nextstate_format_vector initialise_nextstate_formats()
-{
-  nextstate_format_vector result;
-  result.push_back(GS_STATE_VECTOR);
-  result.push_back(GS_STATE_TREE);
-  return result;
-}
-
-static inline
-nextstate_format_vector nextstate_formats()
-{
-  static nextstate_format_vector nextstate_formats = initialise_nextstate_formats();
-  return nextstate_formats;
-}
-
 static void check_lps2lts_specification(std::string const& specification,
                                  const size_t expected_states,
                                  const size_t expected_transitions,
@@ -132,51 +105,36 @@ static void check_lps2lts_specification(std::string const& specification,
     exploration_strategy_vector estrategies(exploration_strategies());
     for (exploration_strategy_vector::const_iterator expl_strategy = estrategies.begin(); expl_strategy != estrategies.end(); ++expl_strategy)
     {
-      nextstate_format_vector nsformats(nextstate_formats());
-      for (nextstate_format_vector::const_iterator state_format = nsformats.begin(); state_format != nsformats.end(); ++state_format)
-      {
-        std::cerr << "AUT FORMAT\n";
-        lts::lts_aut_t result1 = translate_lps_to_lts<lts::lts_aut_t>(lps, *expl_strategy, *rewr_strategy, *state_format, priority_action);
+      std::cerr << "AUT FORMAT\n";
+      lts::lts_aut_t result1 = translate_lps_to_lts<lts::lts_aut_t>(lps, *expl_strategy, *rewr_strategy, priority_action);
 
-        core::garbage_collect();
 
-        BOOST_CHECK_EQUAL(result1.num_states(), expected_states);
-        BOOST_CHECK_EQUAL(result1.num_transitions(), expected_transitions);
-        BOOST_CHECK_EQUAL(result1.num_action_labels(), expected_labels);
+      BOOST_CHECK_EQUAL(result1.num_states(), expected_states);
+      BOOST_CHECK_EQUAL(result1.num_transitions(), expected_transitions);
+      BOOST_CHECK_EQUAL(result1.num_action_labels(), expected_labels);
 
-        std::cerr << "LTS FORMAT\n";
-        lts::lts_lts_t result2 = translate_lps_to_lts<lts::lts_lts_t>(lps, *expl_strategy, *rewr_strategy, *state_format, priority_action);
+      std::cerr << "LTS FORMAT\n";
+      lts::lts_lts_t result2 = translate_lps_to_lts<lts::lts_lts_t>(lps, *expl_strategy, *rewr_strategy, priority_action);
 
-        core::garbage_collect();
 
-        BOOST_CHECK_EQUAL(result2.num_states(), expected_states);
-        BOOST_CHECK_EQUAL(result2.num_transitions(), expected_transitions);
-        BOOST_CHECK_EQUAL(result2.num_action_labels(), expected_labels);
+      BOOST_CHECK_EQUAL(result2.num_states(), expected_states);
+      BOOST_CHECK_EQUAL(result2.num_transitions(), expected_transitions);
+      BOOST_CHECK_EQUAL(result2.num_action_labels(), expected_labels);
 
-        std::cerr << "FSM FORMAT\n";
-        lts::lts_fsm_t result3 = translate_lps_to_lts<lts::lts_fsm_t>(lps, *expl_strategy, *rewr_strategy, *state_format, priority_action);
+      std::cerr << "FSM FORMAT\n";
+      lts::lts_fsm_t result3 = translate_lps_to_lts<lts::lts_fsm_t>(lps, *expl_strategy, *rewr_strategy, priority_action);
 
-        core::garbage_collect();
 
-        BOOST_CHECK_EQUAL(result3.num_states(), expected_states);
-        BOOST_CHECK_EQUAL(result3.num_transitions(), expected_transitions);
-        BOOST_CHECK_EQUAL(result3.num_action_labels(), expected_labels);
-
-        std::cerr << "DOT FORMAT\n";
-        lts::lts_dot_t result4 = translate_lps_to_lts<lts::lts_dot_t>(lps, *expl_strategy, *rewr_strategy, *state_format, priority_action);
-
-        core::garbage_collect();
-
-        BOOST_CHECK_EQUAL(result4.num_states(), expected_states);
-        BOOST_CHECK_EQUAL(result4.num_transitions(), expected_transitions);
-        BOOST_CHECK_EQUAL(result4.num_action_labels(), expected_labels);
+      BOOST_CHECK_EQUAL(result3.num_states(), expected_states);
+      BOOST_CHECK_EQUAL(result3.num_transitions(), expected_transitions);
+      BOOST_CHECK_EQUAL(result3.num_action_labels(), expected_labels);
 
 #ifdef USE_BCG
-        lts::lts_bcg_t result6 = translate_lps_to_lts<lts::lts_bcg_t>(lps, *expl_strategy, *rewr_strategy, *state_format, priority_action);
+      lts::lts_bcg_t result6 = translate_lps_to_lts<lts::lts_bcg_t>(lps, *expl_strategy, *rewr_strategy, priority_action);
 
 //	bool has_i_label = false;
 //	bool has_tau = false;
-//        for( lps::action_label_list::iterator i = lps.action_labels().begin(); i != lps.action_labels().end(); ++i )
+//        for( process::action_label_list::iterator i = lps.action_labels().begin(); i != lps.action_labels().end(); ++i )
 //	{
 //                has_i_label = has_i_label || ( i -> name() == atermpp::aterm_string( "i" ) );
 //	}
@@ -188,11 +146,10 @@ static void check_lps2lts_specification(std::string const& specification,
  //          has_tau = has_tau || i->is_tau() ;
  //       }
 
-        core::garbage_collect();
 
-        std::cerr << "BCG FORMAT\n";
-        BOOST_CHECK_EQUAL(result6.num_states(), expected_states);
-        BOOST_CHECK_EQUAL(result6.num_transitions(), expected_transitions);
+      std::cerr << "BCG FORMAT\n";
+      BOOST_CHECK_EQUAL(result6.num_states(), expected_states);
+      BOOST_CHECK_EQUAL(result6.num_transitions(), expected_transitions);
 //        if( has_i_label )
 //        {
 //          BOOST_CHECK_EQUAL(result6.num_action_labels(), expected_labels );
@@ -202,7 +159,6 @@ static void check_lps2lts_specification(std::string const& specification,
 //            BOOST_CHECK_EQUAL(result6.num_action_labels() , expected_labels + 1 );
 //        }
 #endif
-      }
     }
   }
 }
@@ -276,6 +232,7 @@ BOOST_AUTO_TEST_CASE(test_abp)
     "init P(1, dc, true, 1, dc1, dc2, 1, dc9, 1, dc13, true);\n"
   );
   check_lps2lts_specification(abp, 74, 92, 19);
+  check_lps2lts_specification(abp, 74, 92, 19, "tau");
 }
 
 BOOST_AUTO_TEST_CASE(test_confluence)
@@ -444,7 +401,6 @@ BOOST_AUTO_TEST_CASE(test_well_typedness_of_length_of_list_of_numbers)
   check_lps2lts_specification(spec, 1, 1, 1);
 }
 
-#ifndef MCRL2_SKIP_LONG_TESTS
 #if 0 // This test has been disabled; it was decided not to fix this issue.
 // The following example illustrates that enumeration can sometimes need
 // a large stack depth.
@@ -469,14 +425,180 @@ BOOST_AUTO_TEST_CASE(test_deep_stack)
   check_lps2lts_specification(spec, 3, 524289, 2);
 }
 #endif // false
-#endif // MCRL2_SKIP_LONG_TESTS
+
+
+BOOST_AUTO_TEST_CASE(test_max_states)
+{
+  std::string spec(
+  "act a;\n"
+  "proc P(s: Pos) =\n"
+  "  (s <= 10) -> a . P(s+1);\n"
+  "init P(1);\n");
+
+  lps::specification specification = lps::parse_linear_process_specification(spec);
+
+  lts::lts_generation_options options;
+  options.trace_prefix = "lps2lts_test";
+  options.specification = specification;
+  options.lts = utilities::temporary_filename("lps2lts_test_file");
+  options.max_states = 5;
+
+  lts::lts_aut_t result;
+  options.outformat = result.type();
+  lts::lps2lts_algorithm lps2lts;
+  lps2lts.initialise_lts_generation(&options);
+  lps2lts.generate_lts();
+  lps2lts.finalise_lts_generation();
+  result.load(options.lts);
+  remove(options.lts.c_str()); // Clean up after ourselves
+
+  BOOST_CHECK_LT(result.num_states(), 10);
+}
+
+BOOST_AUTO_TEST_CASE(test_interaction_sum_and_assignment_notation1)
+{
+  std::string spec(
+    "proc B (i:Bool) = sum i:Bool.tau. B();\n"
+    "init B(true);\n"
+  );
+  check_lps2lts_specification(spec, 1, 2, 1);
+}
+
+BOOST_AUTO_TEST_CASE(test_interaction_sum_and_assignment_notation2)
+{
+  std::string spec(
+    "proc B (i:Bool) = sum i:Bool.tau. B(i=i);\n"
+    "init B(true);\n"
+  );
+  check_lps2lts_specification(spec, 2, 4, 1);
+}
+
+BOOST_AUTO_TEST_CASE(test_whether_function_update_is_declared)
+{
+  std::string spec(
+    "sort  T = struct a|b;\n"
+    "act int;\n"
+    "map g: Bool -> T;\n"
+    "var y: Bool;\n"
+    "eqn g(y) = a;\n"
+    "act d: Bool -> T;\n"
+    "proc P(f: Bool -> T) = int . P(f[true->b]);\n"
+    "init P(g) ;\n"
+  );
+  check_lps2lts_specification(spec, 2, 2, 1);
+}
+
+BOOST_AUTO_TEST_CASE(test_whether_bag_enumeration_with_similar_elements_is_allowed)
+{
+  std::string spec(
+    "sort S = struct s1 | s2;\n"
+    "act int;\n"
+    "proc P(b: Bag(S)) = (count(s1, b) < 3) -> int.P();\n"
+    "init P({s1: 1, s1: 1});\n"
+  );
+  check_lps2lts_specification(spec, 1, 1, 1);
+}
+  
+BOOST_AUTO_TEST_CASE(test_whether_functions_to_functions_are_causing_problems)
+{
+  std::string spec(
+    "map f:Nat->Nat->Nat->Nat;\n"
+    "    g:Nat->Nat;\n"
+    "var x,y,z:Nat;"
+    "eqn f(x)(y)(z)=x+y+z;\n"
+    "    f(x)(y)=g;\n"
+    "act int:Nat;\n"
+    "proc P = int(f(1)(2)(3)).P();\n"
+    "init P;\n"
+  );
+  check_lps2lts_specification(spec, 1, 1, 1);
+}
+  
+BOOST_AUTO_TEST_CASE(test_whether_functions_can_be_enumerated)
+{
+  std::string spec(
+    "act int:Bool->Bool;\n"
+    "proc P = sum f:Bool->Bool.int(f).P;\n"
+    "init P;\n"
+  );
+  check_lps2lts_specification(spec, 1, 4, 4);
+}
+  
+BOOST_AUTO_TEST_CASE(test_whether_functions_with_more_arguments_can_be_enumerated)
+{
+  std::string spec(
+    "act int:Bool#Bool#Bool->Bool;\n"
+    "proc P = sum f:Bool#Bool#Bool->Bool.f(true,true,true)->int(f).P;\n"
+    "init P;\n"
+  );
+  check_lps2lts_specification(spec, 1, 128, 128);
+}
+  
+BOOST_AUTO_TEST_CASE(test_whether_finite_sets_can_be_enumerated)
+{
+  std::string spec(
+    "act int:FSet(Bool);\n"
+    "proc P = sum f:FSet(Bool).int(f).P;\n"
+    "init P;\n"
+  );
+  check_lps2lts_specification(spec, 1, 4, 4);
+}
+  
+BOOST_AUTO_TEST_CASE(test_whether_sets_can_be_enumerated)
+{
+  std::string spec(
+    "act int:FSet(Bool);\n"
+    "proc P = sum f:FSet(Bool).int(f).P;\n"
+    "init P;\n"
+  );
+  check_lps2lts_specification(spec, 1, 4, 4);
+}
+  
+BOOST_AUTO_TEST_CASE(test_whether_finite_sets_with_conditions_can_be_enumerated)
+{
+  std::string spec(
+    "act int:FSet(Bool);\n"
+    "proc P = sum f:FSet(Bool).(true in f) -> int(f).P;\n"
+    "init P;\n"
+  );
+  check_lps2lts_specification(spec, 1, 2, 2);
+}
+  
+BOOST_AUTO_TEST_CASE(test_whether_sets_with_conditions_can_be_enumerated)
+{
+  std::string spec(
+    "act int:Set(Bool);\n"
+    "proc P = sum f:Set(Bool).(true in f) -> int(f).P;\n"
+    "init P;\n"
+  );
+  check_lps2lts_specification(spec, 1, 2, 2);
+}
+  
+BOOST_AUTO_TEST_CASE(test_whether_finite_sets_of_functions_can_be_enumerated)
+{
+  std::string spec(
+    "act int:FSet(Bool->Bool);\n"
+    "proc P = sum f:FSet(Bool->Bool).((lambda x:Bool.true) in f) -> int(f).P;\n"
+    "init P;\n"
+  );
+  check_lps2lts_specification(spec, 1, 8, 8);
+}
+  
+BOOST_AUTO_TEST_CASE(test_whether_sets_of_functions_can_be_enumerated)
+{
+  std::string spec(
+    "act int:Set(Bool->Bool);\n"
+    "proc P = sum f:Set(Bool->Bool).((lambda x:Bool.true) in f) -> int(f).P;\n"
+    "init P;\n"
+  );
+  check_lps2lts_specification(spec, 1, 8, 8);
+}
+  
 
 
 boost::unit_test::test_suite* init_unit_test_suite(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
  // Initialise random seed to allow parallel running with lps2lts_test_old
-  std::srand(time(NULL) * 13);
   return 0;
 }
 
