@@ -23,26 +23,22 @@
 #include <boost/algorithm/string/trim.hpp>
 
 #include "mcrl2/atermpp/make_list.h"
-#include "mcrl2/atermpp/map.h"
 #include "mcrl2/data/parse.h"
-#include "mcrl2/data/print.h"
 #include "mcrl2/data/exists.h"
 #include "mcrl2/data/lambda.h"
-#include "mcrl2/data/replace.h"
 #include "mcrl2/data/set.h"
 #include "mcrl2/data/sort_expression.h"
 #include "mcrl2/data/standard.h"
 #include "mcrl2/data/standard_utility.h"
-#include "mcrl2/data/substitutions.h"
 #include "mcrl2/data/detail/data_construction.h"
 #include "mcrl2/pbes/builder.h"
 #include "mcrl2/pbes/pbes.h"
-#include "mcrl2/pbes/replace.h"
 #include "mcrl2/data/set_identifier_generator.h"
 #include "mcrl2/utilities/text_utility.h"
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/utilities/detail/separate_keyword_section.h"
 #include "mcrl2/utilities/exception.h"
+#include "mcrl2/utilities/detail/container_utility.h"
 
 #ifdef MCRL2_ABSINTHE_CHECK_EXPRESSIONS
 #include "mcrl2/data/detail/print_parse_check.h"
@@ -55,7 +51,7 @@ namespace pbes_system {
   template <typename Term>
   std::string print_term(const Term& x)
   {
-    return data::pp(x) + " " + x.to_string();
+    return data::pp(x) + " " + to_string(x);
   }
 
   template <typename Term>
@@ -86,19 +82,11 @@ namespace pbes_system {
 
 namespace detail {
 
-  template <class T> // note, T is only a dummy
-  struct absinthe_globals
-  {
-    static data::data_specification m_dataspec;
-  };
-
-  template <class T>
-  data::data_specification absinthe_globals<T>::m_dataspec = data::data_specification();
-
   inline
   data::data_specification& absinthe_data_specification()
   {
-    return absinthe_globals<size_t>::m_dataspec;
+    static data::data_specification dataspec;
+    return dataspec;
   }
 
 #ifdef MCRL2_ABSINTHE_CHECK_EXPRESSIONS
@@ -126,7 +114,7 @@ namespace detail {
       data::sort_expression s = i->reference();
       if (data::is_structured_sort(s))
       {
-        data::structured_sort ss = s;
+        const data::structured_sort& ss = atermpp::down_cast<data::structured_sort>(s);
         data::function_symbol_vector v = ss.constructor_functions();
         for (data::function_symbol_vector::iterator j = v.begin(); j != v.end(); ++j)
         {
@@ -142,7 +130,7 @@ namespace detail {
   }
 
   inline
-  void print_used_function_symbols(const pbes<>& p)
+  void print_used_function_symbols(const pbes& p)
   {
     mCRL2log(log::debug, "absinthe") << "--- used function symbols ---" << std::endl;
     std::set<data::function_symbol> find_function_symbols = pbes_system::find_function_symbols(p);
@@ -162,7 +150,7 @@ namespace detail {
     }
     else if (data::is_function_sort(s))
     {
-      data::function_sort fs = s;
+      const data::function_sort& fs = atermpp::down_cast<data::function_sort>(s);
       return fs.codomain();
     }
     else if (data::is_container_sort(s))
@@ -177,9 +165,9 @@ namespace detail {
 
 struct absinthe_algorithm
 {
-  typedef atermpp::map<data::sort_expression, data::sort_expression> sort_expression_substitution_map;
-  typedef atermpp::map<data::function_symbol, data::function_symbol> function_symbol_substitution_map;
-  typedef atermpp::map<data::sort_expression, data::function_symbol> abstraction_map;
+  typedef std::map<data::sort_expression, data::sort_expression> sort_expression_substitution_map;
+  typedef std::map<data::function_symbol, data::function_symbol> function_symbol_substitution_map;
+  typedef std::map<data::sort_expression, data::function_symbol> abstraction_map;
 
   struct absinthe_sort_expression_builder: public sort_expression_builder<absinthe_sort_expression_builder>
   {
@@ -230,7 +218,7 @@ struct absinthe_algorithm
         return i->second;
       }
       throw mcrl2::runtime_error("function symbol " + print_symbol(x) + " not present in the function symbol mapping!");
-      return data::data_expression();
+      return data::undefined_data_expression();
     }
 
     //data::data_expression operator()(const data::variable& x)
@@ -245,9 +233,9 @@ struct absinthe_algorithm
     {
       if (data::is_variable(x.head()))
       {
-        data::variable v = x.head();
+        data::variable v = atermpp::down_cast<data::variable>(x.head());
         v = data::variable(v.name(), super::operator()(v.sort()));
-        return data::detail::create_finite_set(data::application(v, x.arguments()));
+        return data::detail::create_finite_set(data::application(v, x.begin(), x.end()));
       }
       else if (data::is_function_symbol(x.head()))
       {
@@ -257,7 +245,7 @@ struct absinthe_algorithm
       {
         throw mcrl2::runtime_error("don't know how to handle arbitrary expression as head: " + data::pp(x));
       }
-      return data::data_expression();
+      return data::undefined_data_expression();
     }
 
     data::data_expression operator()(const data::lambda& x)
@@ -288,7 +276,7 @@ struct absinthe_algorithm
       {
         // check if it is a "ground term", i.e. it does not contain any variables
         abstraction_map::const_iterator i = sigmaH.find(x.sort());
-        if (i != sigmaH.end() && data::find_variables(x).empty())
+        if (i != sigmaH.end() && data::find_all_variables(x).empty())
         {
           data::data_expression_list args = atermpp::make_list(x);
           result = data::detail::create_finite_set(data::application(i->second, args));
@@ -336,11 +324,11 @@ struct absinthe_algorithm
 
     data::variable_list make_variables(const data::data_expression_list& x, const std::string& hint, sort_function sigma) const
     {
-      atermpp::vector<data::variable> result;
-      unsigned int i = 0;
-      for (data::data_expression_list::const_iterator j = x.begin(); j != x.end(); ++i, ++j)
+      std::vector<data::variable> result;
+      std::size_t i = 0;
+      for (auto j = x.begin(); j != x.end(); ++i, ++j)
       {
-        result.push_back(data::variable(hint + boost::lexical_cast<std::string>(i), sigma(j->sort())));
+        result.push_back(data::variable(hint + utilities::number2string(i), sigma(j->sort())));
       }
       return data::variable_list(result.begin(), result.end());
     }
@@ -410,11 +398,11 @@ struct absinthe_algorithm
       data::data_expression q = data::lazy::join_and(z.begin(), z.end());
       if (m_is_over_approximation)
       {
-        result = make_exists(variables, and_(q, propositional_variable_instantiation(x.name(), variables)));
+        result = make_exists(variables, and_(q, propositional_variable_instantiation(x.name(), atermpp::container_cast<data::data_expression_list>(variables))));
       }
       else
       {
-        result = make_forall(variables, imp(q, propositional_variable_instantiation(x.name(), variables)));
+        result = make_forall(variables, imp(q, propositional_variable_instantiation(x.name(), atermpp::container_cast<data::data_expression_list>(variables))));
       }
       return result;
     }
@@ -437,8 +425,7 @@ struct absinthe_algorithm
       x.formula() = super::operator()(x.formula());
     }
 
-    template <typename Container>
-    void operator()(pbes_system::pbes<Container>& x)
+    void operator()(pbes_system::pbes& x)
     {
       super::operator()(x.equations());
       pbes_expression kappa = (*this)(x.initial_state());
@@ -515,7 +502,7 @@ struct absinthe_algorithm
     const data::function_symbol_vector& m = dataspec.user_defined_mappings();
     for (data::function_symbol_vector::const_iterator i = m.begin(); i != m.end(); ++i)
     {
-      data::function_sort f = i->sort();
+      const data::function_sort& f = atermpp::down_cast<data::function_sort>(i->sort());
       if (f.domain().size() != 1)
       {
         throw mcrl2::runtime_error("cannot abstract the function " + data::pp(*i) + " since the arity of the domain is not equal to one!");
@@ -597,11 +584,13 @@ struct absinthe_algorithm
 
     data::function_symbol operator()(const data::function_symbol& f, sort_function sigma) const
     {
+    	using utilities::detail::contains;
+
       //mCRL2log(log::debug, "absinthe") << "lift_function_symbol_1_2 f = " << print_symbol(f) << std::endl;
       std::string name = std::string(f.name());
 
-      bool print_sort = suffix_with_sort.find(std::string(f.name())) != suffix_with_sort.end();
-      std::map<std::string, std::string>::const_iterator i = unprintable.find(name);
+      bool print_sort = contains(suffix_with_sort, std::string(f.name()));
+      auto i = unprintable.find(name);
       if (i != unprintable.end())
       {
         name = i->second;
@@ -622,7 +611,7 @@ struct absinthe_algorithm
         // Apply sigmaS recursively to s
         //   f:        tail:           List(Nat) -> List(Nat)
         //   result:   generated_tail: List(AbsNat) -> Set(List(AbsNat))
-        data::function_sort fs = sigma(s);
+        const data::function_sort& fs = atermpp::down_cast<data::function_sort>(sigma(s));
         return data::function_symbol(name, data::function_sort(fs.domain(), make_set()(fs.codomain())));
       }
       else if (data::is_container_sort(s))
@@ -641,24 +630,26 @@ struct absinthe_algorithm
   {
     data::function_symbol operator()(const data::function_symbol& f) const
     {
+      using namespace data;
       //mCRL2log(log::debug, "absinthe") << "lift_function_symbol_2_3 f = " << print_symbol(f) << std::endl;
       std::string name = "Lift" + boost::algorithm::trim_copy(std::string(f.name()));
-      data::sort_expression s = f.sort();
-      if (data::is_basic_sort(s))
+      const sort_expression &s = f.sort();
+      if (is_basic_sort(s))
       {
-        return data::function_symbol(name, make_set()(s));
+        return function_symbol(name, make_set()(s));
       }
-      else if (data::is_function_sort(s))
+      else if (is_function_sort(s))
       {
-        data::function_sort fs(s);
-        return data::function_symbol(name, data::function_sort(atermpp::apply(fs.domain(), make_set()), fs.codomain()));
+        const function_sort fs(s);
+        const sort_expression_list& sl=fs.domain();
+        return function_symbol(name,
+                 function_sort(sort_expression_list(sl.begin(),sl.end(), make_set()), fs.codomain()));
       }
-      else if (data::is_container_sort(s))
+      else if (is_container_sort(s))
       {
         return data::function_symbol(name, make_set()(s));
       }
       throw mcrl2::runtime_error("absinthe algorithm (lift): unsupported sort " + print_term(s) + " detected!");
-      return data::function_symbol();
     }
   };
 
@@ -668,13 +659,13 @@ struct absinthe_algorithm
     lift_equation_1_2()
     {}
 
-    atermpp::vector<data::variable> make_variables(const data::sort_expression_list& sorts, const std::string& hint, sort_function sigma) const
+    std::vector<data::variable> make_variables(const data::sort_expression_list& sorts, const std::string& hint, sort_function sigma) const
     {
-      atermpp::vector<data::variable> result;
-      unsigned int i = 0;
-      for (data::sort_expression_list::const_iterator j = sorts.begin(); j != sorts.end(); ++i, ++j)
+      std::vector<data::variable> result;
+      std::size_t i = 0;
+      for (auto j = sorts.begin(); j != sorts.end(); ++i, ++j)
       {
-        result.push_back(data::variable(hint + boost::lexical_cast<std::string>(i), sigma(*j)));
+        result.push_back(data::variable(hint + utilities::number2string(i), sigma(*j)));
       }
       return result;
     }
@@ -723,8 +714,8 @@ struct absinthe_algorithm
           throw std::runtime_error("can not generalize functions with abstraction sorts in the domain: " + data::pp(f1) + ": " + data::pp(s1));
         }
 
-        atermpp::vector<data::variable> x = make_variables(fs2.domain(), "x", sigma);
-        variables = atermpp::convert<data::variable_list>(x);
+        data::variable_vector x = make_variables(fs2.domain(), "x", sigma);
+        variables = data::variable_list(x.begin(),x.end());
         lhs = data::application(f2, data::data_expression_list(x.begin(), x.end()));
         data::application f_x(f1, data::data_expression_list(x.begin(), x.end()));
 
@@ -740,7 +731,7 @@ struct absinthe_algorithm
         else
         {
           data::function_symbol h = i->second;
-          rhs = create_finite_set(data::application(h, atermpp::make_list(f_x)));
+          rhs = data::detail::create_finite_set(data::application(h, atermpp::make_list(f_x)));
           //pbes_system::detail::absinthe_check_expression(rhs);
         }
       }
@@ -776,13 +767,13 @@ struct absinthe_algorithm
     lift_equation_2_3()
     {}
 
-    atermpp::vector<data::variable> make_variables(const data::sort_expression_list& sorts, const std::string& hint, sort_function sigma) const
+    std::vector<data::variable> make_variables(const data::sort_expression_list& sorts, const std::string& hint, sort_function sigma) const
     {
-      atermpp::vector<data::variable> result;
-      unsigned int i = 0;
-      for (data::sort_expression_list::const_iterator j = sorts.begin(); j != sorts.end(); ++i, ++j)
+      std::vector<data::variable> result;
+      std::size_t i = 0;
+      for (auto j = sorts.begin(); j != sorts.end(); ++i, ++j)
       {
-        result.push_back(data::variable(hint + boost::lexical_cast<std::string>(i), sigma(*j)));
+        result.push_back(data::variable(hint + utilities::number2string(i), sigma(*j)));
       }
       return result;
     }
@@ -790,11 +781,11 @@ struct absinthe_algorithm
     // Let x = [x1:D1, ..., xn:Dn] and X = [X1:Set(D1), ..., Xn:Set(Dn)]. Returns the expression
     //
     // (x1 in X1 /\ ... /\ xn in Xn)
-    data::data_expression enumerate_domain(const atermpp::vector<data::variable>& x, const atermpp::vector<data::variable>& X) const
+    data::data_expression enumerate_domain(const std::vector<data::variable>& x, const std::vector<data::variable>& X) const
     {
-      atermpp::vector<data::data_expression> a;
-      atermpp::vector<data::variable>::const_iterator i = x.begin();
-      atermpp::vector<data::variable>::const_iterator j = X.begin();
+      std::vector<data::data_expression> a;
+      std::vector<data::variable>::const_iterator i = x.begin();
+      std::vector<data::variable>::const_iterator j = X.begin();
       for (; i != x.end(); ++i, ++j)
       {
         a.push_back(data::detail::create_set_in(*i, *j));
@@ -824,12 +815,12 @@ struct absinthe_algorithm
         data::function_sort fs3(f3.sort());
 
         // TODO: generate these variables in a proper way
-        atermpp::vector<data::variable> x = make_variables(fs2.domain(), "x", sigma);
-        atermpp::vector<data::variable> X = make_variables(fs3.domain(), "X", sigma);
+        std::vector<data::variable> x = make_variables(fs2.domain(), "x", sigma);
+        std::vector<data::variable> X = make_variables(fs3.domain(), "X", sigma);
 
         variables = data::variable_list(X.begin(), X.end());
         lhs = data::application(f3, data::data_expression_list(X.begin(), X.end()));
-        data::variable y("y", data::detail::get_set_sort(fs2.codomain()));
+        data::variable y("y", data::detail::get_set_sort(atermpp::down_cast<data::container_sort>(fs2.codomain())));
         data::data_expression Y = data::application(f2, data::data_expression_list(x.begin(), x.end()));
         data::data_expression body = data::sort_bool::and_(enumerate_domain(x, X), data::detail::create_set_in(y, Y));
         rhs = data::detail::create_set_comprehension(y, data::exists(x, body));
@@ -853,7 +844,7 @@ struct absinthe_algorithm
     std::ostringstream out;
     for (typename Map::const_iterator i = m.begin(); i != m.end(); ++i)
     {
-      out << data::pp(i->first) << " -> " << data::pp(i->second) << std::endl;
+      out << i->first << " -> " << i->second << std::endl;
     }
     return out.str();
   }
@@ -905,8 +896,10 @@ struct absinthe_algorithm
   }
 
   // add lifted mappings and equations to the data specification
-  void lift_data_specification(const pbes<>& p, const abstraction_map& sigmaH, const sort_expression_substitution_map& sigmaS, function_symbol_substitution_map& sigmaF, data::data_specification& dataspec)
+  void lift_data_specification(const pbes& p, const abstraction_map& sigmaH, const sort_expression_substitution_map& sigmaS, function_symbol_substitution_map& sigmaF, data::data_specification& dataspec)
   {
+    using utilities::detail::has_key;
+
     sort_expression_substitution_map sigmaS_consistency = sigmaS; // is only used for consistency checking
     sort_function sigma(sigmaH, sigmaS, sigmaF);
 
@@ -919,43 +912,42 @@ struct absinthe_algorithm
 
     // add List containers for user defined sorts, since they are used in the translation
     const data::sort_expression_vector& sorts = dataspec.user_defined_sorts();
-    for (data::sort_expression_vector::const_iterator i = sorts.begin(); i != sorts.end(); ++i)
+    for (auto i = sorts.begin(); i != sorts.end(); ++i)
     {
       data::sort_expression s = data::container_sort(data::list_container(), *i);
       dataspec.add_context_sort(s);
     }
 
     // add constructor functions of List containers of abstracted sorts to sigmaF
-    for (abstraction_map::const_iterator i = sigmaH.begin(); i != sigmaH.end(); ++i)
+    for (auto i = sigmaH.begin(); i != sigmaH.end(); ++i)
     {
       data::sort_expression s = data::container_sort(data::list_container(), i->first);
       dataspec.add_context_sort(s);
       data::function_symbol_vector list_constructors = dataspec.constructors(s);
-      for (data::function_symbol_vector::iterator j = list_constructors.begin(); j != list_constructors.end(); ++j)
+      for (auto j = list_constructors.begin(); j != list_constructors.end(); ++j)
       {
         data::function_symbol f1 = *j;
         data::function_symbol f2 = lift_function_symbol_1_2()(f1, sigma);
         sigmaF[f1] = f2;
         dataspec.add_mapping(f2);
-mCRL2log(log::debug, "absinthe") << "adding list constructor " << data::pp(f1) << " to sigmaF" << std::endl;
+mCRL2log(log::debug, "absinthe") << "adding list constructor " << f1 << " to sigmaF" << std::endl;
       }
     }
 
-    for (std::set<data::function_symbol>::iterator i = used_function_symbols.begin(); i != used_function_symbols.end(); ++i)
+    for (auto i = used_function_symbols.begin(); i != used_function_symbols.end(); ++i)
     {
-      mCRL2log(log::debug, "absinthe") << "lifting function symbol: " << data::pp(*i) << std::endl;
+      mCRL2log(log::debug, "absinthe") << "lifting function symbol: " << *i << std::endl;
       data::function_symbol f1 = *i;
-      if (sigmaF.find(f1) == sigmaF.end())
+      if (!has_key(sigmaF, f1))
       {
         data::function_symbol f2 = lift_function_symbol_1_2()(f1, sigma);
-        mCRL2log(log::debug, "absinthe") << "lifted function symbol: " << data::pp(f1) << " to " << data::pp(f2) << std::endl;
+        mCRL2log(log::debug, "absinthe") << "lifted function symbol: " << f1 << " to " << f2 << std::endl;
         check_consistency(f1, f2, sigmaS_consistency);
         sigmaF[f1] = f2;
         dataspec.add_mapping(f2);
 
         data::data_equation eq = lift_equation_1_2()(f1, f2, sigma, sigmaH);
         mCRL2log(log::debug, "absinthe") << "adding equation: " << eq << std::endl;
-        mCRL2log(log::debug, "absinthe") << "adding equation: " << data::pp(eq) << std::endl;
         dataspec.add_equation(eq);
       }
     }
@@ -966,7 +958,7 @@ mCRL2log(log::debug, "absinthe") << "adding list constructor " << data::pp(f1) <
       data::function_symbol f2 = i->second;
       data::function_symbol f3 = lift_function_symbol_2_3()(f2);
 
-      mCRL2log(log::debug, "absinthe") << "adding mapping: " << data::pp(f3) << " " << data::pp(f3.sort()) << std::endl;
+      mCRL2log(log::debug, "absinthe") << "adding mapping: " << f3 << " " << f3.sort() << std::endl;
       dataspec.add_mapping(f3);
 
       // update sigmaF
@@ -974,7 +966,7 @@ mCRL2log(log::debug, "absinthe") << "adding list constructor " << data::pp(f1) <
 
       // make an equation for the lifted function symbol f
       data::data_equation eq = lift_equation_2_3()(f2, f3, sigma);
-      mCRL2log(log::debug, "absinthe") << "adding equation: " << data::pp(eq) << std::endl;
+      mCRL2log(log::debug, "absinthe") << "adding equation: " << eq << std::endl;
       dataspec.add_equation(eq);
     }
   }
@@ -1002,7 +994,7 @@ mCRL2log(log::debug, "absinthe") << "adding list constructor " << data::pp(f1) <
     log::mcrl2_logger::set_reporting_level(log::debug, "absinthe");
   }
 
-  void run(pbes<>& p, const std::string& abstraction_text, bool is_over_approximation)
+  void run(pbes& p, const std::string& abstraction_text, bool is_over_approximation)
   {
     // split the string abstraction_text into four different parts
     std::string function_symbol_mapping_text;
@@ -1055,15 +1047,15 @@ mCRL2log(log::debug, "absinthe") << "adding list constructor " << data::pp(f1) <
 
     // 1) create the data specification dataspec, which consists of user_sorts_text, abstract_mapping_text and p.data()
     data::data_specification dataspec = data::parse_data_specification(data::pp(p.data()) + "\n" + user_sorts_text + "\n" + abstraction_mapping_text.substr(3));
-    mCRL2log(log::debug, "absinthe") << "--- data specification 1) ---\n" << data::pp(dataspec) << std::endl;
+    mCRL2log(log::debug, "absinthe") << "--- data specification 1) ---\n" << dataspec << std::endl;
 
     // 2) parse the right hand sides of the function symbol mapping, and add them to dataspec
     parse_right_hand_sides(function_symbol_mapping_text, dataspec);
-    mCRL2log(log::debug, "absinthe") << "--- data specification 2) ---\n" << data::pp(dataspec) << std::endl;
+    mCRL2log(log::debug, "absinthe") << "--- data specification 2) ---\n" << dataspec << std::endl;
 
     // 3) add user_equations_text to dataspec
     dataspec = data::parse_data_specification(data::pp(dataspec) + "\n" + user_equations_text);
-    mCRL2log(log::debug, "absinthe") << "--- data specification 3) ---\n" << data::pp(dataspec) << std::endl;
+    mCRL2log(log::debug, "absinthe") << "--- data specification 3) ---\n" << dataspec << std::endl;
 
     // abstraction functions (specified by the user)
     abstraction_map sigmaH = parse_abstraction_map(pbes_sorts_text + "\n" + user_sorts_text + "\n" + abstraction_mapping_text.substr(3));
@@ -1073,7 +1065,7 @@ mCRL2log(log::debug, "absinthe") << "adding list constructor " << data::pp(f1) <
     for (abstraction_map::const_iterator i = sigmaH.begin(); i != sigmaH.end(); ++i)
     {
       data::function_symbol f = i->second;
-      data::function_sort fs = f.sort();
+      const data::function_sort& fs = atermpp::down_cast<data::function_sort>(f.sort());
       sigmaS[i->first] = fs.codomain();
     }
     mCRL2log(log::debug, "absinthe") << "\n--- sort expression mapping ---\n" << print_mapping(sigmaS) << std::endl;
@@ -1089,18 +1081,18 @@ mCRL2log(log::debug, "absinthe") << "adding list constructor " << data::pp(f1) <
     // after: equations for f3 have been added to dataspec
     // generate mapping f1 -> f2 for missing function symbols
     lift_data_specification(p, sigmaH, sigmaS, sigmaF, dataspec);
-    mCRL2log(log::debug, "absinthe") << "--- data specification 4) ---\n" << data::pp(dataspec) << std::endl;
+    mCRL2log(log::debug, "absinthe") << "--- data specification 4) ---\n" << dataspec << std::endl;
 
     mCRL2log(log::debug, "absinthe") << "\n--- function symbol mapping after lifting ---\n" << print_mapping(sigmaF) << std::endl;
 
-    mCRL2log(log::debug, "absinthe") << "--- pbes before ---\n" << pbes_system::pp(p) << std::endl;
+    mCRL2log(log::debug, "absinthe") << "--- pbes before ---\n" << p << std::endl;
 
     p.data() = dataspec;
 
     // then transform the data expressions and the propositional variable instantiations
     absinthe_data_expression_builder(sigmaH, sigmaS, sigmaF, is_over_approximation)(p);
 
-    mCRL2log(log::debug, "absinthe") << "--- pbes after ---\n" << pbes_system::pp(p) << std::endl;
+    mCRL2log(log::debug, "absinthe") << "--- pbes after ---\n" << p << std::endl;
   }
 };
 
