@@ -17,17 +17,18 @@
 #include <set>
 #include <vector>
 #include <sstream>
-#include <boost/iterator/transform_iterator.hpp>
 #include "mcrl2/atermpp/make_list.h"
 #include "mcrl2/atermpp/detail/aterm_list_utility.h"
 #include "mcrl2/atermpp/container_utility.h"
 #include "mcrl2/data/utility.h"
 #include "mcrl2/data/detail/data_functional.h"
 #include "mcrl2/data/set_identifier_generator.h"
+#include "mcrl2/data/substitutions/mutable_map_substitution.h"
 #include "mcrl2/lps/specification.h"
 #include "mcrl2/lps/replace.h"
 #include "mcrl2/pbes/pbes.h"
 #include "mcrl2/pbes/detail/lps2pbes_utility.h"
+#include "mcrl2/utilities/logger.h"
 
 namespace mcrl2
 {
@@ -62,10 +63,10 @@ class bisimulation_algorithm
     /// \brief Generates a name for an action_list.
     /// \param l A sequence of actions
     /// \return A string representation of the list \p l
-    std::string action_list_name(action_list l) const
+    std::string action_list_name(const process::action_list& l) const
     {
       std::ostringstream out;
-      for (action_list::iterator i = l.begin(); i != l.end(); ++i)
+      for (auto i = l.begin(); i != l.end(); ++i)
       {
         out << (i != l.begin() ? "-" : "") << std::string(i->label().name());
       }
@@ -124,6 +125,19 @@ class bisimulation_algorithm
       }
     }
 
+    // creates the substitution v[i] := e[i]
+    // pre: v.size() == e.size()
+    void make_substitution(const data::variable_list& v, const data::data_expression_list& e, data::mutable_map_substitution<>& result) const
+    {
+      assert(v.size() == e.size());
+      auto vi = v.begin();
+      auto ei = e.begin();
+      for (; vi != v.end(); ++vi, ++ei)
+      {
+        result[*vi] = *ei;
+      }
+    }
+
   public:
     /// \brief Creates a name for the propositional variable Xpq
     /// \param p A linear process
@@ -227,16 +241,16 @@ class bisimulation_algorithm
     /// \param p A linear process specification
     /// \param q A linear process specification
     /// \return A substitution that should be applied to q to remove name clashes between p and q.
-    /// \detail After this substitution the following holds:
+    /// \details After this substitution the following holds:
     /// \f[ ((param(p)\cup glob(p))\cap ((param(q)\cup glob(q))=\emptyset \f]
     /// where param(p) denotes p.process().process_parameters() and glob(p) denotes p.global_variables().
-    atermpp::map<data::variable, data::variable> compute_process_parameter_name_clashes(const specification& p, const specification& q) const
+    data::mutable_map_substitution<> compute_process_parameter_name_clashes(const specification& p, const specification& q) const
     {
-      atermpp::map<data::variable, data::variable> result;
+       data::mutable_map_substitution<> result;
 
       // put the names of variables appearing in p and q in an identifier generator
-      std::set<data::variable> context = lps::find_variables(p);
-      std::set<variable> vars = lps::find_variables(q);
+      std::set<data::variable> context = lps::find_all_variables(p);
+      std::set<variable> vars = lps::find_all_variables(q);
       context.insert(vars.begin(), vars.end());
       data::set_identifier_generator generator;
       for (std::set<data::variable>::iterator i = context.begin(); i != context.end(); ++i)
@@ -244,13 +258,9 @@ class bisimulation_algorithm
         generator.add_identifier(i->name());
       }
 
-      // put the process parameters of q in qvars
-      std::set<data::variable> qvars;
-      data::variable_list pvars = p.process().process_parameters();
-      qvars.insert(pvars.begin(), pvars.end());
-
       // generate renamings for variables appearing in qvars
-      for (std::set<data::variable>::iterator i = qvars.begin(); i != qvars.end(); ++i)
+      data::variable_list qvars = q.process().process_parameters();
+      for (data::variable_list::iterator i = qvars.begin(); i != qvars.end(); ++i)
       {
         data::variable v(generator(i->name()), i->sort());
         if (v != *i)
@@ -266,13 +276,13 @@ class bisimulation_algorithm
     /// \param p A linear process specification
     /// \param q A linear process specification
     /// \return A substitution that should be applied to q to remove name clashes between p and q.
-    atermpp::map<data::variable, data::variable> compute_summand_variable_name_clashes(const specification& p, const specification& q) const
+    data::mutable_map_substitution<> compute_summand_variable_name_clashes(const specification& p, const specification& q) const
     {
-      atermpp::map<data::variable, data::variable> result;
+      data::mutable_map_substitution<> result;
 
       // put the names of variables appearing in p and q in an identifier generator
-      std::set<data::variable> context = lps::find_variables(p);
-      std::set<variable> vars = lps::find_variables(q);
+      std::set<data::variable> context = lps::find_all_variables(p);
+      std::set<variable> vars = lps::find_all_variables(q);
       context.insert(vars.begin(), vars.end());
       data::set_identifier_generator generator;
       for (std::set<data::variable>::iterator i = context.begin(); i != context.end(); ++i)
@@ -310,17 +320,14 @@ class bisimulation_algorithm
     /// \brief Resolves name clashes between model and spec.
     void resolve_name_clashes(const specification& model, specification& spec, bool include_summand_variables = false)
     {
-      atermpp::map<data::variable, data::variable> sigma = compute_process_parameter_name_clashes(model, spec);
-      if (!sigma.empty())
-      {
-        lps::replace_process_parameters(spec, data::make_map_substitution(sigma));
-      }
-
+      data::mutable_map_substitution<> sigma = compute_process_parameter_name_clashes(model, spec);
+      lps::replace_process_parameters(spec, sigma);
       if (include_summand_variables)
       {
         sigma = compute_summand_variable_name_clashes(model, spec);
-        lps::replace_summand_variables(spec, data::make_map_substitution(sigma));
+        lps::replace_summand_variables(spec, sigma);
       }
+      mCRL2log(log::debug) << "bisimulation spec after resolving name clashes:\n" << lps::pp(spec) << std::endl;
     }
 
     /// \brief Initializes the name lookup table.
@@ -342,10 +349,10 @@ class bisimulation_algorithm
     /// \param M A specification
     /// \param S A specification
     /// \return The constructed pbes
-    pbes<> build_pbes(const atermpp::vector<pbes_equation>& equations,
-                      const specification& M,
-                      const specification& S
-                     )
+    pbes build_pbes(const std::vector<pbes_equation>& equations,
+                    const specification& M,
+                    const specification& S
+                   )
     {
       const linear_process& m = M.process();
       const linear_process& s = S.process();
@@ -354,7 +361,7 @@ class bisimulation_algorithm
       data_specification data = M.data();
       propositional_variable_instantiation init(X(m, s), M.initial_process().state(M.process().process_parameters()) + S.initial_process().state(S.process().process_parameters()));
 
-      pbes<> result(data, equations, init);
+      pbes result(data, equations, init);
       assert(result.is_closed());
       return result;
     }
@@ -375,7 +382,7 @@ class branching_bisimulation_algorithm : public bisimulation_algorithm
     pbes_expression match(const linear_process& p, const linear_process& q) const
     {
       namespace z = pbes_expr_optimized;
-      atermpp::vector<pbes_expression> result;
+      std::vector<pbes_expression> result;
       for (my_iterator i = p.action_summands().begin(); i != p.action_summands().end(); ++i)
       {
         data_expression    ci = i->condition();
@@ -400,7 +407,7 @@ class branching_bisimulation_algorithm : public bisimulation_algorithm
       data_expression_list gi = i->next_state(p.process_parameters());
       if (i->is_tau())
       {
-        atermpp::vector<pbes_expression> v;
+        std::vector<pbes_expression> v;
         for (my_iterator j = q.action_summands().begin(); j != q.action_summands().end(); ++j)
         {
           if (!j->is_tau())
@@ -417,7 +424,7 @@ class branching_bisimulation_algorithm : public bisimulation_algorithm
       }
       else
       {
-        atermpp::vector<pbes_expression> v;
+        std::vector<pbes_expression> v;
         for (my_iterator j = q.action_summands().begin(); j != q.action_summands().end(); ++j)
         {
           data_expression      cj = j->condition();
@@ -440,7 +447,7 @@ class branching_bisimulation_algorithm : public bisimulation_algorithm
     pbes_expression close(const linear_process& p, const linear_process& q, my_iterator i) const
     {
       namespace z = pbes_expr_optimized;
-      atermpp::vector<pbes_expression> v;
+      std::vector<pbes_expression> v;
       const variable_list& d  = p.process_parameters();
       const variable_list& d1 = q.process_parameters();
       variable_list e  = i->summation_variables();
@@ -453,7 +460,7 @@ class branching_bisimulation_algorithm : public bisimulation_algorithm
         data_expression       cj = j->condition();
         variable_list         e1 = j->summation_variables();
         data_expression_list  gj = j->next_state(q.process_parameters());
-        pbes_expression       expr = pbes_expr::exists(e1, z::and_(cj, var(Y(p, q, i), d + gj + e)));
+        pbes_expression       expr = pbes_expr::exists(e1, z::and_(cj, var(Y(p, q, i), data_expression_list(d) + gj + data_expression_list(e))));
         v.push_back(expr);
       }
       return z::or_(z::join_or(v.begin(), v.end()), z::and_(var(X(p, q), d + d1), step(p, q, i)));
@@ -465,7 +472,7 @@ class branching_bisimulation_algorithm : public bisimulation_algorithm
     /// \param spec A linear process specification
     /// \return A pbes that expresses branching bisimulation between the
     /// two specifications.
-    pbes<> run(const specification& model, const specification& spec)
+    pbes run(const specification& model, const specification& spec)
     {
       namespace z = pbes_expr_optimized;
 
@@ -477,7 +484,7 @@ class branching_bisimulation_algorithm : public bisimulation_algorithm
 
       const variable_list& d  = m.process_parameters();
       const variable_list& d1 = s.process_parameters();
-      atermpp::vector<pbes_equation> equations;
+      std::vector<pbes_equation> equations;
 
 
       // E1
@@ -507,7 +514,7 @@ class branching_bisimulation_algorithm : public bisimulation_algorithm
 /// \param spec A linear process specification
 /// \return A pbes that expresses branching bisimulation between the two specifications.
 inline
-pbes<> branching_bisimulation(const specification& model, const specification& spec)
+pbes branching_bisimulation(const specification& model, const specification& spec)
 {
   return branching_bisimulation_algorithm().run(model, spec);
 }
@@ -527,7 +534,7 @@ class strong_bisimulation_algorithm : public bisimulation_algorithm
     pbes_expression match(const linear_process& p, const linear_process& q) const
     {
       namespace z = pbes_expr_optimized;
-      atermpp::vector<pbes_expression> result;
+      std::vector<pbes_expression> result;
       for (my_iterator i = p.action_summands().begin(); i != p.action_summands().end(); ++i)
       {
         data_expression  ci = i->condition();
@@ -548,7 +555,7 @@ class strong_bisimulation_algorithm : public bisimulation_algorithm
       namespace z = pbes_expr_optimized;
       data_expression_list gi = i->next_state(p.process_parameters());
 
-      atermpp::vector<pbes_expression> result;
+      std::vector<pbes_expression> result;
       for (my_iterator j = q.action_summands().begin(); j != q.action_summands().end(); ++j)
       {
         data_expression       cj = j->condition();
@@ -566,7 +573,7 @@ class strong_bisimulation_algorithm : public bisimulation_algorithm
     /// \param model A linear process specification
     /// \param spec A linear process specification
     /// \return A pbes that expresses strong bisimulation between stwo specifications.
-    pbes<> run(const specification& model, const specification& spec)
+    pbes run(const specification& model, const specification& spec)
     {
       namespace z = pbes_expr_optimized;
       specification spec1 = spec;
@@ -577,7 +584,7 @@ class strong_bisimulation_algorithm : public bisimulation_algorithm
 
       const variable_list& d  = m.process_parameters();
       const variable_list& d1 = s.process_parameters();
-      atermpp::vector<pbes_equation> equations;
+      std::vector<pbes_equation> equations;
 
 
       // E
@@ -593,7 +600,7 @@ class strong_bisimulation_algorithm : public bisimulation_algorithm
 /// \param spec A linear process specification
 /// \return A pbes that expresses strong bisimulation between the two specifications.
 inline
-pbes<> strong_bisimulation(const specification& model, const specification& spec)
+pbes strong_bisimulation(const specification& model, const specification& spec)
 {
   return strong_bisimulation_algorithm().run(model, spec);
 }
@@ -613,7 +620,7 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
     pbes_expression match(const linear_process& p, const linear_process& q) const
     {
       namespace z = pbes_expr_optimized;
-      atermpp::vector<pbes_expression> result;
+      std::vector<pbes_expression> result;
       for (my_iterator i = p.action_summands().begin(); i != p.action_summands().end(); ++i)
       {
         data_expression    ci = i->condition();
@@ -643,7 +650,7 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
       }
       else
       {
-        atermpp::vector<pbes_expression> v;
+        std::vector<pbes_expression> v;
         for (my_iterator j = q.action_summands().begin(); j != q.action_summands().end(); ++j)
         {
           data_expression      cj = j->condition();
@@ -665,7 +672,7 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
     pbes_expression close1(const linear_process& p, const linear_process& q, my_iterator i) const
     {
       namespace z = pbes_expr_optimized;
-      atermpp::vector<pbes_expression> v;
+      std::vector<pbes_expression> v;
       variable_list e = i->summation_variables();
       const variable_list& d = p.process_parameters();
       const variable_list& d1 = q.process_parameters();
@@ -678,7 +685,7 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
         data_expression      cj = j->condition();
         variable_list        e1 = j->summation_variables();
         data_expression_list gj = j->next_state(d1);
-        pbes_expression      expr = pbes_expr::exists(e1, z::and_(cj, var(Y1(p, q, i), d + gj + e)));
+        pbes_expression      expr = pbes_expr::exists(e1, z::and_(cj, var(Y1(p, q, i), data_expression_list(d) + gj + data_expression_list(e))));
         v.push_back(expr);
       }
       return z::or_(z::join_or(v.begin(), v.end()), step(p, q, i));
@@ -691,11 +698,16 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
     /// \param d A sequence of data expressions
     /// \param d1 A sequence of data expressions
     /// \return The function result
-    pbes_expression close2(const linear_process& p, const linear_process& q, my_iterator i, data_expression_list const& d, data_expression_list const& d1) const
+    pbes_expression close2(const linear_process& p, const linear_process& q, my_iterator i, const data_expression_list& d, const data_expression_list& d1) const
     {
       namespace z = pbes_expr_optimized;
+      const variable_list& parameters = q.process_parameters();
 
-      atermpp::vector<pbes_expression> v;
+      data::mutable_map_substitution<> sigma; // q.process_parameters() := d1
+      make_substitution(parameters, d1, sigma);
+      std::set<data::variable> sigma_variables = data::find_free_variables(d1);
+
+      std::vector<pbes_expression> v;
       for (my_iterator j = q.action_summands().begin(); j != q.action_summands().end(); ++j)
       {
         if (!j->is_tau())
@@ -707,18 +719,17 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
         data_expression      cj  = j->condition();                        // cj == cj(d',e')
         data_expression_list gj  = j->next_state(q.process_parameters()); // gj == gj(d',e')
         variable_list        e1  = j->summation_variables();              // e1 == e'
-        const variable_list& parameters = q.process_parameters();
 
         // replace d' by d1 (if needed)
         if (d1 != data_expression_list(parameters.begin(), parameters.end()))
         {
-          cj = data::replace_free_variables(cj, mcrl2::data::make_sequence_sequence_substitution(q.process_parameters(), d1));
-          gj = data::replace_free_variables(gj, mcrl2::data::make_sequence_sequence_substitution(q.process_parameters(), d1));
+          cj = data::replace_variables_capture_avoiding(cj, sigma, sigma_variables);
+          gj = data::replace_variables_capture_avoiding(gj, sigma, sigma_variables);
         }
 
         // replace e' (e1) by fresh variables e'' (e1_new)
-        std::set<data::variable> used_variables = lps::find_variables(p);
-        std::set<data::variable> tmp = lps::find_variables(q);
+        std::set<data::variable> used_variables = lps::find_all_variables(p);
+        std::set<data::variable> tmp = lps::find_all_variables(q);
         used_variables.insert(tmp.begin(), tmp.end());
 
         std::set<std::string> used_names;
@@ -727,8 +738,11 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
           used_names.insert(std::string(k->name()));
         }
         variable_list e1_new = fresh_variables(e1, used_names);
-        data_expression    cj_new   = data::replace_free_variables(cj, mcrl2::data::make_sequence_sequence_substitution(e1, e1_new));
-        data_expression_list gj_new = data::replace_free_variables(gj, mcrl2::data::make_sequence_sequence_substitution(e1, e1_new));
+        data::mutable_map_substitution<> sigma1;
+        make_substitution(e1, atermpp::container_cast<data::data_expression_list>(e1_new), sigma1);
+        std::set<data::variable> sigma1_variables(e1_new.begin(), e1_new.end());
+        data_expression      cj_new = data::replace_variables_capture_avoiding(cj, sigma1, sigma1_variables);
+        data_expression_list gj_new = data::replace_variables_capture_avoiding(gj, sigma1, sigma1_variables);
 
         pbes_expression expr = pbes_expr::exists(e1_new, z::and_(cj_new, var(Y2(p, q, i), d + gj_new)));
         v.push_back(expr);
@@ -740,7 +754,7 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
     /// \param model A linear process specification
     /// \param spec A linear process specification
     /// \return A pbes that expresses weak bisimulation between two specifications.
-    pbes<> run(const specification& model, const specification& spec)
+    pbes run(const specification& model, const specification& spec)
     {
       namespace z = pbes_expr_optimized;
       specification spec1 = spec;
@@ -751,7 +765,7 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
 
       variable_list const& d  = m.process_parameters();
       variable_list const& d1 = s.process_parameters();
-      atermpp::vector<pbes_equation> equations;
+      std::vector<pbes_equation> equations;
 
       // E1
       equations.push_back(pbes_equation(nu(), propositional_variable(X(m, s), d + d1), z::and_(match(m, s), match(s, m))));
@@ -784,7 +798,7 @@ class weak_bisimulation_algorithm : public bisimulation_algorithm
 /// \param spec A linear process specification
 /// \return A pbes that expresses weak bisimulation between the two specifications.
 inline
-pbes<> weak_bisimulation(const specification& model, const specification& spec)
+pbes weak_bisimulation(const specification& model, const specification& spec)
 {
   return weak_bisimulation_algorithm().run(model, spec);
 }
@@ -801,7 +815,7 @@ class branching_simulation_equivalence_algorithm : public branching_bisimulation
     /// \param model A linear process specification
     /// \param spec A linear process specification
     /// \return A pbes that expresses branching simulation equivalence between two specifications.
-    pbes<> run(const specification& model, const specification& spec)
+    pbes run(const specification& model, const specification& spec)
     {
       namespace z = pbes_expr_optimized;
       specification spec1 = spec;
@@ -812,7 +826,7 @@ class branching_simulation_equivalence_algorithm : public branching_bisimulation
 
       variable_list const& d  = m.process_parameters();
       variable_list const& d1 = s.process_parameters();
-      atermpp::vector<pbes_equation> equations;
+      std::vector<pbes_equation> equations;
 
 
       // E1
@@ -842,7 +856,7 @@ class branching_simulation_equivalence_algorithm : public branching_bisimulation
 /// \param spec A linear process specification
 /// \return A pbes that expresses branching simulation equivalence between the two specifications.
 inline
-pbes<> branching_simulation_equivalence(const specification& model, const specification& spec)
+pbes branching_simulation_equivalence(const specification& model, const specification& spec)
 {
   return branching_simulation_equivalence_algorithm().run(model, spec);
 }
