@@ -12,9 +12,8 @@
 #ifndef MCRL2_PBES_DETAIL_OCCURRING_VARIABLE_VISITOR_H
 #define MCRL2_PBES_DETAIL_OCCURRING_VARIABLE_VISITOR_H
 
-#include "mcrl2/data/detail/data_functional.h"
 #include "mcrl2/pbes/pbes_expression.h"
-#include "mcrl2/pbes/pbes_expression_visitor.h"
+#include "mcrl2/pbes/traverser.h"
 
 namespace mcrl2
 {
@@ -25,19 +24,18 @@ namespace pbes_system
 namespace detail
 {
 
-/// Visitor for collecting the propositional variables that occur in a pbes expression.
-struct occurring_variable_visitor: public pbes_expression_visitor<pbes_expression>
+struct occurring_variable_visitor: public pbes_expression_traverser<occurring_variable_visitor>
 {
+  typedef pbes_expression_traverser<occurring_variable_visitor> super;
+  using super::enter;
+  using super::leave;
+  using super::operator();
+
   std::set<propositional_variable_instantiation> variables;
 
-  /// \brief Visit propositional_variable node
-  /// \param e A PBES expression
-  /// \param v A propositional variable instantiation
-  /// \return The result of visiting the node
-  bool visit_propositional_variable(const pbes_expression& /* e */, const propositional_variable_instantiation& v)
+  void operator()(const propositional_variable_instantiation& x)
   {
-    variables.insert(v);
-    return stop_recursion;
+    variables.insert(x);
   }
 };
 
