@@ -34,7 +34,7 @@ namespace detail
 {
 
 /// \brief Data structure for storing the variables that should be expanded by the finite pbesinst algorithm.
-typedef atermpp::map<core::identifier_string, std::vector<data::variable> > pbes_parameter_map;
+typedef std::map<core::identifier_string, std::vector<data::variable> > pbes_parameter_map;
 
 /// \brief Returns true if the declaration text matches with the variable d.
 inline
@@ -60,10 +60,10 @@ bool match_declaration(const std::string& text, const data::variable& d, const d
 
 /// \brief Find parameter declarations that match a given string.
 inline
-std::vector<data::variable> find_matching_parameters(const pbes<>& p, const std::string& name, const std::set<std::string>& declarations)
+std::vector<data::variable> find_matching_parameters(const pbes& p, const std::string& name, const std::set<std::string>& declarations)
 {
   std::set<data::variable> result;
-  for (atermpp::vector<pbes_equation>::const_iterator i = p.equations().begin(); i != p.equations().end(); ++i)
+  for (std::vector<pbes_equation>::const_iterator i = p.equations().begin(); i != p.equations().end(); ++i)
   {
     propositional_variable X = i->variable();
     if (name == "*" || (name == std::string(X.name())))
@@ -88,7 +88,7 @@ std::vector<data::variable> find_matching_parameters(const pbes<>& p, const std:
 
 /// \brief Parses parameter selection for finite pbesinst algorithm
 inline
-pbes_parameter_map parse_pbes_parameter_map(const pbes<>& p, const std::string& text)
+pbes_parameter_map parse_pbes_parameter_map(const pbes& p, const std::string& text)
 {
   using namespace boost::xpressive;
   pbes_parameter_map result;
@@ -113,7 +113,7 @@ pbes_parameter_map parse_pbes_parameter_map(const pbes<>& p, const std::string&
     match_results<std::string::const_iterator> what;
     if (!regex_match(line, what, sre))
     {
-      std::cerr << "warning: ignoring selection '" << line << "'" << std::endl;
+      mCRL2log(log::warning) << "ignoring selection '" << line << "'" << std::endl;
       continue;
     }
     std::string X = what[1];
@@ -133,7 +133,7 @@ pbes_parameter_map parse_pbes_parameter_map(const pbes<>& p, const std::string&
   {
     std::set<std::string> v = q->second;
     parameter_declarations.erase(q);
-    for (atermpp::vector<pbes_equation>::const_iterator i = p.equations().begin(); i != p.equations().end(); ++i)
+    for (std::vector<pbes_equation>::const_iterator i = p.equations().begin(); i != p.equations().end(); ++i)
     {
       std::string name = i->variable().name();
       parameter_declarations[name].insert(v.begin(), v.end());
