@@ -14,7 +14,6 @@
 #define MCRL2_PBES_DETAIL_PPG_REWRITER_H
 
 #include <stack>
-#include "mcrl2/atermpp/stack.h"
 #include "mcrl2/pbes/traverser.h"
 #include "mcrl2/pbes/pbes_functions.h"
 
@@ -83,12 +82,12 @@ struct ppg_rewriter: public pbes_expression_traverser<ppg_rewriter>
   };
 
   bool result;
-  atermpp::vector<pbes_equation> equations;
+  std::vector<pbes_equation> equations;
   std::stack<expression_mode> mode_stack;
   std::stack<fixpoint_symbol> symbol_stack;
   std::stack<propositional_variable> variable_stack;
   std::stack<data::variable_list> quantifier_variable_stack;
-  atermpp::stack<pbes_expression> expression_stack;
+  std::stack<pbes_expression> expression_stack;
   fresh_variable_name_generator name_generator;
 
   template <typename Container>
@@ -112,16 +111,6 @@ struct ppg_rewriter: public pbes_expression_traverser<ppg_rewriter>
     expression_stack.push(x);
   }
 
-  void enter(const true_& x)
-  {
-    expression_stack.push(x);
-  }
-
-  void enter(const false_& x)
-  {
-    expression_stack.push(x);
-  }
-
   void enter(const propositional_variable_instantiation& x)
   {
     expression_stack.push(x);
@@ -139,7 +128,7 @@ struct ppg_rewriter: public pbes_expression_traverser<ppg_rewriter>
     pbes_equation new_eqn = pbes_equation(symbol, fresh_var, x);
     (*this)(new_eqn);
     propositional_variable_instantiation fresh_var_instantiation =
-        propositional_variable_instantiation(fresh_varname, variable_parameters);
+        propositional_variable_instantiation(fresh_varname, atermpp::container_cast<data::data_expression_list>(variable_parameters));
     return fresh_var_instantiation;
   }
 
@@ -262,10 +251,10 @@ struct ppg_rewriter: public pbes_expression_traverser<ppg_rewriter>
       case EXISTENTIAL:
       case DISJUNCTIVE:
       {
-        atermpp::vector<pbes_expression> conjuncts = pbes_expr::split_conjuncts(x);
+        std::vector<pbes_expression> conjuncts = pbes_expr::split_conjuncts(x);
         bool split = false;
         size_t count = 0;
-        for(atermpp::vector<pbes_expression>::iterator it = conjuncts.begin(); it != conjuncts.end(); ++it)
+        for(std::vector<pbes_expression>::iterator it = conjuncts.begin(); it != conjuncts.end(); ++it)
         {
           if (!is_simple_expression(*it))
           {
@@ -279,9 +268,9 @@ struct ppg_rewriter: public pbes_expression_traverser<ppg_rewriter>
         }
         if (split)
         {
-          atermpp::vector<pbes_expression> simple_conjuncts;
-          atermpp::vector<pbes_expression> new_conjuncts;
-          for(atermpp::vector<pbes_expression>::iterator it = conjuncts.begin(); it != conjuncts.end(); ++it)
+          std::vector<pbes_expression> simple_conjuncts;
+          std::vector<pbes_expression> new_conjuncts;
+          for(std::vector<pbes_expression>::iterator it = conjuncts.begin(); it != conjuncts.end(); ++it)
           {
             if (is_simple_expression(*it))
             {
@@ -351,10 +340,10 @@ struct ppg_rewriter: public pbes_expression_traverser<ppg_rewriter>
       case UNIVERSAL:
       case CONJUNCTIVE:
       {
-        atermpp::vector<pbes_expression> disjuncts = pbes_expr::split_disjuncts(x);
+        std::vector<pbes_expression> disjuncts = pbes_expr::split_disjuncts(x);
         bool split = false;
         size_t count = 0;
-        for(atermpp::vector<pbes_expression>::iterator it = disjuncts.begin(); it != disjuncts.end(); ++it)
+        for(std::vector<pbes_expression>::iterator it = disjuncts.begin(); it != disjuncts.end(); ++it)
         {
           if (!is_simple_expression(*it))
           {
@@ -368,9 +357,9 @@ struct ppg_rewriter: public pbes_expression_traverser<ppg_rewriter>
         }
         if (split)
         {
-          atermpp::vector<pbes_expression> simple_disjuncts;
-          atermpp::vector<pbes_expression> new_disjuncts;
-          for(atermpp::vector<pbes_expression>::iterator it = disjuncts.begin(); it != disjuncts.end(); ++it)
+          std::vector<pbes_expression> simple_disjuncts;
+          std::vector<pbes_expression> new_disjuncts;
+          for(std::vector<pbes_expression>::iterator it = disjuncts.begin(); it != disjuncts.end(); ++it)
           {
             if (is_simple_expression(*it))
             {
@@ -440,12 +429,12 @@ struct ppg_rewriter: public pbes_expression_traverser<ppg_rewriter>
 /// \brief Rewrites a PBES to a PPG.
 /// \param x a PBES
 /// \return a PPG.
-template <typename Container>
-pbes<Container> to_ppg(const pbes<Container>& x)
+inline
+pbes to_ppg(const pbes& x)
 {
   ppg_rewriter f(x.equations());
   f(x);
-  pbes<Container> result(
+  pbes result(
       x.data(),
       f.equations,
       x.initial_state());
