@@ -19,10 +19,11 @@
 #include <vector>
 #include <algorithm>
 #include "mcrl2/data/sort_expression.h"
+#include "mcrl2/data/substitutions/mutable_map_substitution.h"
+#include "mcrl2/pbes/algorithms.h"
 #include "mcrl2/pbes/pbes.h"
-#include "mcrl2/pbes/rewriter.h"
+#include "mcrl2/pbes/rewriters/data_rewriter.h"
 #include "mcrl2/pbes/replace.h"
-#include "mcrl2/pbes/remove_parameters.h"
 #include "mcrl2/utilities/logger.h"
 
 namespace mcrl2
@@ -67,10 +68,10 @@ class pbes_eqelm_algorithm
     typedef std::set<variable_type> equivalence_class;
 
     /// \brief Compares data expressions for equality.
-    DataRewriter m_data_rewriter;
+    const DataRewriter& m_data_rewriter;
 
     /// \brief Compares data expressions for equality.
-    PbesRewriter m_pbes_rewriter;
+    const PbesRewriter& m_pbes_rewriter;
 
     /// \brief The vertices of the grapth, i.e. the equivalence relations.
     /// It stores the equivalence sets for each propositional variable, for example
@@ -79,7 +80,7 @@ class pbes_eqelm_algorithm
 
     /// \brief The edges of the graph.
     /// It is a mapping from X to iocc(X).
-    std::map<string_type, atermpp::set<propositional_variable_type> > m_edges;
+    std::map<string_type, std::set<propositional_variable_type> > m_edges;
 
     /// \brief The parameters of the propositional variable declarations.
     /// These are stored inside a vector, for efficiency reasons.
@@ -104,7 +105,7 @@ class pbes_eqelm_algorithm
         {
           out << ", ";
         }
-        out << print(*i);
+        out << print(atermpp::deprecated_cast<core::identifier_string>(*i));
       }
       return out.str();
     }
@@ -134,7 +135,7 @@ class pbes_eqelm_algorithm
       std::ostringstream out;
       for (typename std::map<string_type, std::vector<equivalence_class> >::const_iterator i = m_vertices.begin(); i != m_vertices.end(); ++i)
       {
-        out << data::pp(i->first) << " -> [ ";
+        out << i->first << " -> [ ";
         const std::vector<equivalence_class>& v = i->second;
         for (typename std::vector<equivalence_class>::const_iterator j = v.begin(); j != v.end(); ++j)
         {
@@ -153,9 +154,9 @@ class pbes_eqelm_algorithm
     std::string print_edges() const
     {
       std::ostringstream out;
-      for (typename std::map<string_type, atermpp::set<propositional_variable_type> >::const_iterator i = m_edges.begin(); i != m_edges.end(); ++i)
+      for (typename std::map<string_type, std::set<propositional_variable_type> >::const_iterator i = m_edges.begin(); i != m_edges.end(); ++i)
       {
-        out << data::pp(i->first) << " -> " << print_set(i->second) << std::endl;
+        out << i->first << " -> " << print_set(i->second) << std::endl;
       }
       return out.str();
     }
@@ -166,7 +167,7 @@ class pbes_eqelm_algorithm
       std::ostringstream out;
       for (typename std::map<string_type, std::vector<equivalence_class> >::const_iterator i = m_vertices.begin(); i != m_vertices.end(); ++i)
       {
-        out << "  vertex " << data::pp(i->first) << ": ";
+        out << "  vertex " << i->first << ": ";
         for (typename std::vector<equivalence_class>::const_iterator j = i->second.begin(); j != i->second.end(); ++j)
         {
           out << print_set(*j) << " ";
@@ -211,12 +212,13 @@ class pbes_eqelm_algorithm
       for (typename std::vector<equivalence_class>::iterator j = cY.begin(); j != cY.end(); ++j)
       {
         equivalence_class& equiv = *j;
-        atermpp::map<data_term_type, equivalence_class> w;
+        std::map<data_term_type, equivalence_class> w;
         for (typename equivalence_class::iterator k = equiv.begin(); k != equiv.end(); ++k)
         {
           size_t p = index_of(*k, m_parameters[Y]);
-          pbes_system::data_rewriter<Term, DataRewriter> rewr(m_data_rewriter);
-          w[rewr(e[p], vX)].insert(*k);
+          pbes_system::data_rewriter<DataRewriter> rewr(m_data_rewriter);
+          pbes_system::pbes_expression e_p = rewr(e[p], vX);
+          w[atermpp::down_cast<const data::data_expression>(e_p)].insert(*k);
         }
         for (typename std::map<data_term_type, equivalence_class>::iterator i = w.begin(); i != w.end(); ++i)
         {
@@ -258,23 +260,23 @@ class pbes_eqelm_algorithm
     /// \brief Chooses one parameter for every equivalence class, and
     /// removes the others. All occurrences of the removed parameters
     /// are replaced by the chosen parameter.
-    template <typename Container>
-    void apply_equivalence_relations(pbes<Container>& p)
+    inline
+    void apply_equivalence_relations(pbes& p)
     {
       // first apply the substitutions to the equations
-      for (typename Container::iterator i = p.equations().begin(); i != p.equations().end(); ++i)
+      for (auto i = p.equations().begin(); i != p.equations().end(); ++i)
       {
         string_type X = i->variable().name();
         data::mutable_map_substitution<> sigma = compute_substitution(X);
         if (!sigma.empty())
         {
-          i->formula() = pbes_system::replace_free_variables(i->formula(), sigma);
+          i->formula() = pbes_system::replace_variables_capture_avoiding(i->formula(), sigma, data::substitution_variables(sigma));
         }
       }
 
       // then remove parameters
       std::map<string_type, std::vector<size_t> > to_be_removed;
-      for (typename Container::const_iterator i = p.equations().begin(); i != p.equations().end(); ++i)
+      for (auto i = p.equations().begin(); i != p.equations().end(); ++i)
       {
         string_type X = i->variable().name();
         const std::vector<equivalence_class>& eq = m_vertices[X];
@@ -286,14 +288,14 @@ class pbes_eqelm_algorithm
           }
         }
       }
-      remove_parameters(p, to_be_removed);
+      pbes_system::algorithms::remove_parameters(p, to_be_removed);
     }
 
   public:
     /// \brief Constructor.
     /// \param datar A data rewriter
     /// \param pbesr A PBES rewriter
-    pbes_eqelm_algorithm(DataRewriter datar, PbesRewriter pbesr)
+    pbes_eqelm_algorithm(const DataRewriter& datar, const PbesRewriter& pbesr)
       : m_data_rewriter(datar),
         m_pbes_rewriter(pbesr)
     {}
@@ -301,15 +303,14 @@ class pbes_eqelm_algorithm
     /// \brief Runs the eqelm algorithm
     /// \param p A pbes
     /// \param ignore_initial_state If true, the initial state is ignored.
-    template <typename Container>
-    void run(pbes<Container>& p, bool ignore_initial_state = false)
+    void run(pbes& p, bool ignore_initial_state = false)
     {
       m_vertices.clear();
       m_edges.clear();
       std::set<string_type> todo;
 
       // compute the vertices and edges of the graph
-      for (typename Container::const_iterator i = p.equations().begin(); i != p.equations().end(); ++i)
+      for (auto i = p.equations().begin(); i != p.equations().end(); ++i)
       {
         string_type name = i->variable().name();
         m_edges[name] = find_propositional_variable_instantiations(i->formula());
@@ -333,7 +334,7 @@ class pbes_eqelm_algorithm
           todo.insert(X);
           m_discovered[X] = true;
           update_equivalence_classes(kappa, vX, todo);
-          mCRL2log(log::debug) << "updated equivalence classes using initial state " << pbes_system::pp(kappa) << "\n" << print_equivalence_classes();
+          mCRL2log(log::debug) << "updated equivalence classes using initial state " << kappa << "\n" << print_equivalence_classes();
         }
       }
 
@@ -349,19 +350,19 @@ class pbes_eqelm_algorithm
 
         string_type X = *todo.begin();
         todo.erase(X);
-        mCRL2log(log::debug) << "choose todo element " << core::pp(X) << "\n";
+        mCRL2log(log::debug) << "choose todo element " << X << "\n";
 
         // create a substitution function that corresponds to cX
         data::mutable_map_substitution<> vX = compute_substitution(X);
-        const atermpp::set<propositional_variable_type>& edges = m_edges[X];
-        for (typename atermpp::set<propositional_variable_type>::const_iterator i = edges.begin(); i != edges.end(); ++i)
+        const std::set<propositional_variable_type>& edges = m_edges[X];
+        for (typename std::set<propositional_variable_type>::const_iterator i = edges.begin(); i != edges.end(); ++i)
         {
           // propagate the equivalence relations in X over the edge Ye
           const propositional_variable_type& Ye = *i;
           if (evaluate_guard(X, Ye))
           {
             update_equivalence_classes(Ye, vX, todo);
-            mCRL2log(log::debug) << "updated equivalence classes using edge " << pbes_system::pp(Ye) << "\n" << print_equivalence_classes();
+            mCRL2log(log::debug) << "updated equivalence classes using edge " << Ye << "\n" << print_equivalence_classes();
           }
         }
       }
