@@ -83,16 +83,6 @@ struct normalize_builder: public pbes_expression_builder<normalize_builder>
     return negated ? data::sort_bool::not_(x) : x;
   }
 
-  pbes_expression operator()(const true_&)
-  {
-    return negated ? tr::false_() : tr::true_();
-  }
-
-  pbes_expression operator()(const false_&)
-  {
-    return negated ? tr::true_() : tr::false_();
-  }
-
   pbes_expression operator()(const not_& x)
   {
     negated = !negated;
@@ -140,7 +130,7 @@ struct normalize_builder: public pbes_expression_builder<normalize_builder>
   {
     if (negated)
     {
-      throw mcrl2::runtime_error(std::string("normalize error: illegal argument ") + x.to_string());
+      throw mcrl2::runtime_error(std::string("normalize error: illegal argument ") + to_string(x));
     }
     return x;
   }
@@ -148,7 +138,7 @@ struct normalize_builder: public pbes_expression_builder<normalize_builder>
 /// \endcond
 
 /// \brief Checks if a pbes expression is normalized
-/// \param t A PBES expression
+/// \param x A PBES expression
 /// \return True if the pbes expression is normalized
 template <typename T>
 bool is_normalized(const T& x)
@@ -163,7 +153,7 @@ bool is_normalized(const T& x)
 /// \param x an object containing pbes expressions
 template <typename T>
 void normalize(T& x,
-               typename boost::disable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+               typename std::enable_if< !std::is_base_of< atermpp::aterm, T >::value>::type* = 0
               )
 {
   normalize_builder f;
@@ -175,7 +165,7 @@ void normalize(T& x,
 /// \param x an object containing pbes expressions
 template <typename T>
 T normalize(const T& x,
-            typename boost::enable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+            typename std::enable_if< std::is_base_of< atermpp::aterm, T >::value>::type* = 0
            )
 {
   normalize_builder f;
