@@ -14,14 +14,11 @@
 
 #include <string>
 #include <sstream>
-#include "mcrl2/atermpp/vector.h"
 #include "mcrl2/data/detail/sequence_algorithm.h"
 #include "mcrl2/data/data_specification.h"
 #include "mcrl2/pbes/fixpoint_symbol.h"
 #include "mcrl2/pbes/pbes_expression.h"
 #include "mcrl2/pbes/propositional_variable.h"
-#include "mcrl2/pbes/pbes_expression_visitor.h"
-#include "mcrl2/pbes/detail/quantifier_visitor.h"
 
 namespace mcrl2
 {
@@ -29,49 +26,14 @@ namespace mcrl2
 namespace pbes_system
 {
 
-namespace detail
-{
-
-struct propositional_variable_visitor: public pbes_expression_visitor<pbes_expression>
-{
-  struct found_propositional_variable
-    {};
-
-  /// \brief Visit propositional_variable node
-  /// \param e A term
-  /// \return The result of visiting the node
-  bool visit_propositional_variable(const pbes_expression& /* e */, const propositional_variable_instantiation& /* v */)
-  {
-    throw found_propositional_variable();
-    return true;
-  }
-};
-
-inline
-bool has_propositional_variables(const pbes_expression& t)
-{
-  propositional_variable_visitor visitor;
-  try
-  {
-    visitor.visit(t);
-  }
-  catch (propositional_variable_visitor::found_propositional_variable&)
-  {
-    return true;
-  }
-  return false;
-}
-
-} // namespace detail
-
 class pbes_equation;
 atermpp::aterm_appl pbes_equation_to_aterm(const pbes_equation& eqn);
+bool is_well_typed(const pbes_equation& eqn);
+bool has_propositional_variables(const pbes_expression& x);
 
 /// \brief pbes equation.
 class pbes_equation
 {
-    friend struct atermpp::aterm_traits<pbes_equation>;
-
   protected:
     /// \brief The fixpoint symbol of the equation
     fixpoint_symbol m_symbol;
@@ -82,32 +44,6 @@ class pbes_equation
     /// \brief The expression on the right hand side of the equation
     pbes_expression m_formula;
 
-    /// \brief Protects the term from being freed during garbage collection.
-    void protect() const
-    {
-      m_symbol.protect();
-      m_variable.protect();
-      m_formula.protect();
-    }
-
-    /// \brief Unprotect the term.
-    /// Releases protection of the term which has previously been protected through a
-    /// call to protect.
-    void unprotect() const
-    {
-      m_symbol.unprotect();
-      m_variable.unprotect();
-      m_formula.unprotect();
-    }
-
-    /// \brief Mark the term for not being garbage collected.
-    void mark() const
-    {
-      m_symbol.mark();
-      m_variable.mark();
-      m_formula.mark();
-    }
-
   public:
     /// \brief The expression type of the equation.
     typedef pbes_expression term_type;
@@ -124,7 +60,7 @@ class pbes_equation
 
     /// \brief Constructor.
     /// \param t A term
-    pbes_equation(atermpp::aterm_appl t)
+    pbes_equation(const atermpp::aterm_appl& t)
     {
       assert(core::detail::check_rule_PBEqn(t));
       atermpp::aterm_appl::iterator i = t.begin();
@@ -133,6 +69,18 @@ class pbes_equation
       m_formula  = pbes_expression(*i);
     }
 
+    /// \brief Constructor.
+    /// \param t1 A term
+    explicit pbes_equation(const atermpp::aterm& t1)
+    {
+      atermpp::aterm_appl t(t1);
+      assert(core::detail::check_rule_PBEqn(t));
+      atermpp::aterm_appl::iterator i = t.begin();
+      m_symbol   = fixpoint_symbol(*i++);
+      m_variable = propositional_variable(*i++);
+      m_formula  = pbes_expression(*i);
+    }
+
     /// \brief Constructor.
     /// \param symbol A fixpoint symbol
     /// \param variable A propositional variable declaration
@@ -190,59 +138,44 @@ class pbes_equation
     /// \brief Returns true if the predicate formula on the right hand side contains no predicate variables.
     // (Comment Wieger: is_const would be a better name)
     /// \return True if the predicate formula on the right hand side contains no predicate variables.
-    bool is_solved() const
+    bool is_solved() const;
+
+    /// \brief Swaps the contents
+    void swap(pbes_equation& other)
     {
-      return !detail::has_propositional_variables(formula());
-    }
-
-    /// \brief Checks if the equation is well typed
-    /// \return True if
-    /// <ul>
-    /// <li>the binding variable parameters have unique names</li>
-    /// <li>the names of the quantifier variables in the equation are disjoint with the binding variable parameter names</li>
-    /// <li>within the scope of a quantifier variable in the formula, no other quantifier variables with the same name may occur</li>
-    /// </ul>
-    bool is_well_typed() const
-    {
-      // check 1)
-      if (data::detail::sequence_contains_duplicates(
-            boost::make_transform_iterator(variable().parameters().begin(), data::detail::variable_name()),
-            boost::make_transform_iterator(variable().parameters().end()  , data::detail::variable_name())
-          )
-         )
-      {
-        std::cerr << "pbes_equation::is_well_typed() failed: the names of the binding variable parameters are not unique" << std::endl;
-        return false;
-      }
-
-      // check 2)
-      detail::quantifier_visitor qvisitor;
-      qvisitor.visit(formula());
-      if (data::detail::sequences_do_overlap(
-            boost::make_transform_iterator(variable().parameters().begin(), data::detail::variable_name()),
-            boost::make_transform_iterator(variable().parameters().end()  , data::detail::variable_name()),
-            boost::make_transform_iterator(qvisitor.variables.begin()     , data::detail::variable_name()),
-            boost::make_transform_iterator(qvisitor.variables.end()       , data::detail::variable_name())
-          )
-         )
-      {
-        std::cerr << "pbes_equation::is_well_typed() failed: the names of the quantifier variables and the names of the binding variable parameters are not disjoint in expression " << pbes_system::pp(formula()) << std::endl;
-        return false;
-      }
-
-      // check 3)
-      detail::quantifier_name_clash_visitor nvisitor;
-      nvisitor.visit(formula());
-      if (nvisitor.result)
-      {
-        std::cerr << "pbes_equation::is_well_typed() failed: the quantifier variable " << data::pp(nvisitor.name_clash) << " occurs within the scope of a quantifier variable with the same name." << std::endl;
-        return false;
-      }
-
-      return true;
+      using std::swap;
+      swap(m_symbol, other.m_symbol);
+      swap(m_variable, other.m_variable);
+      swap(m_formula, other.m_formula);
     }
 };
 
+//--- start generated class pbes_equation ---//
+/// \brief list of pbes_equations
+typedef atermpp::term_list<pbes_equation> pbes_equation_list;
+
+/// \brief vector of pbes_equations
+typedef std::vector<pbes_equation>    pbes_equation_vector;
+
+// prototype declaration
+std::string pp(const pbes_equation& x);
+
+/// \brief Outputs the object to a stream
+/// \param out An output stream
+/// \return The output stream
+inline
+std::ostream& operator<<(std::ostream& out, const pbes_equation& x)
+{
+  return out << pbes_system::pp(x);
+}
+
+/// \brief swap overload
+inline void swap(pbes_equation& t1, pbes_equation& t2)
+{
+  t1.swap(t2);
+}
+//--- end generated class pbes_equation ---//
+
 inline bool
 operator==(const pbes_equation& x, const pbes_equation& y)
 {
@@ -257,19 +190,15 @@ operator!=(const pbes_equation& x, const pbes_equation& y)
   return !(x == y);
 }
 
-/// \brief Conversion to ATermAppl.
-/// \return The specification converted to ATerm format.
+/// \brief Conversion to atermaPpl.
+/// \return The specification converted to aterm format.
 inline
 atermpp::aterm_appl pbes_equation_to_aterm(const pbes_equation& eqn)
 {
-  return core::detail::gsMakePBEqn(eqn.symbol(), eqn.variable(), eqn.formula());
+  return atermpp::aterm_appl(core::detail::function_symbol_PBEqn(), eqn.symbol(), eqn.variable(), eqn.formula());
 }
 
-/// \brief vector of process equations
-typedef atermpp::vector<pbes_equation> pbes_equation_vector;
-
 // template function overloads
-std::string pp(const pbes_equation& x);
 std::string pp(const pbes_equation_vector& x);
 void normalize_sorts(pbes_equation_vector& x, const data::data_specification& dataspec);
 std::set<data::variable> find_free_variables(const pbes_system::pbes_equation& x);
@@ -278,28 +207,4 @@ std::set<data::variable> find_free_variables(const pbes_system::pbes_equation& x
 
 } // namespace mcrl2
 
-/// \cond INTERNAL_DOCS
-namespace atermpp
-{
-
-template<>
-struct aterm_traits<mcrl2::pbes_system::pbes_equation>
-{
-  static void protect(const mcrl2::pbes_system::pbes_equation& t)
-  {
-    t.protect();
-  }
-  static void unprotect(const mcrl2::pbes_system::pbes_equation& t)
-  {
-    t.unprotect();
-  }
-  static void mark(const mcrl2::pbes_system::pbes_equation& t)
-  {
-    t.mark();
-  }
-};
-
-} // namespace atermpp
-/// \endcond
-
 #endif // MCRL2_PBES_PBES_EQUATION_H
