@@ -18,15 +18,15 @@
 #include <vector>
 #include <set>
 #include <sstream>
-#include "mcrl2/atermpp/map.h"
-#include "mcrl2/atermpp/set.h"
-#include "mcrl2/data/classic_enumerator.h"
+#include "mcrl2/data/enumerator.h"
 #include "mcrl2/data/replace.h"
 #include "mcrl2/data/detail/rewrite_container.h"
+#include "mcrl2/data/substitutions/mutable_indexed_substitution.h"
+#include "mcrl2/pbes/algorithms.h"
 #include "mcrl2/pbes/pbes_expression.h"
-#include "mcrl2/pbes/replace.h"
-#include "mcrl2/pbes/detail/data_rewrite_builder.h"
-#include "mcrl2/pbes/detail/instantiate_global_variables.h"
+#include "mcrl2/pbes/rewriters/data_rewriter.h"
+#include "mcrl2/pbes/detail/pbes_parameter_map.h"
+#include "mcrl2/utilities/exception.h"
 #include "mcrl2/utilities/logger.h"
 
 namespace mcrl2
@@ -36,10 +36,10 @@ namespace pbes_system
 {
 
 /// \brief Data structure for storing the indices of the variables that should be expanded by the finite pbesinst algorithm.
-typedef atermpp::map<core::identifier_string, std::vector<size_t> > pbesinst_index_map;
+typedef std::map<core::identifier_string, std::vector<size_t> > pbesinst_index_map;
 
 /// \brief Data structure for storing the variables that should be expanded by the finite pbesinst algorithm.
-typedef atermpp::map<core::identifier_string, std::vector<data::variable> > pbesinst_variable_map;
+typedef std::map<core::identifier_string, std::vector<data::variable> > pbesinst_variable_map;
 
 /// \brief Function object for renaming a propositional variable instantiation
 struct pbesinst_finite_rename
@@ -58,6 +58,14 @@ struct pbesinst_finite_rename
   }
 };
 
+/// \brief Exception that is used to signal an empty parameter selection
+struct empty_parameter_selection: public mcrl2::runtime_error
+{
+  empty_parameter_selection(const std::string& msg)
+    : mcrl2::runtime_error(msg)
+  {}
+};
+
 namespace detail
 {
 
@@ -90,6 +98,8 @@ compose<Function1, Function2> make_compose(const Function1& f1, const Function2&
 
 /// \brief Computes the subset with variables of finite sort and infinite.
 /// \param X A propositional variable instantiation
+/// \param index_map a container storing the indices of the variables that
+///        should be expanded by the finite pbesinst algorithm.
 /// \param finite A sequence of data expressions
 /// \param infinite A sequence of data expressions
 template <typename PropositionalVariable>
@@ -121,24 +131,30 @@ void split_parameters(const PropositionalVariable& X,
 }
 
 /// \brief Visitor that applies a propositional variable substitution to a pbes expression.
-template <typename DataRewriter, typename RenameFunction, typename Substitution>
-struct pbesinst_finite_builder: public pbes_system::detail::data_rewrite_builder<pbes_expression, DataRewriter, Substitution>
+template <typename DataRewriter, typename SubstitutionFunction>
+struct pbesinst_finite_builder: public pbes_system::detail::data_rewriter_builder<pbesinst_finite_builder<DataRewriter, SubstitutionFunction>, DataRewriter, SubstitutionFunction>
 {
-  typedef typename pbes_system::detail::data_rewrite_builder<pbes_expression, DataRewriter, Substitution> super;
+  typedef pbes_system::detail::data_rewriter_builder<pbesinst_finite_builder, DataRewriter, SubstitutionFunction> super;
+  using super::enter;
+  using super::leave;
+  using super::operator();
+  using super::sigma;
+
   typedef core::term_traits<pbes_expression> tr;
 
-  const RenameFunction& m_rename;
+  const pbesinst_finite_rename& m_rename;
   const data::data_specification& m_data_spec;
   const pbesinst_index_map& m_index_map;
   const pbesinst_variable_map& m_variable_map;
 
-  pbesinst_finite_builder(const DataRewriter& r,
-                          const RenameFunction& rho,
+  pbesinst_finite_builder(const DataRewriter& R,
+                          SubstitutionFunction& sigma,
+                          const pbesinst_finite_rename& rho,
                           const data::data_specification& data_spec,
                           const pbesinst_index_map& index_map,
                           const pbesinst_variable_map& variable_map
                          )
-    : super(r),
+    : super(R, sigma),
       m_rename(rho),
       m_data_spec(data_spec),
       m_index_map(index_map),
@@ -151,14 +167,14 @@ struct pbesinst_finite_builder: public pbes_system::detail::data_rewrite_builder
   {
     std::ostringstream out;
     out << "<finite>";
-    for (std::vector<data::data_expression>::const_iterator i = finite_parameters.begin(); i != finite_parameters.end(); ++i)
+    for (auto i = finite_parameters.begin(); i != finite_parameters.end(); ++i)
     {
-      out << data::pp(*i) << " ";
+      out << *i << " ";
     }
     out << "<infinite>";
-    for (std::vector<data::data_expression>::const_iterator i = infinite_parameters.begin(); i != infinite_parameters.end(); ++i)
+    for (auto i = infinite_parameters.begin(); i != infinite_parameters.end(); ++i)
     {
-      out << data::pp(*i) << " ";
+      out << *i << " ";
     }
     out << std::endl;
     return out.str();
@@ -186,22 +202,19 @@ struct pbesinst_finite_builder: public pbes_system::detail::data_rewrite_builder
     return result;
   }
 
-  /// \brief Visit propositional_variable node
-  /// \param x A term
-  /// \return The result of visiting the node
-  pbes_expression visit_propositional_variable(const pbes_expression& x, const propositional_variable_instantiation& v, Substitution& sigma)
+  pbes_expression operator()(const propositional_variable_instantiation& x)
   {
-    mCRL2log(log::debug1) << "visit " << data::pp(x) << "\n";
+    typedef data::enumerator_list_element_with_substitution<> enumerator_element;
 
-    // TODO: this code contains too much conversion between vectors and ATerm lists
+    // TODO: this code contains too much conversion between vectors and aterm lists
     std::vector<data::data_expression> finite_parameters;
     std::vector<data::data_expression> infinite_parameters;
-    split_parameters(v, m_index_map, finite_parameters, infinite_parameters);
+    split_parameters(x, m_index_map, finite_parameters, infinite_parameters);
     mCRL2log(log::debug, "pbesinst_finite") << print_parameters(finite_parameters, infinite_parameters);
-    data::data_expression_list d = atermpp::convert<data::data_expression_list>(finite_parameters);
-    data::data_expression_list e = atermpp::convert<data::data_expression_list>(infinite_parameters);
-    core::identifier_string Xi = v.name();
-    // v = Xi(d,e)
+    data::data_expression_list d = data::data_expression_list(finite_parameters.begin(),finite_parameters.end());
+    data::data_expression_list e = data::data_expression_list(infinite_parameters.begin(),infinite_parameters.end());
+    core::identifier_string Xi = x.name();
+    // x = Xi(d,e)
 
     pbesinst_variable_map::const_iterator vi = m_variable_map.find(Xi);
     std::vector<data::variable> di;
@@ -210,20 +223,29 @@ struct pbesinst_finite_builder: public pbes_system::detail::data_rewrite_builder
       di = vi->second;
     }
 
-    atermpp::set<pbes_expression> result;
-    data::classic_enumerator<> enumerator(m_data_spec, super::m_data_rewriter);
-    for (data::classic_enumerator<>::iterator i=enumerator.begin(di, data::sort_bool::true_());
-              i != enumerator.end(); ++i)
+    std::set<pbes_expression> result;
+    data::enumerator_algorithm_with_iterator<> enumerator(super::R, m_data_spec, super::R);
+    mcrl2::data::mutable_indexed_substitution<> local_sigma;
+    const data::variable_list vl(di.begin(), di.end());
+    std::deque<enumerator_element> enumerator_deque(1, enumerator_element(vl, data::sort_bool::true_()));
+    for (auto i = enumerator.begin(local_sigma, enumerator_deque); i != enumerator.end(); ++i)
     {
-      mCRL2log(log::debug1) << "sigma = " << data::print_substitution(sigma) << "\n";
-      mCRL2log(log::debug1) << "*i    = " << data::print_substitution(*i) << "\n";
+      mCRL2log(log::debug1) << "sigma = " << sigma << "\n";
+      data::mutable_indexed_substitution<> sigma_i;
+      /* data::data_expression_list::const_iterator k = i->begin();
+      for (auto j = di.begin(); j != di.end(); ++j, ++k)
+      {
+        sigma_i[*j]=*k;
+      } */
+      i->add_assignments(vl,sigma_i,super::R);
+      mCRL2log(log::debug1) << "*i    = " << sigma_i << "\n";
       data::data_expression_list d_copy = d;
-      data::detail::rewrite_container(d_copy, super::m_data_rewriter, sigma);
+      data::detail::rewrite_container(d_copy, super::R, sigma);
       data::data_expression_list e_copy = e;
-      data::detail::rewrite_container(e_copy, super::m_data_rewriter, sigma);
+      data::detail::rewrite_container(e_copy, super::R, sigma);
 
-      data::data_expression_list di_copy = atermpp::convert<data::data_expression_list>(di);
-      di_copy = data::replace_free_variables(di_copy, *i);
+      data::data_expression_list di_copy = atermpp::container_cast<data::data_expression_list>(vl);
+      di_copy = data::replace_free_variables(di_copy, sigma_i);
 
       data::data_expression c = make_condition(di_copy, d_copy);
       mCRL2log(log::debug1) << "c = " << data::pp(c) << "\n";
@@ -243,14 +265,18 @@ struct pbesinst_finite_builder: public pbes_system::detail::data_rewrite_builder
     std::vector<data::data_expression> finite_parameters_vector;
     std::vector<data::data_expression> infinite_parameters_vector;
     split_parameters(init, m_index_map, finite_parameters_vector, infinite_parameters_vector);
-    data::data_expression_list finite_parameters = atermpp::convert<data::data_expression_list>(finite_parameters_vector);
-    data::data_expression_list infinite_parameters = atermpp::convert<data::data_expression_list>(infinite_parameters_vector);
+    data::data_expression_list finite_parameters = data::data_expression_list(finite_parameters_vector.begin(),finite_parameters_vector.end());
+    data::data_expression_list infinite_parameters = data::data_expression_list(infinite_parameters_vector.begin(),infinite_parameters_vector.end());
 
-    data::detail::rewrite_container(finite_parameters, super::m_data_rewriter);
-    data::detail::rewrite_container(infinite_parameters, super::m_data_rewriter);
+    data::detail::rewrite_container(finite_parameters, super::R);
+    data::detail::rewrite_container(infinite_parameters, super::R);
     core::identifier_string X = m_rename(init.name(), finite_parameters);
     return propositional_variable_instantiation(X, infinite_parameters);
   }
+
+#ifdef BOOST_MSVC
+#include "mcrl2/core/detail/builder_msvc.inc.h"
+#endif
 };
 
 } // namespace detail
@@ -315,8 +341,7 @@ class pbesinst_finite_algorithm
   public:
 
     /// \brief Constructor.
-    /// \param print_equations If true, the generated equations are printed
-    /// \param print_rewriter_output If true, invocations of the rewriter are printed
+    /// \param rewriter_strategy Strategy to be used for the data rewriter.
     pbesinst_finite_algorithm(data::rewriter::strategy rewriter_strategy = data::jitty)
       : m_rewriter_strategy(rewriter_strategy)
     {}
@@ -324,11 +349,11 @@ class pbesinst_finite_algorithm
     /// \brief Runs the algorithm.
     /// \param p A PBES
     /// \param variable_map A map containing the finite parameters that should be expanded by the algorithm.
-    void run(pbes<>& p,
+    void run(pbes& p,
              const pbesinst_variable_map& variable_map
             )
     {
-      pbes_system::detail::instantiate_global_variables(p);
+      pbes_system::algorithms::instantiate_global_variables(p);
       m_equation_count = 0;
 
       // compute index map corresponding to the variable map
@@ -337,35 +362,38 @@ class pbesinst_finite_algorithm
 
       data::rewriter rewr(p.data(), m_rewriter_strategy);
 
-      typedef data::classic_enumerator<>::substitution_type substitution_type;
-      detail::pbesinst_finite_builder<data::rewriter, pbesinst_finite_rename, substitution_type> visitor(rewr, pbesinst_finite_rename(), p.data(), index_map, variable_map);
-
       // compute new equations
-      atermpp::vector<pbes_equation> equations;
-      for (atermpp::vector<pbes_equation>::const_iterator i = p.equations().begin(); i != p.equations().end(); ++i)
+      std::vector<pbes_equation> equations;
+      for (auto i = p.equations().begin(); i != p.equations().end(); ++i)
       {
         std::vector<data::variable> finite_parameters;
         std::vector<data::variable> infinite_parameters;
         detail::split_parameters(i->variable(), index_map, finite_parameters, infinite_parameters);
         data::variable_list infinite = atermpp::convert<data::variable_list>(infinite_parameters);
 
-        data::classic_enumerator<> enumerator(p.data(),rewr);
-        for (data::classic_enumerator<>::iterator j=enumerator.begin(finite_parameters, data::sort_bool::true_());
-                     j != enumerator.end(); ++j)
+        typedef data::enumerator_list_element_with_substitution<> enumerator_element;
+        data::enumerator_algorithm_with_iterator<> enumerator(rewr, p.data(), rewr);
+        mcrl2::data::mutable_indexed_substitution<> local_sigma;
+        const data::variable_list vl(finite_parameters.begin(), finite_parameters.end());
+        std::deque <enumerator_element> enumerator_deque(1, enumerator_element(vl, data::sort_bool::true_()));
+        for (auto j = enumerator.begin(local_sigma, enumerator_deque); j != enumerator.end(); ++j)
         {
-          // apply the substitution *j
-          // TODO: use a generic substitution routine (does that already exist in the data library?)
+          // apply the substitution contained in the enumerated element.
+          data::mutable_indexed_substitution<> sigma_j;
+          j->add_assignments(vl,sigma_j,rewr);
+
           std::vector<data::data_expression> finite;
           for (std::vector<data::variable>::iterator k = finite_parameters.begin(); k != finite_parameters.end(); ++k)
           {
-            mCRL2log(log::debug1) << "sigma(" << data::pp(*k) << ") = " << data::pp((*j)(*k)) << "\n";
-            finite.push_back((*j)(*k));
+            mCRL2log(log::debug1) << "sigma(" << data::pp(*k) << ") = " << data::pp(sigma_j(*k)) << "\n";
+            finite.push_back(sigma_j(*k));
           }
           core::identifier_string name = pbesinst_finite_rename()(i->variable().name(), finite);
           propositional_variable X(name, infinite);
           mCRL2log(log::debug1) << "formula before = " << pbes_system::pp(i->formula()) << "\n";
-          mCRL2log(log::debug1) << "sigma = " << data::print_substitution(*j) << "\n";
-          pbes_expression formula = visitor(i->formula(), *j);
+          mCRL2log(log::debug1) << "sigma = " << sigma_j << "\n";
+          detail::pbesinst_finite_builder<data::rewriter, data::mutable_indexed_substitution<>> visitor(rewr, sigma_j, pbesinst_finite_rename(), p.data(), index_map, variable_map);
+          pbes_expression formula = visitor(i->formula());
           mCRL2log(log::debug1) << "formula after  = " << pbes_system::pp(formula) << "\n";
           pbes_equation eqn(i->symbol(), X, formula);
           equations.push_back(eqn);
@@ -375,6 +403,8 @@ class pbesinst_finite_algorithm
       }
 
       // compute new initial state
+      data::no_substitution sigma;
+      detail::pbesinst_finite_builder<data::rewriter, data::no_substitution> visitor(rewr, sigma, pbesinst_finite_rename(), p.data(), index_map, variable_map);
       propositional_variable_instantiation initial_state = visitor.visit_initial_state(p.initial_state());
 
       // assign the result
@@ -384,11 +414,11 @@ class pbesinst_finite_algorithm
 
     /// \brief Runs the algorithm.
     /// \param p A PBES
-    void run(pbes<>& p)
+    void run(pbes& p)
     {
       // put all finite variables in a variable map
       pbesinst_variable_map variable_map;
-      for (atermpp::vector<pbes_equation>::const_iterator i = p.equations().begin(); i != p.equations().end(); ++i)
+      for (auto i = p.equations().begin(); i != p.equations().end(); ++i)
       {
         data::variable_list v = i->variable().parameters();
         for (data::variable_list::const_iterator j = v.begin(); j != v.end(); ++j)
@@ -404,6 +434,35 @@ class pbesinst_finite_algorithm
     }
 };
 
+inline
+void pbesinst_finite(pbes& p, data::rewrite_strategy rewrite_strategy, const std::string& finite_parameter_selection)
+{
+  if (finite_parameter_selection.empty())
+  {
+    throw empty_parameter_selection("no finite parameters were selected!");
+  }
+  pbesinst_finite_algorithm algorithm(rewrite_strategy);
+  pbes_system::detail::pbes_parameter_map parameter_map = pbes_system::detail::parse_pbes_parameter_map(p, finite_parameter_selection);
+
+  bool is_empty = true;
+  for (auto i = parameter_map.begin(); i != parameter_map.end(); ++i)
+  {
+    if (!((i->second).empty()))
+    {
+      is_empty = false;
+      break;
+    }
+  }
+  if (is_empty)
+  {
+    mCRL2log(log::verbose) << "Warning: no parameters were found that match the string \"" + finite_parameter_selection + "\"" << std::endl;
+  }
+  else
+  {
+    algorithm.run(p, parameter_map);
+  }
+}
+
 } // namespace pbes_system
 
 } // namespace mcrl2
