@@ -19,10 +19,10 @@
 #include <map>
 #include <vector>
 #include <boost/bind.hpp>
-#include "mcrl2/atermpp/convert.h"
 #include "mcrl2/data/detail/assignment_functional.h"
 #include "mcrl2/pbes/pbes.h"
 #include "mcrl2/pbes/builder.h"
+#include "mcrl2/utilities/detail/container_utility.h"
 
 namespace mcrl2
 {
@@ -42,7 +42,7 @@ template <typename Term>
 atermpp::term_list<Term> remove_elements(atermpp::term_list<Term> l, const std::vector<size_t>& to_be_removed)
 {
   size_t index = 0;
-  atermpp::vector<Term> result;
+  std::vector<Term> result;
   std::vector<size_t>::const_iterator j = to_be_removed.begin();
   for (typename atermpp::term_list<Term>::iterator i = l.begin(); i != l.end(); ++i, ++index)
   {
@@ -55,7 +55,7 @@ atermpp::term_list<Term> remove_elements(atermpp::term_list<Term> l, const std::
       result.push_back(*i);
     }
   }
-  return atermpp::convert< atermpp::term_list< Term > >(result);
+  return atermpp::term_list< Term >(result.begin(),result.end());
 }
 
 template <typename Derived>
@@ -77,7 +77,7 @@ struct remove_parameters_builder: public pbes_system::pbes_expression_builder<De
     return propositional_variable(x.name(), detail::remove_elements(x.parameters(), to_be_removed));
   }
 
-  pbes_expression operator()(const propositional_variable_instantiation& x)
+  propositional_variable_instantiation operator()(const propositional_variable_instantiation& x)
   {
     return propositional_variable_instantiation(x.name(), detail::remove_elements(x.parameters(), to_be_removed));
   }
@@ -88,8 +88,7 @@ struct remove_parameters_builder: public pbes_system::pbes_expression_builder<De
     x.formula() = static_cast<Derived&>(*this)(x.formula());
   }
 
-  template <typename Container>
-  void operator()(pbes<Container>& x)
+  void operator()(pbes& x)
   {
     static_cast<Derived&>(*this)(x.equations());
     x.initial_state() = static_cast<Derived&>(*this)(x.initial_state());
@@ -108,7 +107,7 @@ struct remove_parameters_builder: public pbes_system::pbes_expression_builder<De
 template <typename T>
 T remove_parameters(const T& x,
                     const std::vector<size_t>& to_be_removed,
-                    typename boost::enable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                    typename std::enable_if< std::is_base_of< atermpp::aterm, T >::value>::type* = 0
                    )
 {
   return core::make_apply_builder_arg1<detail::remove_parameters_builder>(to_be_removed)(x);
@@ -121,7 +120,7 @@ T remove_parameters(const T& x,
 template <typename T>
 void remove_parameters(T& x,
                        const std::vector<size_t>& to_be_removed,
-                       typename boost::disable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                       typename std::enable_if< !std::is_base_of< atermpp::aterm, T >::value>::type* = 0
                       )
 {
   core::make_apply_builder_arg1<detail::remove_parameters_builder>(to_be_removed)(x);
@@ -153,7 +152,7 @@ struct map_based_remove_parameters_builder: public pbes_expression_builder<Deriv
 
   propositional_variable operator()(const propositional_variable& x)
   {
-    std::map<core::identifier_string, std::vector<size_t> >::const_iterator i = to_be_removed.find(x.name());
+    auto i = to_be_removed.find(x.name());
     if (i == to_be_removed.end())
     {
       return x;
@@ -161,9 +160,9 @@ struct map_based_remove_parameters_builder: public pbes_expression_builder<Deriv
     return remove_parameters(x, i->second);
   }
 
-  pbes_expression operator()(const propositional_variable_instantiation& x)
+  propositional_variable_instantiation operator()(const propositional_variable_instantiation& x)
   {
-    std::map<core::identifier_string, std::vector<size_t> >::const_iterator i = to_be_removed.find(x.name());
+    auto i = to_be_removed.find(x.name());
     if (i == to_be_removed.end())
     {
       return x;
@@ -180,8 +179,7 @@ struct map_based_remove_parameters_builder: public pbes_expression_builder<Deriv
     x.formula() = static_cast<Derived&>(*this)(x.formula());
   }
 
-  template <typename Container>
-  void operator()(pbes<Container>& x)
+  void operator()(pbes& x)
   {
     static_cast<Derived&>(*this)(x.equations());
     x.initial_state() = static_cast<Derived&>(*this)(x.initial_state());
@@ -197,7 +195,7 @@ struct map_based_remove_parameters_builder: public pbes_expression_builder<Deriv
 template <typename T>
 T remove_parameters(const T& x,
                     const std::map<core::identifier_string, std::vector<size_t> >& to_be_removed,
-                    typename boost::enable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                    typename std::enable_if< std::is_base_of< atermpp::aterm, T >::value>::type* = 0
                    )
 {
   return core::make_apply_builder_arg1<detail::map_based_remove_parameters_builder>(to_be_removed)(x);
@@ -210,7 +208,7 @@ T remove_parameters(const T& x,
 template <typename T>
 void remove_parameters(T& x,
                        const std::map<core::identifier_string, std::vector<size_t> >& to_be_removed,
-                       typename boost::disable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                       typename std::enable_if< !std::is_base_of< atermpp::aterm, T >::value>::type* = 0
                       )
 {
   core::make_apply_builder_arg1<detail::map_based_remove_parameters_builder>(to_be_removed)(x);
@@ -234,7 +232,7 @@ struct set_based_remove_parameters_builder: public pbes_expression_builder<Deriv
     : to_be_removed(to_be_removed_)
   {}
 
-  void remove_parameters(atermpp::set<data::variable>& x) const
+  void remove_parameters(std::set<data::variable>& x) const
   {
     for (std::set<data::variable>::const_iterator i = to_be_removed.begin(); i != to_be_removed.end(); ++i)
     {
@@ -244,10 +242,12 @@ struct set_based_remove_parameters_builder: public pbes_expression_builder<Deriv
 
   data::variable_list operator()(const data::variable_list& l) const
   {
+  	using utilities::detail::contains;
+
     std::vector<data::variable> result;
-    for (data::variable_list::const_iterator i = l.begin(); i != l.end(); ++i)
+    for (auto i = l.begin(); i != l.end(); ++i)
     {
-      if (to_be_removed.find(*i) == to_be_removed.end())
+      if (!contains(to_be_removed, *i))
       {
         result.push_back(*i);
       }
@@ -273,8 +273,7 @@ struct set_based_remove_parameters_builder: public pbes_expression_builder<Deriv
     x.formula() = static_cast<Derived&>(*this)(x.formula());
   }
 
-  template <typename Container>
-  void operator()(pbes<Container>& x)
+  void operator()(pbes& x)
   {
     static_cast<Derived&>(*this)(x.equations());
     x.initial_state() = static_cast<Derived&>(*this)(x.initial_state());
@@ -291,7 +290,7 @@ struct set_based_remove_parameters_builder: public pbes_expression_builder<Deriv
 template <typename T>
 T remove_parameters(const T& x,
                     const std::set<data::variable>& to_be_removed,
-                    typename boost::enable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                    typename std::enable_if< std::is_base_of< atermpp::aterm, T >::value>::type* = 0
                    )
 {
   return core::make_apply_builder_arg1<detail::set_based_remove_parameters_builder>(to_be_removed)(x);
@@ -304,7 +303,7 @@ T remove_parameters(const T& x,
 template <typename T>
 void remove_parameters(T& x,
                        const std::set<data::variable>& to_be_removed,
-                       typename boost::disable_if<typename boost::is_base_of<atermpp::aterm_base, T>::type>::type* = 0
+                       typename std::enable_if< !std::is_base_of< atermpp::aterm, T >::value>::type* = 0
                       )
 {
   core::make_apply_builder_arg1<detail::set_based_remove_parameters_builder>(to_be_removed)(x);
@@ -313,8 +312,8 @@ void remove_parameters(T& x,
 
 /// \cond INTERNAL_DOCS
 // used in pbes.h
-template <typename Container>
-void remove_pbes_parameters(pbes<Container>& x,
+inline
+void remove_pbes_parameters(pbes& x,
                             const std::set<data::variable>& to_be_removed
                            )
 {
