@@ -68,11 +68,11 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  template <typename Container>
-  void operator()(const pbes_system::pbes<Container>& x)
+  void operator()(const pbes_system::pbes& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.equations());
+    static_cast<Derived&>(*this)(x.global_variables());
     static_cast<Derived&>(*this)(x.initial_state());
     static_cast<Derived&>(*this).leave(x);
   }
@@ -84,20 +84,6 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const pbes_system::true_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
-  void operator()(const pbes_system::false_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
   void operator()(const pbes_system::not_& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -148,46 +134,41 @@ struct add_traverser_sort_expressions: public Traverser<Derived>
   void operator()(const pbes_system::pbes_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    pbes_system::pbes_expression result;
     if (data::is_data_expression(x))
     {
-      static_cast<Derived&>(*this)(data::data_expression(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::data_expression>(x));
     }
     else if (pbes_system::is_propositional_variable_instantiation(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::propositional_variable_instantiation(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_true(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::true_(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_false(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::false_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::propositional_variable_instantiation>(x));
     }
     else if (pbes_system::is_not(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::not_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::not_>(x));
     }
     else if (pbes_system::is_and(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::and_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::and_>(x));
     }
     else if (pbes_system::is_or(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::or_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::or_>(x));
     }
     else if (pbes_system::is_imp(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::imp(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::imp>(x));
     }
     else if (pbes_system::is_forall(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::forall(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::forall>(x));
     }
     else if (pbes_system::is_exists(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::exists(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::exists>(x));
+    }
+    else if (data::is_variable(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::variable>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -221,8 +202,7 @@ struct add_traverser_data_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  template <typename Container>
-  void operator()(const pbes_system::pbes<Container>& x)
+  void operator()(const pbes_system::pbes& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.equations());
@@ -237,20 +217,6 @@ struct add_traverser_data_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const pbes_system::true_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
-  void operator()(const pbes_system::false_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
   void operator()(const pbes_system::not_& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -299,46 +265,41 @@ struct add_traverser_data_expressions: public Traverser<Derived>
   void operator()(const pbes_system::pbes_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    pbes_system::pbes_expression result;
     if (data::is_data_expression(x))
     {
-      static_cast<Derived&>(*this)(data::data_expression(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::data_expression>(x));
     }
     else if (pbes_system::is_propositional_variable_instantiation(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::propositional_variable_instantiation(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_true(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::true_(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_false(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::false_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::propositional_variable_instantiation>(x));
     }
     else if (pbes_system::is_not(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::not_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::not_>(x));
     }
     else if (pbes_system::is_and(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::and_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::and_>(x));
     }
     else if (pbes_system::is_or(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::or_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::or_>(x));
     }
     else if (pbes_system::is_imp(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::imp(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::imp>(x));
     }
     else if (pbes_system::is_forall(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::forall(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::forall>(x));
     }
     else if (pbes_system::is_exists(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::exists(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::exists>(x));
+    }
+    else if (data::is_variable(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::variable>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -372,8 +333,7 @@ struct add_traverser_pbes_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  template <typename Container>
-  void operator()(const pbes_system::pbes<Container>& x)
+  void operator()(const pbes_system::pbes& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.equations());
@@ -387,20 +347,6 @@ struct add_traverser_pbes_expressions: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const pbes_system::true_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
-  void operator()(const pbes_system::false_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
   void operator()(const pbes_system::not_& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -449,46 +395,41 @@ struct add_traverser_pbes_expressions: public Traverser<Derived>
   void operator()(const pbes_system::pbes_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    pbes_system::pbes_expression result;
     if (data::is_data_expression(x))
     {
-      static_cast<Derived&>(*this)(data::data_expression(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::data_expression>(x));
     }
     else if (pbes_system::is_propositional_variable_instantiation(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::propositional_variable_instantiation(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_true(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::true_(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_false(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::false_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::propositional_variable_instantiation>(x));
     }
     else if (pbes_system::is_not(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::not_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::not_>(x));
     }
     else if (pbes_system::is_and(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::and_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::and_>(x));
     }
     else if (pbes_system::is_or(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::or_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::or_>(x));
     }
     else if (pbes_system::is_imp(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::imp(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::imp>(x));
     }
     else if (pbes_system::is_forall(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::forall(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::forall>(x));
     }
     else if (pbes_system::is_exists(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::exists(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::exists>(x));
+    }
+    else if (data::is_variable(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::variable>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -530,11 +471,11 @@ struct add_traverser_variables: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  template <typename Container>
-  void operator()(const pbes_system::pbes<Container>& x)
+  void operator()(const pbes_system::pbes& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.equations());
+    static_cast<Derived&>(*this)(x.global_variables());
     static_cast<Derived&>(*this)(x.initial_state());
     static_cast<Derived&>(*this).leave(x);
   }
@@ -546,20 +487,6 @@ struct add_traverser_variables: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const pbes_system::true_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
-  void operator()(const pbes_system::false_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
   void operator()(const pbes_system::not_& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -610,46 +537,41 @@ struct add_traverser_variables: public Traverser<Derived>
   void operator()(const pbes_system::pbes_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    pbes_system::pbes_expression result;
     if (data::is_data_expression(x))
     {
-      static_cast<Derived&>(*this)(data::data_expression(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::data_expression>(x));
     }
     else if (pbes_system::is_propositional_variable_instantiation(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::propositional_variable_instantiation(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_true(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::true_(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_false(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::false_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::propositional_variable_instantiation>(x));
     }
     else if (pbes_system::is_not(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::not_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::not_>(x));
     }
     else if (pbes_system::is_and(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::and_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::and_>(x));
     }
     else if (pbes_system::is_or(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::or_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::or_>(x));
     }
     else if (pbes_system::is_imp(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::imp(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::imp>(x));
     }
     else if (pbes_system::is_forall(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::forall(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::forall>(x));
     }
     else if (pbes_system::is_exists(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::exists(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::exists>(x));
+    }
+    else if (data::is_variable(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::variable>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
@@ -692,11 +614,11 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  template <typename Container>
-  void operator()(const pbes_system::pbes<Container>& x)
+  void operator()(const pbes_system::pbes& x)
   {
     static_cast<Derived&>(*this).enter(x);
     static_cast<Derived&>(*this)(x.equations());
+    static_cast<Derived&>(*this)(x.global_variables());
     static_cast<Derived&>(*this)(x.initial_state());
     static_cast<Derived&>(*this).leave(x);
   }
@@ -709,20 +631,6 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
     static_cast<Derived&>(*this).leave(x);
   }
 
-  void operator()(const pbes_system::true_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
-  void operator()(const pbes_system::false_& x)
-  {
-    static_cast<Derived&>(*this).enter(x);
-    // skip
-    static_cast<Derived&>(*this).leave(x);
-  }
-
   void operator()(const pbes_system::not_& x)
   {
     static_cast<Derived&>(*this).enter(x);
@@ -773,46 +681,41 @@ struct add_traverser_identifier_strings: public Traverser<Derived>
   void operator()(const pbes_system::pbes_expression& x)
   {
     static_cast<Derived&>(*this).enter(x);
-    pbes_system::pbes_expression result;
     if (data::is_data_expression(x))
     {
-      static_cast<Derived&>(*this)(data::data_expression(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::data_expression>(x));
     }
     else if (pbes_system::is_propositional_variable_instantiation(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::propositional_variable_instantiation(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_true(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::true_(atermpp::aterm_appl(x)));
-    }
-    else if (pbes_system::is_false(x))
-    {
-      static_cast<Derived&>(*this)(pbes_system::false_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::propositional_variable_instantiation>(x));
     }
     else if (pbes_system::is_not(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::not_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::not_>(x));
     }
     else if (pbes_system::is_and(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::and_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::and_>(x));
     }
     else if (pbes_system::is_or(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::or_(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::or_>(x));
     }
     else if (pbes_system::is_imp(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::imp(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::imp>(x));
     }
     else if (pbes_system::is_forall(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::forall(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::forall>(x));
     }
     else if (pbes_system::is_exists(x))
     {
-      static_cast<Derived&>(*this)(pbes_system::exists(atermpp::aterm_appl(x)));
+      static_cast<Derived&>(*this)(atermpp::down_cast<pbes_system::exists>(x));
+    }
+    else if (data::is_variable(x))
+    {
+      static_cast<Derived&>(*this)(atermpp::down_cast<data::variable>(x));
     }
     static_cast<Derived&>(*this).leave(x);
   }
