@@ -12,8 +12,6 @@
 #include <string>
 #include <iostream>
 #include <boost/test/minimal.hpp>
-#include "mcrl2/atermpp/aterm_init.h"
-#include "mcrl2/core/garbage_collection.h"
 #include "mcrl2/pbes/abstract.h"
 #include "mcrl2/pbes/txt2pbes.h"
 #include "mcrl2/pbes/detail/pbes_parameter_map.h"
@@ -23,8 +21,8 @@ using namespace mcrl2::pbes_system;
 
 void test_pbesabstract(const std::string& pbes_spec, const std::string& variable_spec, bool value_true)
 {
-  pbes<> p = txt2pbes(pbes_spec);
-  detail::pbes_parameter_map parameter_map = detail::parse_pbes_parameter_map(p, variable_spec);
+  pbes p = txt2pbes(pbes_spec);
+  pbes_system::detail::pbes_parameter_map parameter_map = pbes_system::detail::parse_pbes_parameter_map(p, variable_spec);
   pbes_abstract_algorithm algorithm;
   algorithm.run(p, parameter_map, value_true);
   std::cout << "\n-------------------------------\n" << pbes_system::pp(p) << std::endl;
@@ -68,8 +66,6 @@ void test_pbesabstract()
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT_DEBUG(argc, argv)
-
   test_pbesabstract();
 
   return 0;
