@@ -12,7 +12,6 @@
 // #define MCRL2_PBES_CONSTELM_DEBUG
 
 #include <string>
-#include <boost/foreach.hpp>
 #include <boost/test/minimal.hpp>
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/pbes/eqelm.h"
@@ -20,10 +19,6 @@
 #include "mcrl2/pbes/txt2pbes.h"
 #include "mcrl2/pbes/detail/pbes_property_map.h"
 #include "mcrl2/pbes/pbes_solver_test.h"
-#include "mcrl2/pbes/pbespgsolve.h"
-#include "mcrl2/atermpp/aterm_init.h"
-
-#include "mcrl2/core/garbage_collection.h"
 
 using namespace mcrl2;
 using namespace mcrl2::data;
@@ -36,7 +31,7 @@ std::string t1 =
   "                                        \n"
   " init X(0, 0);                          \n"
   ;
-std::string x1 = "binding_variables = X(m: Nat)";
+std::string x1 = "binding_variables = X(n: Nat)";
 
 // Example provided by Tim Willemse.
 // The parameters n and m are not equivalent, and thus should not
@@ -55,10 +50,10 @@ void test_pbes(const std::string& pbes_spec,
                bool /* remove_equations = true */,
                std::string msg = "")
 {
-  typedef simplifying_rewriter<pbes_expression, data::rewriter> my_pbes_rewriter;
+  typedef simplify_data_rewriter<data::rewriter> my_pbes_rewriter;
 
-  pbes<> p = txt2pbes(pbes_spec);
-  pbes<> q = p;
+  pbes p = txt2pbes(pbes_spec);
+  pbes q = p;
 
   // data rewriter
   data::rewriter datar(q.data());
@@ -92,15 +87,14 @@ void test_pbes(const std::string& pbes_spec,
   }
   BOOST_CHECK(diff.empty());
 
-  core::garbage_collect();
 }
 
 void test_eqelm(const std::string& pbes_spec)
 {
-  typedef simplifying_rewriter<pbes_expression, data::rewriter> my_pbes_rewriter;
+  typedef simplify_data_rewriter<data::rewriter> my_pbes_rewriter;
 
-  pbes<> p = txt2pbes(pbes_spec);
-  pbes<> q = p;
+  pbes p = txt2pbes(pbes_spec);
+  pbes q = p;
 
   data::rewriter datar(q.data());
   my_pbes_rewriter pbesr(datar);
@@ -124,7 +118,7 @@ std::string random1 =
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT_DEBUG(argc, argv)
+  log::mcrl2_logger::set_reporting_level(log::debug);
 
   bool compute_conditions = false;
   bool remove_equations = true;
