@@ -17,13 +17,11 @@
 #include "mcrl2/data/utility.h"
 #include "mcrl2/lps/linearise.h"
 #include "mcrl2/lps/detail/test_input.h"
+#include "mcrl2/modal_formula/parse.h"
 #include "mcrl2/pbes/parelm.h"
 #include "mcrl2/pbes/pbes.h"
 #include "mcrl2/pbes/lps2pbes.h"
-#include "mcrl2/pbes/rename.h"
 #include "mcrl2/pbes/complement.h"
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace std;
 using namespace mcrl2;
@@ -54,7 +52,7 @@ void test_parelm()
   specification spec    = linearise(lps::detail::ABP_SPECIFICATION());
   state_formula formula = state_formulas::parse_state_formula(TRIVIAL_FORMULA, spec);
   bool timed = false;
-  pbes<> p = lps2pbes(spec, formula, timed);
+  pbes p = lps2pbes(spec, formula, timed);
   pbes_parelm_algorithm algorithm;
   algorithm.run(p);
   BOOST_CHECK(p.is_well_typed());
@@ -62,10 +60,7 @@ void test_parelm()
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT_DEBUG(argc, argv)
-
   test_parelm();
-  core::garbage_collect();
 
   return 0;
 }
