@@ -13,8 +13,6 @@
 #include <sstream>
 #include <boost/test/minimal.hpp>
 #include "mcrl2/pbes/parse.h"
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 using namespace mcrl2::pbes_system;
@@ -37,7 +35,7 @@ const std::string VARSPEC =
 
 void test_parse()
 {
-  pbes<> p;
+  pbes p;
   std::stringstream s(PBESSPEC);
   s >> p;
 
@@ -45,12 +43,23 @@ void test_parse()
   std::cout << "x = " << pbes_system::pp(x) << std::endl;
 }
 
+void test_parse_pbes_expression()
+{
+  data::variable_vector vardecl;
+  data::parse_variables("b: Bool; n: Nat;", std::back_inserter(vardecl));
+  std::vector<propositional_variable> propvardecl;
+  propositional_variable X = parse_propositional_variable("X(b: Bool, n: Nat)", vardecl);
+  propositional_variable Y = parse_propositional_variable("Y(n: Nat)", vardecl);
+  propvardecl.push_back(X);
+  propvardecl.push_back(Y);
+  pbes_expression x = parse_pbes_expression("X(true, 2) && Y(n+1)", vardecl, propvardecl);
+  BOOST_CHECK(pbes_system::pp(x) == "X(true, 2) && Y(n + 1)");
+}
+
 int test_main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT_DEBUG(argc, argv)
-
   test_parse();
-  core::garbage_collect();
+  test_parse_pbes_expression();
 
   return 0;
 }
