@@ -18,10 +18,6 @@
 #include "mcrl2/pbes/pbes.h"
 #include "mcrl2/pbes/find.h"
 #include "mcrl2/pbes/parse.h"
-#include "mcrl2/pbes/pbes_expression_with_variables.h"
-#include "mcrl2/pbes/pbes_expression_with_propositional_variables.h"
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace std;
 using namespace mcrl2;
@@ -41,9 +37,9 @@ std::string EXPRESSIONS =
   "  val(n > 3)                             \n"
   ;
 
-void print(atermpp::set<pbes_expression> q)
+void print(std::set<pbes_expression> q)
 {
-  for (atermpp::set<pbes_expression>::iterator i = q.begin(); i != q.end(); ++i)
+  for (std::set<pbes_expression>::iterator i = q.begin(); i != q.end(); ++i)
   {
     std::cout << pbes_system::pp(*i) << std::endl;
   }
@@ -60,9 +56,9 @@ void test_accessors()
   variable d(identifier_string("d"), sort_nat::nat());
   variable_list v = make_list(d);
   pbes_expression z = d;
-  propositional_variable_instantiation X(identifier_string("X"), make_list(d));
+  propositional_variable_instantiation X(identifier_string("X"), atermpp::make_list<data::data_expression>(d));
 
-  atermpp::set<pbes_expression> q;
+  std::set<pbes_expression> q;
   q.insert(x);
   q.insert(y);
   q.insert(z);
@@ -74,7 +70,7 @@ void test_accessors()
     variable_list w;
     identifier_string s;
     data_expression e;
-    atermpp::set<pbes_expression> q1;
+    std::set<pbes_expression> q1;
 
     e = val(z);
 
@@ -116,7 +112,7 @@ void test_accessors()
     BOOST_CHECK(s == identifier_string("X"));
 
     data_expression_list f = param(X);
-    data_expression_list g = make_list(d);
+    data_expression_list g = atermpp::make_list<data::data_expression>(d);
     BOOST_CHECK(f == g);
 
     print(q);
@@ -140,7 +136,7 @@ void test_accessors()
     pbes_expression a;
     identifier_string s;
     data_expression e;
-    atermpp::set<pbes_expression> q1;
+    std::set<pbes_expression> q1;
 
     e = val(z);
 
@@ -157,69 +153,6 @@ void test_accessors()
     q1 = p::split_or(a);
     q1 = p::split_and(a);
   }
-  core::garbage_collect();
-}
-
-void test_pbes_expression_with_variables()
-{
-  typedef core::term_traits<pbes_expression_with_variables> tr;
-
-  const std::string VARSPEC =
-    "datavar         \n"
-    "  m: Nat;       \n"
-    "  n: Nat;       \n"
-    "                \n"
-    "predvar         \n"
-    "  X: Bool, Pos; \n"
-    "  Y: Nat;       \n"
-    ;
-
-  pbes_expression x = parse_pbes_expression("X(true, 2) && Y(n+1) && Y(m)", VARSPEC);
-  pbes_expression_with_variables y(x);
-  BOOST_CHECK(y.variables().size() == 0);
-  std::set<data::variable> v = pbes_system::find_free_variables(y);
-  y.variables() = data::variable_list(v.begin(), v.end());
-  BOOST_CHECK(y.variables().size() == 2);
-
-  x = parse_pbes_expression("forall k:Nat.X(true, 2) && Y(n+1) && Y(k)", VARSPEC);
-  pbes_expression_with_variables z(x);
-  BOOST_CHECK(z.variables().size() == 0);
-  v = pbes_system::find_free_variables(z);
-  z.variables() = data::variable_list(v.begin(), v.end());
-  BOOST_CHECK(z.variables().size() == 1);
-
-  pbes_expression_with_variables yz = tr::and_(y, z);
-  BOOST_CHECK(yz.variables().size() == 2);
-  core::garbage_collect();
-}
-
-void test_pbes_expression_with_propositional_variables()
-{
-  const std::string VARSPEC =
-    "datavar         \n"
-    "  m: Nat;       \n"
-    "  n: Nat;       \n"
-    "                \n"
-    "predvar         \n"
-    "  X: Bool, Pos; \n"
-    "  Y: Nat;       \n"
-    ;
-
-  pbes_expression x = parse_pbes_expression("Y(1) && Y(m)", VARSPEC);
-  std::set<data::variable> vx = pbes_system::find_free_variables(x);
-  std::set<propositional_variable_instantiation> px = find_propositional_variable_instantiations(x);
-  pbes_expression_with_propositional_variables X(x, data::variable_list(vx.begin(), vx.end()), propositional_variable_instantiation_list(px.begin(), px.end()));
-
-  pbes_expression y = parse_pbes_expression("Y(n) || Y(4)", VARSPEC);
-  std::set<data::variable> vy = pbes_system::find_free_variables(y);
-  std::set<propositional_variable_instantiation> py = find_propositional_variable_instantiations(y);
-  pbes_expression_with_propositional_variables Y(y, data::variable_list(vy.begin(), vy.end()), propositional_variable_instantiation_list(py.begin(), py.end()));
-
-  typedef core::term_traits<pbes_expression_with_propositional_variables> tr;
-  pbes_expression_with_propositional_variables Z = tr::and_(X, Y);
-  BOOST_CHECK(Z.variables().size() == 2);
-  BOOST_CHECK(Z.propositional_variables().size() == 4);
-  core::garbage_collect();
 }
 
 void test_term_traits()
@@ -314,17 +247,12 @@ void test_term_traits()
   v = tr::var(x);
   z = tr::arg(x);
 
-  core::garbage_collect();
 }
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT_DEBUG(argc, argv)
-
   test_term_traits();
   test_accessors();
-  test_pbes_expression_with_variables();
-  test_pbes_expression_with_propositional_variables();
 
   return 0;
 }
