@@ -14,8 +14,6 @@
 #include <boost/test/minimal.hpp>
 #include "mcrl2/pbes/txt2pbes.h"
 #include "mcrl2/pbes/detail/pbes_property_map.h"
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 using namespace mcrl2::pbes_system;
@@ -53,15 +51,12 @@ const std::string INFO =
 
 int test_main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
-  pbes<> p = txt2pbes(PBES_SPEC);
+  pbes p = txt2pbes(PBES_SPEC);
   pbes_system::detail::pbes_property_map info1(p);
   std::cerr << info1.to_string() << std::endl;
   pbes_system::detail::pbes_property_map info2(INFO);
   std::string diff = info1.compare(info2);
   BOOST_CHECK(diff.empty());
-  core::garbage_collect();
 
   return 0;
 }
