@@ -19,23 +19,19 @@
 #include <cstdio>
 #include <boost/test/minimal.hpp>
 #include <boost/algorithm/string.hpp>
-#include "mcrl2/atermpp/aterm_init.h"
-#include "mcrl2/atermpp/set.h"
-#include "mcrl2/atermpp/utility.h"
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/core/detail/print_utility.h"
+#include "mcrl2/atermpp/aterm_io.h"
 #include "mcrl2/data/utility.h"
+#include "mcrl2/data/detail/print_utility.h"
 #include "mcrl2/lps/linearise.h"
 #include "mcrl2/lps/detail/test_input.h"
+#include "mcrl2/modal_formula/parse.h"
 #include "mcrl2/pbes/find.h"
 #include "mcrl2/pbes/is_bes.h"
 #include "mcrl2/pbes/pbes.h"
 #include "mcrl2/pbes/parse.h"
 #include "mcrl2/pbes/lps2pbes.h"
-#include "mcrl2/pbes/pbes_expr_builder.h"
 #include "mcrl2/pbes/txt2pbes.h"
 #include "mcrl2/pbes/io.h"
-#include "mcrl2/pbes/rename.h"
 #include "mcrl2/pbes/complement.h"
 #include "mcrl2/pbes/detail/instantiate_global_variables.h"
 
@@ -51,11 +47,9 @@ using lps::linearise;
 using lps::specification;
 using pbes_system::pbes;
 using pbes_system::pbes_expression;
-using pbes_system::pbes_expr_builder;
 using pbes_system::pbes_equation;
 using pbes_system::lps2pbes;
 using pbes_system::propositional_variable_instantiation;
-using pbes_system::detail::make_quantifier_rename_builder;
 
 const std::string SPECIFICATION =
   "act a:Nat;                               \n"
@@ -136,7 +130,7 @@ void test_pbes()
   specification spec = linearise(SPECIFICATION);
   state_formula formula = state_formulas::parse_state_formula(FORMULA2, spec);
   bool timed = false;
-  pbes<> p = lps2pbes(spec, formula, timed);
+  pbes p = lps2pbes(spec, formula, timed);
   pbes_expression e = p.equations().front().formula();
 
   BOOST_CHECK(!is_bes(p));
@@ -144,19 +138,22 @@ void test_pbes()
 
   try
   {
-    p.load("non-existing file");
+    load_pbes(p, "non-existing file");
     BOOST_CHECK(false); // loading is expected to fail
   }
   catch (mcrl2::runtime_error e)
   {
   }
 
-  std::string filename = "write_to_named_text_file.pbes";
+  std::string filename = "write_term_to_text_stream.pbes";
   try
   {
-    atermpp::aterm t = atermpp::make_term("f(x)");
-    atermpp::write_to_named_text_file(t, filename);
-    p.load(filename);
+    atermpp::aterm t = atermpp::read_term_from_string("f(x)");
+    std::ofstream os;
+    os.open(filename.c_str());
+    atermpp::write_term_to_text_stream(t, os);
+    os.close();
+    load_pbes(p, filename);
     BOOST_CHECK(false); // loading is expected to fail
   }
   catch (mcrl2::runtime_error e)
@@ -164,10 +161,9 @@ void test_pbes()
     remove(filename.c_str());
   }
   filename = "pbes_test_file.pbes";
-  p.save(filename);
-  p.load(filename);
+  save_pbes(p, filename);
+  load_pbes(p, filename);
   remove(filename.c_str());
-  core::garbage_collect();
 }
 
 void test_global_variables()
@@ -183,51 +179,12 @@ void test_global_variables()
     "   X1(m, n);                             \n"
     ;
 
-  pbes<> p;
+  pbes p;
   std::stringstream s(TEXT);
   s >> p;
-  atermpp::set<variable> freevars = p.global_variables();
+  std::set<variable> freevars = p.global_variables();
   BOOST_CHECK(freevars.size() == 3);  // The global variable k does not occur in the specification,
   // but occurs in the global variables list.
-  core::garbage_collect();
-}
-
-void test_quantifier_rename_builder()
-{
-  using namespace pbes_system;
-  using namespace pbes_system::pbes_expr;
-  namespace z = pbes_system::pbes_expr;
-
-  variable mN("m", basic_sort("N"));
-  variable nN("n", basic_sort("N"));
-
-  pbes_expression f = data::equal_to(mN, nN);
-  pbes_expression g = data::not_equal_to(mN, nN);
-
-  multiset_identifier_generator generator;
-  generator.add_identifier(identifier_string("n00"));
-  generator.add_identifier(identifier_string("n01"));
-
-  pbes_expression p1 =
-    z::and_(
-      pbes_expr::forall(make_list(nN), pbes_expr::exists(make_list(nN), f)),
-      pbes_expr::forall(make_list(mN), pbes_expr::exists(make_list(mN, nN), g))
-    );
-  pbes_expression q1 = make_quantifier_rename_builder(generator).visit(p1);
-  std::cout << "p1 = " << mcrl2::pbes_system::pp(p1) << std::endl;
-  std::cout << "q1 = " << mcrl2::pbes_system::pp(q1) << std::endl;
-
-  pbes_expression p2 =
-    z::and_(
-      pbes_expr::forall(make_list(nN), pbes_expr::exists(make_list(nN), p1)),
-      pbes_expr::forall(make_list(mN), pbes_expr::exists(make_list(mN, nN), q1))
-    );
-  pbes_expression q2 = rename_quantifier_variables(p2, make_list(variable("n00", basic_sort("N")), variable("n01", basic_sort("N"))));
-  std::cout << "p2 = " << mcrl2::pbes_system::pp(p2) << std::endl;
-  std::cout << "q2 = " << mcrl2::pbes_system::pp(q2) << std::endl;
-
-  // BOOST_CHECK(false);
-  core::garbage_collect();
 }
 
 void test_complement_method_builder()
@@ -244,7 +201,6 @@ void test_complement_method_builder()
   std::cout << "q             = " << mcrl2::pbes_system::pp(q) << std::endl;
   std::cout << "complement(p) = " << mcrl2::pbes_system::pp(complement(p)) << std::endl;
   BOOST_CHECK(complement(p) == q);
-  core::garbage_collect();
 }
 
 void test_pbes_expression()
@@ -255,11 +211,6 @@ void test_pbes_expression()
   pbes_expression e = x1;
   data_expression x2 = mcrl2::pbes_system::accessors::val(e);
   BOOST_CHECK(x1 == x2);
-
-  pbes_expression v_expr = propositional_variable_instantiation("v:V");
-  propositional_variable_instantiation v1 = v_expr;
-  propositional_variable_instantiation v2(v_expr);
-  core::garbage_collect();
 }
 
 void test_trivial()
@@ -267,9 +218,8 @@ void test_trivial()
   specification spec    = linearise(lps::detail::ABP_SPECIFICATION());
   state_formula formula = state_formulas::parse_state_formula(TRIVIAL_FORMULA, spec);
   bool timed = false;
-  pbes<> p = lps2pbes(spec, formula, timed);
+  pbes p = lps2pbes(spec, formula, timed);
   BOOST_CHECK(p.is_well_typed());
-  core::garbage_collect();
 }
 
 void test_instantiate_global_variables()
@@ -284,12 +234,11 @@ void test_instantiate_global_variables()
   specification spec = linearise(spec_text);
   state_formula formula = state_formulas::parse_state_formula(formula_text, spec);
   bool timed = false;
-  pbes<> p = lps2pbes(spec, formula, timed);
+  pbes p = lps2pbes(spec, formula, timed);
   std::cout << "<before>" << mcrl2::pbes_system::pp(p) << std::endl;
   std::cout << "<lps>" << lps::pp(spec) << std::endl;
   pbes_system::detail::instantiate_global_variables(p);
   std::cout << "<after>" << pbes_system::pp(p) << std::endl;
-  core::garbage_collect();
 }
 
 void test_find_sort_expressions()
@@ -299,14 +248,12 @@ void test_find_sort_expressions()
   specification spec    = linearise(lps::detail::ABP_SPECIFICATION());
   state_formula formula = state_formulas::parse_state_formula(TRIVIAL_FORMULA, spec);
   bool timed = false;
-  pbes<> p = lps2pbes(spec, formula, timed);
+  pbes p = lps2pbes(spec, formula, timed);
   std::set<sort_expression> s;
   pbes_system::find_sort_expressions(p, std::inserter(s, s.end()));
-  std::cout << core::detail::print_set(s, data::stream_printer()) << std::endl;
-  core::garbage_collect();
+  std::cout << core::detail::print_set(s) << std::endl;
 }
 
-#ifdef MCRL2_ENABLE_IO_TEST
 void test_io()
 {
   using namespace pbes_system;
@@ -317,14 +264,14 @@ void test_io()
     "                 \n"
     "init X1;         \n"
     ;
-  pbes<> p = txt2pbes(PBES_SPEC);
-  save_pbes(p, "pbes_binary.pbes", pbes_output_pbes, false);
-  save_pbes(p, "pbes_ascii.txt",   pbes_output_pbes, true);
-  save_pbes(p, "pbes_binary.bes",  pbes_output_bes,  false);
-  save_pbes(p, "pbes_ascii.bes",   pbes_output_bes,  true);
-  save_pbes(p, "pbes.cwi",         pbes_output_cwi);
+  pbes p = txt2pbes(PBES_SPEC);
+  save_pbes(p, "pbes.pbes",  pbes_format_internal());
+  load_pbes(p, "pbes.pbes",  pbes_format_internal());
+  save_pbes(p, "pbes.aterm", pbes_format_internal_text());
+  load_pbes(p, "pbes.aterm", pbes_format_internal_text());
+  save_pbes(p, "pbes.txt",   pbes_format_text());
+  load_pbes(p, "pbes.txt",   pbes_format_text());
 }
-#endif // MCRL2_ENABLE_IO_TEST
 
 void test_is_bes()
 {
@@ -336,27 +283,21 @@ void test_is_bes()
     "                  \n"
     "init X;           \n"
     ;
-  pbes<> p = txt2pbes(text);
+  pbes p = txt2pbes(text);
   BOOST_CHECK(is_bes(p));
 }
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT_DEBUG(argc, argv)
-
   test_trivial();
   test_pbes();
   test_global_variables();
-  test_quantifier_rename_builder();
   test_complement_method_builder();
   test_pbes_expression();
   test_instantiate_global_variables();
   test_find_sort_expressions();
   test_is_bes();
-
-#ifdef MCRL2_ENABLE_IO_TEST
   test_io();
-#endif // MCRL2_ENABLE_IO_TEST
 
   return 0;
 }
