@@ -17,8 +17,6 @@
 #include "mcrl2/pbes/remove_parameters.h"
 #include "mcrl2/pbes/detail/test_utility.h"
 #include "mcrl2/atermpp/container_utility.h"
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2::core;
 using namespace mcrl2;
@@ -35,7 +33,7 @@ void test_propositional_variable()
   v.push_back(3);
   propositional_variable X1 = pbes_system::remove_parameters(X, v);
   variable_list d1 = make_list(nat("n"), bool_("b"));
-  BOOST_CHECK(X1 = propvar("X", d1));
+  BOOST_CHECK(X1 == propvar("X", d1));
 }
 
 void test_propositional_variable_instantiation()
@@ -47,7 +45,7 @@ void test_propositional_variable_instantiation()
   v.push_back(3);
   propositional_variable_instantiation X1 = pbes_system::remove_parameters(X, v);
   data_expression_list d1 = atermpp::make_list< data_expression >(nat("n"), bool_("b"));
-  BOOST_CHECK(X1 = propvarinst("X", d1));
+  BOOST_CHECK(X1 == propvarinst("X", d1));
 }
 
 void test_pbes_expression()
@@ -62,7 +60,7 @@ void test_pbes_expression()
   propositional_variable_instantiation x1 = propvarinst("X1", e1);
   propositional_variable_instantiation x2 = propvarinst("X2", e2);
 
-  pbes_expression p = pbes_expr::and_(X1, X2);
+  pbes_expression p = pbes_expr::and_(x1, x2);
 
   std::map<identifier_string, std::vector<size_t> > to_be_removed;
   std::vector<size_t> v1;
@@ -87,17 +85,14 @@ void test_pbes_expression()
     propositional_variable_instantiation x1 = propvarinst("X1", e1);
     propositional_variable_instantiation x2 = propvarinst("X2", e2);
 
-    r = pbes_expr::and_(X1, X2);
+    r = pbes_expr::and_(x1, x2);
   }
   BOOST_CHECK(q == r);
 }
 
 int test_main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT_DEBUG(argc, argv)
-
   test_propositional_variable_instantiation();
-  core::garbage_collect();
 
   return 0;
 }
