@@ -12,21 +12,16 @@
 #ifndef MCRL2_PROCESS_PARSE_H
 #define MCRL2_PROCESS_PARSE_H
 
-//#define MCRL2_NEW_ALPHABET_REDUCE
-
 #include <iostream>
 #include <string>
 #include <sstream>
 #include "mcrl2/core/parser_utility.h"
-#include "mcrl2/lps/action_parse.h"
 #include "mcrl2/utilities/text_utility.h"
 #include "mcrl2/utilities/detail/separate_keyword_section.h"
+#include "mcrl2/process/action_parse.h"
 #include "mcrl2/process/process_specification.h"
 #include "mcrl2/process/typecheck.h"
-#include "mcrl2/process/alphabet_reduction.h"
-#ifdef MCRL2_NEW_ALPHABET_REDUCE
 #include "mcrl2/process/alphabet.h"
-#endif
 
 namespace mcrl2
 {
@@ -34,10 +29,10 @@ namespace mcrl2
 namespace process
 {
 
-struct process_actions: public lps::action_actions
+struct process_actions: public process::action_actions
 {
-  process_actions(const core::parser_table& table_)
-    : lps::action_actions(table_)
+  process_actions(const core::parser& parser_)
+    : process::action_actions(parser_)
   {}
 
   core::identifier_string_list parse_ActIdSet(const core::parse_node& node)
@@ -64,7 +59,8 @@ struct process_actions: public lps::action_actions
   {
     core::identifier_string id = parse_Id(node.child(0));
     core::identifier_string_list ids = parse_IdList(node.child(2));
-    action_name_multiset lhs(atermpp::push_front(ids, id));
+    ids.push_front(id);
+    action_name_multiset lhs(ids);
     core::identifier_string rhs = parse_Id(node.child(4));
     return process::communication_expression(lhs, rhs);
   }
@@ -94,26 +90,31 @@ struct process_actions: public lps::action_actions
     return parse_RenExprList(node.child(1));
   }
 
-  process::process_expression parse_IfThen(const core::parse_node& node)
+  bool is_proc_expr_sum(const core::parse_node& node) const
   {
-    return parse_ProcExprNoIf(node.child(1));
+    return (symbol_name(node).find("ProcExpr") == 0) && (node.child_count() == 3) && (symbol_name(node.child(0)) == "sum") && (symbol_name(node.child(1)) == "VarsDeclList") && (symbol_name(node.child(2)) == ".");
   }
 
-  bool is_proc_expr_if_then(const core::parse_node& node) const
+  bool is_proc_expr_if(const core::parse_node& node) const
   {
-    return (symbol_name(node).find("ProcExpr") == 0) && (node.child_count() == 2) && (symbol_name(node.child(0)) == "DataExprUnit") && (symbol_name(node.child(1)) == "->");
+    return (symbol_name(node).find("ProcExpr") == 0) && (node.child_count() == 2) && (symbol_name(node.child(0)) == "DataExprUnit") && (node.child(1).string() == "->");
   }
 
-  bool is_proc_expr_if_then_else(const core::parse_node& node) const
+  bool is_proc_expr_else(const core::parse_node& node) const
   {
-    return (symbol_name(node).find("ProcExpr") == 0) && (node.child_count() == 2) && (symbol_name(node.child(0)) == "DataExprUnit") && (symbol_name(node.child(1)) == "IfThen");
+    return (symbol_name(node).find("ProcExpr") == 0) && (node.child_count() == 3) && is_proc_expr_if(node.child(0)) && (symbol_name(node.child(1)) == "ProcExpr") && (node.child(2).string() == "<>");
+  }
+
+  // override
+  untyped_parameter_identifier parse_Action(const core::parse_node& node)
+  {
+    return untyped_parameter_identifier(parse_Id(node.child(0)), parse_DataExprList(node.child(1)));
   }
 
-  // TODO: get rid of the gsMakeIdAssignment call
   process::process_expression parse_ProcExpr(const core::parse_node& node)
   {
     if ((node.child_count() == 1) && (symbol_name(node.child(0)) == "Action")) { return parse_Action(node.child(0)); }
-    else if ((node.child_count() == 4) && (symbol_name(node.child(0)) == "Id") && (symbol_name(node.child(1)) == "(") && (symbol_name(node.child(3)) == ")")) { return atermpp::aterm_appl(core::detail::gsMakeIdAssignment(parse_Id(node.child(0)), parse_AssignmentList(node.child(2)))); }
+    else if ((node.child_count() == 4) && (symbol_name(node.child(0)) == "Id") && (symbol_name(node.child(1)) == "(") && (symbol_name(node.child(3)) == ")")) { return untyped_process_assignment(parse_Id(node.child(0)), parse_AssignmentList(node.child(2))); }
     else if ((node.child_count() == 1) && (symbol_name(node.child(0)) == "delta")) { return delta(); }
     else if ((node.child_count() == 1) && (symbol_name(node.child(0)) == "tau")) { return tau(); }
     else if ((node.child_count() == 6) && (symbol_name(node.child(0)) == "block") && (symbol_name(node.child(1)) == "(") && (symbol_name(node.child(2)) == "ActIdSet") && (symbol_name(node.child(3)) == ",") && (symbol_name(node.child(4)) == "ProcExpr") && (symbol_name(node.child(5)) == ")")) { return block(parse_ActIdSet(node.child(2)), parse_ProcExpr(node.child(4))); }
@@ -129,53 +130,26 @@ struct process_actions: public lps::action_actions
     else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExpr") && (node.child(1).string() == "<<") && (symbol_name(node.child(2)) == "ProcExpr")) { return bounded_init(parse_ProcExpr(node.child(0)), parse_ProcExpr(node.child(2))); }
     else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExpr") && (node.child(1).string() == "@") && (symbol_name(node.child(2)) == "DataExprUnit")) { return at(parse_ProcExpr(node.child(0)), parse_DataExprUnit(node.child(2))); }
     else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExpr") && (node.child(1).string() == "|") && (symbol_name(node.child(2)) == "ProcExpr")) { return sync(parse_ProcExpr(node.child(0)), parse_ProcExpr(node.child(2))); }
-    else if ((node.child_count() == 4) && (symbol_name(node.child(0)) == "sum") && (symbol_name(node.child(1)) == "VarsDeclList") && (symbol_name(node.child(2)) == ".") && (symbol_name(node.child(3)) == "ProcExpr")) { return sum(parse_VarsDeclList(node.child(1)), parse_ProcExpr(node.child(3))); }
-    else if ((node.child_count() == 2) && is_proc_expr_if_then(node.child(0)) && (symbol_name(node.child(1)) == "ProcExpr")) { return if_then(parse_DataExprUnit(node.child(0).child(0)), parse_ProcExpr(node.child(1))); }
-    else if ((node.child_count() == 2) && is_proc_expr_if_then_else(node.child(0)) && (symbol_name(node.child(1)) == "ProcExpr")) { return if_then_else(parse_DataExprUnit(node.child(0).child(0)), parse_IfThen(node.child(0).child(1)), parse_ProcExpr(node.child(1))); }
-    report_unexpected_node(node);
-    return process::process_expression();
-  }
-
-  // TODO: get rid of the gsMakeIdAssignment call
-  process::process_expression parse_ProcExprNoIf(const core::parse_node& node)
-  {
-    if ((node.child_count() == 1) && (symbol_name(node.child(0)) == "Action")) { return parse_Action(node.child(0)); }
-    else if ((node.child_count() == 4) && (symbol_name(node.child(0)) == "Id") && (symbol_name(node.child(1)) == "(") && (symbol_name(node.child(3)) == ")")) { return atermpp::aterm_appl(core::detail::gsMakeIdAssignment(parse_Id(node.child(0)), parse_AssignmentList(node.child(2)))); }
-    else if ((node.child_count() == 1) && (symbol_name(node.child(0)) == "delta")) { return delta(); }
-    else if ((node.child_count() == 1) && (symbol_name(node.child(0)) == "tau")) { return tau(); }
-    else if ((node.child_count() == 6) && (symbol_name(node.child(0)) == "block") && (symbol_name(node.child(1)) == "(") && (symbol_name(node.child(2)) == "ActIdSet") && (symbol_name(node.child(3)) == ",") && (symbol_name(node.child(4)) == "ProcExpr") && (symbol_name(node.child(5)) == ")")) { return block(parse_ActIdSet(node.child(2)), parse_ProcExpr(node.child(4))); }
-    else if ((node.child_count() == 6) && (symbol_name(node.child(0)) == "allow") && (symbol_name(node.child(1)) == "(") && (symbol_name(node.child(2)) == "MultActIdSet") && (symbol_name(node.child(3)) == ",") && (symbol_name(node.child(4)) == "ProcExpr") && (symbol_name(node.child(5)) == ")")) { return allow(parse_MultActIdSet(node.child(2)), parse_ProcExpr(node.child(4))); }
-    else if ((node.child_count() == 6) && (symbol_name(node.child(0)) == "hide") && (symbol_name(node.child(1)) == "(") && (symbol_name(node.child(2)) == "ActIdSet") && (symbol_name(node.child(3)) == ",") && (symbol_name(node.child(4)) == "ProcExpr") && (symbol_name(node.child(5)) == ")")) { return hide(parse_ActIdSet(node.child(2)), parse_ProcExpr(node.child(4))); }
-    else if ((node.child_count() == 6) && (symbol_name(node.child(0)) == "rename") && (symbol_name(node.child(1)) == "(") && (symbol_name(node.child(2)) == "RenExprSet") && (symbol_name(node.child(3)) == ",") && (symbol_name(node.child(4)) == "ProcExpr") && (symbol_name(node.child(5)) == ")")) { return rename(parse_RenExprSet(node.child(2)), parse_ProcExpr(node.child(4))); }
-    else if ((node.child_count() == 6) && (symbol_name(node.child(0)) == "comm") && (symbol_name(node.child(1)) == "(") && (symbol_name(node.child(2)) == "CommExprSet") && (symbol_name(node.child(3)) == ",") && (symbol_name(node.child(4)) == "ProcExpr") && (symbol_name(node.child(5)) == ")")) { return comm(parse_CommExprSet(node.child(2)), parse_ProcExpr(node.child(4))); }
-    else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "(") && (symbol_name(node.child(1)) == "ProcExpr") && (symbol_name(node.child(2)) == ")")) { return parse_ProcExpr(node.child(1)); }
-    else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExprNoIf") && (node.child(1).string() == "+") && (symbol_name(node.child(2)) == "ProcExprNoIf")) { return choice(parse_ProcExprNoIf(node.child(0)), parse_ProcExprNoIf(node.child(2))); }
-    else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExprNoIf") && (node.child(1).string() == "||") && (symbol_name(node.child(2)) == "ProcExprNoIf")) { return merge(parse_ProcExprNoIf(node.child(0)), parse_ProcExprNoIf(node.child(2))); }
-    else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExprNoIf") && (node.child(1).string() == "||_") && (symbol_name(node.child(2)) == "ProcExprNoIf")) { return left_merge(parse_ProcExprNoIf(node.child(0)), parse_ProcExprNoIf(node.child(2))); }
-    else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExprNoIf") && (node.child(1).string() == ".") && (symbol_name(node.child(2)) == "ProcExprNoIf")) { return seq(parse_ProcExprNoIf(node.child(0)), parse_ProcExprNoIf(node.child(2))); }
-    else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExprNoIf") && (node.child(1).string() == "<<") && (symbol_name(node.child(2)) == "ProcExprNoIf")) { return bounded_init(parse_ProcExprNoIf(node.child(0)), parse_ProcExprNoIf(node.child(2))); }
-    else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExprNoIf") && (node.child(1).string() == "@") && (symbol_name(node.child(2)) == "DataExprUnit")) { return at(parse_ProcExprNoIf(node.child(0)), parse_DataExprUnit(node.child(2))); }
-    else if ((node.child_count() == 3) && (symbol_name(node.child(0)) == "ProcExprNoIf") && (node.child(1).string() == "|") && (symbol_name(node.child(2)) == "ProcExprNoIf")) { return sync(parse_ProcExprNoIf(node.child(0)), parse_ProcExprNoIf(node.child(2))); }
-    else if ((node.child_count() == 4) && (symbol_name(node.child(0)) == "sum") && (symbol_name(node.child(1)) == "VarsDeclList") && (symbol_name(node.child(2)) == ".") && (symbol_name(node.child(3)) == "ProcExprNoIf")) { return sum(parse_VarsDeclList(node.child(1)), parse_ProcExprNoIf(node.child(3))); }
-    else if ((node.child_count() == 2) && is_proc_expr_if_then_else(node.child(0)) && (symbol_name(node.child(1)) == "ProcExprNoIf")) { return if_then_else(parse_DataExprUnit(node.child(0).child(0)), parse_IfThen(node.child(0).child(1)), parse_ProcExprNoIf(node.child(1))); }
-    report_unexpected_node(node);
-    return process::process_expression();
+    else if ((node.child_count() == 2) && is_proc_expr_if(node.child(0)) && (symbol_name(node.child(1)) == "ProcExpr")) { return if_then(parse_DataExprUnit(node.child(0).child(0)), parse_ProcExpr(node.child(1))); }
+    else if ((node.child_count() == 2) && is_proc_expr_else(node.child(0)) && (symbol_name(node.child(1)) == "ProcExpr")) { return if_then_else(parse_DataExprUnit(node.child(0).child(0).child(0)), parse_ProcExpr(node.child(0).child(1)), parse_ProcExpr(node.child(1))); }
+    else if ((node.child_count() == 2) && is_proc_expr_sum(node.child(0)) && (symbol_name(node.child(1)) == "ProcExpr")) { return sum(parse_VarsDeclList(node.child(0).child(1)), parse_ProcExpr(node.child(1))); }
+    throw core::parse_node_unexpected_exception(m_parser, node);
   }
 
   process::process_equation parse_ProcDecl(const core::parse_node& node)
   {
     core::identifier_string name = parse_Id(node.child(0));
     data::variable_list variables = parse_VarsDeclList(node.child(1));
-    process_identifier id(name, get_sorts(variables));
+    process_identifier id(name, variables);
     return process::process_equation(id, variables, parse_ProcExpr(node.child(3)));
   }
 
-  atermpp::vector<process::process_equation> parse_ProcDeclList(const core::parse_node& node)
+  std::vector<process::process_equation> parse_ProcDeclList(const core::parse_node& node)
   {
     return parse_vector<process::process_equation>(node, "ProcDecl", boost::bind(&process_actions::parse_ProcDecl, this, _1));
   }
 
-  atermpp::vector<process::process_equation> parse_ProcSpec(const core::parse_node& node)
+  std::vector<process::process_equation> parse_ProcSpec(const core::parse_node& node)
   {
     return parse_ProcDeclList(node.child(1));
   }
@@ -206,7 +180,7 @@ struct process_actions: public lps::action_actions
     else if (symbol_name(node) == "GlobVarSpec")
     {
       data::variable_list variables = parse_GlobVarSpec(node);
-      result.global_variables() = atermpp::set<data::variable>(variables.begin(), variables.end());
+      result.global_variables() = std::set<data::variable>(variables.begin(), variables.end());
       return true;
     }
     else if (symbol_name(node) == "ActSpec")
@@ -216,7 +190,7 @@ struct process_actions: public lps::action_actions
     }
     else if (symbol_name(node) == "ProcSpec")
     {
-      atermpp::vector<process::process_equation> eqn = parse_ProcSpec(node);
+      std::vector<process::process_equation> eqn = parse_ProcSpec(node);
       result.equations().insert(result.equations().end(), eqn.begin(), eqn.end());
       return true;
     }
@@ -244,7 +218,7 @@ process_expression parse_process_expression_new(const std::string& text)
   core::parse_node node = p.parse(text, start_symbol_index, partial_parses);
   core::warn_and_or(node);
   core::warn_left_merge_merge(node);
-  process_expression result = process_actions(parser_tables_mcrl2).parse_ProcExpr(node);
+  process_expression result = process_actions(p).parse_ProcExpr(node);
   p.destroy_parse_node(node);
   return result;
 }
@@ -258,7 +232,7 @@ process_specification parse_process_specification_new(const std::string& text)
   core::parse_node node = p.parse(text, start_symbol_index, partial_parses);
   core::warn_and_or(node);
   core::warn_left_merge_merge(node);
-  process_specification result = process_actions(parser_tables_mcrl2).parse_mCRL2Spec(node);
+  process_specification result = process_actions(p).parse_mCRL2Spec(node);
   p.destroy_parse_node(node);
   return result;
 }
@@ -269,12 +243,7 @@ void complete_process_specification(process_specification& x, bool alpha_reduce
   type_check(x);
   if (alpha_reduce)
   {
-#ifdef MCRL2_NEW_ALPHABET_REDUCE
     alphabet_reduce(x);
-#else
-    alphabet_reduction reduce;
-    reduce(x);
-#endif
   }
   process::translate_user_notation(x);
   process::normalize_sorts(x, x.data());
@@ -323,7 +292,7 @@ process_expression parse_process_expression(const std::string& text,
 
 /// \brief Parses and type checks a process expression.
 /// \param[in] text The input text containing a process expression.
-/// \param[in] procspec A process specification used as context
+/// \param[in] procspec_text A textual version of a process specification used as context
 inline
 process_expression parse_process_expression(const std::string& text, const std::string& procspec_text)
 {
@@ -344,6 +313,19 @@ process_expression parse_process_expression(const std::string& text, const std::
   return spec.init();
 }
 
+/// \brief Parses and type checks a process expression. N.B. Very inefficient!
+template <typename VariableContainer>
+process_expression parse_process_expression(const std::string& text, const VariableContainer& variables, const process_specification& procspec)
+{
+  process_specification procspec1 = procspec;
+  auto& globvars = procspec1.global_variables();
+  globvars.insert(variables.begin(), variables.end());
+  std::string ptext = process::pp(procspec1);
+  ptext = utilities::regex_replace("\\binit.*;", "init " + text + ";", ptext);
+  process_specification procspec2 = parse_process_specification(ptext);
+  return procspec2.init();
+}
+
 } // namespace process
 
 } // namespace mcrl2
