@@ -14,10 +14,8 @@
 #include <set>
 #include <vector>
 #include <boost/test/minimal.hpp>
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/atermpp/make_list.h"
-#include "mcrl2/core/garbage_collection.h"
-#include "mcrl2/core/detail/print_utility.h"
+#include "mcrl2/data/detail/print_utility.h"
 #include "mcrl2/process/find.h"
 #include "mcrl2/process/parse.h"
 
@@ -58,9 +56,9 @@ void test_find()
 {
   process_expression x = parse_process_expression("a(m).P(0)", DATA_DECL, PROC_DECL);
 
-  //--- find_variables ---//
+  //--- find_all_variables ---//
   data::variable m = nat("m");
-  std::set<data::variable> v = process::find_variables(x);
+  std::set<data::variable> v = process::find_all_variables(x);
   BOOST_CHECK(v.find(m) != v.end());
 
   //--- find_sort_expressions ---//
@@ -68,7 +66,6 @@ void test_find()
   BOOST_CHECK(std::find(e.begin(), e.end(), data::sort_nat::nat()) != e.end());
   BOOST_CHECK(std::find(e.begin(), e.end(), data::sort_pos::pos()) == e.end());
 
-  core::garbage_collect();
 }
 
 void test_free_variables()
@@ -76,19 +73,17 @@ void test_free_variables()
   using atermpp::make_list;
 
   data::variable b = bool_("b");
-  data::data_expression_list d = make_list(b);
-  process_identifier P(core::identifier_string("P"), make_list(data::sort_bool::bool_()));
+  data::data_expression_list d = make_list<data::data_expression>(b);
+  process_identifier P(core::identifier_string("P"), make_list(b));
   process_instance pi(P, d);
 
   std::set<data::variable> free_variables = process::find_free_variables(pi);
-  std::cout << "free variables: " << core::detail::print_set(free_variables, data::stream_printer()) << std::endl;
+  std::cout << "free variables: " << core::detail::print_set(free_variables) << std::endl;
   BOOST_CHECK(free_variables.find(b) != free_variables.end());
 }
 
 int test_main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv);
-
   test_find();
   test_free_variables();
 
