@@ -11,10 +11,8 @@
 
 #include <iostream>
 #include <boost/test/included/unit_test_framework.hpp>
-#include <mcrl2/lps/parse.h>
-#include <mcrl2/lps/parelm.h>
-#include <mcrl2/lps/specification.h>
-#include "mcrl2/atermpp/aterm_init.h"
+#include "mcrl2/process/parse.h"
+#include "mcrl2/process/typecheck.h"
 #include "mcrl2/utilities/test_utilities.h"
 
 using mcrl2::utilities::collect_after_test_case;
@@ -81,8 +79,8 @@ BOOST_AUTO_TEST_CASE(test_bug_528a)
 {
   test_typechecker_case(
     "sort S = struct c;                  \n"
-    "map succ: S -> S;                   \n"
-    "eqn succ(c) = c;                    \n"
+    "map succ_: S -> S;                   \n"
+    "eqn succ_(c) = c;                    \n"
     "init delta;                         \n",
     true
   );
@@ -92,10 +90,10 @@ BOOST_AUTO_TEST_CASE(test_bug_528b)
 {
   test_typechecker_case(
     "sort S,T;                           \n"
-    "map  count: S # T -> Nat;           \n"
+    "map  count_: S # T -> Nat;           \n"
     "var  x:S;                           \n"
     "     y:T;                           \n"
-    "eqn  count(x, y) = 0;               \n"
+    "eqn  count_(x, y) = 0;               \n"
     "init delta;                         \n",
     true
   );
@@ -105,9 +103,9 @@ BOOST_AUTO_TEST_CASE(test_bug_528c)
 {
   test_typechecker_case(
     "sort S;                             \n"
-    "map  count: S -> Nat;               \n"
+    "map  count_: S -> Nat;               \n"
     "var  x:S;                           \n"
-    "eqn  count(x) = 0;                  \n"
+    "eqn  count_(x) = 0;                  \n"
     "init delta;                         \n",
     true
   );
@@ -345,10 +343,21 @@ BOOST_AUTO_TEST_CASE(test_double_variable_assignment_in_process)
     false);
 }
 
+BOOST_AUTO_TEST_CASE(test_typecheck)
+{
+  std::string text =
+    "act  a;\n"
+    "glob d,c,b: Bool;\n"
+         "n,m: Nat;\n"
+         "p: Pos;\n"
+    "proc P(b,c: Bool) = a . P(c = true);\n"
+    "init delta;\n"
+    ;
+  process::process_specification procspec = process::parse_process_specification_new(text);
+  process::type_check(procspec);
+}
 
 boost::unit_test::test_suite* init_unit_test_suite(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
   return 0;
 }
