@@ -8,8 +8,6 @@
 //
 /// \file bespp.cpp
 
-#include "boost.hpp" // precompiled headers
-
 #define NAME "bespp"
 #define AUTHOR "Jeroen Keiren"
 
@@ -18,10 +16,8 @@
 #include <fstream>
 
 #include "mcrl2/utilities/logger.h"
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/utilities/input_output_tool.h"
 #include "mcrl2/utilities/pbes_input_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
 #include "mcrl2/bes/io.h"
 #include "mcrl2/bes/boolean_equation_system.h"
 #include "mcrl2/bes/print.h"
@@ -75,7 +71,7 @@ class bespp_tool: public bes_input_tool<input_output_tool>
   private:
     void print_specification()
     {
-      bes::boolean_equation_system<> bes;
+      bes::boolean_equation_system bes;
       load_bes(bes,input_filename(), bes_input_format());
 
       mCRL2log(verbose) << "printing BES from " << (input_filename().empty()?"standard input":input_filename())
@@ -102,23 +98,8 @@ class bespp_tool: public bes_input_tool<input_output_tool>
     }
 };
 
-class bespp_gui_tool: public mcrl2_gui_tool<bespp_tool>
-{
-  public:
-    bespp_gui_tool()
-    {
-
-      std::vector<std::string> values;
-
-      values.clear();
-      values.push_back("default");
-      m_gui_options["format"] = create_radiobox_widget(values);
-    }
-};
-
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  return bespp_gui_tool().execute(argc, argv);
+  return bespp_tool().execute(argc, argv);
 }
 
