@@ -8,32 +8,30 @@
 //
 /// \file lps2lts_lts.cpp
 
-#include "boost.hpp" // precompiled headers
-
 // NAME is defined in lps2lts.h
 #define AUTHOR "Muck van Weerdenburg"
 
 #include <string>
 #include <cassert>
 #include <signal.h>
-#include "mcrl2/aterm/aterm2.h"
 
 #include "boost/lexical_cast.hpp"
 
-#include "mcrl2/atermpp/aterm_init.h"
-
 #include "mcrl2/utilities/logger.h"
 
 #include "mcrl2/utilities/input_output_tool.h"
 #include "mcrl2/utilities/rewriter_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
 
-#include "mcrl2/lps/multi_action.h"
-#include "mcrl2/lps/action_parse.h"
+#include "mcrl2/lps/io.h"
+#include "mcrl2/process/action_parse.h"
 
 #include "mcrl2/lts/lts_io.h"
 #include "mcrl2/lts/detail/exploration.h"
 
+#ifdef MCRL2_DISPLAY_REWRITE_STATISTICS
+#include "mcrl2/data/detail/rewrite_statistics.h"
+#endif
+
 #define __STRINGIFY(x) #x
 #define STRINGIFY(x) __STRINGIFY(x)
 
@@ -45,6 +43,7 @@ using namespace mcrl2::lts;
 using namespace mcrl2::lps;
 using namespace mcrl2::log;
 
+static
 std::list<std::string> split_actions(const std::string& s)
 {
   size_t pcount = 0;
@@ -119,7 +118,7 @@ class lps2lts_tool : public lps2lts_base
 
     bool run()
     {
-      m_options.specification.load(m_filename);
+      load_lps(m_options.specification, m_filename);
       m_options.trace_prefix = m_filename.substr(0, m_options.trace_prefix.find_last_of('.'));
 
       // check_whether_actions_on_commandline_exist(m_options.trace_actions, m_options.specification.action_labels());
@@ -168,10 +167,6 @@ class lps2lts_tool : public lps2lts_base
                  "replace free variables in the LPS with dummy values based on the value of BOOL: 'yes' (default) or 'no'", 'y').
       add_option("unused-data",
                  "do not remove unused parts of the data specification", 'u').
-      add_option("state-format", make_enum_argument<NextStateFormat>("NAME")
-                 .add_value(GS_STATE_TREE, true)
-                 .add_value(GS_STATE_VECTOR),
-                 "store state internally in format NAME:", 'f').
       add_option("bit-hash", make_optional_argument("NUM", STRINGIFY(DEFAULT_BITHASHSIZE)),
                  "use bit hashing to store states and store at most NUM states. "
                  "This means that instead of keeping a full record of all states "
@@ -185,8 +180,8 @@ class lps2lts_tool : public lps2lts_base
                  "explore at most NUM states", 'l').
       add_option("todo-max", make_mandatory_argument("NUM"),
                  "keep at most NUM states in todo lists; this option is only relevant for "
-                 "breadth-first search with bithashing, where NUM is the maximum number of "
-                 "states per level, and for depth first, where NUM is the maximum depth").
+                 "breadth-first search, where NUM is the maximum number of states per "
+                 "level, and for depth first search, where NUM is the maximum depth").
       add_option("deadlock",
                  "detect deadlocks (i.e. for every deadlock a message is printed)", 'D').
       add_option("divergence",
@@ -239,7 +234,7 @@ class lps2lts_tool : public lps2lts_base
                  "for visualisation purposes, for instance, but can cause the OUTFILE "
                  "to grow considerably. Note that this option is implicit when writing "
                  "in the AUT format.").
-      add_option("suppress","in verbose mode, do not print progress messages indicating the number of visited states and transitions"
+      add_option("suppress","in verbose mode, do not print progress messages indicating the number of visited states and transitions. "
                  "For large state spaces the number of progress messages can be quite "
                  "horrendous. This feature helps to suppress those. Other verbose messages, "
                  "such as the total number of states explored, just remain visible.").
@@ -281,8 +276,6 @@ class lps2lts_tool : public lps2lts_base
         }
       }
 
-      m_options.stateformat = parser.option_argument_as<NextStateFormat>("state-format");
-
       if (parser.options.count("bit-hash"))
       {
         m_options.bithashing  = true;
@@ -370,48 +363,6 @@ class lps2lts_tool : public lps2lts_base
 
 };
 
-class lps2lts_gui_tool: public mcrl2_gui_tool<lps2lts_tool>
-{
-  public:
-    lps2lts_gui_tool()
-    {
-      std::vector<std::string> values;
-      m_gui_options["action"] = create_textctrl_widget();
-      m_gui_options["bit-hash"] = create_textctrl_widget();
-      m_gui_options["confluence"] = create_textctrl_widget();
-      m_gui_options["deadlock"] = create_checkbox_widget();
-      m_gui_options["error-trace"] = create_checkbox_widget();
-
-
-      values.clear();
-      values.push_back("tree");
-      values.push_back("vector");
-
-      m_gui_options["state-format"] = create_radiobox_widget(values);
-      m_gui_options["divergence"] = create_checkbox_widget();
-      m_gui_options["init-tsize"] = create_textctrl_widget();
-      m_gui_options["max"] = create_textctrl_widget();
-      m_gui_options["no-info"] = create_checkbox_widget();
-
-      add_rewriter_widget();
-
-      values.clear();
-      values.push_back("breadth");
-      values.push_back("depth");
-      values.push_back("prioritized");
-      values.push_back("rprioritized");
-      values.push_back("random");
-      m_gui_options["strategy"] = create_radiobox_widget(values);
-      m_gui_options["suppress"] = create_checkbox_widget();
-      m_gui_options["trace"] = create_textctrl_widget();
-      m_gui_options["todo-max"] = create_textctrl_widget();
-
-      m_gui_options["unused-data"] = create_checkbox_widget();
-      m_gui_options["dummy"] = create_textctrl_widget();
-
-    }
-};
-
 lps2lts_tool *tool_instance;
 
 static
@@ -427,8 +378,7 @@ void premature_termination_handler(int)
 int main(int argc, char** argv)
 {
   int result;
-  MCRL2_ATERMPP_INIT(argc, argv)
-  tool_instance = new lps2lts_gui_tool();
+  tool_instance = new lps2lts_tool();
 
   signal(SIGABRT,premature_termination_handler);
   signal(SIGINT,premature_termination_handler);
@@ -437,6 +387,9 @@ int main(int argc, char** argv)
   try
   {
     result = tool_instance->execute(argc, argv);
+#ifdef MCRL2_DISPLAY_REWRITE_STATISTICS
+    mcrl2::data::detail::display_rewrite_statistics();
+#endif
   }
   catch (...)
   {
