@@ -8,21 +8,17 @@
 //
 /// \file lpsconfcheck.cpp
 
-#include "boost.hpp" // precompiled headers
-
 #include <string>
 #include <fstream>
 
 #include "mcrl2/data/parse.h"
 #include "mcrl2/data/rewriter.h"
-#include "mcrl2/lps/specification.h"
+#include "mcrl2/lps/io.h"
 #include "mcrl2/lps/confluence_checker.h"
 #include "mcrl2/lps/invariant_checker.h"
 #include "mcrl2/utilities/input_output_tool.h"
 #include "mcrl2/utilities/rewriter_tool.h"
 #include "mcrl2/utilities/prover_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 using namespace mcrl2::core;
@@ -41,10 +37,10 @@ using namespace mcrl2::log;
 /// More information about the tool and the classes used can be found in the corresponding man files.
 
 /// \brief The class confcheck_tool uses an instance of the class Confluence_Checker to check which
-/// \brief tau-summands of an LPS are confluent. The tau-actions of all confluent tau-summands can be
-/// \brief renamed to ctau, depending on the flag m_no_marking.
+/// \brief tau-summands of an LPS are confluent. The tau-actions of all confluent tau-summands are
+/// \brief renamed to ctau
 
-class confcheck_tool : public prover_tool< rewriter_tool<input_output_tool> >
+class lpsconfcheck_tool : public prover_tool< rewriter_tool<input_output_tool> >
 {
   protected:
 
@@ -66,9 +62,6 @@ class confcheck_tool : public prover_tool< rewriter_tool<input_output_tool> >
     /// \brief m_invariant_filename is checked.
     bool m_no_check;
 
-    /// \brief The flag indicating whether or not the tau-actions of the confluent summands should be renamed to ctau.
-    bool m_no_marking;
-
     /// \brief The flag indicating whether or not the confluence of a tau-summand regarding all other summands is checked.
     bool m_check_all;
 
@@ -98,7 +91,6 @@ class confcheck_tool : public prover_tool< rewriter_tool<input_output_tool> >
       super::parse_options(parser);
 
       m_no_check            = 0 < parser.options.count("no-check");
-      m_no_marking          = 0 < parser.options.count("no-marking");
       m_generate_invariants = 0 < parser.options.count("generate-invariants");
       m_check_all           = 0 < parser.options.count("check-all");
       m_counter_example     = 0 < parser.options.count("counter-example");
@@ -170,7 +162,7 @@ class confcheck_tool : public prover_tool< rewriter_tool<input_output_tool> >
 
   public:
     /// \brief Constructor setting all flags to their default values.
-    confcheck_tool() : super(
+    lpsconfcheck_tool() : super(
         "lpsconfcheck",
         "Luc Engelen",
         "mark confluent tau-summands of an LPS",
@@ -180,7 +172,6 @@ class confcheck_tool : public prover_tool< rewriter_tool<input_output_tool> >
       m_summand_number(0),
       m_generate_invariants(false),
       m_no_check(false),
-      m_no_marking(false),
       m_check_all(false),
       m_counter_example(false),
       m_dot_file_name(""),
@@ -198,9 +189,8 @@ class confcheck_tool : public prover_tool< rewriter_tool<input_output_tool> >
       mCRL2log(verbose) << "  output file:        " << m_output_filename << std::endl;
       mCRL2log(verbose) << "  data rewriter:      " << m_rewrite_strategy << std::endl;
 
-      lps::specification specification;
-
-      specification.load(m_input_filename);
+      lps::specification spec;
+      load_lps(spec, input_filename());
 
       if (!m_invariant_filename.empty())
       {
@@ -213,25 +203,21 @@ class confcheck_tool : public prover_tool< rewriter_tool<input_output_tool> >
 
         mCRL2log(verbose) << "parsing input file '" <<  m_invariant_filename << "'..." << std::endl;
 
-        m_invariant = parse_data_expression(instream, specification.data());
+        m_invariant = parse_data_expression(instream, spec.data());
 
         instream.close();
       }
 
-      if (check_invariant(specification))
+      if (check_invariant(spec))
       {
         Confluence_Checker v_confluence_checker(
-          specification, rewrite_strategy(),
+          spec, rewrite_strategy(),
           m_time_limit, m_path_eliminator, solver_type(),
-          m_apply_induction, m_no_marking, m_check_all,
+          m_apply_induction, m_check_all,
           m_counter_example, m_generate_invariants, m_dot_file_name);
 
-        specification = lps::specification(v_confluence_checker.check_confluence_and_mark(m_invariant, m_summand_number));
-
-        if (!m_no_marking)
-        {
-          specification.save(m_output_filename);
-        }
+        spec = lps::specification(v_confluence_checker.check_confluence_and_mark(m_invariant, m_summand_number));
+        save_lps(spec, output_filename());
       }
 
       return true;
@@ -264,29 +250,7 @@ class confcheck_tool : public prover_tool< rewriter_tool<input_output_tool> >
 
 };
 
-class lpsconfcheck_gui_tool: public mcrl2_gui_tool<confcheck_tool>
-{
-  public:
-    lpsconfcheck_gui_tool()
-    {
-      m_gui_options["check-all"] = create_checkbox_widget();
-      m_gui_options["counter-example"] = create_checkbox_widget();
-      m_gui_options["generate-invariants"] = create_checkbox_widget();
-      m_gui_options["invariant"] = create_filepicker_widget();
-      m_gui_options["no-marking"] = create_checkbox_widget();
-      m_gui_options["no-check"] = create_checkbox_widget();
-      m_gui_options["induction"] = create_checkbox_widget();
-      m_gui_options["print-dot"] = create_textctrl_widget();
-      add_rewriter_widget();
-      m_gui_options["summand"] = create_textctrl_widget();
-      m_gui_options["time-limit"] = create_textctrl_widget();
-      m_gui_options["smt-solver"] = create_textctrl_widget();
-    }
-};
-
-
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  return lpsconfcheck_gui_tool().execute(argc, argv);
+  return lpsconfcheck_tool().execute(argc, argv);
 }
