@@ -9,17 +9,12 @@
 /// \file lpsinvelm.cpp
 /// \brief Add your file description here.
 
-#include "boost.hpp" // precompiled headers
-
 #include <string>
 #include <fstream>
 
-#include "mcrl2/aterm/aterm_ext.h"
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/utilities/input_output_tool.h"
 #include "mcrl2/utilities/rewriter_tool.h"
 #include "mcrl2/utilities/prover_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
 #include "mcrl2/lps/tools.h"
 
 using namespace mcrl2;
@@ -39,16 +34,12 @@ using namespace mcrl2::utilities::tools;
 
 /// \brief The class invelm_tool takes an invariant and an LPS, and simplifies this LPS using
 /// \brief the invariant.
-class invelm_tool : public prover_tool< rewriter_tool<input_output_tool> >
+class lpsinvelm_tool : public prover_tool< rewriter_tool<input_output_tool> >
 {
   private:
     /// \brief The name of the file containing the invariant.
     std::string m_invariant_file_name;
 
-    /// \brief The number of the summand that is eliminated or simplified. If this number is 0,
-    /// \brief all summands will be simplified or eliminated.
-    size_t m_summand_number;
-
     /// \brief The flag indicating whether or not the invariance of the formula as found in
     /// \brief invelm_tool::m_invariant_file_name is checked.
     bool m_no_check;
@@ -117,19 +108,6 @@ class invelm_tool : public prover_tool< rewriter_tool<input_output_tool> >
       {
         m_dot_file_name = parser.option_argument_as< std::string >("print-dot");
       }
-      if (parser.options.count("summand"))
-      {
-        m_summand_number = parser.option_argument_as< size_t >("summand");
-
-        if (m_summand_number < 1)
-        {
-          throw parser.error("The summand number must be greater than or equal to 1.\n");
-        }
-        else
-        {
-          mCRL2log(verbose) << "Eliminating or simplifying summand number " <<  m_summand_number << "." << std::endl;
-        }
-      }
       if (parser.options.count("time-limit"))
       {
         m_time_limit = parser.option_argument_as< int >("time-limit");
@@ -148,8 +126,6 @@ class invelm_tool : public prover_tool< rewriter_tool<input_output_tool> >
       desc.
       add_option("invariant", make_file_argument("INVFILE"),
                  "use the boolean formula (an mCRL2 data expression of sort Bool) in INVFILE as invariant", 'i').
-      add_option("summand", make_mandatory_argument("NUM"),
-                 "eliminate or simplify the summand with number NUM only", 's').
       add_option("no-check",
                  "do not check if the invariant holds before eliminating unreachable summands", 'n').
       add_option("no-elimination",
@@ -174,7 +150,7 @@ class invelm_tool : public prover_tool< rewriter_tool<input_output_tool> >
 
   public:
     /// \brief Constructor setting all flags to their default values.
-    invelm_tool() : super(
+    lpsinvelm_tool() : super(
         "lpsinvelm",
         "Luc Engelen",
         "check invariants and use these to simplify or eliminate summands of an LPS",
@@ -185,7 +161,6 @@ class invelm_tool : public prover_tool< rewriter_tool<input_output_tool> >
         "If INFILE is present, stdin is used. If OUTFILE is not present, stdout is used.\n"
         "\n"
         "The tool can also be used to simplify the conditions of the summands of the given LPS."),
-      m_summand_number(0),
       m_no_check(false),
       m_no_elimination(false),
       m_simplify_all(false),
@@ -204,7 +179,6 @@ class invelm_tool : public prover_tool< rewriter_tool<input_output_tool> >
                 m_dot_file_name,
                 rewrite_strategy(),
                 solver_type(),
-                m_summand_number,
                 m_no_check,
                 m_no_elimination,
                 m_simplify_all,
@@ -218,28 +192,7 @@ class invelm_tool : public prover_tool< rewriter_tool<input_output_tool> >
     }
 };
 
-class lpsinvelm_giu_tool: public mcrl2_gui_tool<invelm_tool>
-{
-  public:
-    lpsinvelm_giu_tool()
-    {
-
-      m_gui_options["counter-example"] = create_checkbox_widget();
-      m_gui_options["no-elimination"] = create_checkbox_widget();
-      m_gui_options["simplify-all"] = create_checkbox_widget();
-      m_gui_options["no-check"] = create_checkbox_widget();
-      m_gui_options["induction"] = create_checkbox_widget();
-      m_gui_options["print-dot"] = create_textctrl_widget();
-      add_rewriter_widget();
-      m_gui_options["summand"] = create_textctrl_widget();
-      m_gui_options["time-limit"] = create_textctrl_widget();
-      m_gui_options["all-violations"] = create_checkbox_widget();
-      m_gui_options["smt-solver"] = create_textctrl_widget();
-    }
-};
-
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  return lpsinvelm_giu_tool().execute(argc, argv);
+  return lpsinvelm_tool().execute(argc, argv);
 }
