@@ -8,12 +8,8 @@
 //
 /// \file ./lpsparelm.cpp
 
-#include "boost.hpp" // precompiled headers
-
 #include "mcrl2/lps/tools.h"
 #include "mcrl2/utilities/input_output_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 using mcrl2::utilities::tools::input_output_tool;
@@ -45,17 +41,8 @@ class lps_parelm_tool : public input_output_tool
 
 };
 
-class lps_parelm_gui_tool: public mcrl2_gui_tool<lps_parelm_tool>
-{
-  public:
-    lps_parelm_gui_tool()
-    {    }
-};
-
 int main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
-  return lps_parelm_gui_tool().execute(argc, argv);
+  return lps_parelm_tool().execute(argc, argv);
 }
 
