@@ -20,9 +20,10 @@
 //Aterms
 #include <mcrl2/atermpp/aterm.h>
 #include <mcrl2/atermpp/aterm_list.h>
-#include <mcrl2/atermpp/table.h>
 #include <mcrl2/atermpp/algorithm.h>
-#include "mcrl2/atermpp/aterm_init.h"
+
+//LPS
+#include "mcrl2/lps/io.h"
 
 //Tool framework
 #include "mcrl2/utilities/input_output_tool.h"
@@ -87,15 +88,15 @@ class lpsrealelm_tool: public rewriter_tool<input_output_tool >
       mCRL2log(verbose) << "  data rewriter       " << m_rewrite_strategy << std::endl;
       mCRL2log(verbose) << "  max_iterations:     " << max_iterations << std::endl;
 
-      specification lps_specification;
-      lps_specification.load(m_input_filename);
+      specification spec;
+      load_lps(spec, input_filename());
 
       // Untime lps_specification and save the output to a binary file
-      rewriter r=create_rewriter(lps_specification.data());
-      specification new_spec = realelm(lps_specification, max_iterations, r);
+      // rewriter r=create_rewriter(lps_specification.data());
+      specification new_spec = realelm(spec, max_iterations, rewrite_strategy());
 
       mCRL2log(verbose) << "Real time abstraction completed, saving to " << m_output_filename << "\n";
-      new_spec.save(m_output_filename);
+      save_lps(new_spec, output_filename());
 
       return true;
     }
@@ -104,6 +105,5 @@ class lpsrealelm_tool: public rewriter_tool<input_output_tool >
 
 int main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
   return lpsrealelm_tool().execute(argc, argv);
 }
