@@ -8,13 +8,9 @@
 //
 /// \file lpssumelm.cpp
 
-#include "boost.hpp" // precompiled headers
-
 #include "mcrl2/lps/tools.h"
 
 #include "mcrl2/utilities/input_output_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace mcrl2;
 using namespace mcrl2::utilities;
@@ -64,16 +60,7 @@ class sumelm_tool: public input_output_tool
 
 };
 
-class sumelm_gui_tool: public mcrl2_gui_tool<sumelm_tool>
-{
-  public:
-    sumelm_gui_tool()
-    { }
-};
-
 int main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
-  return sumelm_gui_tool().execute(argc, argv);
+  return sumelm_tool().execute(argc, argv);
 }
