@@ -12,13 +12,10 @@
 #define AUTHOR "Muck van Weerdenburg, Jan Friso Groote"
 
 #include <string>
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/utilities/exception.h"
-#include "mcrl2/core/detail/struct_core.h"
 #include "mcrl2/utilities/logger.h"
 #include "mcrl2/utilities/input_output_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
-#include "mcrl2/lps/specification.h"
+#include "mcrl2/lps/io.h"
 #include "mcrl2/lts/lts_equivalence.h"
 #include "mcrl2/lts/lts_io.h"
 #include "mcrl2/lts/detail/lts_convert.h"
@@ -31,12 +28,6 @@ using namespace mcrl2::utilities;
 using namespace mcrl2::core;
 using namespace mcrl2::log;
 
-
-static inline std::string get_base(std::string const& s)
-{
-  return s.substr(0, s.find_last_of('.'));
-}
-
 class t_tool_options
 {
   public:
@@ -79,9 +70,9 @@ class t_tool_options
 
       if (outtype == lts_none)
       {
-        mCRL2log(verbose) << "trying to detect output format by extension..." << std::endl;
+        mCRL2log(verbose) << "Trying to detect output format by extension..." << std::endl;
 
-        outtype = mcrl2::lts::detail::guess_format(outfilename);
+        outtype = mcrl2::lts::detail::guess_format(outfilename,true);
 
         if (outtype == lts_none)
         {
@@ -159,7 +150,7 @@ class ltsconvert_tool : public ltsconvert_base
       if (!tool_options.lpsfile.empty())
       {
         // No lpsfile is given. Only straightforward translations are possible.
-        spec.load(tool_options.lpsfile);
+        load_lps(spec, tool_options.lpsfile);
       }
 
 
@@ -215,7 +206,7 @@ class ltsconvert_tool : public ltsconvert_base
 
       if (tool_options.intype==lts_none)
       {
-        tool_options.intype = mcrl2::lts::detail::guess_format(tool_options.infilename);
+        tool_options.intype = mcrl2::lts::detail::guess_format(tool_options.infilename,false);
       }
       switch (tool_options.intype)
       {
@@ -273,6 +264,10 @@ class ltsconvert_tool : public ltsconvert_base
                       .add_value(lts_eq_branching_bisim_sigref)
                       .add_value(lts_eq_divergence_preserving_branching_bisim)
                       .add_value(lts_eq_divergence_preserving_branching_bisim_sigref)
+                      .add_value(lts_eq_weak_bisim)
+                      .add_value(lts_eq_weak_bisim_sigref)
+                      .add_value(lts_eq_divergence_preserving_weak_bisim)
+                      .add_value(lts_eq_divergence_preserving_weak_bisim_sigref)
                       .add_value(lts_eq_sim)
                       .add_value(lts_eq_trace)
                       .add_value(lts_eq_weak_trace)
@@ -399,44 +394,7 @@ class ltsconvert_tool : public ltsconvert_base
 
 };
 
-class ltsconvert_gui_tool: public mcrl2_gui_tool<ltsconvert_tool>
-{
-  public:
-    ltsconvert_gui_tool()
-    {
-
-      std::vector<std::string> values;
-
-      m_gui_options["determinise"] = create_checkbox_widget();
-
-      values.clear();
-      values.push_back("none");
-      values.push_back("bisim");
-      values.push_back("branching-bisim");
-      values.push_back("dpbranching-bisim");
-      values.push_back("sim");
-      values.push_back("trace");
-      values.push_back("weak-trace");
-      values.push_back("tau-star");
-      m_gui_options["equivalence"] = create_radiobox_widget(values);
-      m_gui_options["lps"] = create_filepicker_widget("LPS files (*.lps)|*.lps|All Files (*.*)|*.*");
-      m_gui_options["no-state"] = create_checkbox_widget();
-      m_gui_options["no-reach"] = create_checkbox_widget();
-      m_gui_options["tau"] = create_textctrl_widget();
-
-      //-iFORMAT, --in=FORMAT    use FORMAT as the input format
-      //-oFORMAT, --out=FORMAT   use FORMAT as the output format
-
-    }
-};
-
-
-
-
-
 int main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
-  return ltsconvert_gui_tool().execute(argc,argv);
+  return ltsconvert_tool().execute(argc,argv);
 }
