@@ -47,7 +47,6 @@ Cluster::Cluster(int r)
   visObject = -1;
   numMarkedTransitions = 0;
   rank = r;
-  selected = false;
   severedDescendantsC = 0;
   bc_radius = 0.0f;
   bc_height = 0.0f;
@@ -503,20 +502,14 @@ bool Cluster::hasMarkedTransition() const
   return (numMarkedTransitions > 0);
 }
 
-void Cluster::addMatchedRule(int mr)
+void Cluster::addMatchedRule(MarkRuleIndex index)
 {
-  matchedRules.insert(mr);
+  matchedRules.insert(index);
 }
 
-void Cluster::removeMatchedRule(int mr)
+void Cluster::removeMatchedRule(MarkRuleIndex index)
 {
-  matchedRules.erase(mr);
-}
-
-
-void Cluster::getMatchedRules(std::vector< int > &mrs)
-{
-  mrs.assign(matchedRules.begin(),matchedRules.end());
+  matchedRules.erase(index);
 }
 
 int Cluster::setActionMark(int l,bool b)
@@ -576,21 +569,6 @@ void Cluster::clearBranchVisObjects()
   branchVisObjects.clear();
 }
 
-void Cluster::select()
-{
-  selected = true;
-}
-
-void Cluster::deselect()
-{
-  selected = false;
-}
-
-bool Cluster::isSelected() const
-{
-  return selected;
-}
-
 int Cluster::getNumMarkedTransitions()
 {
   return numMarkedTransitions;
