@@ -24,22 +24,23 @@ class ToolCatalog
 {
   public:
     ToolCatalog();
-    QString fileType(QString extension);
+    // file types for an extension
+    QStringList fileTypes(QString extension);
 
     void load();
 
     QStringList categories();
 
     QList<ToolInformation> tools(QString category);
-    QList<ToolInformation> tools(QString category, QString extension);
+    //QList<ToolInformation> tools(QString category, QString extension);
 
   private:
     void generateFileTypes();
 
     QDomDocument m_xml;
-    QMap<QString, QString> m_filetypes;
-
-    QMap<QString, QMap<QString, ToolInformation> > m_categories;
+    // Maps extensions to file types
+    QMultiMap<QString, QString> m_filetypes;
+    QMap<QString, QList<ToolInformation> > m_categories;
 };
 
 #endif // TOOLCATALOG_H
