@@ -9,8 +9,6 @@
 /// \file mcrl2i.cpp
 
 
-#include "boost.hpp" // precompiled headers
-
 #define TOOLNAME "mcrl2i"
 #define AUTHORS "Muck van Weerdenburg; Jan Friso Groote"
 
@@ -19,8 +17,6 @@
 #include <string>
 #include <cctype>
 
-#include "boost/utility.hpp"
-
 #include <cstdio>
 #include <cerrno>
 #include <cstdlib>
@@ -28,17 +24,14 @@
 #include <cassert>
 
 #include "mcrl2/data/rewriter.h"
-#include "mcrl2/data/classic_enumerator.h"
+#include "mcrl2/data/enumerator.h"
 #include "mcrl2/data/parse.h"
-#include "mcrl2/core/detail/aterm_io.h"
 #include "mcrl2/data/data_specification.h"
+#include "mcrl2/data/substitutions/mutable_map_substitution.h"
 #include "mcrl2/utilities/input_tool.h"
 #include "mcrl2/utilities/rewriter_tool.h"
-#include "mcrl2/lps/specification.h"
-#include "mcrl2/pbes/pbes.h"
-#include "mcrl2/atermpp/aterm_init.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
-
+#include "mcrl2/lps/io.h"
+#include "mcrl2/pbes/io.h"
 
 using namespace std;
 using namespace mcrl2;
@@ -51,12 +44,12 @@ using mcrl2::utilities::tools::rewriter_tool;
 
 static bool check_whether_variable_string_is_in_use(
   const std::string& s,
-  const atermpp::set < variable > &varset)
+  const std::set < variable > &varset)
 {
-  for (atermpp::set < variable >::const_iterator i=varset.begin();
+  for (std::set < variable >::const_iterator i=varset.begin();
        i!=varset.end(); ++i)
   {
-    if (i->name()==s.c_str())
+    if (i->name().function().name()==s)
     {
       return true;
     }
@@ -88,8 +81,8 @@ static string trim_spaces(const string& str)
 
 static data_expression parse_term(const string& term_string,
                                   const data_specification& spec,
-                                  atermpp::set < variable > context_variables,
-                                  const atermpp::set < variable > &local_variables = atermpp::set < variable >())
+                                  std::set < variable > context_variables,
+                                  const std::set < variable > &local_variables = std::set < variable >())
 {
   context_variables.insert(local_variables.begin(),local_variables.end());
   return parse_data_expression(term_string,context_variables.begin(),context_variables.end(),spec);
@@ -97,7 +90,7 @@ static data_expression parse_term(const string& term_string,
 
 static void declare_variables(
   const string& vars,
-  atermpp::set <variable> &context_variables,
+  std::set <variable> &context_variables,
   data_specification& spec)
 {
   parse_variables(vars + ";",std::inserter(context_variables,context_variables.begin()),
@@ -117,6 +110,24 @@ static bool match_and_remove(string& s, const string& match)
   return false;
 }
 
+static bool add_context_sorts(const std::set<sort_expression> &new_sorts,
+                                    std::set<sort_expression> &context_sorts,
+                                    data_specification &spec)
+{
+  // Check whether new rewrite rules are required, and if so, reinitialise the rewriter with them.
+  bool reinitialise_rewriter=false;
+  for(auto i=new_sorts.begin(); i!=new_sorts.end(); ++i)
+  {
+     if (context_sorts.insert(*i).second)
+     {
+       // The sort was not yet present in the context sorts
+       reinitialise_rewriter=true;
+       spec.add_context_sort(*i);
+     }
+  }
+  return reinitialise_rewriter;
+}
+
 static const std::string help_text=
   "The following commands are available to manipulate mcrl2 data expressions. "
   "Essentially, there are commands to rewrite and type expressions, as well as generating "
@@ -155,7 +166,10 @@ class mcrl2i_tool: public rewriter_tool<input_tool>
     /// Runs the algorithm.
     bool run()
     {
-      atermpp::set < variable > context_variables;
+      std::set < variable > context_variables;
+      std::set < sort_expression > context_sorts;
+      bool need_to_rebuild_rewriter=true;
+
       data_specification spec;
       if (!input_filename().empty())
       {
@@ -163,17 +177,17 @@ class mcrl2i_tool: public rewriter_tool<input_tool>
         {
           // Try to read a linear specification
           mcrl2::lps::specification p;
-          p.load(input_filename());
-          spec=p.data();
+          load_lps(p, input_filename());
+          spec = p.data();
           context_variables = p.global_variables();
         }
-        catch (mcrl2::runtime_error e)
+        catch (mcrl2::runtime_error)
         {
           try
           {
             // Try to read a pbes.
-            mcrl2::pbes_system::pbes <> p;
-            p.load(input_filename());
+            mcrl2::pbes_system::pbes p;
+            load_pbes(p, input_filename());
             spec=p.data();
             context_variables = p.global_variables();
           }
@@ -187,15 +201,12 @@ class mcrl2i_tool: public rewriter_tool<input_tool>
         }
       }
 
-      // Import all standard data types should be available even if they are
-      // not port of the loaded lps or pbes.
-      spec.add_context_sort(sort_real::real_());
 
       std::cout << "mCRL2 interpreter (type h for help)" << std::endl;
 
-      rewriter rewr(spec,m_rewrite_strategy);
+      rewriter rewr; 
 
-      mutable_map_substitution < atermpp::map < variable, data_expression > > assignments;
+      mutable_map_substitution < std::map < variable, data_expression > > assignments;
 
       bool done = false;
       while (!done)
@@ -235,7 +246,7 @@ class mcrl2i_tool: public rewriter_tool<input_tool>
             if (new_strategy!=m_rewrite_strategy)
             {
               m_rewrite_strategy=new_strategy;
-              rewr=rewriter(spec,m_rewrite_strategy);
+              need_to_rebuild_rewriter=true;
             }
           }
           else if (match_and_remove(s,"t ") || match_and_remove(s,"type "))
@@ -250,41 +261,61 @@ class mcrl2i_tool: public rewriter_tool<input_tool>
           else if (match_and_remove(s,"e ") || match_and_remove(s,"eval "))
           {
             data_expression term = parse_term(s,spec,context_variables);
+            std::set<sort_expression> all_sorts=find_sort_expressions(term);
+            need_to_rebuild_rewriter=need_to_rebuild_rewriter||add_context_sorts(all_sorts,context_sorts,spec);
+            if (need_to_rebuild_rewriter)
+            {
+              rewr=rewriter(spec,m_rewrite_strategy);
+              need_to_rebuild_rewriter=false;
+            }
             cout << data::pp(rewr(term,assignments)) << "\n";
           }
           else if (match_and_remove(s,"s ") || match_and_remove(s,"solve "))
           {
-            atermpp::set <variable> vars;
+            std::set <variable> vars;
             string::size_type dotpos=s.find(".");
             if (dotpos==string::npos)
             {
               throw mcrl2::runtime_error("Expect a `.' in the input.");
             }
             parse_variables(s.substr(0,dotpos)+";",std::inserter(vars,vars.begin()),spec);
+            std::set<sort_expression> all_sorts=find_sort_expressions(vars);
             data_expression term = parse_term(s.substr(dotpos+1),spec,context_variables,vars);
             if (term.sort()!=sort_bool::bool_())
             {
               throw mcrl2::runtime_error("expression is not of sort Bool.");
             }
+            find_sort_expressions(term,std::inserter(all_sorts,all_sorts.end()));
+
+            need_to_rebuild_rewriter=need_to_rebuild_rewriter||add_context_sorts(all_sorts,context_sorts,spec);
+            if (need_to_rebuild_rewriter)
+            {
+              rewr=rewriter(spec,m_rewrite_strategy);
+              need_to_rebuild_rewriter=false;
+            }
 
             term=rewr(term);
-            typedef classic_enumerator< rewriter > enumerator_type;
-            enumerator_type enumerator(spec,rewr);
+            typedef enumerator_algorithm_with_iterator<rewriter> enumerator_type;
+            enumerator_type enumerator(rewr, spec, rewr, 10000);
 
-            for (enumerator_type::iterator
-                      i=enumerator.begin(vars,term,10000); // Stop when more than 10000 internal variables are required
-                      i != enumerator.end() ; ++i)
+            data::mutable_indexed_substitution<> sigma;
+            std::deque<enumerator_list_element_with_substitution<> >
+                 enumerator_deque(1, enumerator_list_element_with_substitution<>(variable_list(vars.begin(), vars.end()), term));
+            for (enumerator_type::iterator i = enumerator.begin(sigma, enumerator_deque); i != enumerator.end(); ++i)
             {
+              i->add_assignments(vars,sigma,rewr);
+
               cout << "[";
-              for (atermpp::set< variable >::const_iterator v=vars.begin(); v!=vars.end() ; ++v)
+              for (std::set< variable >::const_iterator v=vars.begin(); v!=vars.end() ; ++v)
               {
-                cout << data::pp(*v) << " := " << data::pp((*i)(*v));
-                if (boost::next(v)!=vars.end())
+                if (v!=vars.begin())
                 {
                   cout << ", ";
                 }
+             
+                cout << data::pp(*v) << " := " << data::pp(sigma(*v));
               }
-              cout << "] evaluates to "<< data::pp(rewr(term,*i)) << "\n";
+              cout << "] evaluates to "<< data::pp(rewr(term,sigma)) << "\n";
             }
           }
           else if (match_and_remove(s,"a ") || match_and_remove(s,"assign "))
@@ -303,6 +334,14 @@ class mcrl2i_tool: public rewriter_tool<input_tool>
             s = s.substr(assign_pos+1);
             data_expression term = parse_term(s,spec,context_variables);
             variable var(varname,term.sort());
+
+            std::set<sort_expression> all_sorts=find_sort_expressions(term);
+            need_to_rebuild_rewriter=need_to_rebuild_rewriter||add_context_sorts(all_sorts,context_sorts,spec);
+            if (need_to_rebuild_rewriter)
+            {
+              rewr=rewriter(spec,m_rewrite_strategy);
+              need_to_rebuild_rewriter=false;
+            }
             term = rewr(term,assignments);
             cout << data::pp(term) << "\n";
             assignments[var]=term;
@@ -323,18 +362,8 @@ class mcrl2i_tool: public rewriter_tool<input_tool>
     }
 };
 
-class mcrl2i_gui_tool: public mcrl2_gui_tool<mcrl2i_tool>
-{
-  public:
-    mcrl2i_gui_tool()
-    {
-      add_rewriter_widget();
-    }
-};
-
 int main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  return mcrl2i_gui_tool().execute(argc, argv);
+  return mcrl2i_tool().execute(argc, argv);
 }
 
