@@ -9,8 +9,6 @@
 /// \file mcrl2parse.cpp
 /// \brief tool for testing the new parser
 
-#include "boost.hpp" // precompiled headers
-
 #define TOOLNAME "mcrl2parse"
 #define AUTHOR "Wieger Wesselink"
 
@@ -22,13 +20,11 @@
 
 #include <boost/algorithm/string/trim.hpp>
 
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/bes/boolean_equation_system.h"
 #include "mcrl2/bes/parse.h"
 #include "mcrl2/bes/print.h"
 #include "mcrl2/core/parse.h"
 #include "mcrl2/core/detail/dparser_functions.h"
-#include "mcrl2/core/typecheck.h"
 #include "mcrl2/core/parser_utility.h"
 #include "mcrl2/data/parse.h"
 #include "mcrl2/data/typecheck.h"
@@ -47,7 +43,6 @@
 #include "mcrl2/process/typecheck.h"
 #include "mcrl2/utilities/detail/separate_keyword_section.h"
 #include "mcrl2/utilities/input_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
 
 using namespace mcrl2;
 using namespace mcrl2::utilities;
@@ -216,12 +211,12 @@ void separate_process_specification(const std::string& text, const std::string&
 }
 
 inline
-void separate_pbes_specification(const std::string& text, const std::string& keyword, pbes_system::pbes<>& pbesspec, std::string& keyword_text)
+void separate_pbes_specification(const std::string& text, const std::string& keyword, pbes_system::pbes& pbesspec, std::string& keyword_text)
 {
   if (!has_keyword(text, keyword))
   {
     keyword_text = text;
-    pbesspec = pbes_system::pbes<>();
+    pbesspec = pbes_system::pbes();
   }
   else
   {
@@ -362,7 +357,7 @@ class mcrl2parse_tool : public input_tool
     bool aterm_format;
     bool warn;
 
-
+    bool dot;
 
     void add_options(interface_description& desc)
     {
@@ -394,6 +389,8 @@ class mcrl2parse_tool : public input_tool
       desc.add_option("check-printer", "compare the results of the old and new pretty printer", 'P');
       desc.add_option("aterm-format", "compare the results in aterm format", 'a');
       desc.add_option("warn", "generate warnings", 'w');
+
+      desc.add_option("dot", "load a dot file using the old and the new parser and check the result");
     }
 
     void parse_options(const command_line_parser& parser)
@@ -448,7 +445,7 @@ class mcrl2parse_tool : public input_tool
       data::data_specification dataspec;
       lps::specification lpsspec;
       process::process_specification procspec;
-      pbes_system::pbes<> pbesspec;
+      pbes_system::pbes pbesspec;
 
       core::parser p(parser_tables_mcrl2, core::detail::ambiguity_fn, core::detail::syntax_error_fn);
       unsigned int start_symbol_index = 0;
@@ -519,7 +516,7 @@ class mcrl2parse_tool : public input_tool
             }
             case besspec_e  :
             {
-              bes::boolean_equation_system<> x = bes::parse_boolean_equation_system_new(text);
+              bes::boolean_equation_system x = bes::parse_boolean_equation_system_new(text);
               atermpp::aterm_appl a = bes::boolean_equation_system_to_aterm(x);
               if (aterm_format)
               {
@@ -578,8 +575,8 @@ class mcrl2parse_tool : public input_tool
             case multact_e  :
             {
               separate_action_specification(text, "multact", lpsspec, text);
-              lps::multi_action x = lps::parse_multi_action_new(text);
-              lps::complete_multi_action(x, lpsspec.action_labels(), lpsspec.data());
+              lps::untyped_multi_action y = lps::detail::parse_multi_action_new(text);
+              lps::multi_action x = lps::detail::complete_multi_action(y, lpsspec.action_labels(), lpsspec.data());
               if (aterm_format)
               {
                 std::cout << lps::detail::multi_action_to_aterm(x) << std::endl;
@@ -606,7 +603,7 @@ class mcrl2parse_tool : public input_tool
             }
             case pbesspec_e :
             {
-              pbes_system::pbes<> x = pbes_system::parse_pbes_new(text);
+              pbes_system::pbes x = pbes_system::parse_pbes_new(text);
               pbes_system::complete_pbes(x);
               if (aterm_format)
               {
@@ -698,7 +695,7 @@ class mcrl2parse_tool : public input_tool
             }
             case besspec_e  :
             {
-              bes::boolean_equation_system<> x = bes::parse_boolean_equation_system_new(text);
+              bes::boolean_equation_system x = bes::parse_boolean_equation_system_new(text);
               std::cout << bes::pp(x) << std::endl;
               break;
             }
@@ -740,7 +737,7 @@ class mcrl2parse_tool : public input_tool
             }
             case pbesspec_e :
             {
-              pbes_system::pbes<> x = pbes_system::parse_pbes(text);
+              pbes_system::pbes x = pbes_system::parse_pbes(text);
               std::cout << pbes_system::pp(x) << std::endl;
               break;
             }
@@ -785,15 +782,7 @@ class mcrl2parse_tool : public input_tool
 
 };
 
-class mcrl2parse_gui_tool: public mcrl2_gui_tool<mcrl2parse_tool>
-{
-  public:
-    mcrl2parse_gui_tool() {}
-};
-
 int main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
-  return mcrl2parse_gui_tool().execute(argc, argv);
+  return mcrl2parse_tool().execute(argc, argv);
 }
