@@ -8,8 +8,6 @@
 /// \file pbes2bool.cpp
 /// \brief Add your file description here.
 
-#include "boost.hpp" // precompiled headers
-
 // ======================================================================
 //
 // file          : pbes2bool
@@ -37,7 +35,6 @@
 #include "mcrl2/utilities/pbes_input_tool.h"
 #include "mcrl2/utilities/rewriter_tool.h"
 #include "mcrl2/utilities/pbes_rewriter_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
 #include "mcrl2/utilities/execution_timer.h"
 
 //Data Framework
@@ -46,24 +43,20 @@
 #include "mcrl2/data/data_equation.h"
 
 //Boolean equation systems
-#include "mcrl2/pbes/normalize.h"
-#include "mcrl2/pbes/utility.h"
-#include "mcrl2/pbes/io.h"
+#include "mcrl2/pbes/algorithms.h"
+#include "mcrl2/bes/io.h"
 #include "mcrl2/bes/bes_deprecated.h"
 #include "mcrl2/bes/boolean_equation_system.h"
 #include "mcrl2/bes/bes2pbes.h"
 #include "mcrl2/pbes/rewrite.h"
 #include "mcrl2/pbes/find.h"
 #include "mcrl2/pbes/detail/instantiate_global_variables.h"
-#include "mcrl2/atermpp/aterm_init.h"
 
 using namespace std;
+using namespace ::bes;
 using namespace mcrl2::utilities;
 using namespace mcrl2::core;
 using bes::bes_expression;
-using namespace ::bes;
-
-// using atermpp::make_substitution;
 
 //Function declarations used by main program
 //------------------------------------------
@@ -75,18 +68,19 @@ using utilities::tools::rewriter_tool;
 using utilities::tools::pbes_rewriter_tool;
 using namespace mcrl2::utilities::tools;
 
-class pbes2bool_tool: public pbes_rewriter_tool<rewriter_tool<pbes_input_tool<input_tool> > >
+class pbes2bool_tool: public rewriter_tool<pbes_input_tool<input_tool> >
 {
   protected:
     // Tool options.
-    ::bes::transformation_strategy opt_strategy; // The strategy
+    ::bes::transformation_strategy opt_transformation_strategy; // The transformation strategy
+    ::bes::search_strategy opt_search_strategy;                 // The search strategy
     bool opt_use_hashtables;                   // The hashtable option
     bool opt_construct_counter_example;        // The counter example option
-    bool opt_store_as_tree;                    // The tree storage option
+    remove_level opt_erase_unused_bes_variables;       // Remove unused bes variables if true
     bool opt_data_elm;                         // The data elimination option
     std::string opt_counter_example_file;      // The counter example file name
 
-    typedef pbes_rewriter_tool<rewriter_tool<pbes_input_tool<input_tool> > > super;
+    typedef rewriter_tool<pbes_input_tool<input_tool> > super;
 
     pbes_system::pbes_rewriter_type default_rewriter() const
     {
@@ -107,10 +101,11 @@ class pbes2bool_tool: public pbes_rewriter_tool<rewriter_tool<pbes_input_tool<in
         "Generate a BES from a PBES and solve it. ",
         "Solves (P)BES from INFILE. "
         "If INFILE is not present, stdin is used. "),
-      opt_strategy(::bes::lazy),
+      opt_transformation_strategy(::bes::lazy),
+      opt_search_strategy(::bes::breadth_first),
       opt_use_hashtables(false),
       opt_construct_counter_example(false),
-      opt_store_as_tree(false),
+      opt_erase_unused_bes_variables(none),
       opt_data_elm(true),
       opt_counter_example_file("")
     {}
@@ -124,15 +119,21 @@ class pbes2bool_tool: public pbes_rewriter_tool<rewriter_tool<pbes_input_tool<in
 
       opt_use_hashtables            = 0 < parser.options.count("hashtables");
       opt_construct_counter_example = 0 < parser.options.count("counter");
-      opt_store_as_tree             = 0 < parser.options.count("tree");
+      opt_erase_unused_bes_variables= parser.option_argument_as<remove_level>("erase");
       opt_data_elm                  = parser.options.count("unused-data") == 0;
-      opt_strategy                  = parser.option_argument_as<transformation_strategy>("strategy");
+      opt_transformation_strategy   = parser.option_argument_as<transformation_strategy>("strategy");
+      opt_search_strategy           = parser.option_argument_as<search_strategy>("search");
 
       if (parser.options.count("output")) // Output format is deprecated.
       {
-        std::string format = parser.option_argument("output");
         throw parser.error("the option --output or -o is deprecated. Use the tool pbes2bes for this functionality. ");
       }
+
+      if (opt_construct_counter_example && opt_erase_unused_bes_variables!=none)
+      {
+        throw parser.error("generating a counter example cannot be combined with erasing bes variables. ");
+      }
+
     }
 
     void add_options(interface_description& desc)
@@ -140,12 +141,19 @@ class pbes2bool_tool: public pbes_rewriter_tool<rewriter_tool<pbes_input_tool<in
       super::add_options(desc);
       desc.
       add_option("strategy", make_enum_argument<transformation_strategy>("STRAT")
-                 .add_value(lazy, true)
-                 .add_value(optimize)
-                 .add_value(on_the_fly)
-                 .add_value(on_the_fly_with_fixed_points),
-                 "use strategy STRAT:",
+                 .add_value(::lazy, true)
+                 .add_value(::optimize)
+                 .add_value(::on_the_fly)
+                 .add_value(::on_the_fly_with_fixed_points),
+                 "use substitution strategy STRAT:",
                  's').
+     add_option("search", make_enum_argument<search_strategy>("SEARCH")
+                 .add_value(breadth_first, true)
+                 .add_value(depth_first)
+                 .add_value(breadth_first_short)
+                 .add_value(depth_first_short),
+                 "use search strategy SEARCH:",
+                 'z').
       add_option("counter",
                  "print at the end a tree labelled with instantiations "
                  "of the left hand side of equations; this tree is an "
@@ -157,13 +165,16 @@ class pbes2bool_tool: public pbes_rewriter_tool<rewriter_tool<pbes_input_tool<in
                  "and translate internal expressions to binary decision "
                  "diagrams (discouraged, due to performance)",
                  'H').
+      add_option("erase", make_enum_argument<remove_level>("LEVEL")
+                 .add_value(none, true)
+                 .add_value(some)
+                 .add_value(all),
+                 "use remove level LEVEL to remove bes variables",
+                 'e').
       add_option("output",
                  make_mandatory_argument("FORMAT"),
                  "use output format FORMAT (this option is deprecated. Use the tool pbes2bes instead).\n",
                  'o').
-      add_option("tree",
-                 "store state in a tree (for memory efficiency)",
-                 't').
       add_option("unused_data",
                  "do not remove unused parts of the data specification",
                  'u');
@@ -176,15 +187,16 @@ class pbes2bool_tool: public pbes_rewriter_tool<rewriter_tool<pbes_input_tool<in
       using namespace utilities;
 
       mCRL2log(verbose) << "pbes2bool parameters:" << std::endl;
-      mCRL2log(verbose) << "  input file:         " << m_input_filename << std::endl;
-      mCRL2log(verbose) << "  data rewriter:      " << m_rewrite_strategy << std::endl;
-      mCRL2log(verbose) << "  pbes rewriter:      " << m_pbes_rewriter_type << std::endl;
-
+      mCRL2log(verbose) << "  input file:            " << m_input_filename << std::endl;
+      mCRL2log(verbose) << "  data rewriter:         " << m_rewrite_strategy << std::endl;
+      mCRL2log(verbose) << "  substitution strategy: " << opt_transformation_strategy << std::endl;
+      mCRL2log(verbose) << "  search strategy:       " << opt_search_strategy << std::endl;
+      mCRL2log(verbose) << "  erase level:           " << opt_erase_unused_bes_variables << std::endl;
       // load the pbes
-      mcrl2::pbes_system::pbes<> p;
-      load_pbes(p, input_filename(), pbes_input_format());
+      mcrl2::pbes_system::pbes p;
+      mcrl2::bes::load_pbes(p, input_filename(), pbes_input_format());
 
-      pbes_system::normalize(p);
+      pbes_system::algorithms::normalize(p);
       pbes_system::detail::instantiate_global_variables(p);
       // data rewriter
 
@@ -193,9 +205,9 @@ class pbes2bool_tool: public pbes_rewriter_tool<rewriter_tool<pbes_input_tool<in
       {
         // Create a rewriter with only the necessary data equations.
         using namespace mcrl2::data;
-        std::set < function_symbol > eqn_symbol_set=pbes_system::find_function_symbols(p.equations());
-        std::set < function_symbol > init_symbol_set=pbes_system::find_function_symbols(p.initial_state());
-        std::set < function_symbol > function_symbol_set;
+        std::set < data::function_symbol > eqn_symbol_set=pbes_system::find_function_symbols(p.equations());
+        std::set < data::function_symbol > init_symbol_set=pbes_system::find_function_symbols(p.initial_state());
+        std::set < data::function_symbol > function_symbol_set;
         set_union(eqn_symbol_set.begin(),eqn_symbol_set.end(),
                   init_symbol_set.begin(),init_symbol_set.end(),
                   inserter(function_symbol_set,function_symbol_set.begin()));
@@ -216,87 +228,13 @@ class pbes2bool_tool: public pbes_rewriter_tool<rewriter_tool<pbes_input_tool<in
         ::bes::boolean_equation_system(
           p,
           datar,
-          opt_strategy,
-          opt_store_as_tree,
+          opt_transformation_strategy,
+          opt_search_strategy,
           opt_construct_counter_example,
+          opt_erase_unused_bes_variables,
           opt_use_hashtables);
       timer().finish("instantiation");
 
-      // pbes rewriter
-      /* The code below can be reactivated, once the pbes_rewriters deliver acceptable performance.
-         As it stands their performance is so bad, that they cannot be used.
-
-      switch (rewriter_type())
-      {
-        case simplify:
-        {
-          simplifying_rewriter<pbes_expression, data::rewriter> pbesr(datar);
-          pbes_rewrite(p,pbesr); // Simplify p such that it does not have to be done
-                             // repeatedly.
-          bes_equations=::bes::boolean_equation_system(
-                            p,
-                            pbesr,
-                            opt_strategy,
-                            opt_store_as_tree,
-                            opt_construct_counter_example,
-                            opt_use_hashtables);
-          break;
-        }
-        case quantifier_finite:
-        {
-          data::number_postfix_generator generator("UNIQUE_PREFIX");
-          data::data_enumerator<> datae(p.data(), datar, generator);
-          data::rewriter_with_variables datarv(datar);
-          bool enumerate_infinite_sorts = false;
-          enumerate_quantifiers_rewriter<pbes_expression, data::rewriter_with_variables,
-                                                             data::data_enumerator<> >
-                          pbesr(datarv, datae, enumerate_infinite_sorts);
-          pbes_rewrite(p,pbesr);  // Simplify p such that this does not need to be done
-                              // repeatedly.
-          bes_equations=::bes::boolean_equation_system(
-                            p,
-                            pbesr,
-                            opt_strategy,
-                            opt_store_as_tree,
-                            opt_construct_counter_example,
-                            opt_use_hashtables);
-          break;
-        }
-        case quantifier_all:
-        {
-          data::number_postfix_generator generator("UNIQUE_PREFIX");
-          data::data_enumerator<> datae(p.data(), datar, generator);
-          data::rewriter_with_variables datarv(datar);
-          const bool enumerate_infinite_sorts1 = false;
-          enumerate_quantifiers_rewriter<pbes_expression, data::rewriter_with_variables,
-                                                             data::data_enumerator<> >
-                          pbesr1(datarv, datae, enumerate_infinite_sorts1);
-          pbes_rewrite(p,pbesr1);  // Simplify p such that this does not need to be done
-                               // repeatedly, without expanding quantifiers over infinite
-                               // domains.
-          const bool enumerate_infinite_sorts2 = true;
-          enumerate_quantifiers_rewriter<pbes_expression, data::rewriter_with_variables,
-                                                             data::data_enumerator<> >
-                          pbesr2(datarv, datae, enumerate_infinite_sorts2);
-          bes_equations=::bes::boolean_equation_system(
-                            p,
-                            pbesr2,
-                            opt_strategy,
-                            opt_store_as_tree,
-                            opt_construct_counter_example,
-                            opt_use_hashtables);
-          break;
-        }
-        case pfnf:
-        {
-          throw mcrl2::runtime_error("The pfnf boolean equation rewriter cannot be used\n");
-        }
-        case prover:
-        {
-          throw mcrl2::runtime_error("The prover based rewriter cannot be used\n");
-        }
-      } */
-
       timer().start("solving");
       bool result=solve_bes(bes_equations,
                             opt_use_hashtables,
@@ -309,48 +247,14 @@ class pbes2bool_tool: public pbes_rewriter_tool<rewriter_tool<pbes_input_tool<in
 
       if (opt_construct_counter_example)
       {
-        bes_equations.print_counter_example(opt_counter_example_file);
+        bes_equations.print_counter_example(opt_counter_example_file,result);
       }
       return true;
     }
 
 };
 
-class pbes2bool_gui_tool: public mcrl2_gui_tool<pbes2bool_tool>
-{
-  public:
-    pbes2bool_gui_tool()
-    {
-
-      std::vector<std::string> values;
-
-      m_gui_options["counter"] = create_checkbox_widget();
-      m_gui_options["hashtables"] = create_checkbox_widget();
-
-      values.clear();
-      values.push_back("simplify");
-      values.push_back("quantifier-all");
-      values.push_back("quantifier-finite");
-      values.push_back("pfnf");
-      m_gui_options["pbes-rewriter"] = create_radiobox_widget(values);
-
-      add_rewriter_widget();
-
-      values.clear();
-      values.push_back("0");
-      values.push_back("1");
-      values.push_back("2");
-      values.push_back("3");
-      m_gui_options["strategy"] = create_radiobox_widget(values);
-
-      m_gui_options["tree"] = create_checkbox_widget();
-      m_gui_options["unused_data"] = create_checkbox_widget();
-    }
-};
-
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-
-  return pbes2bool_gui_tool().execute(argc, argv);
+  return pbes2bool_tool().execute(argc, argv);
 }
