@@ -8,16 +8,14 @@
 //
 /// \file pbesabsint.cpp
 
-#include "boost.hpp" // precompiled headers
-
 #include <iostream>
 #include <string>
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/pbes/absinthe_strategy.h"
 #include "mcrl2/pbes/tools.h"
 #include "mcrl2/utilities/exception.h"
 #include "mcrl2/utilities/input_output_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
+#include "mcrl2/utilities/pbes_output_tool.h"
+#include "mcrl2/utilities/pbes_input_tool.h"
 
 using namespace mcrl2;
 using namespace mcrl2::log;
@@ -26,10 +24,10 @@ using namespace mcrl2::core;
 using namespace mcrl2::utilities;
 using namespace mcrl2::utilities::tools;
 
-class pbes_absinthe_tool: public input_output_tool
+class pbes_absinthe_tool: public pbes_input_tool<pbes_output_tool<input_output_tool> >
 {
   protected:
-    typedef input_output_tool super;
+    typedef pbes_input_tool<pbes_output_tool<input_output_tool> > super;
 
     std::string m_abstraction_file;
     bool m_print_used_function_symbols;
@@ -86,6 +84,8 @@ class pbes_absinthe_tool: public input_output_tool
 
       pbesabsinthe(input_filename(),
                    output_filename(),
+                   pbes_input_format(),
+                   pbes_output_format(),
                    m_abstraction_file,
                    m_strategy,
                    m_print_used_function_symbols,
@@ -96,23 +96,7 @@ class pbes_absinthe_tool: public input_output_tool
 
 };
 
-class pbes_absint_gui_tool: public mcrl2_gui_tool<pbes_absinthe_tool>
-{
-  public:
-    pbes_absint_gui_tool()
-    {
-      m_gui_options["data"] = create_filepicker_widget("Text Files (*.txt)|*.txt|mCRL2 files (*.mcrl2)|*.mcrl2|All Files (*.*)|*.*");
-      std::vector<std::string> values;
-      values.clear();
-      values.push_back("over");
-      values.push_back("under");
-      // TODO: finish the GUI tool
-    }
-};
-
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  pbes_absint_gui_tool tool;
-  return tool.execute(argc, argv);
+  return pbes_absinthe_tool().execute(argc, argv);
 }
