@@ -8,19 +8,17 @@
 //
 /// \file pbesconstelm.cpp
 
-#include "boost.hpp" // precompiled headers
-
 // #define MCRL2_PBES_CONSTELM_DEBUG
 // #define MCRL2_PBES_EXPRESSION_BUILDER_DEBUG
 
 #include <iostream>
 #include <string>
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/pbes/tools.h"
 #include "mcrl2/utilities/input_output_tool.h"
 #include "mcrl2/utilities/rewriter_tool.h"
 #include "mcrl2/utilities/pbes_rewriter_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
+#include "mcrl2/utilities/pbes_input_tool.h"
+#include "mcrl2/utilities/pbes_output_tool.h"
 
 using namespace mcrl2;
 using namespace mcrl2::log;
@@ -29,10 +27,10 @@ using namespace mcrl2::core;
 using namespace mcrl2::utilities;
 using namespace mcrl2::utilities::tools;
 
-class pbes_constelm_tool: public pbes_rewriter_tool<rewriter_tool<input_output_tool> >
+class pbes_constelm_tool: public pbes_input_tool<pbes_output_tool<pbes_rewriter_tool<rewriter_tool<input_output_tool> > > >
 {
   protected:
-    typedef pbes_rewriter_tool<rewriter_tool<input_output_tool> > super;
+    typedef pbes_input_tool<pbes_output_tool<pbes_rewriter_tool<rewriter_tool<input_output_tool> > > > super;
 
     bool m_compute_conditions;
     bool m_remove_redundant_equations;
@@ -72,6 +70,8 @@ class pbes_constelm_tool: public pbes_rewriter_tool<rewriter_tool<input_output_t
 
       pbesconstelm(input_filename(),
                    output_filename(),
+                   pbes_input_format(),
+                   pbes_output_format(),
                    rewrite_strategy(),
                    rewriter_type(),
                    m_compute_conditions,
@@ -82,32 +82,7 @@ class pbes_constelm_tool: public pbes_rewriter_tool<rewriter_tool<input_output_t
 
 };
 
-class pbes_constelm_gui_tool: public mcrl2_gui_tool<pbes_constelm_tool>
-{
-  public:
-    pbes_constelm_gui_tool()
-    {
-
-      std::vector<std::string> values;
-
-      m_gui_options["compute-conditions"] = create_checkbox_widget();
-      m_gui_options["remove-equations"] = create_checkbox_widget();
-
-      values.clear();
-      values.push_back("simplify");
-      values.push_back("quantifier-all");
-      values.push_back("quantifier-finite");
-      values.push_back("pfnf");
-      m_gui_options["pbes-rewriter"] = create_radiobox_widget(values);
-
-      add_rewriter_widget();
-
-    }
-};
-
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  pbes_constelm_gui_tool tool;
-  return tool.execute(argc, argv);
+  return pbes_constelm_tool().execute(argc, argv);
 }
