@@ -27,11 +27,9 @@
 
 //MCRL2-specific
 #include "mcrl2/utilities/logger.h"
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/pbes/tools.h"
 #include "mcrl2/utilities/input_tool.h"
 #include "mcrl2/utilities/pbes_input_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
 
 using namespace std;
 using namespace mcrl2;
@@ -93,17 +91,7 @@ class pbesinfo_tool: public pbes_input_tool<input_tool>
     }
 };
 
-class pbesinfo_gui_tool: public mcrl2_gui_tool<pbesinfo_tool>
-{
-  public:
-    pbesinfo_gui_tool()
-    {
-      m_gui_options["full"] = create_checkbox_widget();
-    }
-};
-
 int main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  return pbesinfo_gui_tool().execute(argc, argv);
+  return pbesinfo_tool().execute(argc, argv);
 }
