@@ -1,7 +1,7 @@
-// Copyright (c) 2009-2011 University of Twente
-// Copyright (c) 2009-2011 Michael Weber <michaelw@cs.utwente.nl>
-// Copyright (c) 2009-2011 Maks Verver <maksverver@geocities.com>
-// Copyright (c) 2009-2011 Eindhoven University of Technology
+// Copyright (c) 2009-2013 University of Twente
+// Copyright (c) 2009-2013 Michael Weber <michaelw@cs.utwente.nl>
+// Copyright (c) 2009-2013 Maks Verver <maksverver@geocities.com>
+// Copyright (c) 2009-2013 Eindhoven University of Technology
 //
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
@@ -13,7 +13,8 @@
 /*! Mix-in class for classes whose operations can be aborted asynchronously.
 
     Classes inheriting Abortable should periodically check whether they are
-    aborted by calling aborted() in time-consuming procedures. */
+    aborted by calling aborted() in time-consuming procedures.
+*/
 class Abortable
 {
 public:
@@ -24,6 +25,7 @@ public:
     bool aborted() { return global_abort_; }
 
 private:
+    //! Global variable indicating whether the process has been aborted.
     static volatile bool global_abort_;
 };
 
