@@ -1,25 +1,29 @@
-// Copyright (c) 2009-2011 University of Twente
-// Copyright (c) 2009-2011 Michael Weber <michaelw@cs.utwente.nl>
-// Copyright (c) 2009-2011 Maks Verver <maksverver@geocities.com>
-// Copyright (c) 2009-2011 Eindhoven University of Technology
+// Copyright (c) 2009-2013 University of Twente
+// Copyright (c) 2009-2013 Michael Weber <michaelw@cs.utwente.nl>
+// Copyright (c) 2009-2013 Maks Verver <maksverver@geocities.com>
+// Copyright (c) 2009-2013 Eindhoven University of Technology
 //
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
 
-#include "ParityGame.h"
+#include <cassert>
 #include "mcrl2/utilities/logger.h"
-#include <assert.h>
+#include <mcrl2/data/rewrite_strategy.h>
+#include <mcrl2/pbes/io.h>
+#include <mcrl2/pbes/parity_game_generator.h>
+#include "ParityGame.h"
+
 
 /* N.B. The PGSolver I/O functions reverse the priorities when reading/writing
    the game description. This is done to preserve solutions, since PGSolver
    considers higher values to dominate lower values, while I assume the opposite
-   (i.e. 0 is the `highest' priority) throughout the rest of the code. */
+   (i.e. 0 is the `highest` priority) throughout the rest of the code. */
 
 void ParityGame::read_pgsolver( std::istream &is,
                                 StaticGraph::EdgeDirection edge_dir )
 {
-    int max_prio = 0;
+    priority_t max_prio = 0;
     std::vector<ParityGameVertex> vertices;
     StaticGraph::edge_list edges;
 
@@ -56,18 +60,23 @@ void ParityGame::read_pgsolver( std::istream &is,
     }
 
     // Invalid vertex (used to mark uninitialized vertices)
-    ParityGameVertex invalid = { (unsigned char)-1, (unsigned char)-1 };
+    ParityGameVertex invalid = { PLAYER_EVEN, (priority_t)-1 };
 
     // Read vertex specs
     while (is)
     {
         verti id;
-        int prio, player;
-        if (!(is >> id >> prio >> player)) break;
+        int prio_raw, player_raw;
+
+        if (!(is >> id >> prio_raw >> player_raw)) break;
+
+        assert(prio_raw >= 0);
+        assert(prio_raw < 65536);
+        priority_t prio = prio_raw;
+
+        assert(player_raw == 0 || player_raw == 1);
+        player_t player = static_cast<player_t>(player_raw);
 
-        assert(prio >= 0);
-        assert(prio < 65536);
-        assert(player == 0 || player == 1);
         if (prio > max_prio) max_prio = prio;
         if (id >= vertices.size()) vertices.resize(id + 1, invalid);
 
@@ -156,7 +165,7 @@ void ParityGame::write_pgsolver(std::ostream &os) const
         os << v << ' ' << (max_prio - priority(v)) << ' ' << player(v);
         StaticGraph::const_iterator it  = graph_.succ_begin(v),
                                     end = graph_.succ_end(v);
-        assert(it != end);
+        assert(it < end);
         os << ' ' << *it++;
         while (it != end) os << ',' << *it++;
         os << ";\n";
@@ -164,19 +173,65 @@ void ParityGame::write_pgsolver(std::ostream &os) const
 }
 
 void ParityGame::read_pbes( const std::string &file_path, verti *goal_vertex,
-                            StaticGraph::EdgeDirection edge_dir )
+                            StaticGraph::EdgeDirection edge_dir,
+                            const std::string &rewrite_strategy )
 {
-#ifdef WITH_MCRL2
-    mcrl2::pbes_system::pbes<> pbes;
-    pbes.load(file_path);  // TODO: handle exceptions raised here?
-    assign_pbes(pbes, goal_vertex, edge_dir);
-#else /* ifdef WITH_MCRL2 */
-    (void)file_path;    // unused
-    (void)edge_dir;     // unused
-    (void)goal_vertex;  // unused
-    Logger::fatal( "ParityGame::read_pbes() called, but "
-                   "compiled without mCRL2 support!\n" );
-#endif /* def WITH_MCRL2 */
+    mcrl2::pbes_system::pbes pbes;
+    load_pbes(pbes, file_path);
+    assign_pbes(pbes, goal_vertex, edge_dir, rewrite_strategy);
+}
+
+void ParityGame::assign_pbes(mcrl2::pbes_system::pbes &pbes, verti *goal_vertex,
+                             StaticGraph::EdgeDirection edge_dir,
+                             const std::string &rewrite_strategy)
+{
+    /* NOTE: this code assumes the vertices generated by parity_game_generator
+             are numbered from 2 to num_vertices-1 with no gaps, with 0 and 1
+             representing true and false (respectively) and 2 representing the
+             initial condition. */
+
+    if (goal_vertex) *goal_vertex = 2;
+
+    // Generate min-priority parity game
+    mcrl2::pbes_system::parity_game_generator pgg( pbes, true, true,
+        mcrl2::data::parse_rewrite_strategy(rewrite_strategy) );
+
+    // Build the edge list
+    StaticGraph::edge_list edges;
+    verti begin = 0, end = 3;
+    for (verti v = begin; v < end; ++v)
+    {
+        std::set<size_t> deps = pgg.get_dependencies(v);
+        for ( std::set<size_t>::const_iterator it = deps.begin();
+                it != deps.end(); ++it )
+        {
+            verti w = (verti)*it;
+            assert(w >= begin);
+            if (w >= end) end = w + 1;
+            edges.push_back(std::make_pair(v - begin, w - begin));
+        }
+    }
+
+    // Determine maximum prioirity
+    int max_prio = 0;
+    for (verti v = begin; v < end; ++v)
+    {
+        max_prio = std::max(max_prio, (int)pgg.get_priority(v));
+    }
+
+    // Assign vertex info and recount cardinalities
+    reset(end - begin, max_prio + 1);
+    for (verti v = begin; v < end; ++v)
+    {
+        bool and_op = pgg.get_operation(v) ==
+                        mcrl2::pbes_system::parity_game_generator::PGAME_AND;
+        vertex_[v - begin].player = and_op ? PLAYER_ODD : PLAYER_EVEN;
+        vertex_[v - begin].priority = pgg.get_priority(v);
+    }
+    recalculate_cardinalities(end - begin);
+
+    // Assign graph
+    graph_.assign(edges, edge_dir);
 }
 
 void ParityGame::read_raw(std::istream &is)
@@ -204,7 +259,7 @@ void ParityGame::write_dot(std::ostream &os) const
     os << "digraph {\n";
     for (verti v = 0; v < graph_.V(); ++v)
     {
-        bool even = player(v) == ParityGame::PLAYER_EVEN;
+        bool even = player(v) == PLAYER_EVEN;
         os << v << " ["
            << "shape=" << (even ? "diamond" : "box") << ", "
            << "label=\"" << priority(v) << " (" << v << ")\"]\n";
@@ -214,7 +269,9 @@ void ParityGame::write_dot(std::ostream &os) const
             for ( StaticGraph::const_iterator it = graph_.succ_begin(v);
                   it != graph_.succ_end(v); ++it )
             {
-                os << v << " -> " << *it << ";\n";
+                os << v << " -> " << *it;
+                // if (*it < v) os << " [color=red]";
+                os << ";\n";
             }
         }
         else
