@@ -11,8 +11,6 @@
 //#define MCRL2_ENUMERATE_QUANTIFIERS_BUILDER_DEBUG
 //#define MCRL2_PBES_EXPRESSION_BUILDER_DEBUG
 
-#include "boost.hpp" // precompiled headers
-
 #include <fstream>
 #include <iostream>
 #include <string>
@@ -26,16 +24,13 @@
 #include "mcrl2/utilities/pbes_input_tool.h"
 #include "mcrl2/utilities/rewriter_tool.h"
 #include "mcrl2/utilities/execution_timer.h"
-#include "mcrl2/atermpp/aterm_init.h"
-//#include "mcrl2/utilities/rewriter_tool.h"
-//#include "mcrl2/utilities/pbes_rewriter_tool.h"
 
 #include "mcrl2/pbes/pbespgsolve.h"
-#include "mcrl2/pbes/io.h"
+#include "mcrl2/pbes/detail/bes_equation_limit.h"
 #include "mcrl2/bes/boolean_equation_system.h"
 #include "mcrl2/bes/bes2pbes.h"
 #include "mcrl2/bes/pg_parse.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
+#include "mcrl2/bes/io.h"
 
 using namespace mcrl2;
 using namespace mcrl2::pbes_system;
@@ -122,10 +117,27 @@ class pg_solver_tool : public rewriter_tool<pbes_input_tool<input_tool> >
       mCRL2log(verbose) << "  verify solution:   " << std::boolalpha << m_options.verify_solution << std::endl;
       mCRL2log(verbose) << "  only generate:   " << std::boolalpha << m_options.only_generate << std::endl;
 
-      pbes<> p;
-      load_pbes(p, input_filename(), pbes_input_format());
+      bool value;
+      if(pbes_input_format() == bes::bes_format_pgsolver())
+      {
+        pbespgsolve_algorithm algorithm(timer(), m_options);
+        ParityGame pg;
+        std::ifstream is(input_filename().c_str());
+        timer().start("load");
+        pg.read_pgsolver(is);
+        timer().finish("load");
 
-      bool value = pbespgsolve(p, timer(), m_options);
+        value = algorithm.run(pg, 0);
+      }
+      else
+      {
+        pbes p;
+        timer().start("load");
+        mcrl2::bes::load_pbes(p, input_filename(), pbes_input_format());
+        timer().finish("load");
+
+        value = pbespgsolve(p, timer(), m_options);
+      }
       std::string result = (value ? "true" : "false");
       mCRL2log(verbose) << "The solution for the initial variable of the pbes is " << result << std::endl;
       std::cout << result << std::endl;
@@ -134,26 +146,7 @@ class pg_solver_tool : public rewriter_tool<pbes_input_tool<input_tool> >
     }
 };
 
-class pg_solver_gui_tool: public mcrl2_gui_tool<pg_solver_tool>
-{
-  public:
-    pg_solver_gui_tool()
-    {
-      m_gui_options["verify"] = create_checkbox_widget();
-
-			std::vector<std::string> values;
-
-      values.clear();
-      values.push_back("spm");
-      values.push_back("altspm");
-      values.push_back("recursive");
-      m_gui_options["solver-type"] = create_radiobox_widget(values);
-    }
-};
-
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  pg_solver_gui_tool tool;
-  return tool.execute(argc, argv);
+  return pg_solver_tool().execute(argc, argv);
 }
