@@ -12,11 +12,8 @@
 #ifndef MCRL2_TOOLS_PBESPGSOLVE_H
 #define MCRL2_TOOLS_PBESPGSOLVE_H
 
-#ifndef WITH_MCRL2
-#define WITH_MCRL2
-#endif
-
 #include <iostream>
+#include "mcrl2/data/rewriter.h"
 #include "mcrl2/utilities/execution_timer.h"
 #include "ParityGame.h"
 #include "SmallProgressMeasures.h"
@@ -168,7 +165,7 @@ class pbespgsolve_algorithm
         // Create a SPM solver factory:
         solver_factory.reset(
           new SmallProgressMeasuresSolverFactory
-                (new PredecessorLiftingStrategyFactory, alternative_solver)
+                (new PredecessorLiftingStrategyFactory, 2, alternative_solver)
         );
       }
       else if (options.solver_type == recursive_solver)
@@ -188,11 +185,6 @@ class pbespgsolve_algorithm
           new ComponentSolverFactory(*solver_factory.release()));
       }
 
-      if (options.use_decycle_solver && options.use_deloop_solver)
-      {
-        throw mcrl2::runtime_error("pbespgsolve: cannot use self-loop removal and cycle removal simultaneously");
-      }
-
       if (options.use_decycle_solver)
       {
         solver_factory.reset(
@@ -206,26 +198,14 @@ class pbespgsolve_algorithm
       }
     }
 
-    template <typename Container>
-    bool run(pbes<Container>& p)
+    bool run(ParityGame& pg, const verti goal_v)
     {
-      m_timer.start("initialization");
-      mCRL2log(log::verbose) << "Generating parity game..."  << std::endl;
-      // Generate the game from a PBES:
-      verti goal_v;
-      ParityGame pg;
-
-      pg.assign_pbes(p, &goal_v, StaticGraph::EDGE_BIDIRECTIONAL, m_options.rewrite_strategy); // N.B. mCRL2 could raise an exception here
-
-      mCRL2log(log::verbose) << "Game: " << pg.graph().V() << " vertices, " << pg.graph().E() << " edges." << std::endl;
-
       if (!m_options.only_generate)
       {
         mCRL2log(log::verbose) << "Solving..." << std::endl;
 
         // Create a solver:
         std::auto_ptr<ParityGameSolver> solver(solver_factory->create(pg));
-        m_timer.finish("initialization");
 
         // Solve the game:
         m_timer.start("solving");
@@ -245,10 +225,25 @@ class pbespgsolve_algorithm
           throw mcrl2::runtime_error("pbespgsolve: verification of the solution failed!\n");
         }
 
-        return pg.winner(solution, goal_v) == ParityGame::PLAYER_EVEN;
+        return pg.winner(solution, goal_v) == PLAYER_EVEN;
       }
       return true;
     }
+
+    bool run(pbes& p)
+    {
+      m_timer.start("initialization");
+      mCRL2log(log::verbose) << "Generating parity game..."  << std::endl;
+      // Generate the game from a PBES:
+      verti goal_v;
+      ParityGame pg;
+
+      pg.assign_pbes(p, &goal_v, StaticGraph::EDGE_BIDIRECTIONAL, data::pp(m_options.rewrite_strategy)); // N.B. mCRL2 could raise an exception here
+      mCRL2log(log::verbose) << "Game: " << pg.graph().V() << " vertices, " << pg.graph().E() << " edges." << std::endl;
+      m_timer.finish("initialization");
+
+      return run(pg, goal_v);
+    }
 };
 
 } // namespace pbes_system
