@@ -8,8 +8,6 @@
 //
 /// \file pbespp.cpp
 
-#include "boost.hpp" // precompiled headers
-
 #define NAME "pbespp"
 #define AUTHOR "Aad Mathijssen and Jeroen Keiren"
 
@@ -18,10 +16,8 @@
 #include <fstream>
 
 #include "mcrl2/utilities/logger.h"
-#include "mcrl2/atermpp/aterm_init.h"
 #include "mcrl2/utilities/input_output_tool.h"
 #include "mcrl2/utilities/pbes_input_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
 #include "mcrl2/pbes/tools.h"
 
 using namespace mcrl2::log;
@@ -48,11 +44,11 @@ class pbespp_tool: public pbes_input_tool<input_output_tool>
 
     bool run()
     {
-      pbespp(input_filename(),
-             output_filename(),
-             pbes_input_format(),
-             format
-            );
+      pbes_system::pbespp(input_filename(),
+                          output_filename(),
+                          pbes_input_format(),
+                          format
+                         );
       return true;
     }
 
@@ -75,24 +71,8 @@ class pbespp_tool: public pbes_input_tool<input_output_tool>
     }
 };
 
-class pbespp_gui_tool: public mcrl2_gui_tool<pbespp_tool>
-{
-  public:
-    pbespp_gui_tool()
-    {
-
-      std::vector<std::string> values;
-
-      values.clear();
-      values.push_back("default");
-      values.push_back("internal-debug");
-      m_gui_options["format"] = create_radiobox_widget(values);
-    }
-};
-
 int main(int argc, char* argv[])
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  return pbespp_gui_tool().execute(argc, argv);
+  return pbespp_tool().execute(argc, argv);
 }
 
