@@ -9,8 +9,6 @@
 /// \file txt2pbes.cpp
 /// \brief Parse a textual description of a BES.
 
-#include "boost.hpp" // precompiled headers
-
 #define NAME "txt2bes"
 #define AUTHOR "Wieger Wesselink"
 
@@ -22,21 +20,21 @@
 //mCRL2 specific
 #include "mcrl2/utilities/text_utility.h"
 #include "mcrl2/utilities/input_output_tool.h"
-#include "mcrl2/utilities/mcrl2_gui_tool.h"
+#include "mcrl2/utilities/pbes_output_tool.h"
 #include "mcrl2/pbes/txt2pbes.h"
-#include "mcrl2/pbes/pbesinstconversion.h"
-#include "mcrl2/atermpp/aterm_init.h"
+#include "mcrl2/bes/pbesinst_conversion.h"
+#include "mcrl2/bes/io.h"
 
 using namespace mcrl2;
 using namespace mcrl2::utilities;
 using namespace mcrl2::utilities::tools;
 
-class txt2pbes_tool: public input_output_tool
+class txt2bes_tool: public bes_output_tool<input_output_tool>
 {
-    typedef input_output_tool super;
+    typedef bes_output_tool<input_output_tool> super;
 
   public:
-    txt2pbes_tool()
+    txt2bes_tool()
       : super(NAME, AUTHOR,
               "parse a textual description of a BES",
               "Parse the textual description of a BES from INFILE and write it to OUTFILE. "
@@ -46,7 +44,7 @@ class txt2pbes_tool: public input_output_tool
 
     bool run()
     {
-      pbes_system::pbes<> p;
+      pbes_system::pbes p;
       if (input_filename().empty())
       {
         p = pbes_system::txt2pbes(std::cin);
@@ -56,21 +54,13 @@ class txt2pbes_tool: public input_output_tool
         std::ifstream from(input_filename().c_str());
         p = pbes_system::txt2pbes(from);
       }
-      bes::boolean_equation_system<> b = pbes_system::pbesinstconversion(p);
-      b.save(output_filename());
+      bes::boolean_equation_system b = bes::pbesinst_conversion(p);
+      bes::save_bes(b, output_filename(), bes_output_format());
       return true;
     }
 };
 
-class txt2pbes_gui_tool: public mcrl2_gui_tool<txt2pbes_tool>
-{
-  public:
-    txt2pbes_gui_tool() {}
-};
-
-
 int main(int argc, char** argv)
 {
-  MCRL2_ATERMPP_INIT(argc, argv)
-  return txt2pbes_gui_tool().execute(argc, argv);
+  return txt2bes_tool().execute(argc, argv);
 }
